/*
 * filedialog.c
 * File Dialog Callbacks and Stuff
 *
 * All file dialog code borrowed HEAVILY from bluefish (see below). Some 
 * slight modifications by me (Luke Freeman - LF) to conform to whiteBOX 
 * uniform look and feel. 
 *
 * Download bluefish at http://bluefish.openoffice.nl
 *
 * Copyright 2000 - 2002, Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 *
 *
 * Bluefish HTML Editor
 *
 * Copyright (C) 1999-2001 Olivier Sessink
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>

#ifdef HAVE_SYS_DIR_H
    #include <sys/dir.h>
#else
    #ifdef HAVE_SYS_NDIR_H
        #include <sys/ndir.h>
    #else
        #include <dirent.h>
    #endif
#endif

#include <sys/stat.h>
#include <sys/types.h>



#include <unistd.h>

#include "filedialog.h"
#include "whiteBOX.h"
#include "msgbox.h"

#define DIRSTR "/"
#define DIRCHR '/'

typedef struct {
	gint multipleselect;
	gchar *filename_to_return;
	GList *filenames_to_return;
	GtkWidget *fs;
} Tfileselect;

/* gchar *ending_slash(gchar *dirname)
 * returns a newly allocated string which is 
 * guaranteed to end on a slash (/) */
gchar *ending_slash(gchar *dirname) {
	/* result should be free-ed !!! */
	if (!dirname) {
		return g_strdup("");
	}

	if (dirname[strlen(dirname)-1] == DIRCHR) {
		return g_strdup(dirname);
	} else {
		return g_strconcat(dirname, DIRSTR, NULL);
	}
}

static void fs_ok_clicked_lcb(GtkWidget * widget, Tfileselect *fileselect)
{
	gchar *dirname, *selected_file;

	selected_file = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect->fs));
  
	if (dir_stat(selected_file)) {
		dirname = ending_slash(selected_file);
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselect->fs), dirname);
		gtk_entry_set_text(GTK_ENTRY(GTK_FILE_SELECTION(fileselect->fs)->selection_entry), "");
		g_free(dirname);
		return;
	}   
  
	if (fileselect->multipleselect) {
		/* multiple files allowed --> scan trough the list for selections */
		dirname = g_dirname(selected_file);
		{
		gint rownum = 0;
		gchar *temp, *totalfilename;
		GList *row = GTK_CLIST(GTK_FILE_SELECTION(fileselect->fs)->file_list)->row_list;
		while (row) {
			if (GTK_CLIST_ROW(row)->state == GTK_STATE_SELECTED) {
				gtk_clist_get_text(GTK_CLIST(GTK_FILE_SELECTION(fileselect->fs)->file_list), rownum, 0, &temp);

				totalfilename = g_strconcat(dirname, DIRSTR, temp, NULL);
				fileselect->filenames_to_return = g_list_append(fileselect->filenames_to_return, totalfilename);
			}

			rownum++;
			row = g_list_next(row);
		}
		}
		g_free(dirname);
	} else {
		/* NO multiple files allowed --> return just one file */
		fileselect->filename_to_return = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect->fs)));
	}
	if ((fileselect->multipleselect 
			&& (g_list_length(fileselect->filenames_to_return) < 1)) 
		|| (!fileselect->multipleselect 
			&& strlen(fileselect->filename_to_return) < 1)){
		
    /* modified to use whiteBOX message box - LF */    
    message_box( 4, "You did not select a filename.", "No file selected" );
	}
	else
	{
	  /* modified so the dialog doesnt close when no file selected - LF */    
    gtk_main_quit();
	  window_destroy(GTK_WIDGET(fileselect->fs));
  }
}

static void close_modal_window_lcb(GtkWidget * widget, gpointer window)
{
	gtk_main_quit();
	window_destroy(window);
}

static void fileselectwin(gchar *setfile, Tfileselect *fileselect, gchar *title) {

	fileselect->fs = gtk_file_selection_new(title);
	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fileselect->fs));
	gtk_signal_connect(GTK_OBJECT(fileselect->fs), "destroy", close_modal_window_lcb, fileselect->fs);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileselect->fs)->cancel_button),
					   "clicked", close_modal_window_lcb, fileselect->fs);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileselect->fs)->ok_button), "clicked", (GtkSignalFunc) fs_ok_clicked_lcb, fileselect);
	if (fileselect->multipleselect) {
		gtk_clist_set_selection_mode(GTK_CLIST(GTK_FILE_SELECTION(fileselect->fs)->file_list), GTK_SELECTION_EXTENDED);
	}
	if (setfile) {
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselect->fs), setfile);
	}
	gtk_window_set_wmclass(GTK_WINDOW(fileselect->fs), "Bluefish", "fileselect");
	gtk_widget_show(fileselect->fs);
	gtk_grab_add(GTK_WIDGET(fileselect->fs));
	gtk_widget_realize(GTK_WIDGET(fileselect->fs));
	//gtk_window_set_transient_for(GTK_WINDOW(fileselect->fs), GTK_WINDOW(main_v->main_window));
}

gchar *return_file_w_title(gchar * setfile, gchar *title) {
	Tfileselect fileselect={0, NULL, NULL, NULL};

	fileselectwin(setfile, &fileselect, title);
	gtk_main();
	return fileselect.filename_to_return;
}

gchar *return_file(gchar * setfile) {
	return return_file_w_title(setfile, "Select file");
}

GList *return_files_w_title(gchar * setfile, gchar *title) {
	Tfileselect fileselect={1, NULL, NULL, NULL};
	
	fileselectwin(setfile, &fileselect, title);
	gtk_main();
	return fileselect.filenames_to_return;
}

GList *return_files(gchar * setfile) {
	return return_files_w_title(setfile, "Select files");
}
