/*
 * main.c
 * Main entry point and application creation 
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <dlfcn.h>

#include <stdio.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <assert.h>
#include <ctype.h>
#include <strings.h>
#include <string.h>

#ifdef HAVE_SYS_DIR_H
    #include <sys/dir.h>
#else
    #ifdef HAVE_SYS_NDIR_H
        #include <sys/ndir.h>
    #else
        #include <dirent.h>
    #endif
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "msgbox.h"
#include "wbmain.h"
#include "interface.h"
#include "support.h"
#include "plugin.h"
#include "whiteBOX.h"

char *str_plugintname = "";

void
enable_plugin( int plug_num, void *openso, GtkWidget *ptr_to_main, 
               const char *plg_name, char *plg_loc, int argc, char* argv[] )
{
    
  void *(*get_plug)(void);
  
  if(( get_plug = dlsym( openso, "get_plugin_data" )) != NULL )
  {
    plugin_data *pp = calloc( 1, sizeof( plugin_data ));  
    GtkWidget* (*run_plug)(GtkWidget *wgt_main);
    void (*init)(GtkWidget *wnd_main, int argc, char* argv[]);
    
    GtkWidget* wgt_nbkmain = lookup_widget( ptr_to_main, "nbk_main" );
    GtkWidget* lbl_tabpage;    
    
    pp = (plugin_data *) get_plug();
    pp->plugin_handle = openso;

    /* grab main plugin data */
    run_plug = dlsym( openso, "entry_point" );
    pp->plugin_entry = run_plug( ptr_to_main );    
        
    /* print debugging stuff */
    if( DEBUG_FLAG )
    {
      g_print( "Plugin Information\n" );
      g_print( "------------------\n" );
      g_print( " - internal name: %s\n",   pp->plugin_intname );
      g_print( " - description:   %s\n",   pp->plugin_description );
      g_print( " - author:        %s\n\n", pp->plugin_author );
      g_print( " - posn:          %d\n\n", plug_num );
    }
    str_plugintname = pp->plugin_intname;
    
    gtk_container_add( GTK_CONTAINER( wgt_nbkmain ), pp->plugin_entry );

    lbl_tabpage = gtk_label_new ( plg_name );
    gtk_widget_set_name (lbl_tabpage, "lbl_tabpage");
    gtk_widget_ref (lbl_tabpage);
    gtk_object_set_data_full (GTK_OBJECT (ptr_to_main), "lbl_tabpage", lbl_tabpage,
                             (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (lbl_tabpage);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_nbkmain), 
    gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_nbkmain), plug_num), 
                               lbl_tabpage);

    /* do initialisation */
    if(( init = dlsym( openso, "init" )) != NULL ) {
      init( (GtkWidget *) ptr_to_main, argc, argv );
    }
    
    pp->plugin_location = plg_loc;
    pp->plugin_status = "idle";
    
    lst_plugins = g_list_append( lst_plugins, pp );
  
  }
  else 
  {
    char msg[255];
    
    strcpy( msg, "Could not open the following plugin:\n\n" );
    strcat( msg, plg_name );
    strcat( msg, "\n\nThe plugin will NOT be loaded.\nPlease verify the plugin details and reload whiteBOX." );
    
    message_box( 4, msg, "Error loading plugin .." ); 
  }

}

int
main (int argc, char *argv[])
{
  
  GtkWidget *wnd_main;
  GtkWidget *wgt_nbkmain;

  void *openso;

  int i = 0;     /* counter */

  FILE *in_file;
  gchar row[2][250];
  gchar *tt[1][2];
  gchar *substring;

  char image_line[1024];
  char image_line2[1024];

  int posn;

  char bgset_dir[255];
  struct stat s;
 
  FILE *cfg_file;
  char cfg_line[255];
  
  char plug_list[255];

  int c = 1;
  
#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
#endif

  gtk_set_locale ();
  gtk_init (&argc, &argv);
  
  add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
  add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");
 
    
  /* create the main window and grab a reference to the menu tree */
  wnd_main  = create_wnd_main ();

  wgt_nbkmain = lookup_widget( wnd_main, "nbk_main" );  

  /* get whitebox configuration directory */
  strncpy( bgset_dir, getenv("HOME"), 255 );
  strncat( bgset_dir, "/.whiteBOX", 255 );

  /* some config files */
  strncpy( plug_list, bgset_dir, 255 );
  strncat( plug_list, "/pluginlist", 255 );


  if( stat( bgset_dir, &s ) == -1 ) 
  {
    
    message_box( 4, "Thanks for trying out whiteBOX ..\n\nThe default configuration files will now be created.",
                 "Making configuration files .." );
    
    /* make directory */
    if( DEBUG_FLAG )
      g_print( "creating whiteBOX directory ..\n" );

    mkdir( bgset_dir, 0700 );   /* create the whiteBOX user dir */
    
    cfg_file = fopen( plug_list, "w" );
    if( cfg_file != NULL )
    {
      if( DEBUG_FLAG )
        g_print( "setting configuration defaults ..\n" );
        
      strncpy( cfg_line, "Menu Editor;", 255 );
      strncat( cfg_line, PLUGIN_DIR "/libwbmenu.so", 255 );
      fprintf( cfg_file, "%s\n", cfg_line );
      
      strncpy( cfg_line, "Style Editor;", 255 );
      strncat( cfg_line, PLUGIN_DIR "/libwbstyle.so", 255 );
      fprintf( cfg_file, "%s\n", cfg_line );

      strncpy( cfg_line, "Background Tool;", 255 );
      strncat( cfg_line, PLUGIN_DIR "/libwbbgset.so", 255 );
      fprintf( cfg_file, "%s\n", cfg_line );

      strncpy( cfg_line, "Keybinding Tool;", 255 );
      strncat( cfg_line, PLUGIN_DIR "/libwbkeybind.so", 255 );
      fprintf( cfg_file, "%s\n", cfg_line );

      fclose( cfg_file );
    }
    else
    {
      if( DEBUG_FLAG ) 
        g_print( "cannot write to file %s ..\n", plug_list );
    }
  }
  
  
  
  
  
  
  /* read in the plug-in information */
  in_file = fopen( plug_list, "r" );
	
  if( in_file != NULL )
  {
    if( DEBUG_FLAG )
      g_print ("file opened ..\n" );
    
    while( !feof( in_file ))
    {
      
      posn = 0;

      if( fgets( image_line, 1024, in_file ))
      {
        strcpy( image_line2, strtok( image_line, "\n" ));
        strcpy( row[posn], strtok( image_line2, ";" ));         
        
        if( DEBUG_FLAG )
          g_print( "0. strn = %s\n", row[posn] );

        while(( substring = strtok( NULL, ";" )) != NULL )
        {
          posn++;
          strcpy( row[posn], substring);

          if( DEBUG_FLAG ) 
          {
            g_print( "posn = %d\n", posn );
            g_print( "strn = %s\n", row[posn] );
          }
	      
        }

        tt[0][0] = row[0];  /* show name */
        tt[0][1] = row[1];  /* show location */
                  
        openso = NULL;
	openso = dlopen( tt[0][1], RTLD_NOW | RTLD_GLOBAL );
  
        if ( openso )
        {
          if( DEBUG_FLAG )
            g_print( "plugin %s..\n", tt[0][1] );
          
          enable_plugin( i, openso, wnd_main, tt[0][0], tt[0][1], argc, argv );

          i++;   /* increment */
        }
        else
        {
          if( DEBUG_FLAG )
          {
            g_print( "err: %s\n", dlerror() );
            g_print( "could not open file ..\n" );
          }
        }  
        
      }

    }
    
    fclose( in_file );  /* close the file */
    
  }


  gtk_widget_show (wnd_main);

  gtk_main ();
  return 0;
}

