/*
 * msgbox.c
 * Message box stuff
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include "msgbox.h"
#include "support.h"

void
on_mbx_button_clicked( GtkWidget *widget, gpointer data )
{

  GtkWidget *this_wnd;

  retval = (int) data;
  gtk_main_quit();

  /* destroy the dialog */
  this_wnd = GTK_WIDGET( GTK_WIDGET( GTK_WIDGET( GTK_WIDGET( GTK_WIDGET( widget )->parent )->parent )->parent )->parent );

  gtk_widget_unrealize( (GtkWidget *) this_wnd );
  gtk_widget_hide( (GtkWidget *) this_wnd );
    
}

int
message_box( int type, const char *content, const char *title )
{


  GtkWidget *dlg_msgbox;
  GtkWidget *vbx_msgbox;
  GtkWidget *hbx_mbxcontent;
  GtkWidget *pxp_mbxicon;
  GtkWidget *lbl_mbxinfo;
  GtkWidget *daa_msgbox;
  GtkWidget *hbx_mbxbtns;
  GtkWidget *btn_mbxyes;
  GtkWidget *btn_mbxno;
  GtkWidget *btn_mbxok;
  GtkWidget *btn_mbxcancel;

  dlg_msgbox = gtk_dialog_new ();
  gtk_widget_set_name (dlg_msgbox, "dlg_msgbox");
  gtk_object_set_data (GTK_OBJECT (dlg_msgbox), "dlg_msgbox", dlg_msgbox);
  gtk_window_set_title (GTK_WINDOW (dlg_msgbox), title );
  gtk_window_set_position (GTK_WINDOW (dlg_msgbox), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (dlg_msgbox), TRUE);
  gtk_window_set_policy (GTK_WINDOW (dlg_msgbox), TRUE, TRUE, FALSE);

  vbx_msgbox = GTK_DIALOG (dlg_msgbox)->vbox;
  gtk_widget_set_name (vbx_msgbox, "vbx_msgbox");
  gtk_object_set_data (GTK_OBJECT (dlg_msgbox), "vbx_msgbox", vbx_msgbox);
  gtk_widget_show (vbx_msgbox);

  hbx_mbxcontent = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbx_mbxcontent, "hbx_mbxcontent");
  gtk_widget_ref (hbx_mbxcontent);
  gtk_object_set_data_full (GTK_OBJECT (dlg_msgbox), "hbx_mbxcontent", hbx_mbxcontent,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_set_border_width (GTK_CONTAINER (hbx_mbxcontent), 15);
  gtk_widget_show (hbx_mbxcontent);
  gtk_box_pack_start (GTK_BOX (vbx_msgbox), hbx_mbxcontent, TRUE, TRUE, 0);

  pxp_mbxicon = create_pixmap (dlg_msgbox, "exclaim1.xpm");
  gtk_widget_set_name (pxp_mbxicon, "pxp_mbxicon");
  gtk_widget_ref (pxp_mbxicon);
  gtk_object_set_data_full (GTK_OBJECT (dlg_msgbox), "pxp_mbxicon", pxp_mbxicon,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pxp_mbxicon);
  gtk_box_pack_start (GTK_BOX (hbx_mbxcontent), pxp_mbxicon, FALSE, TRUE, 12);

  lbl_mbxinfo = gtk_label_new ( content );
  gtk_widget_set_name (lbl_mbxinfo, "lbl_mbxinfo");
  gtk_widget_ref (lbl_mbxinfo);
  gtk_object_set_data_full (GTK_OBJECT (dlg_msgbox), "lbl_mbxinfo", lbl_mbxinfo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_label_set_justify( (GtkLabel *) lbl_mbxinfo, GTK_JUSTIFY_LEFT );
  gtk_widget_show (lbl_mbxinfo);
  gtk_box_pack_start (GTK_BOX (hbx_mbxcontent), lbl_mbxinfo, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_mbxinfo), 0, 0);

  daa_msgbox = GTK_DIALOG (dlg_msgbox)->action_area;
  gtk_widget_set_name (daa_msgbox, "daa_msgbox");
  gtk_object_set_data (GTK_OBJECT (dlg_msgbox), "daa_msgbox", daa_msgbox);
  gtk_widget_show (daa_msgbox);
  gtk_container_set_border_width (GTK_CONTAINER (daa_msgbox), 10);

  hbx_mbxbtns = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbx_mbxbtns, "hbx_mbxbtns");
  gtk_widget_ref (hbx_mbxbtns);
  gtk_object_set_data_full (GTK_OBJECT (dlg_msgbox), "hbx_mbxbtns", hbx_mbxbtns,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbx_mbxbtns);
  gtk_box_pack_start (GTK_BOX (daa_msgbox), hbx_mbxbtns, TRUE, TRUE, 0);

  if( type == 1 || type == 3 )
  {
    btn_mbxyes = gtk_button_new_with_label (("Yes"));
    gtk_widget_set_name (btn_mbxyes, "btn_mbxyes");
    gtk_widget_ref (btn_mbxyes);
    gtk_object_set_data_full (GTK_OBJECT (dlg_msgbox), "btn_mbxyes", btn_mbxyes,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_set_usize( btn_mbxyes, 78, 32 );
    gtk_widget_show (btn_mbxyes);
    gtk_box_pack_start (GTK_BOX (hbx_mbxbtns), btn_mbxyes, FALSE, FALSE, 0);
    GTK_WIDGET_SET_FLAGS (btn_mbxyes, GTK_CAN_DEFAULT);
    
    gtk_signal_connect (GTK_OBJECT (btn_mbxyes), "clicked",
                        GTK_SIGNAL_FUNC (on_mbx_button_clicked),
                        (gpointer) 1 );
    if( type == 1 )
      gtk_widget_grab_default( btn_mbxyes );
    
  }

  if( type == 2 || type == 3 )
  {
    btn_mbxno = gtk_button_new_with_label (("No"));
    gtk_widget_set_name (btn_mbxno, "btn_mbxno");
    gtk_widget_ref (btn_mbxno);
    gtk_object_set_data_full (GTK_OBJECT (dlg_msgbox), "btn_mbxno", btn_mbxno,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_set_usize( btn_mbxno, 78, 32 );
    gtk_widget_show (btn_mbxno);
    gtk_box_pack_start (GTK_BOX (hbx_mbxbtns), btn_mbxno, FALSE, FALSE, 0);
    GTK_WIDGET_SET_FLAGS (btn_mbxno, GTK_CAN_DEFAULT);
    
    gtk_signal_connect (GTK_OBJECT (btn_mbxno), "clicked",
                        GTK_SIGNAL_FUNC (on_mbx_button_clicked),
                        (gpointer) 2 );
    
    if( type == 2 || type == 3 )
      gtk_widget_grab_default( btn_mbxno );
    
    
  }

  if( type == 4 || type == 6 )
  {
    btn_mbxok = gtk_button_new_with_label (("Ok"));
    gtk_widget_set_name (btn_mbxok, "btn_mbxok");
    gtk_widget_ref (btn_mbxok);
    gtk_object_set_data_full (GTK_OBJECT (dlg_msgbox), "btn_mbxok", btn_mbxok,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_set_usize (btn_mbxok, 78, 32);
    gtk_widget_show (btn_mbxok);
    gtk_box_pack_start (GTK_BOX (hbx_mbxbtns), btn_mbxok, FALSE, FALSE, 0);
    GTK_WIDGET_SET_FLAGS (btn_mbxok, GTK_CAN_DEFAULT);
    
    gtk_signal_connect (GTK_OBJECT (btn_mbxok), "clicked",
                        GTK_SIGNAL_FUNC (on_mbx_button_clicked),
                        (gpointer) 4 );

    if( type == 4 )
      gtk_widget_grab_default( btn_mbxok );

    
  }

  if( type == 5 || type == 6 )
  {
    btn_mbxcancel = gtk_button_new_with_label (("Cancel"));
    gtk_widget_set_name (btn_mbxcancel, "btn_mbxcancel");
    gtk_widget_ref (btn_mbxcancel);
    gtk_object_set_data_full (GTK_OBJECT (dlg_msgbox), "btn_mbxcancel", btn_mbxcancel,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_set_usize( btn_mbxcancel, 78, 32 );
    gtk_widget_show (btn_mbxcancel);
    gtk_box_pack_start (GTK_BOX (hbx_mbxbtns), btn_mbxcancel, FALSE, FALSE, 0);
    GTK_WIDGET_SET_FLAGS (btn_mbxcancel, GTK_CAN_DEFAULT);

    gtk_signal_connect (GTK_OBJECT (btn_mbxcancel), "clicked",
                        GTK_SIGNAL_FUNC (on_mbx_button_clicked),
                        (gpointer) 5 );
    
    if( type == 5 || type == 6 )
      gtk_widget_grab_default( btn_mbxcancel );


  }
  
  gtk_widget_show( dlg_msgbox );

  gtk_main();

  return retval;

}
