/*
 * options.c
 * Options screen callbacks and stuff
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>

#ifdef HAVE_SYS_DIR_H
    #include <sys/dir.h>
#else
    #ifdef HAVE_SYS_NDIR_H
        #include <sys/ndir.h>
    #else
        #include <dirent.h>
    #endif
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <dlfcn.h>

#include "options.h"
#include "support.h"
#include "msgbox.h"
#include "interface.h"

#include "filedialog.h"
#include "plugin.h"
#include "whiteBOX.h"

#define WB_DEBUG 1

GList *opt_plug_list;   /* copy of main plugin list (in case we cancel) */

void
write_config_file( const char *cfg_file, GtkWidget *ptr_tomain )
{

  GtkCList *lst_plug;     /* main list */
  int i = 0;                  /* initialise counter */

  char *row_data[1][2];
    
  FILE *out_file;             /* the file we are saving this stuff to */

  lst_plug = (GtkCList *) lookup_widget( (GtkWidget *) ptr_tomain, "lst_optplugins" );

  out_file = fopen( cfg_file, "w" );

  for( i = 0; i < lst_plug->rows; i++ )
  {
	  gtk_clist_get_text( lst_plug, i, 0, &row_data[0][0] );	
    gtk_clist_get_text( lst_plug, i, 1, &row_data[0][1] );	
    fprintf( out_file, "%s;%s\n", row_data[0][0], row_data[0][1] );
  }
  
  fclose( out_file );

}

void
relabel_plugins( GtkWidget *cur_wnd, GtkWidget *nbk_plugins )
{

  int i = 0;
  int x = 0;
  GtkWidget *lst_plug = lookup_widget( cur_wnd, "lst_optplugins" );
  
  /* do plugin reordering */
  for( i = 0; i < g_list_length( opt_plug_list ); i++ )
  {
    char *plug_name;
    GtkWidget *child;
    gpointer *pp = g_list_nth_data( opt_plug_list, i );
        
    if( ((plugin_data *) pp)->plugin_handle )
    {
      gtk_clist_get_text( (GtkCList *) lst_plug, i, 0, &plug_name );	
      child = gtk_notebook_get_nth_page( (GtkNotebook *) nbk_plugins, x );
          
      gtk_notebook_set_tab_label_text( (GtkNotebook *) nbk_plugins, child, plug_name );
      x++;
    }
  }
  
}

void
reorder_plugins( GtkWidget *cur_wnd, GtkWidget *nbk_plugins )
{

  int i;
  
  /* do plugin reordering */
  for( i = 0; i < g_list_length( opt_plug_list ); i++ )
  {
    gpointer *pp = g_list_nth_data( opt_plug_list, i );
        
    if( ((plugin_data *) pp)->plugin_handle )
    {
      GtkWidget *wgt = (GtkWidget *) ((plugin_data *) pp)->plugin_entry;
      
      if( wgt->parent != NULL ) 
      {    
        
        GtkWidget *page;
        int x = 0;
      
        page = gtk_notebook_get_nth_page( (GtkNotebook *) nbk_plugins, i );
      
        if( wgt != page )
        {      
          
          for( x = 0; x < g_list_length( opt_plug_list ); x++ ) 
          {
            GtkWidget *p = gtk_notebook_get_nth_page( (GtkNotebook *) nbk_plugins, x );
          
            if( p == wgt ) {
              g_print( "found at %d ..\n", x );
              gtk_notebook_reorder_child( (GtkNotebook *) nbk_plugins, p, i );
            }
          }
        }
    
      } 
    }
    else
    {
      g_print( "skipping ..\n" );
    }
  }  
  
  
}

void
save_options( GtkWidget *ptr_wnd )
{
  
  GtkWidget *this_wnd    = GTK_WIDGET(GTK_WIDGET(GTK_WIDGET( ptr_wnd )->parent)->parent);
  char save_file[255];       /* filename to save to */

  GtkWidget *ptr_to_main = (GtkWidget *) gtk_object_get_data( (GtkObject *) this_wnd, "wnd_main" );
  GtkWidget *wgt_nbkmain = lookup_widget( ptr_to_main, "nbk_main" );
  
  strncpy( save_file, getenv("HOME"), 255 );
  strncat( save_file, "/.whiteBOX/pluginlist", 255 );  
      
  write_config_file( save_file, ptr_wnd );  /* write plugin list */
   
  /* do some housekeeping */
  reorder_plugins( this_wnd, wgt_nbkmain );   /* reorder the plugins  */
  relabel_plugins( this_wnd, wgt_nbkmain );   /* re-label the plugins */
  
  
  /* 
   * TODO
   *  - unload deleted plugins
   *  - load added plugins 
   */
  
  
  lst_plugins = g_list_copy( opt_plug_list );  

}

void
on_btn_optsave_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{

  save_options( (GtkWidget *) button );
  
}

void
on_wnd_options_show                    (GtkWidget       *widget,
                                        gpointer         user_data)
{

  FILE *in_file;
  gchar row[2][250];
  gchar *tt[1][2];
  gchar *substring;

  GtkWidget *wgt_pluginlist = lookup_widget( widget, "lst_optplugins" );

  char image_line[1024];
  char image_line2[1024];

  int posn;
  int row_num;

  char *home_dir = getenv( "HOME" );
  char plugin_list[255];

    
  /* determine relative locations of config files */
  strcpy( plugin_list, home_dir );
  strcat( plugin_list, "/.whiteBOX/pluginlist" );


  /* open the plugin list */
  in_file = fopen( plugin_list, "r" );
	
  if( in_file != NULL )
  {
    while( !feof( in_file ))
    {
      posn = 0;

      if( fgets( image_line, 1024, in_file ))
      {
        strcpy( image_line2, strtok( image_line, "\n" ));
        strcpy( row[posn], strtok( image_line2, ";" ));         

         while(( substring = strtok( NULL, ";" )) != NULL )
        {
          posn++;
          strcpy( row[posn], substring);
	}

        tt[0][0] = row[0];  /* show key */
        tt[0][1] = row[1];  /* show location */
			
        /* create a new row in list */
        row_num = gtk_clist_append( (GtkCList *) wgt_pluginlist, tt[0] );

      }
    }
    
    fclose( in_file );  /* close the file */
    
  }
  else 
  {
    g_print( "could not open file ..\n" );
  }

  /* copy plugin list */
  opt_plug_list = g_list_copy( lst_plugins );
  
}

void
on_btn_optplugadd_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget *wgt_dlgaddplugin;
  GtkWidget *this_wnd;
    
  wgt_dlgaddplugin = create_dlg_addplugin();
  this_wnd = GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET( button )->parent)->parent)->parent)->parent);
  
  gtk_object_set_data( (GtkObject *) wgt_dlgaddplugin , "wnd_parent", (GtkWidget *) button );
  gtk_object_set_data( (GtkObject *) wgt_dlgaddplugin , "wnd_main", 
    (GtkWidget *) gtk_object_get_data( (GtkObject *) this_wnd, "wnd_main" ));
  
  gtk_object_set_data( (GtkObject *) wgt_dlgaddplugin,  "type", "new" );  
  
  gtk_widget_show( wgt_dlgaddplugin );

}


void
on_btn_optplugremove_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkCList *lst_optplugins = (GtkCList *) lookup_widget( (GtkWidget *) button, "lst_optplugins" );
  int row;

  if( lst_optplugins->selection ) 
  {
    if( message_box( 3, "Are you sure you want to remove this plugin?\n\nNote: the plugin file will not be deleted.", 
                     "Remove Plugin" ) == 1 )
    {
      gpointer *pp;
      
      row = GPOINTER_TO_INT(( lst_optplugins->selection )->data );
      pp = g_list_nth_data( opt_plug_list, (row ) );
     
      opt_plug_list = g_list_remove( opt_plug_list, pp );
      gtk_clist_remove( lst_optplugins, row );
    }
  }

}

void
on_btn_browseplug_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget *wgt_txtplugloc  = lookup_widget( (GtkWidget *) button, "txt_plugloc" );
  gchar *file = NULL;
  
  file = return_file_w_title( NULL, "Please select a plugin .." );

  if( file )
    gtk_entry_set_text( (GtkEntry *) wgt_txtplugloc, file );
  
}


void
on_btn_cancelplug_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget *this_wnd;

  this_wnd   = GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET( button )->parent)->parent)->parent)->parent);

  gtk_widget_unrealize( (GtkWidget *) this_wnd );
  gtk_widget_hide( (GtkWidget *) this_wnd );  
  
  gtk_widget_destroy( (GtkWidget *) this_wnd );
  
}


void
on_btn_addplug_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget *wgt_pluglist;
  GtkWidget *wgt_txtplugname = lookup_widget( (GtkWidget *) button, "txt_plugname" );
  GtkWidget *wgt_txtplugloc  = lookup_widget( (GtkWidget *) button, "txt_plugloc" );
  
  GtkWidget *parent_loc;
  GtkWidget *this_wnd;
  GtkWidget *ptr_to_main;

  char *row_data[1][2];
  int row;

  gchar *plug_name;
  gchar *plug_loc;
  
  gchar *type;  
 
  
  this_wnd    = GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET( button )->parent)->parent)->parent)->parent);
  parent_loc  = (GtkWidget *) gtk_object_get_data( (GtkObject *) this_wnd, "wnd_parent" );
  ptr_to_main = (GtkWidget *) gtk_object_get_data( (GtkObject *) this_wnd, "wnd_main" );
  type        = gtk_object_get_data( (GtkObject *) this_wnd, "type" );
  
  wgt_pluglist = lookup_widget( parent_loc, "lst_optplugins" );

  plug_name = g_strdup( gtk_entry_get_text( (GtkEntry *) wgt_txtplugname ));
  plug_loc  = g_strdup( gtk_entry_get_text( (GtkEntry *) wgt_txtplugloc ));  

  /* check to see if user has entered both name and location */
  if( strlen( plug_name ) > 0 && strlen( plug_loc ) > 0 )
  {
    row_data[0][0] = plug_name;
    row_data[0][1] = plug_loc;
  }
  else
  {
    /* print appropriate error message */
    if( strlen( plug_name ) <= 0 && strlen( plug_loc ) <= 0 )
      message_box( 4, "Please enter both a name and location\nfor this plugin and try again.", "No name or location specified." );
    else if( strlen( plug_name ) <= 0 )
      message_box( 4, "Please enter a name for this plugin and try again.", "No name specified." );
    else if( strlen( plug_loc ) <= 0 )
      message_box( 4, "Please enter a location for this plugin and try again.", "No location specified." );
    
    
    return; 
  }
     
  if( !strcmp( type, "new" ))
  {
    plugin_data *new_p = calloc( 1, sizeof( plugin_data ));
    
    row = gtk_clist_append( (GtkCList *) wgt_pluglist, row_data[0] );

    new_p->plugin_handle      = NULL;
    new_p->plugin_intname     = "Unknown";
    new_p->plugin_description = "Plugin information will not be retrieved until\n you restart/refresh.";
    new_p->plugin_author      = "Unknown";
    new_p->plugin_entry       = NULL;
    new_p->plugin_location    = plug_loc;
    new_p->plugin_status      = "add";
    
    opt_plug_list = g_list_append( opt_plug_list, new_p );
    g_print( "add: new length = %d\n", g_list_length( opt_plug_list ));
  }
  else if( !strcmp( type, "edit" ))
  {  
    row = GPOINTER_TO_INT( (GTK_CLIST(wgt_pluglist)->selection)->data );

    gtk_clist_set_text( (GtkCList *) wgt_pluglist, row, 0, row_data[0][0] );
    gtk_clist_set_text( (GtkCList *) wgt_pluglist, row, 1, row_data[0][1] );
  }
  
  /* kill window */
  gtk_widget_unrealize( (GtkWidget *) this_wnd );
  gtk_widget_hide( (GtkWidget *) this_wnd );

  gtk_widget_destroy( (GtkWidget *) this_wnd );
  
}

void
run_command( const char *cmd_to_run )
{

  char *shell = "/bin/sh";
  
  int pid; /* process identifier */
  pid = fork();

  if ( pid < 0 )
  { 
    g_print( "Error: could not fork process. Exiting ...\n" );
    gtk_main_quit();
  }

  if ( pid == 0 )
  { 
    /* child process */
    execl( shell, shell, "-c", cmd_to_run, NULL );
    
    /* needs error checking */
    
  } 
  else
  { 
    /* parent process */
    g_print ("hmm" );
    gtk_main_quit();  
  }    
  
}

void
on_btn_optplugrefresh_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{

  if( message_box( 6, "whiteBOX will now restart in order to make\nthe necessary changes.\nYou will lose any unsaved changes.\n\nIf whiteBOX fails to restart please do so manually.", 
      "Ok to quit?" ) == 4 )
  {

    char save_file[255];       /* filename to save to */
  
    strncpy( save_file, getenv("HOME"), 255 );
    strncat( save_file, "/.whiteBOX/pluginlist", 255 );  
      
    write_config_file( save_file, (GtkWidget *) button );  /* write plugin list */   
  
    /* bad bad very bad, well .. lazy, way of doing things :) */
    run_command( "whiteBOX" );  
    gtk_main_quit();
  }
  else
    return;
}


void
on_btn_optok_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget *this_wnd    = GTK_WIDGET(GTK_WIDGET(GTK_WIDGET( button )->parent)->parent);
  
  save_options( (GtkWidget *) button );
  
  gtk_widget_unrealize( this_wnd );
  gtk_widget_hide( this_wnd );  
  
  gtk_widget_destroy( this_wnd );  
  
}

void
on_btn_optplugedit_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  
  GtkWidget *wgt_dlgaddplugin;
   
  GtkWidget *wgt_txtplugname;
  GtkWidget *wgt_txtplugloc;
  GtkCList   *lst_plug;
    
  char *row_data[1][2];
  int cur_row = 0;
  
  lst_plug = (GtkCList *) lookup_widget( (GtkWidget *) button, "lst_optplugins" );
  
  wgt_dlgaddplugin = create_dlg_addplugin();
  gtk_object_set_data( (GtkObject *) wgt_dlgaddplugin , "wnd_parent", (GtkWidget *) button );

  wgt_txtplugname = lookup_widget( wgt_dlgaddplugin, "txt_plugname" );
  wgt_txtplugloc  = lookup_widget( wgt_dlgaddplugin, "txt_plugloc" );

  if( GTK_CLIST( lst_plug )->selection ) 
    cur_row = GPOINTER_TO_INT(( GTK_CLIST( lst_plug )->selection)->data );
  else {
    message_box( 4, "Please select a plugin from the list and try again.", "Error." );
    return;
  }
  
  /* get plugin information */
	gtk_clist_get_text( lst_plug, cur_row, 0, &row_data[0][0] );	
  gtk_clist_get_text( lst_plug, cur_row, 1, &row_data[0][1] );	  
  
  gtk_entry_set_text( (GtkEntry *) wgt_txtplugname, row_data[0][0] );
  gtk_entry_set_text( (GtkEntry *) wgt_txtplugloc,  row_data[0][1] );
   
  gtk_object_set_data( (GtkObject *) wgt_dlgaddplugin,
                       "type", "edit" );
  gtk_widget_show( wgt_dlgaddplugin ); 	  
  
}

void
on_btn_optcancel_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget *this_wnd; 
  
  this_wnd   = GTK_WIDGET(GTK_WIDGET(GTK_WIDGET( button )->parent)->parent);
 
    
  gtk_widget_unrealize( this_wnd );
  gtk_widget_hide( this_wnd );  
  
  gtk_widget_destroy( this_wnd );
  
}

void
on_btn_tbrplugup_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkCList  *lst_mainlist = (GtkCList *) lookup_widget( (GtkWidget *) button, "lst_optplugins" );
  
  if( lst_mainlist->selection ) 
  {
    
    gpointer *pp;
    
    int i;
    int row = 0;
    
    row = GPOINTER_TO_INT( (lst_mainlist->selection)->data );

    if( row == 0 )
      return;
    
    gtk_clist_row_move( lst_mainlist, row, ( row - 1 ));

    /* need to re-sort plugin list */

    for( i = 0; i < g_list_length( opt_plug_list ); i++ ) 
    {
           
      if( i == row ) {
        pp = g_list_nth_data( opt_plug_list, (i - 1) );
        opt_plug_list = g_list_remove( opt_plug_list, pp );
        opt_plug_list = g_list_insert( opt_plug_list, pp, i );
        i++;
      }
         
    }    
    
    
  }    
  
}



void
on_btn_tbrplugdown_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkCList  *lst_mainlist = (GtkCList *) lookup_widget( (GtkWidget *) button, "lst_optplugins" );
   
  if( lst_mainlist->selection ) 
  {
  
    gpointer *pp;
    int i;
    
    int row = 0;
    int count = 0;
    
    row = GPOINTER_TO_INT( (lst_mainlist->selection)->data );
    count = GPOINTER_TO_INT( lst_mainlist->rows );
       
    if( (row + 1) == count )
      return;
    
    gtk_clist_row_move( lst_mainlist, row, ( row + 1 ));

    /* need to re-sort plugin list */
    for( i = 0; i < g_list_length( opt_plug_list ); i++ ) 
    {
           
      if( i == row ) {
        pp = g_list_nth_data( opt_plug_list, (i + 1) );
        opt_plug_list = g_list_remove( opt_plug_list, pp );
        opt_plug_list = g_list_insert( opt_plug_list, pp, i );
        i++;
      }
         
    }    
    
    
  }    
 
}


void
on_btn_tbrpluginfo_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkCList   *lst_plug;
  
  char *row_data[1][2];
  int cur_row = 0;
  char plug_info[255];
  
  plugin_data *pp;
  
  lst_plug = (GtkCList *) lookup_widget( (GtkWidget *) button, "lst_optplugins" );

  if( GTK_CLIST( lst_plug )->selection ) 
    cur_row = GPOINTER_TO_INT(( GTK_CLIST( lst_plug )->selection)->data );
  else {
    message_box( 4, "Please select a plugin from the list and try again.", "Error." );
    return;
  }
    
	gtk_clist_get_text( lst_plug, cur_row, 0, &row_data[0][0] );	
  gtk_clist_get_text( lst_plug, cur_row, 1, &row_data[0][1] );	  
  
  pp = (plugin_data *) g_list_nth_data( opt_plug_list, cur_row );
  
  strcpy( plug_info, "Plugin Label: " );
  strcat( plug_info, row_data[0][0] );
  strcat( plug_info, "\nPlugin Location: " );
  strcat( plug_info, row_data[0][1] );
  
  strcat( plug_info, "\n\nInternal Name: " );
  strcat( plug_info, pp->plugin_intname );
  strcat( plug_info, "\nAuthor: " );
  strcat( plug_info, pp->plugin_author );  
  strcat( plug_info, "\nDescription: " );
  strcat( plug_info, pp->plugin_description );  
   
  message_box( 4, plug_info, "Plugin Information .." );
  
}

