/*
 * bgset.c
 * BG tool callbacks and stuff
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>
#include <glib.h>
#include <gtk/gtkcolorsel.h>

#include <unistd.h>
#include <string.h>
#include <assert.h>

#include <malloc.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

#include "bgset.h"
#include "image.h"
#include "../../support.h"
#include "../../msgbox.h"
#include "../../filedialog.h"
#include "../../colourdialog.h"


bg_image *
new_bg_image (gchar * key, gchar * loc, gchar * bg,
	      gchar * xpos, gchar * ypos, gchar * zpos, gchar * scale,
	      gchar * opacity)
{

	bg_image *new_image = calloc (1, sizeof (bg_image));

	assert (key);		/* see if any errors occur */

	/* set the item data */
	new_image->img_key = g_strdup (key);
	new_image->img_loc = g_strdup (loc);
	new_image->img_bg = g_strdup (bg);
	new_image->img_x = g_strdup (xpos);
	new_image->img_y = g_strdup (ypos);
	new_image->img_z = g_strdup (zpos);
	new_image->img_scale = g_strdup (scale);
	new_image->img_opacity = g_strdup (opacity);

	return new_image;

}

void
save_images (GtkWidget * ptr_main)
{

	GtkCList *lst_main;	/* main list */
	int i = 0;		/* initialise counter */

	char save_file[255];	/* filename to save to -- this needs changin' */

	FILE *out_file;		/* the file we are saving this stuff to */


	bg_image *bg_save;

	char *scale = NULL;


	if (BGSET_DEBUG)
		g_print ("home: %s\n", g_get_home_dir ());

	strncpy (save_file, getenv ("HOME"), 255);
	strncat (save_file, "/.whiteBOX/bgset/bgfiles", 255);

	lst_main =
		(GtkCList *) lookup_widget ((GtkWidget *) ptr_main,
					    "lst_images");

	if (BGSET_DEBUG)
		g_print ("writing to %s ..\n", save_file);

	out_file = fopen (save_file, "w");

	for (i = 0; i < lst_main->rows; i++)
	{

		bg_save = gtk_clist_get_row_data (lst_main, i);

		if (!bg_save->img_scale || strlen (bg_save->img_scale) == 0)
			scale = "0";
		else
			scale = bg_save->img_scale;

		fprintf (out_file, "%s;%s;%s;%s;%s;%s;%s;%s\n",
			 bg_save->img_key, bg_save->img_loc, bg_save->img_bg,
			 bg_save->img_x, bg_save->img_y, bg_save->img_z,
			 scale, bg_save->img_opacity);

	}

	fclose (out_file);

}

void
save_single (char *loc, char *bg, char *pos, char *xpos, char *ypos,
	     char *scale, char *opc)
{

	FILE *out_file;
	char save_file[255];

	strncpy (save_file, getenv ("HOME"), 255);
	strncat (save_file, "/.whiteBOX/bgset/bgsetsingle", 255);

	if ((out_file = fopen (save_file, "w")) != NULL)
	{

		if (strlen (xpos) == 0)
			xpos = "0";

		if (strlen (ypos) == 0)
			ypos = "0";

		if (strlen (scale) == 0)
			scale = "0";


		fprintf (out_file, "%s;%s;%s;%s;%s;%s;%s\n",
			 loc, bg, pos, xpos, ypos, scale, opc);

		fclose (out_file);

	}
}

char *
get_long_mode (char *posz)
{

	if (!strcmp (posz, "N"))
		return "Custom";

	if (!strcmp (posz, "C"))
		return "Center";

	if (!strcmp (posz, "T"))
		return "Tiled";

	if (!strcmp (posz, "S"))
		return "Scaled";

	if (!strcmp (posz, "SP"))
		return "Scaled Perspective";

	if (!strcmp (posz, "SC"))
		return "Scaled Custom";

	if (!strcmp (posz, "SN"))
		return "Scaled Custom A";


}

int
get_align_mode (gchar * posz)
{


	if (!strcmp (posz, "N"))
		return 6;

	if (!strcmp (posz, "C"))
		return 0;

	if (!strcmp (posz, "T"))
		return 1;

	if (!strcmp (posz, "S"))
		return 2;

	if (!strcmp (posz, "SP"))
		return 3;

	if (!strcmp (posz, "SC"))
		return 4;

	if (!strcmp (posz, "SN"))
		return 5;

	else
		return 0;	/* default to center for now */

}



void
set_background (char *file, char *bgcolour, char *posz,
		char *posx, char *posy, char *scale, char *opacity)
{


	char image_bg[6];
	char new_bg[8];


	GdkWindow *win = GDK_ROOT_PARENT ();	/* root window */
	GdkVisual *gvis = gdk_window_get_visual (win);	/* ?? */
	GdkGC *gc;		/* ?? */

	GdkPixmap *p;		/* pixmap storage for loaded image */
	GdkPixmap *desk_out;	/* final pixmap - display */
	GdkPixbuf *im = NULL;	/* the loaded image */

	GdkColormap *cmap = gdk_colormap_get_system ();	/* the system colour map */
	GdkColor immg;		/* the background colour */

	gchar *buf;		/* ?? */

	gint image_w = 0;	/* the image width */
	gint image_h = 0;	/* the image height */
	gint root_x = 0;	/* centered x position of image */
	gint root_y = 0;	/* centered y position of image */
	gint s_width = gdk_screen_width ();	/* screen width */
	gint s_height = gdk_screen_height ();	/* screen height */

	gint cust_opacity = 255;
	gint algn_mode;

	int cust_x;
	int cust_y;
	double cust_mod;

	Pixmap pp;

	/* get custom co-ordinates */
	cust_x = atoi (posx);
	cust_y = atoi (posy);

	cust_mod = strtod (scale, NULL);


	/* get a value that the computer convert to an unsigned int */
	sscanf (bgcolour, "#%s", image_bg);
	strcpy (new_bg, "0x");
	strcat (new_bg, image_bg);

	if (BGSET_DEBUG)
	{
		//g_print( "\nbg: %s\n", new_bg );
		//g_print( "\nbg: %lu\n", (strtoul( new_bg, NULL, 16 )));
	}

	/* Load the image specified as the first argument */
	im = gdk_pixbuf_new_from_file (file);

	/* get the width and height of the image */
	image_w = gdk_pixbuf_get_width (im);
	image_h = gdk_pixbuf_get_height (im);

	/* figure out the center x/y co-ordinates */
	root_x = (s_width - image_w) / 2;
	root_y = (s_height - image_h) / 2;

	/* render the image to the correct width + height and store it for display */
	im = gdk_pixbuf_scale_simple (im, image_w, image_h,
				      GDK_INTERP_BILINEAR);

	if (BGSET_DEBUG)
		g_print ("height: %d\n", image_h);


	/* if image has alpha levels or user given an opacity level */
	if (gdk_pixbuf_get_has_alpha (im) || (cust_opacity))
	{

		guint32 check_colour;

		if (!bgcolour || (strlen (bgcolour) == 0))
			strcpy (new_bg, "0xFFFFFF");

		check_colour = strtoul (new_bg, NULL, 0);	/* convert colour to guint32 */

		/* routine to provide better colour matching .. ?? 
		 *   -- need a check here as my maths are pretty bad and the conversion might 
		 *      be incorrect to start with (hence this becomes redundant) 
		 */

		if (strcmp (new_bg, "0xFFFFFF")
		    && strcmp (new_bg, "0x000000"))
		{
			check_colour += 65535;	/* equivalent to adding 1 to each of the RGB segments of the colour ?? */
		}

		//g_print( "set bg: bg opacity to int = %d\n", (atoi( opacity ) * 255)/100 );

		/* render the alpha background of the transparent image and the alpha level of the image */
		im = gdk_pixbuf_composite_color_simple (im, image_w, image_h,
							GDK_INTERP_BILINEAR,
							(atoi (opacity) *
							 255) / 100, 8,
							check_colour,
							check_colour);

	}


	im = gdk_pixbuf_add_alpha (im, TRUE, 255, 255, 255);

	gdk_pixbuf_render_pixmap_and_mask (im, &p, NULL, 128);

	algn_mode = get_align_mode (posz);

	if (BGSET_DEBUG)
		g_print ("align: %d\n", algn_mode);

	/*!!!!!!!!! */

	/* the crux of it all */
	if (algn_mode == 0 || algn_mode > 1)
	{

		buf = calloc (1, s_width * s_width);	/* storage for pixmap data : empty */
		gc = gdk_gc_new (win);	/* set up a graphics context to draw the BG */

		/* get the background colour */
		if (!bgcolour || (strlen (bgcolour) == 0))
			gdk_color_parse ("#FFFFFF", &immg);	/* default to white as a background colour */
		else
			gdk_color_parse (bgcolour, &immg);

		gdk_colormap_alloc_color (cmap, &immg, FALSE, TRUE);

		/* create the pixmap we want to display as our background.
		 * basically we want to create the desktop (image, bg colour, alignment) that we will 
		 * display 
		 */
		desk_out =
			gdk_pixmap_create_from_data (win, buf, s_width,
						     s_height, gvis->depth,
						     &immg, &immg);

		/* center the image */
		if (algn_mode == 0)
			gdk_draw_pixmap (desk_out, gc, p, 0, 0, root_x,
					 root_y, image_w, image_h);

		/* align the image to a set of custom co-ordinates */
		if (algn_mode == 6)
			gdk_draw_pixmap (desk_out, gc, p, 0, 0, cust_x,
					 cust_y, image_w, image_h);

		/* scale the image to the desktop */
		if (algn_mode == 2)
		{

			/* re-render to the image to the width & height of the screen */
			im = gdk_pixbuf_scale_simple (im, s_width, s_height,
						      GDK_INTERP_BILINEAR);
			gdk_pixbuf_render_pixmap_and_mask (im, &p, NULL, 128);

			gdk_draw_pixmap (desk_out, gc, p, 0, 0, 0, 0, s_width,
					 s_height);

		}


		/* perspective scale the image */
		if (algn_mode == 3 || algn_mode == 4 || algn_mode == 5)
		{
			double mod_val = 0;
			int scaled_width = 0, scaled_height = 0;

			/* determine which value to scale the image by */
			if (image_w > image_h)
				mod_val = (double) s_width / (double) image_w;
			else
				mod_val =
					(double) s_height / (double) image_h;

			/* scale the image to a custom percentage */
			if (algn_mode == 4 || algn_mode == 5)
				mod_val = cust_mod;

			/* scale the image */
			scaled_width = image_w * mod_val;
			scaled_height = image_h * mod_val;

			if (BGSET_DEBUG)
			{
				g_print ("h: %d,  w: %d\n", scaled_height,
					 scaled_width);
				g_print ("val:  %f\n", mod_val);
			}

			/* re-render to the image to the width & height of the screen */
			im = gdk_pixbuf_scale_simple (im, scaled_width,
						      scaled_height,
						      GDK_INTERP_BILINEAR);
			gdk_pixbuf_render_pixmap_and_mask (im, &p, NULL, 128);

			/* get the new center position for the scaled image */
			if (algn_mode != 5)
			{
				root_x = (s_width - scaled_width) / 2;
				root_y = (s_height - scaled_height) / 2;
			}
			else
			{
				root_x = cust_x;
				root_y = cust_y;
			}

			if (algn_mode == 3)
				gdk_draw_pixmap (desk_out, gc, p, 0, 0,
						 root_x, root_y, s_width,
						 s_height);
			if (algn_mode == 4 || algn_mode == 5)
				gdk_draw_pixmap (desk_out, gc, p, 0, 0,
						 root_x, root_y, scaled_width,
						 scaled_height);

		}

		/* set the background image */
		gdk_window_set_back_pixmap( win, desk_out, FALSE );
		gdk_gc_destroy (gc);	/* free up the unused graphic context */

	}

	if (algn_mode == 1)
	{
		gdk_window_set_back_pixmap( win, p, FALSE );
	}

	gdk_window_clear(win);
	gdk_pixbuf_unref (im);
	gdk_pixmap_unref (p);

}


char *
get_pos_type (char *pos_type)
{

	char *pos_z = "";

	if (!strcmp (pos_type, "Tiled"))
		pos_z = "T";
	else if (!strcmp (pos_type, "Center"))
		pos_z = "C";
	else if (!strcmp (pos_type, "Scaled"))
		pos_z = "S";
	else if (!strcmp (pos_type, "Scaled Perspective"))
		pos_z = "SP";
	else if (!strcmp (pos_type, "Scaled Custom"))
		pos_z = "SC";
	else if (!strcmp (pos_type, "Scaled Custom A"))
		pos_z = "SN";
	else if (!strcmp (pos_type, "Custom"))
		pos_z = "N";


	return pos_z;

}

void
on_btn_setbg_clicked (GtkButton * button, gpointer user_data)
{

	GtkNotebook *nbk_bgmain = (GtkNotebook *) lookup_widget ((GtkWidget *) button, "nbk_bgmain");
	
	/* stuff necessary for regular file */
	GtkEntry *txt_bgnormloc = (GtkEntry *) lookup_widget ((GtkWidget *) button, "txt_bgnormloc");
	GtkEntry *txt_bgnormcol = (GtkEntry *) lookup_widget ((GtkWidget *) button, "txt_bgnormcol");
	GtkEntry *txt_cbobgnormpos = (GtkEntry *) lookup_widget ((GtkWidget *) button, "txt_cbobgnormpos");
	GtkEntry *txt_bgnormxpos = (GtkEntry *) lookup_widget ((GtkWidget *) button, "txt_bgnormxpos");
	GtkEntry *txt_bgnormypos = (GtkEntry *) lookup_widget ((GtkWidget *) button, "txt_bgnormypos");
	GtkEntry *txt_bgnormscale = (GtkEntry *) lookup_widget ((GtkWidget *) button, "txt_bgnormscale");
	GtkRange *hsl_bgnormopacity = (GtkRange *) lookup_widget ((GtkWidget *) button, "hsl_bgnormopacity");
	GtkAdjustment *adj = gtk_range_get_adjustment (hsl_bgnormopacity);
	
	/* stuff necessary for saved files */
	GtkCList *lst_main;
	int cur_pos = 0;
	bg_image *bg_set;
	
	int tab_pos = gtk_notebook_get_current_page (nbk_bgmain);
	char sld_s_val[4];
	
	int error_present = 0;

	if (BGSET_DEBUG)
		g_print ("tab pos = %d\n", tab_pos);

	switch (tab_pos)
	{

		case 0:

			/* check the entry for missing information */
			if (!strcmp(gtk_entry_get_text(txt_cbobgnormpos), "Tiled") || 
				!strcmp(gtk_entry_get_text(txt_cbobgnormpos), "Center") ||
				!strcmp(gtk_entry_get_text(txt_cbobgnormpos), "Scaled") ||
				!strcmp(gtk_entry_get_text(txt_cbobgnormpos), "Scaled Perspective"))
			{
				if (strlen(gtk_entry_get_text(txt_bgnormloc)) == 0 ||
					strlen(gtk_entry_get_text(txt_bgnormcol)) == 0)
				{
					error_present = 1;
				}
			}
			else if(!strcmp(gtk_entry_get_text(txt_cbobgnormpos), "Scaled Custom"))
			{
				if (strlen(gtk_entry_get_text(txt_bgnormloc)) == 0 ||
					strlen(gtk_entry_get_text(txt_bgnormcol)) == 0 || 
					strlen(gtk_entry_get_text(txt_bgnormscale)) == 0)				
				{
					error_present = 1;
				}
			}
			else if(!strcmp(gtk_entry_get_text(txt_cbobgnormpos), "Scaled Custom A"))
			{
				if (strlen(gtk_entry_get_text(txt_bgnormloc)) == 0 ||
					strlen(gtk_entry_get_text(txt_bgnormcol)) == 0 || 
					strlen(gtk_entry_get_text(txt_bgnormscale)) == 0 ||
					strlen(gtk_entry_get_text(txt_bgnormxpos)) == 0 ||	
					strlen(gtk_entry_get_text(txt_bgnormypos)) == 0)
				{
					error_present = 1;
				}
				
			}
			else if(!strcmp(gtk_entry_get_text(txt_cbobgnormpos), "Custom"))
			{
				if (strlen(gtk_entry_get_text(txt_bgnormloc)) == 0 ||
					strlen(gtk_entry_get_text(txt_bgnormcol)) == 0 || 
					strlen(gtk_entry_get_text(txt_bgnormxpos)) == 0 ||	
					strlen(gtk_entry_get_text(txt_bgnormypos)) == 0)
				{
					error_present = 1;
				}
			}


			/* if some key information is missing, then display a message */
			if (error_present == 1)
			{
				message_box (4,
					"You have not entered all of the required information.\n\nPlease all required information and try again.",
					"Insufficient Information");
				return;
			}

			
			sprintf (sld_s_val, "%d", (int)adj->value);		/* get the opacity value (%) */

			/* set the background */
			set_background (gtk_entry_get_text (txt_bgnormloc),
							gtk_entry_get_text (txt_bgnormcol),
							get_pos_type (gtk_entry_get_text (txt_cbobgnormpos)),
							gtk_entry_get_text (txt_bgnormxpos),
							gtk_entry_get_text (txt_bgnormypos),
							gtk_entry_get_text (txt_bgnormscale),
							sld_s_val);

			/* save the details of the background */
			save_single (gtk_entry_get_text (txt_bgnormloc),
						 gtk_entry_get_text (txt_bgnormcol),
						 get_pos_type (gtk_entry_get_text(txt_cbobgnormpos)),
						 gtk_entry_get_text (txt_bgnormxpos),
						 gtk_entry_get_text (txt_bgnormypos),
						 gtk_entry_get_text (txt_bgnormscale), sld_s_val);


			break;



		case 1:

			lst_main = (GtkCList *) lookup_widget ((GtkWidget *) button, "lst_images");

			/* check to see if a selection has been made */
			if (lst_main->selection)
				cur_pos = GPOINTER_TO_INT ((GTK_CLIST (lst_main)->selection)->data);
			else
			{
				/* show error message -- no selection made */
				message_box (4,
					"You have not selected an item.\n\nPlease select an image from the list and try again.",
					"No item selected");
				
				break;

			}			
			
			save_images ((GtkWidget *) button);		/* save the image data in case there is a crash */
			bg_set = gtk_clist_get_row_data (lst_main, cur_pos);	/* get the information on the bg to set */

			/* set the background */
			set_background (bg_set->img_loc, bg_set->img_bg,
							bg_set->img_z, bg_set->img_x, bg_set->img_y,
							bg_set->img_scale, bg_set->img_opacity);

			break;



		case 2:
			
			/* group background -- not completed */
			g_print ("set bg: group mode\n");
			break;

	
	}

}



void
on_btn_addpic_clicked (GtkButton * button, gpointer user_data)
{

	GtkWidget *wnd_additem;

	wnd_additem = create_wnd_additem ();
	gtk_object_set_data ((GtkObject *) wnd_additem,
			     "ptr_to_main", button);
	gtk_object_set_data ((GtkObject *) wnd_additem, "type", "new");

	gtk_widget_show (wnd_additem);

}


void
on_btn_set_clicked (GtkButton * button, gpointer user_data)
{

	GtkWidget *wgt_mainlist;
	GtkWidget *parent_loc;
	GtkWidget *this_wnd;

	GtkWidget *wgt_txtimagekey;
	GtkWidget *wgt_txtimageloc;
	GtkWidget *wgt_txtimagebg;

	GtkWidget *wgt_txtcboimagepos;
	GtkWidget *wgt_txtimagecustx;
	GtkWidget *wgt_txtimagecusty;

	GtkWidget *wgt_txtimagecustscale;

	char *row_data[1][8];
	char *pos_type;
	char *type;

	int row;
	int error_present = 0;

	char sld_s_val[3];

	bg_image *bg_store;

	GtkRange *wgt_hscopacity =
		(GtkRange *) lookup_widget ((GtkWidget *) button,
					    "hsc_opacity");
	GtkAdjustment *adj = gtk_range_get_adjustment (wgt_hscopacity);


	if (BGSET_DEBUG)
		g_print ("store image: slider value: %d\n", (int) adj->value);


	this_wnd =
		GTK_WIDGET (GTK_WIDGET (GTK_WIDGET (button)->parent)->parent);
	parent_loc =
		(GtkWidget *) gtk_object_get_data ((GtkObject *) this_wnd,
						   "ptr_to_main");
	type = gtk_object_get_data ((GtkObject *) this_wnd, "type");

	wgt_mainlist = lookup_widget ((GtkWidget *) parent_loc, "lst_images");

	wgt_txtimagekey =
		lookup_widget ((GtkWidget *) button, "txt_imagekey");
	wgt_txtimageloc =
		lookup_widget ((GtkWidget *) button, "txt_imageloc");
	wgt_txtimagebg = lookup_widget ((GtkWidget *) button, "txt_imagebg");

	wgt_txtcboimagepos =
		lookup_widget ((GtkWidget *) button, "txt_cboimagepos");
	wgt_txtimagecustx =
		lookup_widget ((GtkWidget *) button, "txt_imagecustx");
	wgt_txtimagecusty =
		lookup_widget ((GtkWidget *) button, "txt_imagecusty");

	wgt_txtimagecustscale =
		lookup_widget ((GtkWidget *) button, "txt_imagecustscale");



	/* check the entry for missing information */
	if (!strcmp(gtk_entry_get_text((GtkEntry *) wgt_txtcboimagepos), "Tiled") || 
		!strcmp(gtk_entry_get_text((GtkEntry *) wgt_txtcboimagepos), "Center") ||
		!strcmp(gtk_entry_get_text((GtkEntry *) wgt_txtcboimagepos), "Scaled") ||
		!strcmp(gtk_entry_get_text((GtkEntry *) wgt_txtcboimagepos), "Scaled Perspective"))
	{
		if (strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimageloc)) == 0 ||
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagebg)) == 0 ||
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagekey)) == 0 )
		{
			error_present = 1;
		}
	}
	else if(!strcmp(gtk_entry_get_text((GtkEntry *) wgt_txtcboimagepos), "Scaled Custom"))
	{
		if (strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimageloc)) == 0 ||
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagebg)) == 0 || 
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagecustscale)) == 0 ||
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagekey)) == 0 )				
		{
			error_present = 1;
		}
	}
	else if(!strcmp(gtk_entry_get_text((GtkEntry *) wgt_txtcboimagepos), "Scaled Custom A"))
	{
		if (strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimageloc)) == 0 ||
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagebg)) == 0 || 
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagecustscale)) == 0 ||
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagecustx)) == 0 ||	
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagecusty)) == 0 ||
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagekey)) == 0 )
		{
			error_present = 1;
		}
		
	}
	else if(!strcmp(gtk_entry_get_text((GtkEntry *) wgt_txtcboimagepos), "Custom"))
	{
		if (strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimageloc)) == 0 ||
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagebg)) == 0 || 
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagecustx)) == 0 ||	
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagecusty)) == 0 ||
			strlen(gtk_entry_get_text((GtkEntry *) wgt_txtimagekey)) == 0 )
		{
			error_present = 1;
		}
	}


	/* if some key information is missing, then display a message */
	if (error_present == 1)
	{
		message_box (4,
			"You have not entered all of the required information.\n\nPlease all required information and try again.",
			"Insufficient Information");
		return;
	}




	pos_type = gtk_entry_get_text ((GtkEntry *) wgt_txtcboimagepos);

	row_data[0][0] = gtk_entry_get_text ((GtkEntry *) wgt_txtimagekey);
	row_data[0][1] = gtk_entry_get_text ((GtkEntry *) wgt_txtimageloc);
	row_data[0][2] = gtk_entry_get_text ((GtkEntry *) wgt_txtimagebg);

	if (!strcmp (pos_type, "Custom"))
	{
		row_data[0][3] =
			gtk_entry_get_text ((GtkEntry *) wgt_txtimagecustx);
		row_data[0][4] =
			gtk_entry_get_text ((GtkEntry *) wgt_txtimagecusty);

		row_data[0][5] = "N";
	}
	else
	{

		row_data[0][3] = "0";
		row_data[0][4] = "0";

		if (!strcmp (pos_type, "Tiled"))
			row_data[0][5] = "T";
		else if (!strcmp (pos_type, "Center"))
			row_data[0][5] = "C";
		else if (!strcmp (pos_type, "Scaled"))
			row_data[0][5] = "S";
		else if (!strcmp (pos_type, "Scaled Perspective"))
			row_data[0][5] = "SP";
		else if (!strcmp (pos_type, "Scaled Custom"))
			row_data[0][5] = "SC";
		else if (!strcmp (pos_type, "Scaled Custom A"))
		{
			row_data[0][3] =
				gtk_entry_get_text ((GtkEntry *)
						    wgt_txtimagecustx);
			row_data[0][4] =
				gtk_entry_get_text ((GtkEntry *)
						    wgt_txtimagecusty);
			row_data[0][5] = "SN";
		}
	}

	row_data[0][6] =
		gtk_entry_get_text ((GtkEntry *) wgt_txtimagecustscale);

	sprintf (sld_s_val, "%d", (int) adj->value);
	row_data[0][7] = sld_s_val;

	g_print ("store image: slider to char = %s\n", row_data[0][7]);

	bg_store =
		new_bg_image (row_data[0][0], row_data[0][1], row_data[0][2],
			      row_data[0][3], row_data[0][4], row_data[0][5],
			      row_data[0][6], row_data[0][7]);


	/* if adding a new item */
	if (!strcmp (type, "new"))
	{
		row = gtk_clist_append ((GtkCList *) wgt_mainlist,
					row_data[0]);
		gtk_clist_set_row_data ((GtkCList *) wgt_mainlist, row,
					bg_store);

		gtk_clist_set_text ((GtkCList *) wgt_mainlist, row, 0,
				    row_data[0][0]);
		gtk_clist_set_text ((GtkCList *) wgt_mainlist, row, 1,
				    row_data[0][1]);
		gtk_clist_set_text ((GtkCList *) wgt_mainlist, row, 2,
				    row_data[0][2]);
		gtk_clist_set_text ((GtkCList *) wgt_mainlist, row, 3,
				    row_data[0][5]);

	}

	/* if editing an existing item */
	else
	{
		row = GPOINTER_TO_INT ((GTK_CLIST (wgt_mainlist)->selection)->
				       data);

		gtk_clist_set_row_data ((GtkCList *) wgt_mainlist, row,
					bg_store);

		gtk_clist_set_text ((GtkCList *) wgt_mainlist, row, 0,
				    row_data[0][0]);
		gtk_clist_set_text ((GtkCList *) wgt_mainlist, row, 1,
				    row_data[0][1]);
		gtk_clist_set_text ((GtkCList *) wgt_mainlist, row, 2,
				    row_data[0][2]);
		gtk_clist_set_text ((GtkCList *) wgt_mainlist, row, 3,
				    row_data[0][5]);

	}

	gtk_widget_unrealize ((GtkWidget *) this_wnd);
	gtk_widget_hide ((GtkWidget *) this_wnd);
	gtk_widget_destroy ((GtkWidget *) this_wnd);

	g_print ("saving images ..\n");
	save_images ((GtkWidget *) parent_loc);

}


void
on_btn_editpic_clicked (GtkButton * button, gpointer user_data)
{

	GtkWidget *wnd_additem;

	GtkCList *lst_mainlist =
		(GtkCList *) lookup_widget ((GtkWidget *) button,
					    "lst_images");

	GtkEntry *wgt_txtimagekey;
	GtkEntry *wgt_txtimageloc;
	GtkEntry *wgt_txtimagebg;
	GtkEntry *wgt_txtcboimagepos;
	GtkEntry *wgt_txtimagecustx;
	GtkEntry *wgt_txtimagecusty;
	GtkEntry *wgt_txtimagecustscale;

	int row;

	GtkRange *wgt_hscopacity;
	GtkAdjustment *adj;

	bg_image *bg_edit;

	if (lst_mainlist->selection)
	{

		wnd_additem = create_wnd_additem ();
		gtk_object_set_data ((GtkObject *) wnd_additem,
				     "ptr_to_main", button);
		gtk_object_set_data ((GtkObject *) wnd_additem,
				     "type", "edit");


		wgt_hscopacity =
			(GtkRange *) lookup_widget ((GtkWidget *) wnd_additem,
						    "hsc_opacity");
		adj = gtk_range_get_adjustment (wgt_hscopacity);


		wgt_txtimagekey =
			(GtkEntry *) lookup_widget ((GtkWidget *) wnd_additem,
						    "txt_imagekey");
		wgt_txtimageloc =
			(GtkEntry *) lookup_widget ((GtkWidget *) wnd_additem,
						    "txt_imageloc");
		wgt_txtimagebg =
			(GtkEntry *) lookup_widget ((GtkWidget *) wnd_additem,
						    "txt_imagebg");

		wgt_txtcboimagepos =
			(GtkEntry *) lookup_widget ((GtkWidget *) wnd_additem,
						    "txt_cboimagepos");
		wgt_txtimagecustx =
			(GtkEntry *) lookup_widget ((GtkWidget *) wnd_additem,
						    "txt_imagecustx");
		wgt_txtimagecusty =
			(GtkEntry *) lookup_widget ((GtkWidget *) wnd_additem,
						    "txt_imagecusty");

		wgt_txtimagecustscale =
			(GtkEntry *) lookup_widget ((GtkWidget *) wnd_additem,
						    "txt_imagecustscale");

		if (lst_mainlist->selection)
			row = GPOINTER_TO_INT ((lst_mainlist->selection)->
					       data);
		else
			row = 0;

		bg_edit = gtk_clist_get_row_data (lst_mainlist, row);

		gtk_entry_set_text (wgt_txtimagekey, bg_edit->img_key);
		gtk_entry_set_text (wgt_txtimageloc, bg_edit->img_loc);
		gtk_entry_set_text (wgt_txtimagebg, bg_edit->img_bg);

		if (!strcmp (bg_edit->img_z, "N"))
		{
			gtk_entry_set_text (wgt_txtcboimagepos, "Custom");
			gtk_entry_set_text (wgt_txtimagecustx,
					    bg_edit->img_x);
			gtk_entry_set_text (wgt_txtimagecusty,
					    bg_edit->img_y);
			gtk_entry_set_text (wgt_txtimagecustscale, "");
		}
		else if (!strcmp (bg_edit->img_z, "T"))
		{
			gtk_entry_set_text (wgt_txtcboimagepos, "Tiled");
			gtk_entry_set_text (wgt_txtimagecustx, "");
			gtk_entry_set_text (wgt_txtimagecusty, "");
			gtk_entry_set_text (wgt_txtimagecustscale, "");
		}
		else if (!strcmp (bg_edit->img_z, "C"))
		{
			gtk_entry_set_text (wgt_txtcboimagepos, "Center");
			gtk_entry_set_text (wgt_txtimagecustx, "");
			gtk_entry_set_text (wgt_txtimagecusty, "");
			gtk_entry_set_text (wgt_txtimagecustscale, "");
		}
		else if (!strcmp (bg_edit->img_z, "S"))
		{
			gtk_entry_set_text (wgt_txtcboimagepos, "Scaled");
			gtk_entry_set_text (wgt_txtimagecustx, "");
			gtk_entry_set_text (wgt_txtimagecusty, "");
			gtk_entry_set_text (wgt_txtimagecustscale, "");
		}

		else if (!strcmp (bg_edit->img_z, "SP"))
		{
			gtk_entry_set_text (wgt_txtcboimagepos,
					    "Scaled Perspective");
			gtk_entry_set_text (wgt_txtimagecustx, "");
			gtk_entry_set_text (wgt_txtimagecusty, "");
			gtk_entry_set_text (wgt_txtimagecustscale, "");
		}

		else if (!strcmp (bg_edit->img_z, "SC"))
		{
			gtk_entry_set_text (wgt_txtcboimagepos,
					    "Scaled Custom");
			gtk_entry_set_text (wgt_txtimagecustx, "");
			gtk_entry_set_text (wgt_txtimagecusty, "");
			gtk_entry_set_text (wgt_txtimagecustscale,
					    bg_edit->img_scale);
		}

		else if (!strcmp (bg_edit->img_z, "SN"))
		{
			gtk_entry_set_text (wgt_txtcboimagepos,
					    "Scaled Custom A");
			gtk_entry_set_text (wgt_txtimagecustx,
					    bg_edit->img_x);
			gtk_entry_set_text (wgt_txtimagecusty,
					    bg_edit->img_y);
			gtk_entry_set_text (wgt_txtimagecustscale,
					    bg_edit->img_scale);
		}

		gtk_adjustment_set_value (adj, atoi (bg_edit->img_opacity));

		gtk_widget_show (wnd_additem);

	}
	else
	{
		message_box (4,
			     "You have not selected an item.\n\nPlease select an image from the list and try again.",
			     "No item selected");
		return;
	}

}



void
on_btn_cancel_clicked (GtkButton * button, gpointer user_data)
{

	GtkWidget *this_wnd;

	this_wnd =
		GTK_WIDGET (GTK_WIDGET (GTK_WIDGET (button)->parent)->parent);

	gtk_widget_unrealize ((GtkWidget *) this_wnd);
	gtk_widget_hide ((GtkWidget *) this_wnd);
	gtk_widget_destroy ((GtkWidget *) this_wnd);


}

gboolean
on_wnd_additem_destroy_event (GtkWidget * widget,
			      GdkEvent * event, gpointer user_data)
{

	GtkWidget *this_wnd;

	this_wnd =
		GTK_WIDGET (GTK_WIDGET (GTK_WIDGET (widget)->parent)->parent);

	gtk_widget_unrealize ((GtkWidget *) this_wnd);
	gtk_widget_hide ((GtkWidget *) this_wnd);

	return FALSE;
}



void
on_lst_images_drag_begin (GtkWidget * widget,
			  GdkDragContext * drag_context, gpointer user_data)
{

	g_print ("draaaaaag...\n");

}


void
on_btn_delpic_clicked (GtkButton * button, gpointer user_data)
{

	GtkCList *lst_mainlist =
		(GtkCList *) lookup_widget ((GtkWidget *) button,
					    "lst_images");
	int row;

	if (lst_mainlist->selection)
	{
		message_box (3, "Are you sure you want to delete this image?",
			     "Delete Item");

		if (retval == 1)
		{
			row = GPOINTER_TO_INT ((lst_mainlist->selection)->
					       data);
			gtk_clist_remove (lst_mainlist, row);
		}
	}
	else
	{
		message_box (4,
			     "You have not selected an item.\n\nPlease select an image from the list and try again.",
			     "No item selected");
		return;
	}

	save_images ((GtkWidget *) button);

}


void
on_btn_delall_clicked (GtkButton * button, gpointer user_data)
{

	GtkCList *lst_mainlist =
		(GtkCList *) lookup_widget ((GtkWidget *) button,
					    "lst_images");

	message_box (3, "Are you sure you want to delete all of your images?",
		     "Delete All");

	if (retval == 1)
	{
		gtk_clist_clear (lst_mainlist);
	}

	save_images ((GtkWidget *) button);
}


void
on_btn_imagebrowse_clicked (GtkButton * button, gpointer user_data)
{


	GtkEntry *txt_itemtext =
		(GtkEntry *) lookup_widget ((GtkWidget *) button,
					    "txt_imageloc");

	gchar *cur_file = NULL;	/* grabs the current filename if any */
	gchar *file = NULL;

	/* check to see if there is a current file showing */
	if (strlen (gtk_entry_get_text (txt_itemtext)) != 0)
		cur_file = gtk_entry_get_text (txt_itemtext);
	else
		cur_file = NULL;

	/* open the file dialog */
	file = return_file_w_title (NULL, "Please select an image ..");

	if (file)
	{
		gtk_entry_set_text (txt_itemtext, file);	/* set the image loc */
	}

}


void
on_btn_bgcolour_clicked (GtkButton * button, gpointer user_data)
{

	GtkEntry *txt_itembg =
		(GtkEntry *) lookup_widget ((GtkWidget *) button,
					    "txt_imagebg");

	cdlg_struct cdlg;	/* stores file dialog return values */
	char *item_clr;		/* grabs the current filename if any */

	/* open the file dialog */
	cdlg = colour_dialog ("Please select a colour");

	/* if ok button pressed 
	 *   -- could also use "if( fdlg.fdlg_file )"
	 */
	if (cdlg.cdlg_rval == 1)
	{
		gtk_entry_set_text (txt_itembg, cdlg.cdlg_colour);	/* set the image loc */
	}

}


void
on_txt_cboimagepos_changed (GtkEditable * editable, gpointer user_data)
{
	char *posn_type;

	GtkEntry *xpos =
		(GtkEntry *) lookup_widget ((GtkWidget *) editable,
					    "txt_imagecustx");
	GtkEntry *ypos =
		(GtkEntry *) lookup_widget ((GtkWidget *) editable,
					    "txt_imagecusty");
	GtkEntry *scale =
		(GtkEntry *) lookup_widget ((GtkWidget *) editable,
					    "txt_imagecustscale");

	posn_type = gtk_entry_get_text (GTK_ENTRY (editable));

	if (!strcmp (posn_type, "Center") ||
	    !strcmp (posn_type, "Tiled") ||
	    !strcmp (posn_type, "Custom") ||
	    !strcmp (posn_type, "Scaled") ||
	    !strcmp (posn_type, "Scaled Perspective"))
	{
		gtk_widget_set_sensitive ((GtkWidget *) scale, FALSE);
	}
	else
	{
		gtk_widget_set_sensitive ((GtkWidget *) scale, TRUE);
	}

	if (!strcmp (posn_type, "Custom") ||
	    !strcmp (posn_type, "Scaled Custom A"))
	{
		gtk_widget_set_sensitive ((GtkWidget *) xpos, TRUE);
		gtk_widget_set_sensitive ((GtkWidget *) ypos, TRUE);
	}
	else
	{
		gtk_widget_set_sensitive ((GtkWidget *) xpos, FALSE);
		gtk_widget_set_sensitive ((GtkWidget *) ypos, FALSE);
	}

}

void
on_wnd_additem_show (GtkWidget * widget, gpointer user_data)
{

	GdkPixbuf *im = NULL;
	GdkPixmap *p;
	GdkBitmap *b;

	GtkPixmap *px;
	GtkEntry *wgt_txtimgloc =
		(GtkEntry *) lookup_widget (widget, "txt_imageloc");

	if (strlen (gtk_entry_get_text ((GtkEntry *) wgt_txtimgloc)) != 0)
	{

		im = gdk_pixbuf_new_from_file (gtk_entry_get_text
					       ((GtkEntry *) wgt_txtimgloc));
		im = gdk_pixbuf_composite_color_simple (im, 152, 184,
							GDK_INTERP_NEAREST,
							255, 8, 0xFF0000,
							0xFF0000);
		im = gdk_pixbuf_scale_simple (im, 152, 184,
					      GDK_INTERP_NEAREST);

		gdk_pixbuf_render_pixmap_and_mask (im, &p, &b, 255);

		px = (GtkPixmap *) lookup_widget (widget, "pxp_preview");

		gtk_pixmap_set (px, p, b);

	}

}


void
on_txt_imageloc_changed (GtkEditable * editable, gpointer user_data)
{

	GdkPixbuf *im = NULL;
	GdkPixmap *p;
	GdkBitmap *b;

	GtkPixmap *px;
	GtkEntry *wgt_txtimgloc =
		(GtkEntry *) lookup_widget ((GtkWidget *) editable,
					    "txt_imageloc");

	if (strlen (gtk_entry_get_text ((GtkEntry *) wgt_txtimgloc)) != 0)
	{

		im = gdk_pixbuf_new_from_file (gtk_entry_get_text
					       ((GtkEntry *) wgt_txtimgloc));

		if (im)
		{
			im = gdk_pixbuf_composite_color_simple (im, 152, 184,
								GDK_INTERP_NEAREST,
								255, 8,
								0xFF0000,
								0xFF0000);
			im = gdk_pixbuf_scale_simple (im, 152, 184,
						      GDK_INTERP_NEAREST);

			gdk_pixbuf_render_pixmap_and_mask (im, &p, &b, 255);

			px = (GtkPixmap *) lookup_widget ((GtkWidget *)
							  editable,
							  "pxp_preview");

			gtk_pixmap_set (px, p, b);
		}

	}

}


void
on_btn_aboutclose_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *wgt_dlgabout;

	wgt_dlgabout =
		GTK_WIDGET (GTK_WIDGET (GTK_WIDGET (button)->parent)->parent);

	gtk_widget_unrealize (wgt_dlgabout);
	gtk_widget_hide (wgt_dlgabout);

}



void
on_btn_optcancel_clicked (GtkButton * button, gpointer user_data)
{

	GtkWidget *wgt_wndoptions;

	wgt_wndoptions =
		GTK_WIDGET (GTK_WIDGET
			    (GTK_WIDGET (GTK_WIDGET (button)->parent)->
			     parent)->parent);

	gtk_widget_unrealize (wgt_wndoptions);
	gtk_widget_hide (wgt_wndoptions);


}


void
on_btn_bgnormbrowse_clicked (GtkButton * button, gpointer user_data)
{

	GtkEntry *txt_itemtext =
		(GtkEntry *) lookup_widget ((GtkWidget *) button,
					    "txt_bgnormloc");

	gchar *cur_file = NULL;	/* grabs the current filename if any */
	gchar *file = NULL;

	/* check to see if there is a current file showing */
	if (strlen (gtk_entry_get_text (txt_itemtext)) != 0)
		cur_file = gtk_entry_get_text (txt_itemtext);
	else
		cur_file = NULL;

	/* open the file dialog */
	file = return_file_w_title (cur_file, "Please select an image ..");

	if (file)
	{
		gtk_entry_set_text (txt_itemtext, file);	/* set the image loc */
	}

}


void
on_btn_bgnormcol_clicked (GtkButton * button, gpointer user_data)
{


	GtkEntry *txt_itembg =
		(GtkEntry *) lookup_widget ((GtkWidget *) button,
					    "txt_bgnormcol");

	cdlg_struct cdlg;	/* stores file dialog return values */
	char *item_clr;		/* grabs the current filename if any */

	/* open the file dialog */
	cdlg = colour_dialog ("Please select a colour");

	/* if ok button pressed 
	 *   -- could also use "if( fdlg.fdlg_file )"
	 */
	if (cdlg.cdlg_rval == 1)
	{
		gtk_entry_set_text (txt_itembg, cdlg.cdlg_colour);	/* set the image loc */
	}

}


void
on_txt_bgnormloc_changed (GtkEditable * editable, gpointer user_data)
{

	GdkPixbuf *im = NULL;
	GdkPixmap *p;
	GdkBitmap *b;

	GtkPixmap *px;
	GtkEntry *wgt_txtimgloc =
		(GtkEntry *) lookup_widget ((GtkWidget *) editable,
					    "txt_bgnormloc");

	if (strlen (gtk_entry_get_text ((GtkEntry *) wgt_txtimgloc)) != 0)
	{

		im = gdk_pixbuf_new_from_file (gtk_entry_get_text
					       ((GtkEntry *) wgt_txtimgloc));

		if (im)
		{
			im = gdk_pixbuf_composite_color_simple (im, 100, 130,
								GDK_INTERP_NEAREST,
								255, 8,
								0xFF0000,
								0xFF0000);
			im = gdk_pixbuf_scale_simple (im, 100, 130,
						      GDK_INTERP_NEAREST);

			gdk_pixbuf_render_pixmap_and_mask (im, &p, &b, 255);

			px = (GtkPixmap *) lookup_widget ((GtkWidget *)
							  editable,
							  "pxp_bgnormprev");

			gtk_pixmap_set (px, p, b);
		}

	}

}


void
on_txt_cbobgnormpos_changed (GtkEditable * editable, gpointer user_data)
{

	char *posn_type;

	GtkEntry *xpos =
		(GtkEntry *) lookup_widget ((GtkWidget *) editable,
					    "txt_bgnormxpos");
	GtkEntry *ypos =
		(GtkEntry *) lookup_widget ((GtkWidget *) editable,
					    "txt_bgnormypos");
	GtkEntry *scale =
		(GtkEntry *) lookup_widget ((GtkWidget *) editable,
					    "txt_bgnormscale");

	posn_type = gtk_entry_get_text (GTK_ENTRY (editable));

	if (!strcmp (posn_type, "Center") ||
	    !strcmp (posn_type, "Tiled") ||
	    !strcmp (posn_type, "Custom") ||
	    !strcmp (posn_type, "Scaled") ||
	    !strcmp (posn_type, "Scaled Perspective"))
	{
		gtk_widget_set_sensitive ((GtkWidget *) scale, FALSE);
	}
	else
	{
		gtk_widget_set_sensitive ((GtkWidget *) scale, TRUE);
	}

	if (!strcmp (posn_type, "Custom") ||
	    !strcmp (posn_type, "Scaled Custom A"))
	{
		gtk_widget_set_sensitive ((GtkWidget *) xpos, TRUE);
		gtk_widget_set_sensitive ((GtkWidget *) ypos, TRUE);
	}
	else
	{
		gtk_widget_set_sensitive ((GtkWidget *) xpos, FALSE);
		gtk_widget_set_sensitive ((GtkWidget *) ypos, FALSE);
	}


}


void
on_btn_tbrsavedup_clicked (GtkButton * button, gpointer user_data)
{

	GtkCList *lst_mainlist =
		(GtkCList *) lookup_widget ((GtkWidget *) button,
					    "lst_images");

	int row;

	if (lst_mainlist->selection)
	{
		row = GPOINTER_TO_INT ((lst_mainlist->selection)->data);
		gtk_clist_row_move (lst_mainlist, row, (row - 1));
	}

	save_images ((GtkWidget *) button);

}


void
on_btn_tbrsaveddown_clicked (GtkButton * button, gpointer user_data)
{

	GtkCList *lst_mainlist =
		(GtkCList *) lookup_widget ((GtkWidget *) button,
					    "lst_images");

	int row;

	if (lst_mainlist->selection)
	{
		row = GPOINTER_TO_INT ((lst_mainlist->selection)->data);
		gtk_clist_row_move (lst_mainlist, row, (row + 1));

	}

	save_images ((GtkWidget *) button);
}
