/*
 * image.c
 * whiteBOX bgset image routines 
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>

#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "image.h"

int set_background_image( const char *image_location, int algn_mode , 
		int cust_x, int cust_y, gchar *bg_colour, double cust_mod, 
		int cust_opacity )
{

  /*
   * set_background_image
   * 
   * arguments:
   *   - image_location  = a string representing the image location 
   *     (eg:/home/luke/directory/image1.jpg)
   *
   *   - algn_mode = the requested alignment mode
   *       0 = center
   *       1 = tiled
   *       2 = scaled (both)
   *       3 = scaled (perspective)
   *       4 = scaled (custom)
   *       5 = scaled (custom + align)
   *	   6 = custom
   *
   *   - cust_x    = user specified x position of image
   *   - cust_y    = user specified y position of image
   *   - cust_mod  = user specified scaling modifier (eg: 2x, 0.5x)
   *   - bg_colour = the background colour
   */  
	
  GdkWindow      *win = GDK_ROOT_PARENT();		/* root window */
  GdkVisual      *gvis = gdk_window_get_visual(win);    /* ?? */
  GdkGC          *gc;                                   /* ?? */

  GdkPixmap      *p;                                   /* pixmap storage for loaded image */
  GdkPixmap      *desk_out;                            /* final pixmap - display */
  GdkPixbuf      *im = NULL;                                  /* the loaded image */

  GdkColormap    *cmap = gdk_colormap_get_system();    /* the system colour map */
  GdkColor	 immg;                                 /* the background colour */

  gchar          *buf;                                 /* ?? */

  gint           image_w  = 0;                         /* the image width */
  gint           image_h  = 0;                         /* the image height */
  gint           root_x   = 0;                         /* centered x position of image */
  gint           root_y   = 0;			       /* centered y position of image */	
  gint           s_width  = gdk_screen_width();	       /* screen width */
  gint           s_height = gdk_screen_height();       /* screen height */

  char           image_bg[6];                            /* converted bg colour for strtoul */
  char           *new_bg;
  guint32        colour_val;

  char ni[255];

  /* get a value that the computer canvert to an unsigned int */ 
  //  sscanf( bg_colour, "#%s", image_bg );
  //sprintf( new_bg, "0x%s", image_bg );
  sscanf( bg_colour, "#%s", image_bg );
  strcpy( new_bg, "0x" );
  strcat( new_bg, image_bg );

  colour_val = strtoul( new_bg, NULL, 0 );
 
  strcpy( ni, image_location );
  
  /* Load the image specified as the first argument */
  im = gdk_pixbuf_new_from_file( image_location );

  /* get the width and height of the image */
  image_w = gdk_pixbuf_get_width(im);
  image_h = gdk_pixbuf_get_height(im);
  
  /* figure out the center x/y co-ordinates */
  root_x = (s_width - image_w) / 2;
  root_y = (s_height - image_h) / 2;

  /* render the image to the correct width + height and store it for display */
  im = gdk_pixbuf_scale_simple( im, image_w, image_h, GDK_INTERP_NEAREST );


  /* if image has alpha levels or user given an opacity level */
  if( gdk_pixbuf_get_has_alpha( im ) ||
    ( cust_opacity )) {
	  
    im = gdk_pixbuf_composite_color_simple( im, image_w, image_h, GDK_INTERP_NEAREST,
         cust_opacity, 8, colour_val, colour_val );

    
  }

    
  gdk_pixbuf_render_pixmap_and_mask( im, &p, NULL, 128 ); 
  

  
  /* the crux of it all */
  if( algn_mode == 0 || algn_mode > 1 )
  {

    buf  = calloc(1, s_width * s_width );  /* storage for pixmap data : empty */
    gc   = gdk_gc_new(win);                /* set up a graphics context to draw the BG */
  
    /* get the background colour */
    if( !bg_colour )
      gdk_color_parse( "#FFFFFF", &immg );      /* default to white as a background colour */
    else
      gdk_color_parse( bg_colour, &immg ); 
    
    gdk_colormap_alloc_color( cmap, &immg, FALSE, TRUE );
  
    /* create the pixmap we want to display as our background.
     * basically we want to create the desktop (image, bg colour, alignment) that we will 
     * display 
     */
    desk_out = gdk_pixmap_create_from_data( win, buf, s_width, s_height, gvis->depth,
		  &immg, &immg );

    /* center the image */
    if( algn_mode == 0 )
      gdk_draw_pixmap( desk_out, gc, p, 0, 0, root_x, root_y, image_w, image_h );

    /* align the image to a set of custom co-ordinates */
    if( algn_mode == 6 )
      gdk_draw_pixmap( desk_out, gc, p, 0, 0, cust_x, cust_y, image_w, image_h );

    /* scale the image to the desktop */
    if( algn_mode == 2 ) {        

      /* re-render to the image to the width & height of the screen */
      im = gdk_pixbuf_scale_simple( im, s_width, s_height, GDK_INTERP_BILINEAR );
      gdk_pixbuf_render_pixmap_and_mask( im, &p, NULL, 128 );       
 
      gdk_draw_pixmap( desk_out, gc, p, 0, 0, 0, 0, s_width, s_height );

    }

    /* perspective scale the image */
    if( algn_mode == 3 || algn_mode == 4 || algn_mode == 5 )
    {
      double mod_val = 0; 
      int scaled_width = 0, scaled_height = 0;

      /* determine which value to scale the image by */
      if( image_w > image_h )
        mod_val = (double) s_width / (double) image_w;
      else
        mod_val =  (double) s_height / (double) image_h;

      /* scale the image to a custom percentage */
      if( algn_mode == 4 || algn_mode == 5 )
        mod_val = cust_mod;
      
      /* scale the image */
      scaled_width  = image_w * mod_val;
      scaled_height = image_h * mod_val;

      //g_print( "h: %d,  w: %d\n", scaled_height, scaled_width );
      //g_print( "val:  %f\n", mod_val );

      /* re-render to the image to the width & height of the screen */
      im = gdk_pixbuf_scale_simple( im, scaled_width, scaled_height, GDK_INTERP_BILINEAR );
      gdk_pixbuf_render_pixmap_and_mask( im, &p, NULL, 128 );		  
      
      /* get the new center position for the scaled image */
      if( algn_mode != 5 ) {
        root_x = (s_width - scaled_width) / 2;
        root_y = (s_height - scaled_height) / 2;
      }
      else {
        root_x = cust_x;
        root_y = cust_y;
      }

      if( algn_mode == 3 )
        gdk_draw_pixmap( desk_out, gc, p, 0, 0, root_x, root_y, s_width, s_height );
      if( algn_mode == 4 || algn_mode == 5 )
        gdk_draw_pixmap( desk_out, gc, p, 0, 0, root_x, root_y, scaled_width, scaled_height );

    }  
	    
    /* set the background image */
    gdk_window_set_back_pixmap( win, desk_out, FALSE );

    gdk_gc_destroy(gc);     /* free up the unused graphic context */
  }

  
  if( algn_mode == 1 )
  {  
    gdk_window_set_back_pixmap( win, p, FALSE );
  }

  gdk_window_clear(win);
  gdk_pixbuf_unref(im);
  gdk_pixmap_unref(p);
  
  //g_print ("ended ..\n" );

  return TRUE;
  
}


