/*
 * libwbbgset.c
 * BG tool plugin entry point and initialisation
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_SYS_DIR_H
    #include <sys/dir.h>
#else
    #ifdef HAVE_SYS_NDIR_H
        #include <sys/ndir.h>
    #else
        #ifdef HAVE_DIRENT_H
            #include <dirent.h>
        #else
            #include <sys/dirent.h>
        #endif
    #endif
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "bgset.h"
#include "../../support.h"
#include "../../msgbox.h"
#include "../../plugin.h"

plugin_data bgplg = {
  NULL,
  "wb_bgset_01",
  NULL,
  "whiteBOX background tool component",
  "Luke Freeman",
  NULL,
  NULL
};

plugin_data *get_plugin_data( void )
{
  return &bgplg;
}

void
init( GtkWidget *wnd_main, int argc, char* argv[] )
{

	/*
	 * init
	 * intitialisation for bgset plugin
	 */


	bg_image *bg_load;  
	char bgset_dir[255];
	char image_line[1024];
	char image_line2[1024];
	char save_file[255];
	FILE *in_file;
	gchar row[8][250];
	gchar *tt[1][4];
	gchar *substring;
	int i;
	int posn = 0;
	int row_num;	
	struct stat s;

	GtkWidget *wgt_mainlist = lookup_widget( wnd_main, "lst_images" );

	/* do some checks so that everything works nicely */
	strncpy( bgset_dir, getenv("HOME"), 255 );
	strncat( bgset_dir, "/.whiteBOX/bgset/", 255 );

	/* check to see if the bgset plug-in directory is present */
	if( stat( bgset_dir, &s ) == -1 ) 
	{
		/* make directory */
		if( BGSET_DEBUG )
		  g_print( "creating directory .. %s\n", bgset_dir );

		mkdir( bgset_dir, 0700 );
	}
	else
	{
		strncpy( save_file, getenv("HOME"), 255 );
		strncat( save_file, "/.whiteBOX/bgset/bgfiles", 255 );

		in_file = fopen( save_file, "r" );

		if( in_file != NULL )
		{
			while( !feof( in_file ))
			{
				posn = 0;
				if( fgets( image_line, 1024, in_file ))
				{
					strcpy( image_line2, strtok( image_line, "\n" ));
					strcpy( row[posn], strtok( image_line2, ";" ));         

					while(( substring = strtok( NULL, ";" )) != NULL )
					{
						posn++;
						strcpy( row[posn], substring);
					}

					/* create a new 'bg image' to be stored for this row */
					bg_load = new_bg_image( row[0], row[1], row[2], row[3], row[4], 
											row[5], row[6], row[7] );


					tt[0][0] = row[0];  /* show key */
					tt[0][1] = row[1];  /* show location */
					tt[0][2] = row[2];  /* show bg colour */
					tt[0][3] = row[5];  /* show positioning scheme */

					/* create a new row in list and attack bg image data to it */
					row_num = gtk_clist_append( (GtkCList *) wgt_mainlist, tt[0] );
					gtk_clist_set_row_data( (GtkCList *) wgt_mainlist, row_num, bg_load );

				}

			}

			fclose( in_file );  /* close the file */

		}

		strncpy( save_file, getenv("HOME"), 255 );
		strncat( save_file, "/.whiteBOX/bgset/bgsetsingle", 255 );

		/* open the file */
		in_file = fopen( save_file, "r" );

		if( in_file != NULL )
		{

			/* stuff necessary for regular file */
			GtkEntry *txt_bgnormloc     = (GtkEntry *) lookup_widget( wnd_main, "txt_bgnormloc" );
			GtkEntry *txt_bgnormcol     = (GtkEntry *) lookup_widget( wnd_main, "txt_bgnormcol" );
			GtkEntry *txt_cbobgnormpos  = (GtkEntry *) lookup_widget( wnd_main, "txt_cbobgnormpos" );
			GtkEntry *txt_bgnormxpos    = (GtkEntry *) lookup_widget( wnd_main, "txt_bgnormxpos" );
			GtkEntry *txt_bgnormypos    = (GtkEntry *) lookup_widget( wnd_main, "txt_bgnormypos" );
			GtkEntry *txt_bgnormscale   = (GtkEntry *) lookup_widget( wnd_main, "txt_bgnormscale" );

			GtkRange *hsl_bgnormopacity = (GtkRange *) lookup_widget( wnd_main, "hsl_bgnormopacity" );
			GtkAdjustment *adj = gtk_range_get_adjustment( hsl_bgnormopacity );    

			if( fgets( image_line, 1024, in_file ))
			{	
				posn = 0;
				
				strcpy( image_line2, strtok( image_line, "\n" ));
				strcpy( row[posn], strtok( image_line2, ";" ));         

				while(( substring = strtok( NULL, ";" )) != NULL )
				{
					posn++;
					strcpy( row[posn], substring);

				}

				/* show the single background data */
				gtk_entry_set_text( txt_bgnormloc, row[0] );
				gtk_entry_set_text( txt_bgnormcol, row[1] );
				gtk_entry_set_text( txt_cbobgnormpos, get_long_mode( row[2] ));

				if( !strcmp( row[2], "N" ) ||
					!strcmp( row[2], "SN" ))
				{
					gtk_entry_set_text( txt_bgnormxpos, row[3] );
					gtk_entry_set_text( txt_bgnormypos, row[4] );

				}

				if( !strcmp( row[2], "SC" ) ||
					!strcmp( row[2], "SN" ))
				{
					gtk_entry_set_text( txt_bgnormscale, row[5] );
				}

				gtk_adjustment_set_value( adj, atoi( row[6] ));

			}

			fclose( in_file );  /* close the file */

		}      
	}


	/* parse the command line options */ 

	for( i = 1; i < argc; i++ ) 
	{
		if( !strcmp(argv[i], "-bgt" ) || !strcmp(argv[i], "--bg-tab" )) 
		{
                    GtkWidget *wgt_nbkmain    = lookup_widget( wnd_main, "nbk_main" );
                    GtkWidget *wgt_vbxbgmain = lookup_widget( wnd_main, "vbx_bgmain" );
		
                    gtk_notebook_set_page( (GtkNotebook *) wgt_nbkmain, 
			gtk_notebook_page_num( (GtkNotebook *) wgt_nbkmain, GTK_WIDGET( wgt_vbxbgmain )->parent ));



		}
		else if( !strcmp(argv[i], "-bgs" ) || !strcmp(argv[i], "--bg-single" )) 
		{
			/* set background and quit */
			set_background( row[0], row[1], row[2], 
							row[3], row[4], row[5], row[6] );
			exit(0);
		}
	}
}


GtkWidget *
entry_point( GtkWidget *wnd_main )
{

  GtkWidget *vbx_bgmain;
  GtkWidget *nbk_bgmain;
  GtkWidget *tbl_bgnorm;
  GtkWidget *lbl_bgnormloc;
  GtkWidget *lbl_bgnormcol;
  GtkWidget *lbl_bgnormpos;
  GtkWidget *lbl_bgnormcust;
  GtkWidget *lbl_bgnormscale;
  GtkWidget *lbl_bgnormopacity;
  GtkWidget *btn_bgnormbrowse;
  GtkWidget *btn_bgnormcol;
  GtkWidget *cbo_bgnormpos;
  GList *cbo_bgnormpos_items = NULL;
  GtkWidget *txt_cbobgnormpos;
  GtkWidget *lbl_bgnormcustx;
  GtkWidget *lbl_bgnormcusty;
  GtkWidget *pxp_bgnormprev;
  GtkWidget *txt_bgnormypos;
  GtkWidget *txt_bgnormxpos;
  GtkWidget *txt_bgnormloc;
  GtkWidget *txt_bgnormcol;
  GtkWidget *lbl_bgnormscaleins;
  GtkWidget *txt_bgnormscale;
  GtkWidget *hsl_bgnormopacity;
  GtkWidget *lbl_bgnormprev;
  GtkWidget *lbl_bgnosave;
  GtkWidget *hbx_nbksavedbg;
  GtkWidget *scw_images;
  GtkWidget *lst_images;
  GtkWidget *lbl_clmID;
  GtkWidget *lbl_clmloc;
  GtkWidget *lbl_clmbgcol;
  GtkWidget *lbl_clmposz;
  GtkWidget *vbx_nbksavebg;
  GtkWidget *btn_addpic;
  GtkWidget *btn_editpic;
  GtkWidget *btn_delpic;
  GtkWidget *btn_delall;
  GtkWidget *tbr_savedmove;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *btn_tbrsavedup;
  GtkWidget *btn_tbrsaveddown;
  GtkWidget *lbl_bgnormtab;
  GtkWidget *hbx_btns;
  GtkWidget *btn_setbg;


  vbx_bgmain = gtk_vbox_new (FALSE, 3);
  gtk_widget_set_name (vbx_bgmain, "vbx_bgmain");
  gtk_widget_ref (vbx_bgmain);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "vbx_bgmain", vbx_bgmain,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbx_bgmain);
  //gtk_container_add (GTK_CONTAINER (wnd_main), vbx_bgmain);
  gtk_container_set_border_width (GTK_CONTAINER (vbx_bgmain), 4);

  nbk_bgmain = gtk_notebook_new ();
  gtk_widget_set_name (nbk_bgmain, "nbk_bgmain");
  gtk_widget_ref (nbk_bgmain);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "nbk_bgmain", nbk_bgmain,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (nbk_bgmain);
  gtk_box_pack_start (GTK_BOX (vbx_bgmain), nbk_bgmain, TRUE, TRUE, 0);
  gtk_widget_set_usize (nbk_bgmain, 440, 248);

  tbl_bgnorm = gtk_table_new (6, 7, FALSE);
  gtk_widget_set_name (tbl_bgnorm, "tbl_bgnorm");
  gtk_widget_ref (tbl_bgnorm);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbl_bgnorm", tbl_bgnorm,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbl_bgnorm);
  gtk_container_add (GTK_CONTAINER (nbk_bgmain), tbl_bgnorm);
  gtk_container_set_border_width (GTK_CONTAINER (tbl_bgnorm), 50);
  gtk_table_set_row_spacings (GTK_TABLE (tbl_bgnorm), 8);
  gtk_table_set_col_spacings (GTK_TABLE (tbl_bgnorm), 6);

  lbl_bgnormloc = gtk_label_new (_("Location:"));
  gtk_widget_set_name (lbl_bgnormloc, "lbl_bgnormloc");
  gtk_widget_ref (lbl_bgnormloc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormloc", lbl_bgnormloc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormloc);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), lbl_bgnormloc, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_bgnormloc), 7.45058e-09, 0.5);

  lbl_bgnormcol = gtk_label_new (_("Background Colour:"));
  gtk_widget_set_name (lbl_bgnormcol, "lbl_bgnormcol");
  gtk_widget_ref (lbl_bgnormcol);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormcol", lbl_bgnormcol,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormcol);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), lbl_bgnormcol, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_bgnormcol), 7.45058e-09, 0.5);

  lbl_bgnormpos = gtk_label_new (_("Position:"));
  gtk_widget_set_name (lbl_bgnormpos, "lbl_bgnormpos");
  gtk_widget_ref (lbl_bgnormpos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormpos", lbl_bgnormpos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormpos);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), lbl_bgnormpos, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_bgnormpos), 7.45058e-09, 0.5);

  lbl_bgnormcust = gtk_label_new (_("Custom Position:"));
  gtk_widget_set_name (lbl_bgnormcust, "lbl_bgnormcust");
  gtk_widget_ref (lbl_bgnormcust);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormcust", lbl_bgnormcust,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormcust);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), lbl_bgnormcust, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_bgnormcust), 7.45058e-09, 0.5);

  lbl_bgnormscale = gtk_label_new (_("Custom Scale:"));
  gtk_widget_set_name (lbl_bgnormscale, "lbl_bgnormscale");
  gtk_widget_ref (lbl_bgnormscale);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormscale", lbl_bgnormscale,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormscale);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), lbl_bgnormscale, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_bgnormscale), 7.45058e-09, 0.5);

  lbl_bgnormopacity = gtk_label_new (_("Opacity:"));
  gtk_widget_set_name (lbl_bgnormopacity, "lbl_bgnormopacity");
  gtk_widget_ref (lbl_bgnormopacity);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormopacity", lbl_bgnormopacity,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormopacity);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), lbl_bgnormopacity, 0, 1, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_bgnormopacity), 7.45058e-09, 0.5);

  btn_bgnormbrowse = gtk_button_new_with_label (_(".."));
  gtk_widget_set_name (btn_bgnormbrowse, "btn_bgnormbrowse");
  gtk_widget_ref (btn_bgnormbrowse);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_bgnormbrowse", btn_bgnormbrowse,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_bgnormbrowse);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), btn_bgnormbrowse, 5, 6, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  btn_bgnormcol = gtk_button_new_with_label (_("?"));
  gtk_widget_set_name (btn_bgnormcol, "btn_bgnormcol");
  gtk_widget_ref (btn_bgnormcol);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_bgnormcol", btn_bgnormcol,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_bgnormcol);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), btn_bgnormcol, 5, 6, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  cbo_bgnormpos = gtk_combo_new ();
  gtk_widget_set_name (cbo_bgnormpos, "cbo_bgnormpos");
  gtk_widget_ref (cbo_bgnormpos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "cbo_bgnormpos", cbo_bgnormpos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_bgnormpos);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), cbo_bgnormpos, 2, 5, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (cbo_bgnormpos, 72, -2);
  gtk_combo_set_value_in_list (GTK_COMBO (cbo_bgnormpos), TRUE, TRUE);
  gtk_combo_set_use_arrows_always (GTK_COMBO (cbo_bgnormpos), TRUE);
  cbo_bgnormpos_items = g_list_append (cbo_bgnormpos_items, (gpointer) _("Tiled"));
  cbo_bgnormpos_items = g_list_append (cbo_bgnormpos_items, (gpointer) _("Center"));
  cbo_bgnormpos_items = g_list_append (cbo_bgnormpos_items, (gpointer) _("Scaled"));
  cbo_bgnormpos_items = g_list_append (cbo_bgnormpos_items, (gpointer) _("Scaled Perspective"));
  cbo_bgnormpos_items = g_list_append (cbo_bgnormpos_items, (gpointer) _("Scaled Custom"));
  cbo_bgnormpos_items = g_list_append (cbo_bgnormpos_items, (gpointer) _("Scaled Custom A"));
  cbo_bgnormpos_items = g_list_append (cbo_bgnormpos_items, (gpointer) _("Custom"));
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_bgnormpos), cbo_bgnormpos_items);
  g_list_free (cbo_bgnormpos_items);

  txt_cbobgnormpos = GTK_COMBO (cbo_bgnormpos)->entry;
  gtk_widget_set_name (txt_cbobgnormpos, "txt_cbobgnormpos");
  gtk_widget_ref (txt_cbobgnormpos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_cbobgnormpos", txt_cbobgnormpos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_cbobgnormpos);
  gtk_entry_set_text (GTK_ENTRY (txt_cbobgnormpos), _("Tiled"));

  lbl_bgnormcustx = gtk_label_new (_("X:"));
  gtk_widget_set_name (lbl_bgnormcustx, "lbl_bgnormcustx");
  gtk_widget_ref (lbl_bgnormcustx);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormcustx", lbl_bgnormcustx,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormcustx);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), lbl_bgnormcustx, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_bgnormcustx), 0, 0.5);

  lbl_bgnormcusty = gtk_label_new (_("Y:"));
  gtk_widget_set_name (lbl_bgnormcusty, "lbl_bgnormcusty");
  gtk_widget_ref (lbl_bgnormcusty);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormcusty", lbl_bgnormcusty,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormcusty);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), lbl_bgnormcusty, 3, 4, 3, 4,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_bgnormcusty), 0, 0.5);

  pxp_bgnormprev = create_pixmap (wnd_main, NULL);
  gtk_widget_set_name (pxp_bgnormprev, "pxp_bgnormprev");
  gtk_widget_ref (pxp_bgnormprev);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "pxp_bgnormprev", pxp_bgnormprev,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pxp_bgnormprev);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), pxp_bgnormprev, 6, 7, 0, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (pxp_bgnormprev, 100, 130);

  txt_bgnormypos = gtk_entry_new ();
  gtk_widget_set_name (txt_bgnormypos, "txt_bgnormypos");
  gtk_widget_ref (txt_bgnormypos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_bgnormypos", txt_bgnormypos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_bgnormypos);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), txt_bgnormypos, 4, 5, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_bgnormypos, 34, -2);
  gtk_widget_set_sensitive( txt_bgnormypos, FALSE );

  txt_bgnormxpos = gtk_entry_new ();
  gtk_widget_set_name (txt_bgnormxpos, "txt_bgnormxpos");
  gtk_widget_ref (txt_bgnormxpos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_bgnormxpos", txt_bgnormxpos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_bgnormxpos);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), txt_bgnormxpos, 2, 3, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_bgnormxpos, 34, -2);
  gtk_widget_set_sensitive( txt_bgnormxpos, FALSE );

  txt_bgnormloc = gtk_entry_new ();
  gtk_widget_set_name (txt_bgnormloc, "txt_bgnormloc");
  gtk_widget_ref (txt_bgnormloc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_bgnormloc", txt_bgnormloc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_bgnormloc);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), txt_bgnormloc, 2, 5, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  txt_bgnormcol = gtk_entry_new ();
  gtk_widget_set_name (txt_bgnormcol, "txt_bgnormcol");
  gtk_widget_ref (txt_bgnormcol);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_bgnormcol", txt_bgnormcol,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_bgnormcol);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), txt_bgnormcol, 2, 5, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  lbl_bgnormscaleins = gtk_label_new (_("x (original)"));
  gtk_widget_set_name (lbl_bgnormscaleins, "lbl_bgnormscaleins");
  gtk_widget_ref (lbl_bgnormscaleins);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormscaleins", lbl_bgnormscaleins,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormscaleins);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), lbl_bgnormscaleins, 3, 5, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (lbl_bgnormscaleins, 72, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_bgnormscaleins), 7.45058e-09, 0.5);

  txt_bgnormscale = gtk_entry_new ();
  gtk_widget_set_name (txt_bgnormscale, "txt_bgnormscale");
  gtk_widget_ref (txt_bgnormscale);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_bgnormscale", txt_bgnormscale,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_bgnormscale);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), txt_bgnormscale, 2, 3, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_bgnormscale, 46, -2);
  gtk_widget_set_sensitive( txt_bgnormscale, FALSE );

  hsl_bgnormopacity = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (100, 0, 100, 15, 30, 0)));
  gtk_widget_set_name (hsl_bgnormopacity, "hsl_bgnormopacity");
  gtk_widget_ref (hsl_bgnormopacity);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "hsl_bgnormopacity", hsl_bgnormopacity,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hsl_bgnormopacity);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), hsl_bgnormopacity, 2, 6, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_scale_set_value_pos (GTK_SCALE (hsl_bgnormopacity), GTK_POS_RIGHT);
  gtk_scale_set_digits (GTK_SCALE (hsl_bgnormopacity), 0);

  lbl_bgnormprev = gtk_label_new (_("preview"));
  gtk_widget_set_name (lbl_bgnormprev, "lbl_bgnormprev");
  gtk_widget_ref (lbl_bgnormprev);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormprev", lbl_bgnormprev,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormprev);
  gtk_table_attach (GTK_TABLE (tbl_bgnorm), lbl_bgnormprev, 6, 7, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  lbl_bgnosave = gtk_label_new (_("Background"));
  gtk_widget_set_name (lbl_bgnosave, "lbl_bgnosave");
  gtk_widget_ref (lbl_bgnosave);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnosave", lbl_bgnosave,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnosave);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbk_bgmain), gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbk_bgmain), 0), lbl_bgnosave);

  hbx_nbksavedbg = gtk_vbox_new (FALSE, 3);
  gtk_widget_set_name (hbx_nbksavedbg, "hbx_nbksavedbg");
  gtk_widget_ref (hbx_nbksavedbg);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "hbx_nbksavedbg", hbx_nbksavedbg,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbx_nbksavedbg);
  gtk_container_add (GTK_CONTAINER (nbk_bgmain), hbx_nbksavedbg);
  gtk_container_set_border_width (GTK_CONTAINER (hbx_nbksavedbg), 7);

  scw_images = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scw_images, "scw_images");
  gtk_widget_ref (scw_images);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "scw_images", scw_images,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scw_images);
  gtk_box_pack_start (GTK_BOX (hbx_nbksavedbg), scw_images, TRUE, TRUE, 0);
  gtk_widget_set_usize (scw_images, 472, 144);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw_images), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  lst_images = gtk_clist_new (4);
  gtk_widget_set_name (lst_images, "lst_images");
  gtk_widget_ref (lst_images);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lst_images", lst_images,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lst_images);
  gtk_container_add (GTK_CONTAINER (scw_images), lst_images);
  gtk_clist_set_column_width (GTK_CLIST (lst_images), 0, 71);
  gtk_clist_set_column_width (GTK_CLIST (lst_images), 1, 224);
  gtk_clist_set_column_width (GTK_CLIST (lst_images), 2, 59);
  gtk_clist_set_column_width (GTK_CLIST (lst_images), 3, 18);
  gtk_clist_column_titles_show (GTK_CLIST (lst_images));

  lbl_clmID = gtk_label_new (_("ID"));
  gtk_widget_set_name (lbl_clmID, "lbl_clmID");
  gtk_widget_ref (lbl_clmID);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_clmID", lbl_clmID,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_clmID);
  gtk_clist_set_column_widget (GTK_CLIST (lst_images), 0, lbl_clmID);

  lbl_clmloc = gtk_label_new (_("Location"));
  gtk_widget_set_name (lbl_clmloc, "lbl_clmloc");
  gtk_widget_ref (lbl_clmloc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_clmloc", lbl_clmloc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_clmloc);
  gtk_clist_set_column_widget (GTK_CLIST (lst_images), 1, lbl_clmloc);

  lbl_clmbgcol = gtk_label_new (_("BG Colour"));
  gtk_widget_set_name (lbl_clmbgcol, "lbl_clmbgcol");
  gtk_widget_ref (lbl_clmbgcol);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_clmbgcol", lbl_clmbgcol,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_clmbgcol);
  gtk_clist_set_column_widget (GTK_CLIST (lst_images), 2, lbl_clmbgcol);

  lbl_clmposz = gtk_label_new (_("Pos"));
  gtk_widget_set_name (lbl_clmposz, "lbl_clmposz");
  gtk_widget_ref (lbl_clmposz);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_clmposz", lbl_clmposz,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_clmposz);
  gtk_clist_set_column_widget (GTK_CLIST (lst_images), 3, lbl_clmposz);

  vbx_nbksavebg = gtk_hbox_new (FALSE, 4);
  gtk_widget_set_name (vbx_nbksavebg, "vbx_nbksavebg");
  gtk_widget_ref (vbx_nbksavebg);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "vbx_nbksavebg", vbx_nbksavebg,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbx_nbksavebg);
  gtk_box_pack_start (GTK_BOX (hbx_nbksavedbg), vbx_nbksavebg, FALSE, FALSE, 0);
  gtk_widget_set_usize (vbx_nbksavebg, -2, 26);
  gtk_container_set_border_width (GTK_CONTAINER (vbx_nbksavebg), 2);

  btn_addpic = gtk_button_new_with_label (_("Add"));
  gtk_widget_set_name (btn_addpic, "btn_addpic");
  gtk_widget_ref (btn_addpic);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_addpic", btn_addpic,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_addpic);
  gtk_box_pack_start (GTK_BOX (vbx_nbksavebg), btn_addpic, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_addpic, 86, 24);

  btn_editpic = gtk_button_new_with_label (_("Edit"));
  gtk_widget_set_name (btn_editpic, "btn_editpic");
  gtk_widget_ref (btn_editpic);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_editpic", btn_editpic,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_editpic);
  gtk_box_pack_start (GTK_BOX (vbx_nbksavebg), btn_editpic, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_editpic, 86, 24);

  btn_delpic = gtk_button_new_with_label (_("Delete"));
  gtk_widget_set_name (btn_delpic, "btn_delpic");
  gtk_widget_ref (btn_delpic);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_delpic", btn_delpic,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_delpic);
  gtk_box_pack_start (GTK_BOX (vbx_nbksavebg), btn_delpic, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_delpic, 86, 24);

  btn_delall = gtk_button_new_with_label (_("Delete All"));
  gtk_widget_set_name (btn_delall, "btn_delall");
  gtk_widget_ref (btn_delall);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_delall", btn_delall,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_delall);
  gtk_box_pack_start (GTK_BOX (vbx_nbksavebg), btn_delall, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_delall, 86, 23);

  tbr_savedmove = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gtk_widget_set_name (tbr_savedmove, "tbr_savedmove");
  gtk_widget_ref (tbr_savedmove);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbr_savedmove", tbr_savedmove,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbr_savedmove);
  gtk_box_pack_start (GTK_BOX (vbx_nbksavebg), tbr_savedmove, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (tbr_savedmove), 6);

  tmp_toolbar_icon = create_pixmap (wnd_main, "arrowup.xpm");
  btn_tbrsavedup = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_savedmove),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("button2"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (btn_tbrsavedup, "btn_tbrsavedup");
  gtk_widget_ref (btn_tbrsavedup);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_tbrsavedup", btn_tbrsavedup,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_tbrsavedup);
  gtk_widget_set_usize (btn_tbrsavedup, 24, -2);

  gtk_toolbar_append_space (GTK_TOOLBAR (tbr_savedmove));

  tmp_toolbar_icon = create_pixmap (wnd_main, "arrowdn.xpm");
  btn_tbrsaveddown = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_savedmove),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("button3"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (btn_tbrsaveddown, "btn_tbrsaveddown");
  gtk_widget_ref (btn_tbrsaveddown);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_tbrsaveddown", btn_tbrsaveddown,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_tbrsaveddown);

  lbl_bgnormtab = gtk_label_new (_("Saved BG's"));
  gtk_widget_set_name (lbl_bgnormtab, "lbl_bgnormtab");
  gtk_widget_ref (lbl_bgnormtab);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_bgnormtab", lbl_bgnormtab,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_bgnormtab);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbk_bgmain), gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbk_bgmain), 1), lbl_bgnormtab);

  hbx_btns = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbx_btns, "hbx_btns");
  gtk_widget_ref (hbx_btns);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "hbx_btns", hbx_btns,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbx_btns);
  gtk_box_pack_start (GTK_BOX (vbx_bgmain), hbx_btns, FALSE, FALSE, 6);

  btn_setbg = gtk_button_new_with_label (_("Set Background"));
  gtk_widget_set_name (btn_setbg, "btn_setbg");
  gtk_widget_ref (btn_setbg);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_setbg", btn_setbg,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_setbg);
  gtk_box_pack_start (GTK_BOX (hbx_btns), btn_setbg, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_setbg, 120, 24);


  gtk_signal_connect (GTK_OBJECT (btn_bgnormbrowse), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_bgnormbrowse_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_bgnormcol), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_bgnormcol_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (txt_cbobgnormpos), "changed",
                      GTK_SIGNAL_FUNC (on_txt_cbobgnormpos_changed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (txt_bgnormloc), "changed",
                      GTK_SIGNAL_FUNC (on_txt_bgnormloc_changed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_addpic), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_addpic_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_editpic), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_editpic_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_delpic), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_delpic_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_delall), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_delall_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_tbrsavedup), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_tbrsavedup_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_tbrsaveddown), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_tbrsaveddown_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_setbg), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_setbg_clicked),
                      NULL);
                      
  return vbx_bgmain;


}

GtkWidget*
create_wnd_additem (void)
{
  GtkWidget *wnd_additem;
  GtkWidget *fxd_mainadd;
  GtkWidget *txt_imagekey;
  GtkWidget *txt_imageloc;
  GtkWidget *btn_set;
  GtkWidget *btn_cancel;
  GtkWidget *txt_imagecustx;
  GtkWidget *txt_imagecusty;
  GtkWidget *btn_imagebrowse;
  GtkWidget *txt_imagebg;
  GtkWidget *btn_bgcolour;
  GtkWidget *txt_imagecustscale;
  GtkWidget *vpt_preview;
  GtkWidget *pxp_preview;
  GtkWidget *lbl_imagebg;
  GtkWidget *lbl_imagepos;
  GtkWidget *lbl_imagecustpos;
  GtkWidget *lbl_imagecustscale;
  GtkWidget *lbl_imageopacity;
  GtkWidget *lbl_imagecustx;
  GtkWidget *lbl_imagecusty;
  GtkWidget *cbo_imagepos;
  GList *cbo_imagepos_items = NULL;
  GtkWidget *txt_cboimagepos;
  GtkWidget *lbl_preview;
  GtkWidget *lbl_scletimes;
  GtkWidget *hsc_opacity;
  GtkWidget *lbl_imageloc;
  GtkWidget *lbl_imagekey;

  wnd_additem = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (wnd_additem, "wnd_additem");
  gtk_object_set_data (GTK_OBJECT (wnd_additem), "wnd_additem", wnd_additem);
  gtk_widget_set_usize (wnd_additem, 508, 222);
  gtk_window_set_title (GTK_WINDOW (wnd_additem), _("Add an image"));
  gtk_window_set_position (GTK_WINDOW (wnd_additem), GTK_WIN_POS_MOUSE);
  gtk_window_set_policy (GTK_WINDOW (wnd_additem), FALSE, FALSE, FALSE);

  fxd_mainadd = gtk_fixed_new ();
  gtk_widget_set_name (fxd_mainadd, "fxd_mainadd");
  gtk_widget_ref (fxd_mainadd);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "fxd_mainadd", fxd_mainadd,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fxd_mainadd);
  gtk_container_add (GTK_CONTAINER (wnd_additem), fxd_mainadd);

  txt_imagekey = gtk_entry_new ();
  gtk_widget_set_name (txt_imagekey, "txt_imagekey");
  gtk_widget_ref (txt_imagekey);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "txt_imagekey", txt_imagekey,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), txt_imagekey, 136, 8);
  gtk_widget_set_uposition (txt_imagekey, 136, 8);
  gtk_widget_set_usize (txt_imagekey, 196, 19);
  gtk_widget_show( txt_imagekey );

  txt_imageloc = gtk_entry_new ();
  gtk_widget_set_name (txt_imageloc, "txt_imageloc");
  gtk_widget_ref (txt_imageloc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "txt_imageloc", txt_imageloc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_imageloc);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), txt_imageloc, 136, 32);
  gtk_widget_set_uposition (txt_imageloc, 136, 32);
  gtk_widget_set_usize (txt_imageloc, 172, 19);

  btn_set = gtk_button_new_with_label (_("Set"));
  gtk_widget_set_name (btn_set, "btn_set");
  gtk_widget_ref (btn_set);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "btn_set", btn_set,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_set);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), btn_set, 8, 184);
  gtk_widget_set_uposition (btn_set, 8, 184);
  gtk_widget_set_usize (btn_set, 84, 24);

  btn_cancel = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_set_name (btn_cancel, "btn_cancel");
  gtk_widget_ref (btn_cancel);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "btn_cancel", btn_cancel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_cancel);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), btn_cancel, 96, 184);
  gtk_widget_set_uposition (btn_cancel, 96, 184);
  gtk_widget_set_usize (btn_cancel, 84, 24);

  txt_imagecustx = gtk_entry_new ();
  gtk_widget_set_name (txt_imagecustx, "txt_imagecustx");
  gtk_widget_ref (txt_imagecustx);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "txt_imagecustx", txt_imagecustx,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_imagecustx);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), txt_imagecustx, 136, 104);
  gtk_widget_set_uposition (txt_imagecustx, 136, 104);
  gtk_widget_set_usize (txt_imagecustx, 54, 19);
  gtk_widget_set_sensitive (txt_imagecustx, FALSE);

  txt_imagecusty = gtk_entry_new ();
  gtk_widget_set_name (txt_imagecusty, "txt_imagecusty");
  gtk_widget_ref (txt_imagecusty);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "txt_imagecusty", txt_imagecusty,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_imagecusty);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), txt_imagecusty, 224, 104);
  gtk_widget_set_uposition (txt_imagecusty, 224, 104);
  gtk_widget_set_usize (txt_imagecusty, 54, 19);
  gtk_widget_set_sensitive (txt_imagecusty, FALSE);

  btn_imagebrowse = gtk_button_new_with_label (_(".."));
  gtk_widget_set_name (btn_imagebrowse, "btn_imagebrowse");
  gtk_widget_ref (btn_imagebrowse);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "btn_imagebrowse", btn_imagebrowse,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_imagebrowse);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), btn_imagebrowse, 312, 32);
  gtk_widget_set_uposition (btn_imagebrowse, 312, 32);
  gtk_widget_set_usize (btn_imagebrowse, 20, 19);

  txt_imagebg = gtk_entry_new ();
  gtk_widget_set_name (txt_imagebg, "txt_imagebg");
  gtk_widget_ref (txt_imagebg);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "txt_imagebg", txt_imagebg,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_imagebg);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), txt_imagebg, 136, 56);
  gtk_widget_set_uposition (txt_imagebg, 136, 56);
  gtk_widget_set_usize (txt_imagebg, 172, 19);

  btn_bgcolour = gtk_button_new_with_label (_("?"));
  gtk_widget_set_name (btn_bgcolour, "btn_bgcolour");
  gtk_widget_ref (btn_bgcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "btn_bgcolour", btn_bgcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_bgcolour);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), btn_bgcolour, 312, 56);
  gtk_widget_set_uposition (btn_bgcolour, 312, 56);
  gtk_widget_set_usize (btn_bgcolour, 20, 19);

  txt_imagecustscale = gtk_entry_new ();
  gtk_widget_set_name (txt_imagecustscale, "txt_imagecustscale");
  gtk_widget_ref (txt_imagecustscale);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "txt_imagecustscale", txt_imagecustscale,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_imagecustscale);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), txt_imagecustscale, 136, 128);
  gtk_widget_set_uposition (txt_imagecustscale, 136, 128);
  gtk_widget_set_usize (txt_imagecustscale, 54, 19);
  gtk_widget_set_sensitive( txt_imagecustscale, FALSE );

  vpt_preview = gtk_viewport_new (NULL, NULL);
  gtk_widget_set_name (vpt_preview, "vpt_preview");
  gtk_widget_ref (vpt_preview);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "vpt_preview", vpt_preview,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vpt_preview);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), vpt_preview, 344, 8);
  gtk_widget_set_uposition (vpt_preview, 344, 8);
  gtk_widget_set_usize (vpt_preview, 152, 184);
  gtk_viewport_set_shadow_type (GTK_VIEWPORT (vpt_preview), GTK_SHADOW_ETCHED_IN);

  pxp_preview = create_pixmap (wnd_additem, NULL);
  gtk_widget_set_name (pxp_preview, "pxp_preview");
  gtk_widget_ref (pxp_preview);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "pxp_preview", pxp_preview,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pxp_preview);
  gtk_container_add (GTK_CONTAINER (vpt_preview), pxp_preview);
  gtk_widget_set_usize (pxp_preview, 152, 184);

  lbl_imagebg = gtk_label_new (_("Background Colour:"));
  gtk_widget_set_name (lbl_imagebg, "lbl_imagebg");
  gtk_widget_ref (lbl_imagebg);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_imagebg", lbl_imagebg,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_imagebg);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_imagebg, 8, 58);
  gtk_widget_set_uposition (lbl_imagebg, 8, 58);
  gtk_widget_set_usize (lbl_imagebg, 112, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_imagebg), 7.45058e-09, 0.5);

  lbl_imagepos = gtk_label_new (_("Position:"));
  gtk_widget_set_name (lbl_imagepos, "lbl_imagepos");
  gtk_widget_ref (lbl_imagepos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_imagepos", lbl_imagepos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_imagepos);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_imagepos, 8, 82);
  gtk_widget_set_uposition (lbl_imagepos, 8, 82);
  gtk_widget_set_usize (lbl_imagepos, 112, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_imagepos), 7.45058e-09, 0.5);

  lbl_imagecustpos = gtk_label_new (_("Custom Position:"));
  gtk_widget_set_name (lbl_imagecustpos, "lbl_imagecustpos");
  gtk_widget_ref (lbl_imagecustpos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_imagecustpos", lbl_imagecustpos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_imagecustpos);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_imagecustpos, 8, 106);
  gtk_widget_set_uposition (lbl_imagecustpos, 8, 106);
  gtk_widget_set_usize (lbl_imagecustpos, 104, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_imagecustpos), 7.45058e-09, 0.5);

  lbl_imagecustscale = gtk_label_new (_("Custom Scale:"));
  gtk_widget_set_name (lbl_imagecustscale, "lbl_imagecustscale");
  gtk_widget_ref (lbl_imagecustscale);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_imagecustscale", lbl_imagecustscale,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_imagecustscale);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_imagecustscale, 8, 128);
  gtk_widget_set_uposition (lbl_imagecustscale, 8, 128);
  gtk_widget_set_usize (lbl_imagecustscale, 101, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_imagecustscale), 7.45058e-09, 0.5);

  lbl_imageopacity = gtk_label_new (_("Opacity:"));
  gtk_widget_set_name (lbl_imageopacity, "lbl_imageopacity");
  gtk_widget_ref (lbl_imageopacity);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_imageopacity", lbl_imageopacity,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_imageopacity);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_imageopacity, 9, 155);
  gtk_widget_set_uposition (lbl_imageopacity, 9, 155);
  gtk_widget_set_usize (lbl_imageopacity, 64, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_imageopacity), 7.45058e-09, 0.5);

  lbl_imagecustx = gtk_label_new (_("X:"));
  gtk_widget_set_name (lbl_imagecustx, "lbl_imagecustx");
  gtk_widget_ref (lbl_imagecustx);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_imagecustx", lbl_imagecustx,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_imagecustx);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_imagecustx, 116, 106);
  gtk_widget_set_uposition (lbl_imagecustx, 116, 106);
  gtk_widget_set_usize (lbl_imagecustx, 18, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_imagecustx), 7.45058e-09, 0.5);

  lbl_imagecusty = gtk_label_new (_("Y:"));
  gtk_widget_set_name (lbl_imagecusty, "lbl_imagecusty");
  gtk_widget_ref (lbl_imagecusty);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_imagecusty", lbl_imagecusty,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_imagecusty);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_imagecusty, 202, 106);
  gtk_widget_set_uposition (lbl_imagecusty, 202, 106);
  gtk_widget_set_usize (lbl_imagecusty, 18, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_imagecusty), 7.45058e-09, 0.5);

  cbo_imagepos = gtk_combo_new ();
  gtk_widget_set_name (cbo_imagepos, "cbo_imagepos");
  gtk_widget_ref (cbo_imagepos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "cbo_imagepos", cbo_imagepos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_imagepos);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), cbo_imagepos, 136, 79);
  gtk_widget_set_uposition (cbo_imagepos, 136, 79);
  gtk_widget_set_usize (cbo_imagepos, 196, 22);
  gtk_combo_set_value_in_list (GTK_COMBO (cbo_imagepos), TRUE, TRUE);
  gtk_combo_set_use_arrows_always (GTK_COMBO (cbo_imagepos), TRUE);
  cbo_imagepos_items = g_list_append (cbo_imagepos_items, (gpointer) _("Tiled"));
  cbo_imagepos_items = g_list_append (cbo_imagepos_items, (gpointer) _("Center"));
  cbo_imagepos_items = g_list_append (cbo_imagepos_items, (gpointer) _("Scaled"));
  cbo_imagepos_items = g_list_append (cbo_imagepos_items, (gpointer) _("Scaled Perspective"));
  cbo_imagepos_items = g_list_append (cbo_imagepos_items, (gpointer) _("Scaled Custom"));
  cbo_imagepos_items = g_list_append (cbo_imagepos_items, (gpointer) _("Scaled Custom A"));
  cbo_imagepos_items = g_list_append (cbo_imagepos_items, (gpointer) _("Custom"));
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_imagepos), cbo_imagepos_items);
  g_list_free (cbo_imagepos_items);

  txt_cboimagepos = GTK_COMBO (cbo_imagepos)->entry;
  gtk_widget_set_name (txt_cboimagepos, "txt_cboimagepos");
  gtk_widget_ref (txt_cboimagepos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "txt_cboimagepos", txt_cboimagepos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_cboimagepos);
  gtk_entry_set_text (GTK_ENTRY (txt_cboimagepos), _("Tiled"));

  lbl_preview = gtk_label_new (_("preview"));
  gtk_widget_set_name (lbl_preview, "lbl_preview");
  gtk_widget_ref (lbl_preview);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_preview", lbl_preview,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_preview);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_preview, 344, 192);
  gtk_widget_set_uposition (lbl_preview, 344, 192);
  gtk_widget_set_usize (lbl_preview, 152, 16);

  lbl_scletimes = gtk_label_new (_("x (original)"));
  gtk_widget_set_name (lbl_scletimes, "lbl_scletimes");
  gtk_widget_ref (lbl_scletimes);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_scletimes", lbl_scletimes,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_scletimes);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_scletimes, 200, 131);
  gtk_widget_set_uposition (lbl_scletimes, 200, 131);
  gtk_widget_set_usize (lbl_scletimes, 72, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_scletimes), 7.45058e-09, 0.5);

  hsc_opacity = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (100, 0, 100, 15, 30, 0)));
  gtk_widget_set_name (hsc_opacity, "hsc_opacity");
  gtk_widget_ref (hsc_opacity);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "hsc_opacity", hsc_opacity,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hsc_opacity);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), hsc_opacity, 136, 152);
  gtk_widget_set_uposition (hsc_opacity, 136, 152);
  gtk_widget_set_usize (hsc_opacity, 192, 24);
  gtk_scale_set_value_pos (GTK_SCALE (hsc_opacity), GTK_POS_RIGHT);
  gtk_scale_set_digits (GTK_SCALE (hsc_opacity), 0);

  lbl_imageloc = gtk_label_new (_("Image Location:"));
  gtk_widget_set_name (lbl_imageloc, "lbl_imageloc");
  gtk_widget_ref (lbl_imageloc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_imageloc", lbl_imageloc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_imageloc);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_imageloc, 8, 34);
  gtk_widget_set_uposition (lbl_imageloc, 8, 34);
  gtk_widget_set_usize (lbl_imageloc, 112, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_imageloc), 7.45058e-09, 0.5);

  lbl_imagekey = gtk_label_new (_("Image Key:"));
  gtk_widget_set_name (lbl_imagekey, "lbl_imagekey");
  gtk_widget_ref (lbl_imagekey);
  gtk_object_set_data_full (GTK_OBJECT (wnd_additem), "lbl_imagekey", lbl_imagekey,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_imagekey);
  gtk_fixed_put (GTK_FIXED (fxd_mainadd), lbl_imagekey, 8, 10);
  gtk_widget_set_uposition (lbl_imagekey, 8, 10);
  gtk_widget_set_usize (lbl_imagekey, 112, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_imagekey), 7.45058e-09, 0.5);

  gtk_signal_connect (GTK_OBJECT (wnd_additem), "destroy_event",
                      GTK_SIGNAL_FUNC (on_wnd_additem_destroy_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (wnd_additem), "show",
                      GTK_SIGNAL_FUNC (on_wnd_additem_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (txt_imageloc), "changed",
                      GTK_SIGNAL_FUNC (on_txt_imageloc_changed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_set), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_set_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_cancel), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_cancel_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_imagebrowse), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_imagebrowse_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_bgcolour), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_bgcolour_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (txt_cboimagepos), "changed",
                      GTK_SIGNAL_FUNC (on_txt_cboimagepos_changed),
                      NULL);

  return wnd_additem;
}

GtkWidget*
create_wnd_about (void)
{
  GtkWidget *wnd_about;
  GtkWidget *fxd_about;
  GtkWidget *btn_aboutclose;
  GtkWidget *pxp_logo;
  GtkWidget *lbl_abouttitle;
  GtkWidget *lbl_aboutcrux;

  wnd_about = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (wnd_about, "wnd_about");
  gtk_object_set_data (GTK_OBJECT (wnd_about), "wnd_about", wnd_about);
  gtk_widget_set_usize (wnd_about, 377, 294);
  gtk_window_set_title (GTK_WINDOW (wnd_about), _("About: bgset"));
  gtk_window_set_position (GTK_WINDOW (wnd_about), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (wnd_about), TRUE);
  gtk_window_set_policy (GTK_WINDOW (wnd_about), FALSE, FALSE, FALSE);

  fxd_about = gtk_fixed_new ();
  gtk_widget_set_name (fxd_about, "fxd_about");
  gtk_widget_ref (fxd_about);
  gtk_object_set_data_full (GTK_OBJECT (wnd_about), "fxd_about", fxd_about,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fxd_about);
  gtk_container_add (GTK_CONTAINER (wnd_about), fxd_about);

  btn_aboutclose = gtk_button_new_with_label (_("close"));
  gtk_widget_set_name (btn_aboutclose, "btn_aboutclose");
  gtk_widget_ref (btn_aboutclose);
  gtk_object_set_data_full (GTK_OBJECT (wnd_about), "btn_aboutclose", btn_aboutclose,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_aboutclose);
  gtk_fixed_put (GTK_FIXED (fxd_about), btn_aboutclose, 24, 256);
  gtk_widget_set_uposition (btn_aboutclose, 24, 256);
  gtk_widget_set_usize (btn_aboutclose, 328, 24);

  pxp_logo = create_pixmap (wnd_about, "logo1b.xpm");
  gtk_widget_set_name (pxp_logo, "pxp_logo");
  gtk_widget_ref (pxp_logo);
  gtk_object_set_data_full (GTK_OBJECT (wnd_about), "pxp_logo", pxp_logo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pxp_logo);
  gtk_fixed_put (GTK_FIXED (fxd_about), pxp_logo, 8, 8);
  gtk_widget_set_uposition (pxp_logo, 8, 8);
  gtk_widget_set_usize (pxp_logo, 344, 64);

  lbl_abouttitle = gtk_label_new (_("bgset - desktop image manipulator"));
  gtk_widget_set_name (lbl_abouttitle, "lbl_abouttitle");
  gtk_widget_ref (lbl_abouttitle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_about), "lbl_abouttitle", lbl_abouttitle,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_abouttitle);
  gtk_fixed_put (GTK_FIXED (fxd_about), lbl_abouttitle, 24, 88);
  gtk_widget_set_uposition (lbl_abouttitle, 24, 88);
  gtk_widget_set_usize (lbl_abouttitle, 312, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_abouttitle), 7.45058e-09, 0.5);

  lbl_aboutcrux = gtk_label_new (_("bgset is an comprehensive tool for manipulating your desktop.\n\nCopyright (C) 2001, Luke Freeman.\n\nAuthor:   Luke Freeman\n\nBuild:  1       Revision:   0\nBuild Date:   Mon Aug  6 20:31:52 EST 2001\n\nLicence:  GPL"));
  gtk_widget_set_name (lbl_aboutcrux, "lbl_aboutcrux");
  gtk_widget_ref (lbl_aboutcrux);
  gtk_object_set_data_full (GTK_OBJECT (wnd_about), "lbl_aboutcrux", lbl_aboutcrux,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_aboutcrux);
  gtk_fixed_put (GTK_FIXED (fxd_about), lbl_aboutcrux, 24, 112);
  gtk_widget_set_uposition (lbl_aboutcrux, 24, 112);
  gtk_widget_set_usize (lbl_aboutcrux, 320, 144);
  gtk_label_set_justify (GTK_LABEL (lbl_aboutcrux), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (lbl_aboutcrux), TRUE);
  gtk_misc_set_alignment (GTK_MISC (lbl_aboutcrux), 7.45058e-09, 2.23517e-08);
  gtk_misc_set_padding (GTK_MISC (lbl_aboutcrux), 0, 2);

  gtk_signal_connect (GTK_OBJECT (btn_aboutclose), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_aboutclose_clicked),
                      NULL);

  return wnd_about;
}

GtkWidget*
create_wnd_options (void)
{
  GtkWidget *wnd_options;
  GtkWidget *fxd_options;
  GtkWidget *chk_saveonquit;
  GtkWidget *chk_saveonset;
  GtkWidget *lbl_optsave;
  GtkWidget *chk_quitonset;
  GSList *one_group = NULL;
  GtkWidget *rdo_savebefore;
  GtkWidget *rdo_saveafter;
  GtkWidget *rdo_saveboth;
  GtkWidget *hbuttonbox1;
  guint btn_optcancel_key;
  GtkWidget *btn_optcancel;
  guint btn_optapply_key;
  GtkWidget *btn_optapply;
  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new ();

  wnd_options = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (wnd_options, "wnd_options");
  gtk_object_set_data (GTK_OBJECT (wnd_options), "wnd_options", wnd_options);
  gtk_widget_set_usize (wnd_options, 405, 237);
  gtk_window_set_title (GTK_WINDOW (wnd_options), _("bgset: options"));
  gtk_window_set_position (GTK_WINDOW (wnd_options), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (wnd_options), TRUE);
  gtk_window_set_policy (GTK_WINDOW (wnd_options), FALSE, FALSE, FALSE);

  fxd_options = gtk_fixed_new ();
  gtk_widget_set_name (fxd_options, "fxd_options");
  gtk_widget_ref (fxd_options);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "fxd_options", fxd_options,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fxd_options);
  gtk_container_add (GTK_CONTAINER (wnd_options), fxd_options);

  chk_saveonquit = gtk_check_button_new_with_label (_("Save image information on quit"));
  gtk_widget_set_name (chk_saveonquit, "chk_saveonquit");
  gtk_widget_ref (chk_saveonquit);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "chk_saveonquit", chk_saveonquit,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (chk_saveonquit);
  gtk_fixed_put (GTK_FIXED (fxd_options), chk_saveonquit, 8, 72);
  gtk_widget_set_uposition (chk_saveonquit, 8, 72);
  gtk_widget_set_usize (chk_saveonquit, 388, 21);

  chk_saveonset = gtk_check_button_new_with_label (_("Save image information when setting background"));
  gtk_widget_set_name (chk_saveonset, "chk_saveonset");
  gtk_widget_ref (chk_saveonset);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "chk_saveonset", chk_saveonset,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (chk_saveonset);
  gtk_fixed_put (GTK_FIXED (fxd_options), chk_saveonset, 8, 16);
  gtk_widget_set_uposition (chk_saveonset, 8, 16);
  gtk_widget_set_usize (chk_saveonset, 388, 21);

  lbl_optsave = gtk_label_new (_("image data save:"));
  gtk_widget_set_name (lbl_optsave, "lbl_optsave");
  gtk_widget_ref (lbl_optsave);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "lbl_optsave", lbl_optsave,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_optsave);
  gtk_fixed_put (GTK_FIXED (fxd_options), lbl_optsave, 8, 43);
  gtk_widget_set_uposition (lbl_optsave, 8, 43);
  gtk_widget_set_usize (lbl_optsave, 104, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_optsave), 7.45058e-09, 0.5);

  chk_quitonset = gtk_check_button_new_with_label (_("Quit program after setting background"));
  gtk_widget_set_name (chk_quitonset, "chk_quitonset");
  gtk_widget_ref (chk_quitonset);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "chk_quitonset", chk_quitonset,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (chk_quitonset);
  gtk_fixed_put (GTK_FIXED (fxd_options), chk_quitonset, 8, 96);
  gtk_widget_set_uposition (chk_quitonset, 8, 96);
  gtk_widget_set_usize (chk_quitonset, 388, 21);

  rdo_savebefore = gtk_radio_button_new_with_label (one_group, _("before set"));
  one_group = gtk_radio_button_group (GTK_RADIO_BUTTON (rdo_savebefore));
  gtk_widget_set_name (rdo_savebefore, "rdo_savebefore");
  gtk_widget_ref (rdo_savebefore);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "rdo_savebefore", rdo_savebefore,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (rdo_savebefore);
  gtk_fixed_put (GTK_FIXED (fxd_options), rdo_savebefore, 112, 40);
  gtk_widget_set_uposition (rdo_savebefore, 112, 40);
  gtk_widget_set_usize (rdo_savebefore, 90, 21);

  rdo_saveafter = gtk_radio_button_new_with_label (one_group, _("after set"));
  one_group = gtk_radio_button_group (GTK_RADIO_BUTTON (rdo_saveafter));
  gtk_widget_set_name (rdo_saveafter, "rdo_saveafter");
  gtk_widget_ref (rdo_saveafter);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "rdo_saveafter", rdo_saveafter,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (rdo_saveafter);
  gtk_fixed_put (GTK_FIXED (fxd_options), rdo_saveafter, 208, 40);
  gtk_widget_set_uposition (rdo_saveafter, 208, 40);
  gtk_widget_set_usize (rdo_saveafter, 92, 21);

  rdo_saveboth = gtk_radio_button_new_with_label (one_group, _("both"));
  one_group = gtk_radio_button_group (GTK_RADIO_BUTTON (rdo_saveboth));
  gtk_widget_set_name (rdo_saveboth, "rdo_saveboth");
  gtk_widget_ref (rdo_saveboth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "rdo_saveboth", rdo_saveboth,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (rdo_saveboth);
  gtk_fixed_put (GTK_FIXED (fxd_options), rdo_saveboth, 304, 40);
  gtk_widget_set_uposition (rdo_saveboth, 304, 40);
  gtk_widget_set_usize (rdo_saveboth, 92, 21);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_set_name (hbuttonbox1, "hbuttonbox1");
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "hbuttonbox1", hbuttonbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_fixed_put (GTK_FIXED (fxd_options), hbuttonbox1, 202, 200);
  gtk_widget_set_uposition (hbuttonbox1, 202, 200);
  gtk_widget_set_usize (hbuttonbox1, 200, 29);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox1), 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox1), 98, 33);

  btn_optcancel = gtk_button_new_with_label ("");
  btn_optcancel_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (btn_optcancel)->child),
                                   _("_Cancel"));
  gtk_widget_add_accelerator (btn_optcancel, "clicked", accel_group,
                              btn_optcancel_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
  gtk_widget_set_name (btn_optcancel, "btn_optcancel");
  gtk_widget_ref (btn_optcancel);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "btn_optcancel", btn_optcancel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_optcancel);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), btn_optcancel);
  GTK_WIDGET_SET_FLAGS (btn_optcancel, GTK_CAN_DEFAULT);

  btn_optapply = gtk_button_new_with_label ("");
  btn_optapply_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (btn_optapply)->child),
                                   _("_Apply"));
  gtk_widget_add_accelerator (btn_optapply, "clicked", accel_group,
                              btn_optapply_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
  gtk_widget_set_name (btn_optapply, "btn_optapply");
  gtk_widget_ref (btn_optapply);
  gtk_object_set_data_full (GTK_OBJECT (wnd_options), "btn_optapply", btn_optapply,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_optapply);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), btn_optapply);
  GTK_WIDGET_SET_FLAGS (btn_optapply, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (btn_optcancel), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_optcancel_clicked),
                      NULL);

  gtk_window_add_accel_group (GTK_WINDOW (wnd_options), accel_group);

  return wnd_options;
}

GtkWidget*
create_dialog1 (void)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *dialog_action_area1;
  GtkWidget *hbox3;
  GtkWidget *button16;

  dialog1 = gtk_dialog_new ();
  gtk_widget_set_name (dialog1, "dialog1");
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  gtk_window_set_title (GTK_WINDOW (dialog1), _("dialog1"));
  gtk_window_set_policy (GTK_WINDOW (dialog1), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
  gtk_widget_set_name (dialog_vbox1, "dialog_vbox1");
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
  gtk_widget_set_name (dialog_action_area1, "dialog_action_area1");
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox3, "hbox3");
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbox3, TRUE, TRUE, 0);

  button16 = gtk_button_new_with_label (_("button16"));
  gtk_widget_set_name (button16, "button16");
  gtk_widget_ref (button16);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button16", button16,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button16);
  gtk_box_pack_start (GTK_BOX (hbox3), button16, FALSE, FALSE, 0);

  return dialog1;
}

