
/*
 * keybindcb.c
 * Keybinding plugin callbacks
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_SYS_DIR_H
    #include <sys/dir.h>
#else
    #ifdef HAVE_SYS_NDIR_H
        #include <sys/ndir.h>
    #else
        #ifdef HAVE_DIRENT_H
            #include <dirent.h>
        #else
            #include <sys/dirent.h>
        #endif
    #endif
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "keybindcb.h"
#include "convertwin.h"
#include "keybind.h"
#include "keybindfunc.h"

#include "../../support.h"
#include "../../msgbox.h"

int    lose_focus    = 0;    /* catch double capture events when entry loses focus */
int    focus_out     = 0;
int    binds_changed = 0;    /* prompt to save when starting new file */
int    keys_type;            /* the file type of the file currently in use */
gchar  *keys_file     = NULL; /* the file currently in use */
GList *keys_list     = NULL; /* the linked list containing key binding information*/

int stopcapture = 0;
int int_triggerinfochange = 0;

void
set_keys_type (int int_keystype)
{
    keys_type = int_keystype;
}

void
set_keys_file (const char *str_keysfile)
{
    keys_file = str_keysfile;
    g_print ("keys file = %s\n", keys_file);
}

void
on_btn_keyadd_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  /* grab widgets with key bind information */
  GtkWidget *wgt_tgbkeybindctrl    = NULL;
  GtkWidget *wgt_tgbkeybindalt     = NULL;
  GtkWidget *wgt_tgbkeybindshift   = NULL;
  GtkWidget *wgt_tgbkeybindwinkey  = NULL;
  GtkWidget *wgt_txtkeybindkey     = NULL;
  GtkWidget *wgt_txtkeybindcmd     = NULL;
  GtkWidget *wgt_txtcbokeybindfunc = NULL;
  GtkWidget *wgt_tgbkeybindcapture = NULL;
  GtkWidget *wgt_lstkeysaved       = NULL;
  GtkWidget *wgt_frmkeybindinfo    = NULL;
  GtkWidget *wgt_frmkeyfileinfo    = NULL;

  /* grab all the widgets needed in the save */
  wgt_tgbkeybindctrl    = lookup_widget( (GtkWidget *) button, "tgb_keybindctrl" );
  wgt_tgbkeybindalt     = lookup_widget( (GtkWidget *) button, "tgb_keybindalt" );
  wgt_tgbkeybindshift   = lookup_widget( (GtkWidget *) button, "tgb_keybindshift" );
  wgt_tgbkeybindwinkey  = lookup_widget( (GtkWidget *) button, "tgb_keybindwinkey" );
  wgt_txtkeybindkey     = lookup_widget( (GtkWidget *) button, "txt_keybindkey" );
  wgt_txtkeybindcmd     = lookup_widget( (GtkWidget *) button, "txt_keybindcmd" );
  wgt_txtcbokeybindfunc = lookup_widget( (GtkWidget *) button, "txt_cbokeybindfunc" );
  wgt_tgbkeybindcapture = lookup_widget( (GtkWidget *) button, "tgb_keybindcapture" );
  wgt_lstkeysaved       = lookup_widget( (GtkWidget *) button, "lst_keysaved" );
  wgt_frmkeybindinfo    = lookup_widget( (GtkWidget *) button, "frm_keybindinfo" );
  wgt_frmkeyfileinfo    = lookup_widget( (GtkWidget *) button, "frm_keyfileinfo" );
  
  gtk_clist_unselect_all( (GtkCList *) wgt_lstkeysaved );   /* unselect any items */

  /* reset binding information */
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindcapture,   FALSE );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl,   FALSE );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt,    FALSE );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift,  FALSE );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, FALSE );

  gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindkey, "" );
  //gtk_entry_set_text( (GtkEntry *) wgt_txtcbokeybindfunc, "BigNudgeDown" );

  /* show the key bind capture frame */
  gtk_widget_show( wgt_frmkeybindinfo );
  gtk_widget_hide( wgt_frmkeyfileinfo );
}


void
on_btn_keyremove_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *wgt_lstkeysaved = NULL;    /* the list of key bindings */

  char str_noselErrMsg[125];            /* error message for no selection */

  int selRow;                           /* the selected row */
  
  key_binding *key_bind;                /* the key binding information */


  wgt_lstkeysaved    = lookup_widget( (GtkWidget *) button, "lst_keysaved" );

  /* the error message to show if there was no selected node */
  strcpy( str_noselErrMsg, "You have not selected an item from the list\n" );
  strcat( str_noselErrMsg, "of key bindings.\n" );
  strcat( str_noselErrMsg, "Please make a selection from the list and\n" );
  strcat( str_noselErrMsg, "try again." );

  /* if there is a selected node .. continue */
  if( GTK_CLIST( wgt_lstkeysaved )->selection )
  {
    selRow = (int) (GTK_CLIST( wgt_lstkeysaved )->selection)->data;  /* get the selected row */
    /* get the key binfing information */
    key_bind = (key_binding *) gtk_clist_get_row_data( (GtkCList *) wgt_lstkeysaved, selRow );
    gtk_clist_remove( (GtkCList *) wgt_lstkeysaved, selRow );       /* remove the item from the list of bindings */
    keys_list = g_list_remove( keys_list, (gpointer *) key_bind );  /* remove the item from the linked list */
    key_bind = NULL;    /* clean up */
  }
  else
  {
    /* no selected node .. fail */
    message_box( 4, str_noselErrMsg, "Error!" );
  }
}


void
on_btn_keyclearall_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  int  int_retval = 0;
  char str_warnmsg[250] ;
  GtkWidget *wgt_lstkeysaved;
  
  wgt_lstkeysaved = lookup_widget( (GtkWidget *) button, "lst_keysaved" );

  strcpy( str_warnmsg, "Are you sure?\n" );
  strcat( str_warnmsg, "This will remove all of your saved\n" );
  strcat( str_warnmsg, "keybindings.\n" );
  
  
  int_retval = message_box( 3, str_warnmsg, "Warning!" );
  
  if( int_retval == 1 )
  {
    gtk_clist_clear( (GtkCList *) wgt_lstkeysaved );
    g_list_free( keys_list );
    keys_list = NULL;
  }

  binds_changed = 1;
}


void
on_btn_tbrkeynavup_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  
  
    GtkWidget *wgt_lstkeysaved;
    int int_currow;
    gpointer *ptr0;
    gpointer *ptr1;
    
    wgt_lstkeysaved = lookup_widget( (GtkWidget *) button, "lst_keysaved" );
  
    int_currow = get_selected_row_number ((GtkCList *) wgt_lstkeysaved);
  
    if (int_currow > 0)
    {
        if (GTK_CLIST(wgt_lstkeysaved)->rows != 1)
        {
            //keys_list = my_g_list_swap_prev (keys_list, g_list_nth(keys_list, int_currow));
        
            keys_list = reorder_keys_list (keys_list, int_currow, PREV_FLAG);
            gtk_clist_swap_rows ((GtkCList *) wgt_lstkeysaved, int_currow, (int_currow - 1));
        }
    }
}


void
on_btn_tbrkeynavdown_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{

    GtkWidget *wgt_lstkeysaved;
    int int_currow;
    wgt_lstkeysaved = lookup_widget( (GtkWidget *) button, "lst_keysaved" );
  
    int_currow = get_selected_row_number ((GtkCList *) wgt_lstkeysaved);
  
    if (int_currow >= 0)
    {
        if (int_currow != GTK_CLIST(wgt_lstkeysaved)->rows)
        {
            keys_list = reorder_keys_list (keys_list, int_currow, NEXT_FLAG);
            gtk_clist_swap_rows ((GtkCList *) wgt_lstkeysaved, int_currow, (int_currow + 1));
        }
    }

}


void
on_btn_tbrkeynew_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{

    gchar *file;
    int int_retval = 0;
    char str_warnmsg[250] ;
    GtkWidget *wgt_lstkeysaved;

    /* the frame widgets (bind/file information) */
    GtkWidget *wgt_frmkeybindinfo;
    GtkWidget *wgt_frmkeyfileinfo;

    GtkWidget *wgt_tgbkeybindcapture = NULL;



    wgt_lstkeysaved = lookup_widget( (GtkWidget *) button, "lst_keysaved" );
    
    strcpy( str_warnmsg, "The current file has been modified since\n" );
    strcat( str_warnmsg, "you last saved it.\n\n" );
    strcat( str_warnmsg, "Do you wish to save your file before\n" );
    strcat( str_warnmsg, "continuing (clicking NO will erase all\n" );
    strcat( str_warnmsg, "changes)." );
    
    if (binds_changed)
    {
        int_retval = message_box( 3, str_warnmsg, "Warning!" );
        
        if( int_retval == 1 )
        {
            if (!keys_file)
                file = return_file_w_title( NULL, "Select keybind config file to save .." );
            else
                file = keys_file;
            
            if( keys_type == BBKEYS_FILE )
                save_bbkeys_rc (file, keys_list);
            else if (keys_type == FLUX_FILE)
                save_fluxkeys_rc (file, keys_list);        
        }    
    }

    gtk_clist_clear( (GtkCList *) wgt_lstkeysaved );
    g_list_free( keys_list );
    keys_list = NULL;
    keys_file = NULL;
    binds_changed = 0;


    wgt_tgbkeybindcapture = lookup_widget( button, "tgb_keybindcapture" );

    /* kill capture when we lose focus -- dont want weird situations */
    if( gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindcapture ))
    {
        gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindcapture , FALSE );
    }	
    else
    {
        /* grab the frames */
        wgt_frmkeybindinfo = lookup_widget( (GtkWidget *) button, "frm_keybindinfo" );
        wgt_frmkeyfileinfo = lookup_widget( (GtkWidget *) button, "frm_keyfileinfo" );
    
        /* hide the frames */
        gtk_widget_hide( wgt_frmkeybindinfo );
        gtk_widget_hide( wgt_frmkeyfileinfo );
    }


}


void
load_keysrc_file( GtkWidget *wgt_ref, const char *str_filename )
{
  GtkWidget *wgt_lstkeysaved;
  
  FILE *key_file;       /* file to write to */
  
  int file_type;
  int i;
  int row_num;

  char  *rowitem[2];
  gchar *file = NULL;
  

  struct stat s;

  wgt_lstkeysaved = lookup_widget( wgt_ref, "lst_keysaved" );

  if( str_filename )
  {
    if (( stat( str_filename, &s ) == -1 ) || ( S_ISDIR( s.st_mode )))
    {
      if( message_box( 4, "The file you have selected is invalid, a directory,\nor you do not have write permission.\n\nPlease select a new file and try again.", "Invalid File" ) == MBX_OK )
        return;
    }
    else
    {
      if(( key_file = fopen( str_filename, "r" )))
      {
        file_type = determine_file_type(key_file);    /* determine the file type */
        keys_type  = file_type;                        /* set the file type global */
        
        /* reopen the file */
        freopen( file, "r", key_file );
    
        /* open a bbkeys file */
        if( file_type == BBKEYS_FILE )
          keys_list = read_bbkeys_rc( key_file );
    
        /* open a fluxbox file */
        if( file_type == FLUX_FILE )
          keys_list = read_fluxbox_rc( key_file );
        
    
        /* populate the key bindings list */
        for( i = 0; i < g_list_length( keys_list ); i++ )
        {
          key_binding *rr = g_list_nth_data( keys_list, i );
          char kb_string[200];
          
          memset(kb_string, '\0', 200);
        
          if( rr->ctl_mod )
            strcpy( kb_string, "Control+" );
          
          if( rr->alt_mod )
            strcat( kb_string, "Alt+" );
          
          if( rr->sft_mod )
            strcat( kb_string, "Shift+" );
          
          if( rr->win_mod )
            strcat( kb_string, "Win+" );
    
          strcat( kb_string, gdk_keyval_name( rr->key_val ));
          
          if( !rr->exec_command || strlen(rr->exec_command) == 0)
          {
            rr->exec_command = NULL;
            rowitem[0] = rr->action_type;
            rowitem[1] = kb_string;
            
            row_num = gtk_clist_append( (GtkCList *) wgt_lstkeysaved, rowitem );
            gtk_clist_set_row_data( (GtkCList *) wgt_lstkeysaved, row_num, rr );
            
            g_print( "%s: %s: %d\n", rr->action_type, kb_string, row_num );
          }
          else
          {
            char row_text[200];
            
            strcpy( row_text, rr->action_type );
            strcat( row_text, " (" );
            strcat( row_text, rr->exec_command );
            strcat( row_text, ")" );
            
            rowitem[0] = row_text;
            rowitem[1] = kb_string;
            
            row_num = gtk_clist_append( (GtkCList *) wgt_lstkeysaved, rowitem );
            gtk_clist_set_row_data( (GtkCList *) wgt_lstkeysaved, row_num, rr );
            
            g_print( "%s(%s): %s\n", rr->action_type, rr->exec_command, kb_string );
          }
        }
     
        fclose( key_file );
        
        set_keys_file (g_strdup(str_filename));
        
      }
    }
  }
}



void
on_btn_tbrkeyload_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{

    char *file;
    char  str_noSaveErrMsg[250];
    GtkWidget *wgt_lstkeysaved;
    
    wgt_lstkeysaved = lookup_widget( (GtkWidget *) button, "lst_keysaved" );

    strcpy( str_noSaveErrMsg, "The current file has been modified since\n" );
    strcat( str_noSaveErrMsg, "you last saved it.\n\n" );
    strcat( str_noSaveErrMsg, "Do you wish to save your file before\n" );
    strcat( str_noSaveErrMsg, "continuing (clicking NO will erase all\n" );
    strcat( str_noSaveErrMsg, "changes)." );

    if( binds_changed )
    {
        if (message_box( 3, str_noSaveErrMsg, "Warning!" ) == 1)
        {
            if (!keys_file)
                file = return_file_w_title( NULL, "Select keybind config file to save .." );
            else
                file = keys_file;
                
            if( keys_type == BBKEYS_FILE )
                save_bbkeys_rc( file, keys_list );
            else if( keys_type == FLUX_FILE )
                save_fluxkeys_rc( file, keys_list );
        }
    }

    gtk_clist_clear( (GtkCList *) wgt_lstkeysaved );
    g_list_free( keys_list );
    keys_list = NULL;

    file = return_file_w_title( NULL, "Select keybind config file to load.." );
    load_keysrc_file( (GtkWidget *) button, file );
  
}


void
on_btn_tbrkeysave_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  gchar *file;

  /* do save as */
  if( !keys_file )
    file = return_file_w_title( NULL, "Select the file to copy to .." );
  else
    file = keys_file;

  if( keys_type == BBKEYS_FILE )
    save_bbkeys_rc( file, keys_list );
  else if( keys_type == FLUX_FILE )
    save_fluxkeys_rc( file, keys_list );
    
  keys_file = file;
}


void
on_btn_tbrkeysaveas_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{

    gchar *file;
    
    file = return_file_w_title( NULL, "Select keybind config file .." );
    g_print ("%d\n", keys_type);

    if( keys_type == BBKEYS_FILE )
        save_bbkeys_rc( file, keys_list );
    else if( keys_type == FLUX_FILE )
        save_fluxkeys_rc( file, keys_list );

}


void
on_btn_tbrkeyfileinfo_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *wgt_frmkeybindinfo    = NULL;
  GtkWidget *wgt_frmkeyfileinfo    = NULL;
  GtkWidget *wgt_lblkeyfile        = NULL;
  GtkWidget *wgt_txtcbokeyfiletype = NULL;
    
  wgt_frmkeybindinfo    = lookup_widget( (GtkWidget *) button, "frm_keybindinfo" );
  wgt_frmkeyfileinfo    = lookup_widget( (GtkWidget *) button, "frm_keyfileinfo" );
  wgt_lblkeyfile        = lookup_widget( (GtkWidget *) button, "lbl_keyfile" );
  wgt_txtcbokeyfiletype = lookup_widget( (GtkWidget *) button, "txt_cbokeyfiletype" );

  g_print ("%s\n", keys_file);

  if( keys_file != NULL )
    gtk_label_set_text( (GtkLabel *) wgt_lblkeyfile, keys_file );
  else
    gtk_label_set_text( (GtkLabel *) wgt_lblkeyfile, "No file specified .." );
  

  if( keys_type == BBKEYS_FILE )
  {
    int_triggerinfochange = 1;
    gtk_entry_set_text( (GtkEntry *) wgt_txtcbokeyfiletype, "BB Keys" );
    int_triggerinfochange = 0;
  }
  else if( keys_type == FLUX_FILE )
  {
    int_triggerinfochange = 1;
    gtk_entry_set_text( (GtkEntry *) wgt_txtcbokeyfiletype, "Fluxbox" );
    int_triggerinfochange = 0;
  }
 
  
  gtk_widget_hide( wgt_frmkeybindinfo );
  gtk_widget_show( wgt_frmkeyfileinfo );
}


void
on_btn_tbrkeyconvert_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
  /*
  GtkWidget *wgt_wndkeyconvert;
  
  wgt_wndkeyconvert = create_wnd_keyconvert();
  gtk_widget_show( wgt_wndkeyconvert );
  */

  message_box( 4, "This function has not yet been\nimplemented.\n\nSorry ..", "Function not implemented!" );

}

gboolean
on_txt_keybindkey_key_press_event      (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  /* grab widgets with key bind information */
  GtkWidget *wgt_tgbkeybindctrl    = NULL;
  GtkWidget *wgt_tgbkeybindalt     = NULL;
  GtkWidget *wgt_tgbkeybindshift   = NULL;
  GtkWidget *wgt_tgbkeybindwinkey  = NULL;
  GtkWidget *wgt_tgbkeybindcapture = NULL;
  
  char str_keyErrMsg[125];
  
  wgt_tgbkeybindctrl    = lookup_widget( widget, "tgb_keybindctrl" );
  wgt_tgbkeybindalt     = lookup_widget( widget, "tgb_keybindalt" );
  wgt_tgbkeybindshift   = lookup_widget( widget, "tgb_keybindshift" );
  wgt_tgbkeybindwinkey  = lookup_widget( widget, "tgb_keybindwinkey" );
  wgt_tgbkeybindcapture = lookup_widget( widget, "tgb_keybindcapture" );


  /* if not in capture mode then kill the signal prematurely  
       -- no work to be done 
   */
  gtk_signal_emit_stop_by_name((GtkObject *) widget, "key_press_event" );

  if( !gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindcapture ))
    return FALSE;

  
  /* if handling a modifier  -- is this complete ..? */
  if(( event->keyval >= 65505 ) && ( event->keyval <= 65516 ))
  {
    switch( event->keyval )
    {
      /* SHIFT keys */
      case 65505:
      case 65506:
            
        gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift, TRUE );
        break;
      
      /* CTRL keys */
      case 65507:
      case 65508:
      
        gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl, TRUE );
        break;
                  
      
      /* ALT keys */
      case 65511:
      case 65512:
      case 65513:
      case 65514:
      
        gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt, TRUE );
        break;
    
    
      /* WIN keys */
      case 65515:
      case 65516:
      
        gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, TRUE );
        break;
    }
  }
  else
  {
    /* clear out the current entry then show keyval name */
    gtk_entry_set_text( (GtkEntry *) widget, "" );
    gtk_entry_set_text( (GtkEntry *) widget, gdk_keyval_name( gdk_keyval_to_lower( event->keyval )));


    /* if spacebar: kill process and then do a save 
        -- will refocus capture button if not caught (duh) */
    if( event->keyval == 32 || event->keyval == 65293 ) 
    {
      gtk_signal_emit_stop_by_name((GtkObject *) widget, "key_press_event" );
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindcapture, FALSE );

      return FALSE;
    }

    gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindcapture, FALSE );
    gtk_widget_grab_focus( wgt_tgbkeybindcapture );
  
    gdk_keyboard_ungrab( GDK_CURRENT_TIME );
    
  }

  return FALSE;
}


gboolean
on_txt_keybindkey_focus_out_event      (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
  
	GtkWidget *wgt_tgbkeybindcapture = NULL;
	wgt_tgbkeybindcapture = lookup_widget( widget, "tgb_keybindcapture" );


	/* kill capture when we lose focus -- dont want weird situations */
	if( gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindcapture ))
	{
		lose_focus = 1;
		gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindcapture , FALSE );
	}	

	return FALSE;
}


gboolean
on_txt_keybindkey_key_release_event    (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  /* grab widgets with key bind information */
  GtkWidget *wgt_tgbkeybindctrl    = NULL;
  GtkWidget *wgt_tgbkeybindalt     = NULL;
  GtkWidget *wgt_tgbkeybindshift   = NULL;
  GtkWidget *wgt_tgbkeybindwinkey  = NULL;
  GtkWidget *wgt_tgbkeybindcapture = NULL;
      
  wgt_tgbkeybindctrl    = lookup_widget( widget, "tgb_keybindctrl" );
  wgt_tgbkeybindalt     = lookup_widget( widget, "tgb_keybindalt" );
  wgt_tgbkeybindshift   = lookup_widget( widget, "tgb_keybindshift" );
  wgt_tgbkeybindwinkey  = lookup_widget( widget, "tgb_keybindwinkey" );
  wgt_tgbkeybindcapture = lookup_widget( widget, "tgb_keybindcapture" );
  
  
  /* check if in capture mode 
       -- we want to kill capture if the user has keyed-up */
  if( gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindcapture ))
  {
    /* if the user hasn't assigned a non-modifier binding then reset everything */
    char *str_key = gtk_entry_get_text((GtkEntry *) widget );
    
    //g_print( "txt length: %d\n",  event->keyval );
    
    if( strlen(str_key) == 0 )
    {
      /* reset everything */
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl, FALSE );
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt, FALSE );
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift, FALSE );
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, FALSE );
      
      /* kill capture mode */
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindcapture, FALSE );
    }
  }  
  
  gdk_keyboard_ungrab( GDK_CURRENT_TIME );
  
  return FALSE;
}


void
on_btn_keyinfosave_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  /* grab widgets with key bind information */
  GtkWidget *wgt_tgbkeybindctrl    = NULL;
  GtkWidget *wgt_tgbkeybindalt     = NULL;
  GtkWidget *wgt_tgbkeybindshift   = NULL;
  GtkWidget *wgt_tgbkeybindwinkey  = NULL;
  GtkWidget *wgt_txtkeybindkey     = NULL;
  GtkWidget *wgt_txtkeybindcmd     = NULL;
  GtkWidget *wgt_txtcbokeybindfunc = NULL;
  GtkWidget *wgt_tgbkeybindcapture = NULL;
  GtkWidget *wgt_lstkeysaved       = NULL;
  GtkWidget *wgt_frmkeybindinfo    = NULL;
  GtkWidget *wgt_frmkeyfileinfo    = NULL;

  /* store error messages */
  char str_fcnErrMsg[125];
  char str_keyErrMsg[125];
  char str_asnErrMsg[125];
  char str_cmdErrMsg[125];
  
  char *rowitem[2];          /* the text to add to the list box */
  char str_entry[1];  


  int  int_hasError = 0;     /* whether an error was present in the binding information */
  int  int_keyVal;           /* the int conversion of the key value */
  int  int_modCount = 0;     /* count of the modifiers */
  int  row_num;              /* row number of inserted item */

  gboolean bln_ctlkey = FALSE;
  gboolean bln_altkey = FALSE;
  gboolean bln_sftkey = FALSE;
  gboolean bln_winkey = FALSE;
  char *str_actiontype;
  char *str_execcommand;
	

  key_binding *key_bind;     /* the key-bind information to save*/

  /* grab all the widgets needed in the save */
  wgt_tgbkeybindctrl    = lookup_widget( (GtkWidget *) button, "tgb_keybindctrl" );
  wgt_tgbkeybindalt     = lookup_widget( (GtkWidget *) button, "tgb_keybindalt" );
  wgt_tgbkeybindshift   = lookup_widget( (GtkWidget *) button, "tgb_keybindshift" );
  wgt_tgbkeybindwinkey  = lookup_widget( (GtkWidget *) button, "tgb_keybindwinkey" );
  wgt_txtkeybindkey     = lookup_widget( (GtkWidget *) button, "txt_keybindkey" );
  wgt_txtkeybindcmd     = lookup_widget( (GtkWidget *) button, "txt_keybindcmd" );
  wgt_txtcbokeybindfunc = lookup_widget( (GtkWidget *) button, "txt_cbokeybindfunc" );
  wgt_tgbkeybindcapture = lookup_widget( (GtkWidget *) button, "tgb_keybindcapture" );
  wgt_lstkeysaved       = lookup_widget( (GtkWidget *) button, "lst_keysaved" );
  wgt_frmkeybindinfo    = lookup_widget( (GtkWidget *) button, "frm_keybindinfo" );
  wgt_frmkeyfileinfo    = lookup_widget( (GtkWidget *) button, "frm_keyfileinfo" );

  /* define all of the error messages */
  /* define the error message for multiple function assignments */
  strcpy( str_fcnErrMsg, "This function has already been assigned to\n" );
  strcat( str_fcnErrMsg, "another key binding.\n\n" );
  strcat( str_fcnErrMsg, "Functions cannot be assigned to multiple key\n" );
  strcat( str_fcnErrMsg, "bindings." );

  /* define the error message for multiple key binding assignments */
  strcpy( str_keyErrMsg, "The key binding you are trying to assign\n" );
  strcat( str_keyErrMsg, "has already been assigned.\n\n" );
  strcat( str_keyErrMsg, "Please select another key binding for this\n" );
  strcat( str_keyErrMsg, "function." );

  /* define the error message for no key value assignment */
  strcpy( str_asnErrMsg, "You have not properly assigned a key binding\n" );
  strcat( str_asnErrMsg, "for this function.\n\n" );
  strcat( str_asnErrMsg, "Please select a key binding for this\n" );
  strcat( str_asnErrMsg, "function." );

  /* define the error message for no command specified */
  strcpy( str_cmdErrMsg, "You have not properly assigned a key binding\n" );
  strcat( str_cmdErrMsg, "for this function.\n\n" );
  strcat( str_cmdErrMsg, "Please select a key binding for this\n" );
  strcat( str_cmdErrMsg, "function." );



  /* stop the capture if trying to save  -- you shouldn't be doing this anyway */
  if( gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindcapture ))
    gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindcapture, FALSE );


  strcpy( str_entry, gtk_entry_get_text( (GtkEntry *) wgt_txtkeybindkey ));

  /* if a key value has been specified .. continue */
  if( strlen( str_entry ) != 0 )
  {

    /* convert the key value from a char to an int */
    int_keyVal = gdk_keyval_from_name( str_entry );
    int_keyVal = (int) gdk_keyval_to_lower( int_keyVal );
    
  
    /* determine which modifiers have been set */
    if( gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindctrl ))
    {
	  bln_ctlkey = TRUE;
      int_modCount++;
    }
    
    if( gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindalt ))
    {
      bln_altkey = TRUE;
      int_modCount++;
    }

    if( gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindshift ))
    {
      bln_sftkey = TRUE;
      int_modCount++;
    }

    if( gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindwinkey ))
    {
      bln_winkey = TRUE;
      int_modCount++;
    }

    /* get the action type */
    str_actiontype = gtk_entry_get_text( (GtkEntry *) wgt_txtcbokeybindfunc );

  
    /* if action type is ExecCommand then get the command as well */
    if( !strcmp( str_actiontype , "ExecCommand" ))
    {
      str_execcommand = gtk_entry_get_text( (GtkEntry *) wgt_txtkeybindcmd );
    }
  
    /* if no command has been specified then fail */
    if(( !strcmp( str_actiontype , "ExecCommand" )) &&
       ( strlen( str_execcommand ) == 0 ))
    {
      message_box( 4, "No Command Specified.", "Error!" );
      int_hasError = 1;
    }


    if( int_hasError != 1 )
    {

      char kb_string[200];
      
      strcpy( kb_string, "" );
      
      if( bln_ctlkey )
        strcat( kb_string, "Control+" );
      
      if( bln_altkey )
        strcat( kb_string, "Alt+" );
      
      if( bln_sftkey )
        strcat( kb_string, "Shift+" );
      
      if( bln_winkey )
        strcat( kb_string, "Win+" );
        
      strcat( kb_string, gdk_keyval_name( int_keyVal ));
      
      if( strcmp( str_actiontype , "ExecCommand" ))
      {
        
        str_execcommand = NULL;
        
        rowitem[0] = str_actiontype;
        rowitem[1] = kb_string;
      }
      else
      {
        char row_text[200];
        
        strcpy( row_text, str_actiontype );
        strcat( row_text, " (" );
        strcat( row_text, str_execcommand );
        strcat( row_text, ")" );      

        rowitem[0] = row_text;
        rowitem[1] = kb_string;
      }


      /* if there is a selected node .. do an edit */
      if( GTK_CLIST( wgt_lstkeysaved )->selection )
      {
        key_binding *rem_bind = NULL;
        int int_selRow;

        int_selRow = (GTK_CLIST( wgt_lstkeysaved )->selection)->data;  /* get the selected row */


		key_bind = create_binding( int_keyVal, bln_altkey, bln_ctlkey, bln_sftkey, bln_winkey, 
                           str_actiontype, str_execcommand );

        /* check to see if the key binding has been used before */
        if(( locate_key_binding( key_bind, keys_list ) >= 0 ) &&
           ( locate_key_binding( key_bind, keys_list ) != int_selRow ))
        {
          message_box( 4, str_keyErrMsg, "Error!" );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, FALSE );
          gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindkey, "" );
          int_hasError = 1;
        }
      
        /* check to see if the function has been assigned to another key binding */
        if(( locate_function( key_bind, keys_list ) >= 0 ) &&
           ( locate_function( key_bind, keys_list ) != int_selRow ))
        {
          message_box( 4, str_fcnErrMsg, "Error!" );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, FALSE );
          gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindkey, "" );
          int_hasError = 1;
        }

        if( int_hasError != 1 )
        {
          /* get the key binding information */
          rem_bind = (key_binding *) gtk_clist_get_row_data( (GtkCList *) wgt_lstkeysaved, int_selRow );
          keys_list = g_list_remove( keys_list, (gpointer *) rem_bind );  /* remove the item from the linked list */
          rem_bind = NULL;    /* clean up */
          
          gtk_clist_set_text( (GtkCList *) wgt_lstkeysaved, int_selRow, 0, rowitem[0] );
          gtk_clist_set_text( (GtkCList *) wgt_lstkeysaved, int_selRow, 1, rowitem[1] );
          
          gtk_clist_set_row_data( (GtkCList *) wgt_lstkeysaved, int_selRow, key_bind );
          keys_list = g_list_insert( keys_list, key_bind, int_selRow );
        }
    
    
      }
      else
      {

		key_bind = create_binding( int_keyVal, bln_altkey, bln_ctlkey, bln_sftkey, bln_winkey, 
                           str_actiontype, str_execcommand );

        /* check to see if the key binding has been used before */
        if( locate_key_binding( key_bind, keys_list ) >= 0 )
        {
          message_box( 4, str_keyErrMsg, "Add Error!" );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, FALSE );
          gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindkey, "" );
          int_hasError = 1;
        }
      

        /* check to see if the function has been assigned to another key binding */
        if( locate_function( key_bind, keys_list ) >= 0 )
        {
          message_box( 4, str_fcnErrMsg, "Add Error!" );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift, FALSE );
          gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, FALSE );
          gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindkey, "" );
          int_hasError = 1;
        }
 
		g_print( "locate function done ..\n");


        /* add the key binding to the list ONLY if no errors have occured */
        if( int_hasError != 1 )
        {

          /* add a new item to the list */
          row_num = gtk_clist_append( (GtkCList *) wgt_lstkeysaved, rowitem );
          gtk_clist_set_row_data( (GtkCList *) wgt_lstkeysaved, row_num, key_bind );
          keys_list = g_list_append( keys_list, key_bind );
          gtk_clist_select_row( (GtkCList *) wgt_lstkeysaved, row_num, 1 );
        }
      }



      /* reset binding information */
      /*gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl,   FALSE );
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt,    FALSE );
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift,  FALSE );
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, FALSE );
    
      gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindkey, "" );
      gtk_entry_set_text( (GtkEntry *) wgt_txtcbokeybindfunc, "BigNudgeDown" );*/
    
      binds_changed = 1;    /* will prompt for save when selecting new or open */
        
      /* hide the key binding infomation frame */
      //gtk_widget_hide( wgt_frmkeybindinfo );
      //gtk_widget_hide( wgt_frmkeyfileinfo );
    }
  }
  else
  {
    /* fail because no the key binding has not been properly assigned */
    message_box( 4, str_asnErrMsg, "Error!" );
  }
}


void
on_btn_keyinfocancel_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	/* the frame widgets (bind/file information) */
	GtkWidget *wgt_frmkeybindinfo;
	GtkWidget *wgt_frmkeyfileinfo;

	GtkWidget *wgt_tgbkeybindcapture = NULL;

	wgt_tgbkeybindcapture = lookup_widget( button, "tgb_keybindcapture" );

	/* kill capture when we lose focus -- dont want weird situations */
	if( gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindcapture ))
	{
		gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindcapture , FALSE );
	}	
	else
	{
		/* grab the frames */
		wgt_frmkeybindinfo = lookup_widget( (GtkWidget *) button, "frm_keybindinfo" );
		wgt_frmkeyfileinfo = lookup_widget( (GtkWidget *) button, "frm_keyfileinfo" );
	  	
		/* hide the frames */
		gtk_widget_hide( wgt_frmkeybindinfo );
		gtk_widget_hide( wgt_frmkeyfileinfo );
    }

}


void
on_tgb_keybindcapture_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *wgt_wndmain;
	
	GtkWidget *wgt_txtkeybindkey;
	GtkWidget *wgt_tgbkeybindctrl;
	GtkWidget *wgt_tgbkeybindalt;
	GtkWidget *wgt_tgbkeybindshift;
	GtkWidget *wgt_tgbkeybindwinkey;


	wgt_wndmain = (GtkWidget *) GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET((GtkWidget *) button)->parent)->parent)->parent)->parent)->parent)->parent)->parent;


	wgt_txtkeybindkey     = lookup_widget( (GtkWidget *) button, "txt_keybindkey" );
	wgt_tgbkeybindctrl	  = lookup_widget( (GtkWidget *) button, "tgb_keybindctrl" );
	wgt_tgbkeybindalt	  = lookup_widget( (GtkWidget *) button, "tgb_keybindalt" );
	wgt_tgbkeybindshift   = lookup_widget( (GtkWidget *) button, "tgb_keybindshift" );
	wgt_tgbkeybindwinkey  = lookup_widget( (GtkWidget *) button, "tgb_keybindwinkey" );
	
	if( gtk_toggle_button_get_active( (GtkToggleButton *) button ))
	{
		/* reset binding information */
		gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl,   FALSE );
		gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt,    FALSE );
		gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift,  FALSE );
		gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, FALSE );
		gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindkey, "" );
		gdk_keyboard_grab( GTK_WIDGET(wgt_wndmain)->window, FALSE, GDK_CURRENT_TIME );
	}

	gtk_widget_grab_focus( wgt_txtkeybindkey );
		
}

void
on_lst_keysaved_select_row             (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  GtkWidget *wgt_frmkeybindinfo = NULL;
  GtkWidget *wgt_frmkeyfileinfo = NULL;
  
  GtkWidget *wgt_txtkeybindkey  = NULL;
  
  GtkWidget *wgt_txtcbokeybindfunc = NULL;
  
  GtkWidget *wgt_tgbkeybindctrl = NULL;
  GtkWidget *wgt_tgbkeybindalt = NULL;
  GtkWidget *wgt_tgbkeybindshift = NULL;
  GtkWidget *wgt_tgbkeybindwinkey = NULL;
  
  GtkWidget *wgt_txtkeybindcmd = NULL;
  GtkWidget *wgt_lblkeybindcmd = NULL;
    
  key_binding *bind = NULL;
  
  
  wgt_frmkeybindinfo    = lookup_widget( (GtkWidget *) clist, "frm_keybindinfo" );
  wgt_frmkeyfileinfo    = lookup_widget( (GtkWidget *) clist, "frm_keyfileinfo" );
  
  wgt_txtkeybindkey     = lookup_widget( (GtkWidget *) clist, "txt_keybindkey" );
  
  wgt_txtcbokeybindfunc = lookup_widget( (GtkWidget *) clist, "txt_cbokeybindfunc" );

  wgt_tgbkeybindctrl    = lookup_widget( (GtkWidget *) clist, "tgb_keybindctrl" );
  wgt_tgbkeybindalt     = lookup_widget( (GtkWidget *) clist, "tgb_keybindalt" );
  wgt_tgbkeybindshift   = lookup_widget( (GtkWidget *) clist, "tgb_keybindshift" );
  wgt_tgbkeybindwinkey  = lookup_widget( (GtkWidget *) clist, "tgb_keybindwinkey" );
  
  wgt_txtkeybindcmd     = lookup_widget( (GtkWidget *) clist, "txt_keybindcmd" );
  wgt_lblkeybindcmd     = lookup_widget( (GtkWidget *) clist, "lbl_keybindcmd" );
  

  bind = g_list_nth_data( keys_list, row );

  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl, TRUE );
  //gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl, bind->ctl_mod );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt, bind->alt_mod );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift, bind->sft_mod );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, bind->win_mod );

  gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindkey, gdk_keyval_name( gdk_keyval_to_lower( bind->key_val )));
  gtk_entry_set_text( (GtkEntry *) wgt_txtcbokeybindfunc, bind->action_type );

  if( !strcmp( bind->action_type, "ExecCommand" ))
  {
    gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindcmd, bind->exec_command );
    gtk_widget_show( wgt_txtkeybindcmd );
    gtk_widget_show( wgt_lblkeybindcmd );
  }
  else
  {
    gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindcmd, "" );
    gtk_widget_hide( wgt_txtkeybindcmd );
    gtk_widget_hide( wgt_lblkeybindcmd );
  }

  if( !GTK_WIDGET_VISIBLE( wgt_frmkeybindinfo ))
    gtk_widget_show( wgt_frmkeybindinfo );

  gtk_widget_hide( wgt_frmkeyfileinfo );  
}


void
on_lst_keysaved_unselect_row           (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  /* grab widgets with key bind information */
  GtkWidget *wgt_tgbkeybindctrl    = NULL;
  GtkWidget *wgt_tgbkeybindalt     = NULL;
  GtkWidget *wgt_tgbkeybindshift   = NULL;
  GtkWidget *wgt_tgbkeybindwinkey  = NULL;
  GtkWidget *wgt_txtkeybindkey     = NULL;
  GtkWidget *wgt_txtkeybindcmd     = NULL;
  GtkWidget *wgt_txtcbokeybindfunc = NULL;
  GtkWidget *wgt_tgbkeybindcapture = NULL;
  GtkWidget *wgt_lstkeysaved       = NULL;
  GtkWidget *wgt_frmkeybindinfo    = NULL;
  GtkWidget *wgt_frmkeyfileinfo    = NULL;

  /* grab all the widgets needed in the save */
  wgt_tgbkeybindctrl    = lookup_widget( (GtkWidget *) clist, "tgb_keybindctrl" );
  wgt_tgbkeybindalt     = lookup_widget( (GtkWidget *) clist, "tgb_keybindalt" );
  wgt_tgbkeybindshift   = lookup_widget( (GtkWidget *) clist, "tgb_keybindshift" );
  wgt_tgbkeybindwinkey  = lookup_widget( (GtkWidget *) clist, "tgb_keybindwinkey" );
  wgt_txtkeybindkey     = lookup_widget( (GtkWidget *) clist, "txt_keybindkey" );
  wgt_txtkeybindcmd     = lookup_widget( (GtkWidget *) clist, "txt_keybindcmd" );
  wgt_txtcbokeybindfunc = lookup_widget( (GtkWidget *) clist, "txt_cbokeybindfunc" );
  wgt_tgbkeybindcapture = lookup_widget( (GtkWidget *) clist, "tgb_keybindcapture" );
  wgt_lstkeysaved       = lookup_widget( (GtkWidget *) clist, "lst_keysaved" );
  wgt_frmkeybindinfo    = lookup_widget( (GtkWidget *) clist, "frm_keybindinfo" );
  wgt_frmkeyfileinfo    = lookup_widget( (GtkWidget *) clist, "frm_keyfileinfo" );

  /* reset binding information */
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindcapture,   FALSE );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindctrl,   FALSE );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindalt,    FALSE );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindshift,  FALSE );
  gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tgbkeybindwinkey, FALSE );

  gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindkey, "" );
  gtk_entry_set_text( (GtkEntry *) wgt_txtcbokeybindfunc, "BigNudgeDown" );

  /* show the key bind capture frame */
  gtk_widget_hide( wgt_frmkeybindinfo );
  gtk_widget_hide( wgt_frmkeyfileinfo );
}


void
on_txt_cbokeybindfunc_changed          (GtkEditable     *editable,
                                        gpointer         user_data)
{
  char *str_function;
  GtkWidget *wgt_txtkeybindcmd = NULL; 
  GtkWidget *wgt_lblkeybindcmd = NULL; 

  lose_focus = 0;

  wgt_txtkeybindcmd     = lookup_widget( (GtkWidget *) editable, "txt_keybindcmd" );
  wgt_lblkeybindcmd     = lookup_widget( (GtkWidget *) editable, "lbl_keybindcmd" );

  str_function = gtk_entry_get_text( (GtkEntry *) editable );
  
  if( !strcmp( str_function, "ExecCommand" ))
  {
    gtk_widget_show( wgt_txtkeybindcmd );
    gtk_widget_show( wgt_lblkeybindcmd );
  }
  else
  {
    gtk_entry_set_text( (GtkEntry *) wgt_txtkeybindcmd, "" );
    gtk_widget_hide( wgt_txtkeybindcmd );
    gtk_widget_hide( wgt_lblkeybindcmd );
  }
}

void
on_txt_cbokeyfiletype_changed			(GtkEditable     *editable,
                                        gpointer         user_data)
{

    GtkWidget *lst_cbokeyFuncs;
    GtkWidget *cbo_keyFuncs = lookup_widget (editable, "cbo_keybindfunc");
    GList *cbo_keybindfunc_items = NULL;
    int int_incompat = 0;

    char str_errMsg[255];

    /* aaaaand again to catch those nasty double trigger events when you hold down the mouse on the combo */
    if (int_triggerinfochange >= 1) {
        gtk_signal_handler_block_by_func (GTK_OBJECT(editable), GTK_SIGNAL_FUNC(on_txt_cbokeyfiletype_changed),
            user_data);
        
        if (keys_type == BBKEYS_FILE)
            gtk_entry_set_text( (GtkEntry *) editable, "BB Keys" );
        else if (keys_type == FLUX_FILE)
            gtk_entry_set_text( (GtkEntry *) editable, "Fluxbox" );
        
        gtk_signal_emit_stop_by_name (GTK_OBJECT(editable), "changed");
        gtk_signal_handler_unblock_by_func (GTK_OBJECT(editable), GTK_SIGNAL_FUNC(on_txt_cbokeyfiletype_changed),
            user_data);

        int_triggerinfochange--;
        return;
    }

    if (!strcmp( gtk_entry_get_text( (GtkEntry *) editable), "BB Keys" ) && (keys_type == BBKEYS_FILE))
        return;
    if (!strcmp( gtk_entry_get_text( (GtkEntry *) editable), "Fluxbox" ) && (keys_type == FLUX_FILE))
        return;


    if (keys_type == BBKEYS_FILE)
        int_incompat = check_available_items (keys_list, FLUX_FILE);
    else if (keys_type == FLUX_FILE)
        int_incompat = check_available_items (keys_list, BBKEYS_FILE);

    if (int_incompat > 0) {

        if (message_box (3, "Your keybindings list contains bindings that\nare incompatable for this file format.\n\nDo you wish to contine (invalid keybindings\nWILL be retained)?", 
                         "Warning!") == 2)
        {
            int_triggerinfochange = 1;
            if (keys_type == BBKEYS_FILE) {
                gtk_signal_handler_block_by_func (GTK_OBJECT(editable), GTK_SIGNAL_FUNC(on_txt_cbokeyfiletype_changed),
                    user_data);
                gtk_entry_set_text( (GtkEntry *) editable, "BB Keys" );
                gtk_signal_emit_stop_by_name (GTK_OBJECT(editable), "changed");
                gtk_signal_handler_unblock_by_func (GTK_OBJECT(editable), GTK_SIGNAL_FUNC(on_txt_cbokeyfiletype_changed),
                    user_data);                
            }
            else if (keys_type == FLUX_FILE)
            {
                gtk_signal_handler_block_by_func (GTK_OBJECT(editable), GTK_SIGNAL_FUNC(on_txt_cbokeyfiletype_changed),
                    user_data);
                gtk_entry_set_text( (GtkEntry *) editable, "Fluxbox" );
                gtk_signal_emit_stop_by_name (GTK_OBJECT(editable), "changed");
                gtk_signal_handler_unblock_by_func (GTK_OBJECT(editable), GTK_SIGNAL_FUNC(on_txt_cbokeyfiletype_changed),
                    user_data);
            }
            
            return;
        
        }
    }

    if( !strcmp( gtk_entry_get_text( (GtkEntry *) editable), "BB Keys" ))
    {
        keys_type = BBKEYS_FILE;
        lst_cbokeyFuncs = GTK_COMBO(cbo_keyFuncs)->list;
        gtk_list_clear_items (lst_cbokeyFuncs, 0, -1);
        
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeDown"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeLeft"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeRight"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeUp"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Close"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ExecCommand"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("HorizontalDecrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("HorizontalIncrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Lower"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeHorizontal"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeVertical"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Minimize"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWindowAllWorkspaces"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeDown"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeLeft"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeUp"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeRight"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Raise"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ShadeWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("StickWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ToggleDecor"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("VerticalDecrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("VerticalIncrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace1"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace2"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace3"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace4"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace5"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace6"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace7"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace8"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace9"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace10"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace11"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace12"));
       
        gtk_combo_set_popdown_strings (GTK_COMBO(cbo_keyFuncs), cbo_keybindfunc_items);
        g_list_free (cbo_keybindfunc_items);

    }
    else
    {
        keys_type = FLUX_FILE;
        lst_cbokeyFuncs = GTK_COMBO(cbo_keyFuncs)->list;
        gtk_list_clear_items (lst_cbokeyFuncs, 0, -1);
        
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeDown"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeLeft"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeRight"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeUp"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Close"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ExecCommand"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("HorizontalDecrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("HorizontalIncrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("KillWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("LeftWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Lower"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeHorizontal"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeVertical"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Minimize"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextTab"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeDown"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeLeft"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeUp"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeRight"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevTab"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Raise"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("RightWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("RootMenu"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ShadeWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("StickWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ToggleDecor"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("VerticalDecrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("VerticalIncrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace1"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace2"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace3"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace4"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace5"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace6"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace7"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace8"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace9"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace10"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace11"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace12"));
        
        gtk_combo_set_popdown_strings (GTK_COMBO(cbo_keyFuncs), cbo_keybindfunc_items);
        g_list_free (cbo_keybindfunc_items);
    }


}

void    
on_keybutton_pressed                     (GtkButton       *button,
                                          gpointer         user_data)
{

  /* 
   * function: on_keybutton_pressed
   *
   * Generic function to handle hotkey button "pressed" events.
   * Prevent hotkey states if not in capture mode. This should never
   * be able to be set manually (ie: pressed).
   */
      

  GtkWidget *wgt_tgbkeybindcapture = lookup_widget( (GtkWidget *) button, "tgb_keybindcapture" );    /* capture toggle */
  
  
  /* if not in capture mode than reset button state */
  if( !gtk_toggle_button_get_active( (GtkToggleButton *) wgt_tgbkeybindcapture )) 
  {
    /* note: emit_stop doesn't kill event so need to reset -- why ?? */
    gtk_signal_emit_stop_by_name( (GtkObject *) button, "pressed" );
    gtk_toggle_button_set_active( (GtkToggleButton *) button, !gtk_toggle_button_get_active ((GtkToggleButton *) button));
    
    return;
  }


}

