
/*
 * keybindfunc.c
 * Keybinding config file manipulation routines
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_SYS_DIR_H
    #include <sys/dir.h>
#else
    #ifdef HAVE_SYS_NDIR_H
        #include <sys/ndir.h>
    #else
        #ifdef HAVE_DIRENT_H
            #include <dirent.h>
        #else
            #include <sys/dirent.h>
        #endif
    #endif
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include "keybindfunc.h"
#include "keybind.h"

#include "../../msgbox.h"
#include "../../support.h"


int
get_selected_row_number (GtkCList *clist)
{
	GList *selection;
	
	selection = GTK_CLIST(clist)->selection;
	
        /* nothing is selected */
	if(!selection)
		return -1;
	
        return GPOINTER_TO_INT(selection->data);
}

key_binding *
create_binding( int key_val, gboolean alt_mod, gboolean ctl_mod, gboolean sft_mod, 
                gboolean win_mod, char *action_type, char *exec_command  ) 
{

  /*
   * function:    create_binding
   * description: safely create a new keybinding
   * 
   */

  //key_binding *new_item = calloc( 1, sizeof( key_binding ));

	key_binding *new_item;

	new_item = g_new( key_binding, 1 );

	assert( key_val );	  /* see if any errors occur */

	/* set the item data */
	
	new_item->key_val      = key_val;
	new_item->alt_mod      = alt_mod;
	new_item->ctl_mod      = ctl_mod;
	new_item->sft_mod      = sft_mod;
	new_item->win_mod      = win_mod;
	new_item->action_type  = g_strdup( action_type );
	new_item->exec_command = g_strdup( exec_command );

	return new_item;

}


int
locate_key_binding( key_binding *key_to_find, GList *lst_to_search )
{
  
  int list_length = g_list_length( lst_to_search );
  int i = 0;
  
  
  for( i = 0; i < list_length; i++ )
  {

    key_binding *tmp_key = g_list_nth_data( lst_to_search, i );
    
	g_print( "%d\n", tmp_key->key_val );
	g_print( "%s\n", tmp_key->action_type );


	if(( tmp_key->key_val == key_to_find->key_val ) &&
       ( tmp_key->alt_mod == key_to_find->alt_mod ) &&
       ( tmp_key->ctl_mod == key_to_find->ctl_mod ) &&
       ( tmp_key->sft_mod == key_to_find->sft_mod ) &&
       ( tmp_key->win_mod == key_to_find->win_mod ))
    {
      

		return i;
    }

  }


  return -1;

}


int
locate_function( key_binding *key_to_find, GList *lst_to_search )
{
  
  
	int list_length = g_list_length( lst_to_search );
	int i = 0;
	int x = 0;
	
	
	for( i = 0; i < list_length; i++ )
	{
		key_binding *tmp_key = g_list_nth_data( lst_to_search, i );
	
		if( !strcmp( key_to_find->action_type, tmp_key->action_type ))
		{
			if( !strcmp( key_to_find->action_type, "ExecCommand" ))
				return -1;
			else
				return i;
	
		}
	}	

	return -1;

}


int 
determine_file_type( FILE *tmp_file )
{

  char key_line[255];
  int found_line;
  int file_type;

  found_line = 0; 
    
  while( !found_line && !feof(tmp_file))
  {
   
    fgets( key_line, 1024, tmp_file );
    
    if( !fnmatch( BBCONF_STRING1, key_line, 0 ) || !fnmatch( BBCONF_STRING2, key_line, 0 ))
    {
      file_type = BBKEYS_FILE;
      found_line = 1;
    }

    if( !fnmatch( FLUXBOX_STRING1, key_line, 0 ) || !fnmatch( FLUXBOX_STRING2, key_line, 0 ))
    {
      file_type = FLUX_FILE;
      found_line = 1;
    }

  }
  
  
  if( found_line )
    return file_type;
  else
    return -1;

}


GList *
read_fluxbox_rc( FILE *tmp_file )
{

    GList *tmp_list = NULL;
    
    char key_line[255];
    
    char key_value[8];
    char key_modstr[80];
    char key_action[80];
    char key_exec[500];
    
    
    char* key_cap_pos;
    char* key_mod_pos;
    char* key_act_pos;
    char* key_exec_pos;
    
    gboolean alt_mod = FALSE;
    gboolean ctl_mod = FALSE;
    gboolean sft_mod = FALSE;
    gboolean win_mod = FALSE;
    
    
    int conv_keyval;
    
    key_binding *kb;
    
    int n;
    char tmp_action[80];
    
    
    if( !tmp_file )
    return NULL;
    
    
    fgets( key_line, 1024, tmp_file );

    while( !feof( tmp_file ))
    {
     
        memset(key_value, '\0', 8);
        memset(key_modstr, '\0', 80);
        memset(key_action, '\0', 80);
        memset(key_exec, '\0', 500);
        memset(tmp_action, '\0', 80);
         
        if( !fnmatch( FLUXBOX_STRING1, key_line, 0 ) || !fnmatch( FLUXBOX_STRING2, key_line, 0 )) 
        {
                                  
            key_mod_pos = strchr (key_line, ':');
            strncpy ( key_modstr, key_line, (strcspn( key_line, ":" ) - 2));
            strncpy( key_value, key_mod_pos - 1, strcspn((key_mod_pos - 1), ":"));
            if (!fnmatch( FLUXBOX_STRING2, key_line, 0 )) 
                strncpy( key_action, key_mod_pos + 1, strcspn((key_mod_pos + 1), " "));
            else
                strncpy( key_action, key_mod_pos + 1, strcspn((key_mod_pos + 1), "\0"));
            
            conv_keyval = gdk_keyval_to_lower (gdk_keyval_from_name(key_value));
            
            if( strstr( key_modstr, "Control" ))
                ctl_mod = TRUE;
            else
                ctl_mod = FALSE;
            
            if( strstr( key_modstr, "Mod1" ))
                alt_mod = TRUE;
            else
                alt_mod = FALSE;
            
            if( strstr( key_modstr, "Shift" ))
                sft_mod = TRUE;
            else
                sft_mod = FALSE;
            
            if( strstr( key_modstr, "Mod4" ))
                win_mod = TRUE;
            else
                win_mod = FALSE;
                        
          
            if( !fnmatch( FLUXBOX_STRING2, key_line, 0 ))
            {

                key_exec_pos = strchr( key_line + 1, ':' );
                key_exec_pos = strchr( key_exec_pos + 1, ' ' );
                strncpy( key_exec, key_exec_pos + 1, strcspn( key_exec_pos + 1, "\0" ));
                
                /* set keybinding properties */
                kb = create_binding( conv_keyval, alt_mod, ctl_mod, sft_mod, win_mod, 
                                 g_strstrip(key_action), g_strstrip(key_exec) );

            }
            else
            {
                /* set keybinding properties */
                kb = create_binding( conv_keyval, alt_mod, ctl_mod, sft_mod, win_mod, 
                    g_strstrip(key_action), NULL );
            }
    
            tmp_list = g_list_append( tmp_list, (gpointer) kb );
        }

        fgets( key_line, 1024, tmp_file );

    }

  return tmp_list;

}




GList *
read_bbkeys_rc( FILE *tmp_file )
{

  GList *tmp_list = NULL;
  
  char key_line[255];

  char key_value[80];
  char key_modstr[80];
  char key_action[80];
  char key_exec[500];


  char* key_cap_pos;
  char* key_mod_pos;
  char* key_act_pos;
  char* key_exec_pos;
  
  gboolean alt_mod = FALSE;
  gboolean ctl_mod = FALSE;
  gboolean sft_mod = FALSE;
  gboolean win_mod = FALSE;
  

  int conv_keyval;

  key_binding *kb;

 
  if( !tmp_file )
    return NULL;


  fgets( key_line, 1024, tmp_file );

  while( !feof( tmp_file ))
  {
     
    memset(key_value, '\0', 80);
    memset(key_modstr, '\0', 80);
    memset(key_action, '\0', 80);
    memset(key_exec, '\0', 500);
 
 
    if( !fnmatch( BBCONF_STRING1, key_line, 0 ) || !fnmatch( BBCONF_STRING2, key_line, 0 ))
    {
                              
      key_cap_pos = strchr( key_line, '(' );
      strncpy( key_value, key_cap_pos + 1, strcspn( key_cap_pos + 1, ")" ));
          
      key_mod_pos = strchr( key_cap_pos + 1, '(' );
      strncpy( key_modstr, key_mod_pos + 1, strcspn( key_mod_pos + 1, ")" ));
          
      key_act_pos = strchr( key_mod_pos + 1, '(' );
      strncpy( key_action, key_act_pos + 1, strcspn( key_act_pos + 1, ")" ));
      
      
      conv_keyval = gdk_keyval_to_lower( gdk_keyval_from_name( key_value ));
      
      
      if( strstr( key_modstr, "Control" ))
        ctl_mod = TRUE;
      else
        ctl_mod = FALSE;
      
      if( strstr( key_modstr, "Mod1" ))
        alt_mod = TRUE;
      else
        alt_mod = FALSE;

      if( strstr( key_modstr, "Shift" ))
        sft_mod = TRUE;
      else
        sft_mod = FALSE;

      if( strstr( key_modstr, "Mod4" ))
        win_mod = TRUE;
      else
        win_mod = FALSE;



      /* set keybinding properties */
      kb = create_binding( conv_keyval, alt_mod, ctl_mod, sft_mod, win_mod, 
                           key_action, NULL );
      
          
      if( !fnmatch( BBCONF_STRING2, key_line, 0 ))
      {

        key_exec_pos = strchr( key_act_pos + 1, '(' );
        strncpy( key_exec, key_exec_pos + 1, strcspn( key_exec_pos + 1, ")" ));

        /* set keybinding properties */
        kb = create_binding( conv_keyval, alt_mod, ctl_mod, sft_mod, win_mod, 
                             key_action, key_exec );

      }
    
    
      tmp_list = g_list_append( tmp_list, (gpointer) kb );
    
    }

    fgets( key_line, 1024, tmp_file );

  }

  return tmp_list;

}

gboolean
save_bbkeys_rc( char *file_name, GList *k_list )
{

  FILE *out_file;       /* file to write to */
  char key_line[255];   /* config line to write */
  int num_mods = 0;     /* number of modifiers present */
  int i = 0;            /* counter */

  if(( out_file = fopen( file_name, "w" )) != NULL )
  {
    /* print obligatory "created by .." message and key binding */
    fprintf( out_file, "# bbKeys configuration file\n" );
    fprintf( out_file, "# Generated by whiteBOX version 0.5\n\n" );
 
    for( i = 0; i < g_list_length( k_list ); i++ )
    {
      key_binding *k_val;
      
      k_val = g_list_nth_data( k_list, i );
  
      /* determine how many modifiers have been set */
      if( k_val->alt_mod == TRUE ) 
      {
        g_print( "is true %d\n", i );
        num_mods++; 
      }
      
      if( k_val->ctl_mod == TRUE )  
      {
        g_print( "is true %d\n", i );
        num_mods++; 
      }
      
      if( k_val->sft_mod == TRUE ) 
      {
        g_print( "is true %d\n", i );        
        num_mods++; 
      }

      if( k_val->win_mod == TRUE ) 
      {
        g_print( "is true %d\n", i );
        num_mods++; 
      }
  
      /* write the key binding information */
      strcpy( key_line, "KeyToGrab(" );
      strcat( key_line,  gdk_keyval_name( gdk_keyval_to_lower( k_val->key_val )));
      strcat( key_line, "), " );
    
      /* write out all of the selected modifiers */
      strcat( key_line, "WithModifier(" );
    
      if( k_val->ctl_mod ) 
      {
        strcat( key_line, "Control" );
        if( num_mods > 1 ) 
        {
          strcat( key_line, "+" );
          g_print( "ctl: num_mods %d\n", num_mods );
          num_mods--;
        }
      }
    
      if( k_val->alt_mod )
      {
        strcat( key_line, "Mod1" );
        if( num_mods > 1 )
        {
          strcat( key_line, "+" );
          g_print( "alt: num_mods %d\n", num_mods );
          num_mods--;
        }
      }
    
      if( k_val->sft_mod )
      {
        strcat( key_line, "Shift" );
        if( num_mods > 1 )
        {
          g_print( "sft: num_mods %d\n", num_mods );
          strcat( key_line, "+" );
          num_mods--;
        }
      }
    
      if( k_val->win_mod )
        strcat( key_line, "Mod4" );
    
      strcat( key_line, "), " );
      strcat( key_line, "WithAction(" );
      strcat( key_line, k_val->action_type );
      strcat( key_line, ")" );
  
      if( k_val->exec_command != NULL )
      {
        strcat( key_line, ", DoThis(" );
        strcat( key_line, k_val->exec_command );
        strcat( key_line, ")" );
      }
  
      fprintf( out_file, "%s\n", key_line );
      
      num_mods = 0;
    }
      
    fclose( out_file );
  }

  return TRUE;

}


gboolean
save_fluxkeys_rc( char *file_name, GList *k_list )
{

  FILE *out_file;       /* file to write to */
  char key_line[255];   /* config line to write */
  int num_mods = 0;     /* number of modifiers present */
  int i = 0;            /* counter */

  if(( out_file = fopen( file_name, "w" )) != NULL )
  {
    /* print obligatory "created by .." message and key binding */
    fprintf( out_file, "# Fluxbox keybindings configuration file\n" );
    fprintf( out_file, "# Generated by whiteBOX version 0.5\n\n" );
 
    for( i = 0; i < g_list_length( k_list ); i++ )
    {
      key_binding *k_val;
      
      strcpy( key_line, "" );
      k_val = g_list_nth_data( k_list, i );
  
      /* determine how many modifiers have been set */
      if( k_val->alt_mod == TRUE ) 
        num_mods++; 
      
      if( k_val->ctl_mod == TRUE )  
        num_mods++; 
      
      if( k_val->sft_mod == TRUE ) 
        num_mods++; 

      if( k_val->win_mod == TRUE ) 
        num_mods++; 

      
    
      /* write out all of the selected modifiers */
      if( k_val->ctl_mod ) 
      {
        strcat( key_line, "Control" );
        if( num_mods > 1 ) 
        {
          strcat( key_line, " " );
          num_mods--;
        }
      }
    
      if( k_val->alt_mod )
      {
        strcat( key_line, "Mod1" );
        if( num_mods > 1 )
        {
          strcat( key_line, " " );
          num_mods--;
        }
      }
    
      if( k_val->sft_mod )
      {
        strcat( key_line, "Shift" );
        if( num_mods > 1 )
        {
          strcat( key_line, " " );
          num_mods--;
        }
      }
    
      if( k_val->win_mod )
        strcat( key_line, "Mod4" );
    
        strcat (key_line, " ");
    
      /* write the key binding information */
      strcat( key_line,  gdk_keyval_name( gdk_keyval_to_lower( k_val->key_val )));
        strcat (key_line, ":");
      strcat( key_line, k_val->action_type );
  
      if( k_val->exec_command != NULL )
      {
        strcat( key_line, " " );
        strcat( key_line, k_val->exec_command );
      }
  
      fprintf( out_file, "%s\n", key_line );
      
      num_mods = 0;
    }
      
    fclose( out_file );
  }

  return TRUE;

}


GList *
reorder_keys_list (GList *lst_keybindings, int cur_index, int reorder_flag)
{

    /*
     * reorder_keys_list
     */

    GList *dl;
    GList *t;

    dl = g_list_nth (lst_keybindings, cur_index);

    if (reorder_flag == PREV_FLAG)
    {
        if(!dl->prev)
            return lst_keybindings;
            
        if(dl->next)
            dl->next->prev = dl->prev;
            
        t = dl->next;
        dl->next = dl->prev;
        dl->prev->next = t;
        
        if(dl->prev->prev)
                dl->prev->prev->next = dl;
        
        t = dl->prev->prev;
        dl->prev->prev = dl;
        dl->prev = t;
    
        if(lst_keybindings == dl->next)
                return dl;
        
        return lst_keybindings;
    }
    else if (reorder_flag == NEXT_FLAG)
    {
    
	if(!dl->next)
            return lst_keybindings;
                
	if(dl->prev)
            dl->prev->next = dl->next;
               
	t = dl->prev;
	dl->prev = dl->next;
	dl->next->prev = t;
	
        if(dl->next->next)
            dl->next->next->prev = dl;
                
	t = dl->next->next;
	dl->next->next = dl;
	dl->next = t;

	if(lst_keybindings == dl)
		return dl->prev;
	
        return lst_keybindings;    
    
    }
    
    
}

int
check_available_items (GList *lst_availkeys, int int_keysType)
{

    GList *lst_alloweditems = NULL;
    GList *lst_current = NULL;
    GList *lst_check = NULL;
    int int_incompat = 0;
    gboolean *bln_found = FALSE;
    
    int x;

    if (int_keysType == BBKEYS_FILE)
    {
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("BigNudgeDown"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("BigNudgeLeft"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("BigNudgeRight"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("BigNudgeUp"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Close"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("ExecCommand"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("HorizontalDecrement"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("HorizontalIncrement"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Lower"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("MaximizeHorizontal"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("MaximizeVertical"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("MaximizeWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Minimize"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NextWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NextWindowAllWorkspaces"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NextWorkspace"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NudgeDown"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NudgeLeft"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NudgeUp"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NudgeRight"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("PrevWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("PrevWorkspace"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Raise"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("ShadeWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("StickWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("ToggleDecor"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("VerticalDecrement"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("VerticalIncrement"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace1"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace2"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace3"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace4"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace5"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace6"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace7"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace8"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace9"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace10"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace11"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace12"));
    }
    else if (int_keysType == FLUX_FILE)
    {
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("BigNudgeDown"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("BigNudgeLeft"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("BigNudgeRight"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("BigNudgeUp"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Close"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("ExecCommand"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("HorizontalDecrement"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("HorizontalIncrement"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("KillWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("LeftWorkspace"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Lower"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("MaximizeHorizontal"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("MaximizeVertical"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("MaximizeWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Minimize"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NextTab"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NextWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NextWorkspace"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NudgeDown"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NudgeLeft"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NudgeUp"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("NudgeRight"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("PrevTab"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("PrevWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("PrevWorkspace"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Raise"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("RightWorkspace"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("RootMenu"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("ShadeWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("StickWindow"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("ToggleDecor"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("VerticalDecrement"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("VerticalIncrement"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace1"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace2"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace3"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace4"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace5"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace6"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace7"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace8"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace9"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace10"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace11"));
        lst_alloweditems = g_list_append (lst_alloweditems, (gpointer) _("Workspace12"));    
    }
 
    
    /* check for incompatable key bindings */
    lst_current = lst_availkeys;

    while (lst_current)
    {
        key_binding *kb = (key_binding *) (lst_current->data);
        bln_found = FALSE;
        lst_check = lst_alloweditems;
        while (lst_check)
        {
            if (!strcmp (kb->action_type, (char *) lst_check->data))
                bln_found = TRUE;
            lst_check = lst_check->next;
        }
        
        if (!bln_found)
            int_incompat++;
        
        lst_current = lst_current->next;
    }
 
 
 
    g_list_free (lst_check);
    g_list_free (lst_alloweditems);
         
    return (int_incompat);
}
