/*
 * libkeybind.c
 * Keybinding plugin entry point and initialisation
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_SYS_DIR_H
    #include <sys/dir.h>
#else
    #ifdef HAVE_SYS_NDIR_H
        #include <sys/ndir.h>
    #else
        #ifdef HAVE_DIRENT_H
            #include <dirent.h>
        #else
            #include <sys/dirent.h>
        #endif
    #endif
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "keybindcb.h"
#include "keybind.h"

#include "../../support.h"
#include "../../msgbox.h"
#include "../../plugin.h"


plugin_data kbplg = {
  NULL,
  "wb_kbind_01",
  NULL,
  "whiteBOX keybinding tool component",
  "Luke Freeman",
  NULL,
  NULL
};

plugin_data *get_plugin_data( void )
{
  return &kbplg;
}

void
init( GtkWidget *wnd_main, int argc, char* argv[] )
{
    int i, x;                 /* counters */
    int int_fileexists;
    char str_fdefrc[5][255];
    char str_bdefrc[255]; 
    char *fle = NULL;
    struct stat s;
    int int_keystype = BBKEYS_FILE;

    GList *cbo_keybindfunc_items = NULL;
    GtkWidget *lst_cbokeyFuncs;
    GtkWidget *cbo_keyFuncs = lookup_widget (wnd_main, "cbo_keybindfunc");


    /* set up default locations to check for file */
    /* fluxbox */
    strncpy( str_fdefrc[0], getenv("HOME"), 255 );
    strncat( str_fdefrc[0] ,"/.fluxbox/keys", 255 );
    strncpy( str_fdefrc[1] ,"/usr/local/share/fluxbox/keys", 255 );
    strncpy( str_fdefrc[2] ,"/usr/share/fluxbox/keys", 255 );
    strncpy( str_fdefrc[3] ,"/usr/X11R6/share/fluxbox/keys", 255 );
    
    /* blackbox */
    strncpy( str_bdefrc, getenv("HOME"), 255 );
    strncat( str_bdefrc ,"/.bbkeysrc", 255 );
    
    
    /* parse the command line options */ 
    for( i = 1; i < argc; i++ ) 
    {
        x = 0;
    
        if( !strcmp(argv[i], "-b" ))
        {
            set_keys_type (BBKEYS_FILE);
            int_keystype = BBKEYS_FILE;
        
            if (( stat( str_bdefrc, &s ) != -1 ) && ( !S_ISDIR( s.st_mode )))
                int_fileexists = 1;
                
            if( int_fileexists == 1 ) {
                set_keys_file (g_strdup(str_bdefrc));
                load_keysrc_file( (GtkWidget *) wnd_main, str_bdefrc );
            }
                
        }
        else if( !strcmp(argv[i], "-f" ))
        {
            set_keys_type (FLUX_FILE);
            int_keystype = FLUX_FILE;
            
            while( int_fileexists == 0 )
            {
                if (( stat( str_fdefrc[x], &s ) != -1 ) && ( !S_ISDIR( s.st_mode )))
                    int_fileexists = 1;
                else
                    x++;
            }
        
            if( int_fileexists == 1 ) {
                    load_keysrc_file( (GtkWidget *) wnd_main, str_fdefrc[x] );
                    //set_keys_file (str_fdefrc[x]);
            }
        }
        /* load a keys rc file */       
        else if( !strcmp(argv[i], "-kf" ) || !strcmp(argv[i], "--keys-file" )) 
        {
            i++;
            if (( stat( argv[i], &s ) != -1 ) && ( !S_ISDIR( s.st_mode ))) {
                //set_keys_file (argv[i]);
                int_keystype = determine_file_type (wnd_main, argv[i]);
                load_keysrc_file ((GtkWidget *) wnd_main, argv[i]);
            }
        }
        else if( !strcmp( argv[i], "-kbt" ) || !strcmp( argv[i], "--keys-tab" ))
        {
            GtkWidget *wgt_nbkmain    = lookup_widget( wnd_main, "nbk_main" );
            GtkWidget *wgt_fxdkeymain = lookup_widget( wnd_main, "fxd_keymain" );
        
            gtk_notebook_set_page( (GtkNotebook *) wgt_nbkmain, 
            gtk_notebook_page_num( (GtkNotebook *) wgt_nbkmain, GTK_WIDGET( wgt_fxdkeymain )->parent ));
        }
    }
    
    /* set up the default function list */
    if (int_keystype == BBKEYS_FILE)
    {
        
        lst_cbokeyFuncs = GTK_COMBO(cbo_keyFuncs)->list;
        gtk_list_clear_items (lst_cbokeyFuncs, 0, -1);
        
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeDown"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeLeft"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeRight"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeUp"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Close"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ExecCommand"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("HorizontalDecrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("HorizontalIncrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Lower"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeHorizontal"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeVertical"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Minimize"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWindowAllWorkspaces"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeDown"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeLeft"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeUp"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeRight"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Raise"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ShadeWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("StickWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ToggleDecor"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("VerticalDecrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("VerticalIncrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace1"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace2"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace3"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace4"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace5"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace6"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace7"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace8"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace9"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace10"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace11"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace12"));
       
        gtk_combo_set_popdown_strings (GTK_COMBO(cbo_keyFuncs), cbo_keybindfunc_items);
        g_list_free (cbo_keybindfunc_items);

    }
    else
    {
        lst_cbokeyFuncs = GTK_COMBO(cbo_keyFuncs)->list;
        gtk_list_clear_items (lst_cbokeyFuncs, 0, -1);
        
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeDown"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeLeft"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeRight"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeUp"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Close"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ExecCommand"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("HorizontalDecrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("HorizontalIncrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("KillWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("LeftWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Lower"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeHorizontal"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeVertical"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Minimize"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextTab"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeDown"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeLeft"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeUp"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeRight"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevTab"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Raise"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("RightWorkspace"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("RootMenu"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ShadeWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("StickWindow"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ToggleDecor"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("VerticalDecrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("VerticalIncrement"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace1"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace2"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace3"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace4"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace5"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace6"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace7"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace8"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace9"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace10"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace11"));
        cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace12"));
        
        gtk_combo_set_popdown_strings (GTK_COMBO(cbo_keyFuncs), cbo_keybindfunc_items);
        g_list_free (cbo_keybindfunc_items);
    }    
    
    
 }

GtkWidget *
entry_point( GtkWidget *wnd_main )
{

  GtkWidget *fxd_keymain;
  GtkWidget *btn_keyadd;
  GtkWidget *btn_keyremove;
  GtkWidget *btn_keyclearall;
  GtkWidget *lbl_keynonode;
  GtkWidget *tbr_keynav;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *btn_tbrkeynavup;
  GtkWidget *btn_tbrkeynavdown;
  GtkWidget *scw_keysaved;
  GtkWidget *lst_keysaved;
  GtkWidget *lbl_lstkeysavedfunc;
  GtkWidget *lbl_lstkeysavedbind;
  GtkWidget *frm_keyfileinfo;
  GtkWidget *fxd_keyfileinfo;
  GtkWidget *lbl_keyfiletype;
  GtkWidget *cbo_keyfiletype;
  GList *cbo_keyfiletype_items = NULL;
  GtkWidget *txt_cbokeyfiletype;
  GtkWidget *lbl_keyfilehed;
  GtkWidget *lbl_keyfile;
  GtkWidget *tbr_keymain;
  GtkWidget *btn_tbrkeynew;
  GtkWidget *btn_tbrkeyload;
  GtkWidget *btn_tbrkeysave;
  GtkWidget *btn_tbrkeysaveas;
  GtkWidget *btn_tbrkeyfileinfo;
  GtkWidget *frm_keybindinfo;
  GtkWidget *fxd_keybindinfo;
  GtkWidget *tgb_keybindctrl;
  GtkWidget *tgb_keybindalt;
  GtkWidget *tgb_keybindshift;
  GtkWidget *txt_keybindkey;
  GtkWidget *txt_keybindcmd;
  GtkWidget *btn_keyinfosave;
  GtkWidget *btn_keyinfocancel;
  GtkWidget *lbl_keybindcombo;
  GtkWidget *tgb_keybindwinkey;
  GtkWidget *tgb_keybindcapture;
  GtkWidget *lbl_keybindfunc;
  GtkWidget *cbo_keybindfunc;
  GtkWidget *txt_cbokeybindfunc;
  GtkWidget *lbl_keybindcmd;
  GList *cbo_keybindfunc_items = NULL;  

  fxd_keymain = gtk_fixed_new ();
  gtk_widget_ref (fxd_keymain);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "fxd_keymain", fxd_keymain,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fxd_keymain);

  btn_keyadd = gtk_button_new_with_label (_("Add"));
  gtk_widget_ref (btn_keyadd);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_keyadd", btn_keyadd,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_keyadd);
  gtk_fixed_put (GTK_FIXED (fxd_keymain), btn_keyadd, 8, 320);
  gtk_widget_set_uposition (btn_keyadd, 8, 320);
  gtk_widget_set_usize (btn_keyadd, 80, 22);

  btn_keyremove = gtk_button_new_with_label (_("Remove"));
  gtk_widget_ref (btn_keyremove);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_keyremove", btn_keyremove,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_keyremove);
  gtk_fixed_put (GTK_FIXED (fxd_keymain), btn_keyremove, 93, 320);
  gtk_widget_set_uposition (btn_keyremove, 93, 320);
  gtk_widget_set_usize (btn_keyremove, 80, 22);

  btn_keyclearall = gtk_button_new_with_label (_("Clear All"));
  gtk_widget_ref (btn_keyclearall);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_keyclearall", btn_keyclearall,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_keyclearall);
  gtk_fixed_put (GTK_FIXED (fxd_keymain), btn_keyclearall, 178, 320);
  gtk_widget_set_uposition (btn_keyclearall, 178, 320);
  gtk_widget_set_usize (btn_keyclearall, 80, 22);

  lbl_keynonode = gtk_label_new (_("Please select an item from the list\nor press Add to add a new binding"));
  gtk_widget_ref (lbl_keynonode);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_keynonode", lbl_keynonode,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_keynonode);
  gtk_fixed_put (GTK_FIXED (fxd_keymain), lbl_keynonode, 384, 136);
  gtk_widget_set_uposition (lbl_keynonode, 384, 136);
  gtk_widget_set_usize (lbl_keynonode, 200, 32);

  tbr_keynav = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gtk_widget_ref (tbr_keynav);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbr_keynav", tbr_keynav,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbr_keynav);
  gtk_fixed_put (GTK_FIXED (fxd_keymain), tbr_keynav, 263, 320);
  gtk_widget_set_uposition (tbr_keynav, 263, 320);
  gtk_widget_set_usize (tbr_keynav, 60, 22);

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_up-16.xpm");
  btn_tbrkeynavup = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_keynav),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("button9"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (btn_tbrkeynavup);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_tbrkeynavup", btn_tbrkeynavup,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_tbrkeynavup);
  gtk_widget_set_usize (btn_tbrkeynavup, 22, 22);

  gtk_toolbar_append_space (GTK_TOOLBAR (tbr_keynav));

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_down-16.xpm");
  btn_tbrkeynavdown = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_keynav),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("button10"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (btn_tbrkeynavdown);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_tbrkeynavdown", btn_tbrkeynavdown,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_tbrkeynavdown);
  gtk_widget_set_usize (btn_tbrkeynavdown, 22, 22);

  scw_keysaved = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scw_keysaved);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "scw_keysaved", scw_keysaved,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scw_keysaved);
  gtk_fixed_put (GTK_FIXED (fxd_keymain), scw_keysaved, 8, 8);
  gtk_widget_set_uposition (scw_keysaved, 8, 8);
  gtk_widget_set_usize (scw_keysaved, 328, 304);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw_keysaved), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  lst_keysaved = gtk_clist_new (2);
  gtk_widget_ref (lst_keysaved);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lst_keysaved", lst_keysaved,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lst_keysaved);
  gtk_container_add (GTK_CONTAINER (scw_keysaved), lst_keysaved);
  gtk_widget_set_usize (lst_keysaved, 304, -2);
  gtk_clist_set_column_width (GTK_CLIST (lst_keysaved), 0, 187);
  gtk_clist_set_column_width (GTK_CLIST (lst_keysaved), 1, 77);
  gtk_clist_column_titles_show (GTK_CLIST (lst_keysaved));

  lbl_lstkeysavedfunc = gtk_label_new (_("Function"));
  gtk_widget_ref (lbl_lstkeysavedfunc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_lstkeysavedfunc", lbl_lstkeysavedfunc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_lstkeysavedfunc);
  gtk_clist_set_column_widget (GTK_CLIST (lst_keysaved), 0, lbl_lstkeysavedfunc);

  lbl_lstkeysavedbind = gtk_label_new (_("Binding"));
  gtk_widget_ref (lbl_lstkeysavedbind);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_lstkeysavedbind", lbl_lstkeysavedbind,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_lstkeysavedbind);
  gtk_clist_set_column_widget (GTK_CLIST (lst_keysaved), 1, lbl_lstkeysavedbind);

  frm_keyfileinfo = gtk_frame_new (_("File Information"));
  gtk_widget_ref (frm_keyfileinfo);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "frm_keyfileinfo", frm_keyfileinfo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_fixed_put (GTK_FIXED (fxd_keymain), frm_keyfileinfo, 344, 64);
  gtk_widget_set_uposition (frm_keyfileinfo, 344, 64);
  gtk_widget_set_usize (frm_keyfileinfo, 312, 192);
  gtk_frame_set_label_align (GTK_FRAME (frm_keyfileinfo), 0.01, 0.5);

  fxd_keyfileinfo = gtk_fixed_new ();
  gtk_widget_ref (fxd_keyfileinfo);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "fxd_keyfileinfo", fxd_keyfileinfo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fxd_keyfileinfo);
  gtk_container_add (GTK_CONTAINER (frm_keyfileinfo), fxd_keyfileinfo);

  lbl_keyfiletype = gtk_label_new (_("File Type:"));
  gtk_widget_ref (lbl_keyfiletype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_keyfiletype", lbl_keyfiletype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_keyfiletype);
  gtk_fixed_put (GTK_FIXED (fxd_keyfileinfo), lbl_keyfiletype, 8, 16);
  gtk_widget_set_uposition (lbl_keyfiletype, 8, 16);
  gtk_widget_set_usize (lbl_keyfiletype, 64, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_keyfiletype), 7.45058e-09, 0.5);

  cbo_keyfiletype = gtk_combo_new ();
  gtk_widget_ref (cbo_keyfiletype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "cbo_keyfiletype", cbo_keyfiletype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_keyfiletype);
  gtk_fixed_put (GTK_FIXED (fxd_keyfileinfo), cbo_keyfiletype, 72, 16);
  gtk_widget_set_uposition (cbo_keyfiletype, 72, 16);
  gtk_widget_set_usize (cbo_keyfiletype, 152, 19);
  cbo_keyfiletype_items = g_list_append (cbo_keyfiletype_items, (gpointer) _("BB Keys"));
  cbo_keyfiletype_items = g_list_append (cbo_keyfiletype_items, (gpointer) _("Fluxbox"));
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_keyfiletype), cbo_keyfiletype_items);
  g_list_free (cbo_keyfiletype_items);

  txt_cbokeyfiletype = GTK_COMBO (cbo_keyfiletype)->entry;
  gtk_widget_ref (txt_cbokeyfiletype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_cbokeyfiletype", txt_cbokeyfiletype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_cbokeyfiletype);
  gtk_entry_set_text (GTK_ENTRY (txt_cbokeyfiletype), _("BB Keys"));

  lbl_keyfilehed = gtk_label_new (_("Keys File:"));
  gtk_widget_ref (lbl_keyfilehed);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_keyfilehed", lbl_keyfilehed,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_keyfilehed);
  gtk_fixed_put (GTK_FIXED (fxd_keyfileinfo), lbl_keyfilehed, 8, 40);
  gtk_widget_set_uposition (lbl_keyfilehed, 8, 40);
  gtk_widget_set_usize (lbl_keyfilehed, 56, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_keyfilehed), 7.45058e-09, 0.5);

  lbl_keyfile = gtk_label_new (_("no key file loaded .."));
  gtk_widget_ref (lbl_keyfile);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_keyfile", lbl_keyfile,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_keyfile);
  gtk_fixed_put (GTK_FIXED (fxd_keyfileinfo), lbl_keyfile, 72, 40);
  gtk_widget_set_uposition (lbl_keyfile, 72, 40);
  gtk_widget_set_usize (lbl_keyfile, 216, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_keyfile), 7.45058e-09, 0.5);

  tbr_keymain = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_widget_ref (tbr_keymain);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbr_keymain", tbr_keymain,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbr_keymain);
  gtk_fixed_put (GTK_FIXED (fxd_keymain), tbr_keymain, 344, 8);
  gtk_widget_set_uposition (tbr_keymain, 344, 8);
  gtk_widget_set_usize (tbr_keymain, 312, 48);
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (tbr_keymain), GTK_RELIEF_NONE);

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_new-16.xpm");
  btn_tbrkeynew = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_keymain),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("New"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (btn_tbrkeynew);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_tbrkeynew", btn_tbrkeynew,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_tbrkeynew);

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_open-16.xpm");
  btn_tbrkeyload = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_keymain),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Load"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (btn_tbrkeyload);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_tbrkeyload", btn_tbrkeyload,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_tbrkeyload);
  gtk_widget_set_usize (btn_tbrkeyload, 40, -2);

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_save-16.xpm");
  btn_tbrkeysave = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_keymain),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Save"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (btn_tbrkeysave);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_tbrkeysave", btn_tbrkeysave,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_tbrkeysave);

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_save_as-16.xpm");
  btn_tbrkeysaveas = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_keymain),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Copy"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (btn_tbrkeysaveas);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_tbrkeysaveas", btn_tbrkeysaveas,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_tbrkeysaveas);

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_preferences-16.xpm");
  btn_tbrkeyfileinfo = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_keymain),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Info"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (btn_tbrkeyfileinfo);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_tbrkeyfileinfo", btn_tbrkeyfileinfo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_tbrkeyfileinfo);

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_convert-16.xpm");

  frm_keybindinfo = gtk_frame_new (_("Bind Information"));
  gtk_widget_ref (frm_keybindinfo);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "frm_keybindinfo", frm_keybindinfo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_fixed_put (GTK_FIXED (fxd_keymain), frm_keybindinfo, 344, 64);
  gtk_widget_set_uposition (frm_keybindinfo, 344, 64);
  gtk_widget_set_usize (frm_keybindinfo, 312, 192);
  gtk_frame_set_label_align (GTK_FRAME (frm_keybindinfo), 0.01, 0.5);

  fxd_keybindinfo = gtk_fixed_new ();
  gtk_widget_ref (fxd_keybindinfo);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "fxd_keybindinfo", fxd_keybindinfo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fxd_keybindinfo);
  gtk_container_add (GTK_CONTAINER (frm_keybindinfo), fxd_keybindinfo);

  tgb_keybindctrl = gtk_toggle_button_new_with_label (_("Ctrl"));
  gtk_widget_ref (tgb_keybindctrl);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tgb_keybindctrl", tgb_keybindctrl,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tgb_keybindctrl);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), tgb_keybindctrl, 8, 40);
  gtk_widget_set_uposition (tgb_keybindctrl, 8, 40);
  gtk_widget_set_usize (tgb_keybindctrl, 32, 19);

  tgb_keybindalt = gtk_toggle_button_new_with_label (_("Alt"));
  gtk_widget_ref (tgb_keybindalt);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tgb_keybindalt", tgb_keybindalt,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tgb_keybindalt);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), tgb_keybindalt, 48, 40);
  gtk_widget_set_uposition (tgb_keybindalt, 48, 40);
  gtk_widget_set_usize (tgb_keybindalt, 34, 19);

  tgb_keybindshift = gtk_toggle_button_new_with_label (_("Shift"));
  gtk_widget_ref (tgb_keybindshift);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tgb_keybindshift", tgb_keybindshift,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tgb_keybindshift);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), tgb_keybindshift, 88, 40);
  gtk_widget_set_uposition (tgb_keybindshift, 88, 40);
  gtk_widget_set_usize (tgb_keybindshift, 34, 19);

  txt_keybindkey = gtk_entry_new_with_max_length (25);
  gtk_widget_ref (txt_keybindkey);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_keybindkey", txt_keybindkey,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_keybindkey);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), txt_keybindkey, 170, 40);
  gtk_widget_set_uposition (txt_keybindkey, 170, 40);
  gtk_widget_set_usize (txt_keybindkey, 78, 19);
  gtk_widget_set_events (txt_keybindkey, GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);

  txt_keybindcmd = gtk_entry_new ();
  gtk_widget_ref (txt_keybindcmd);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_keybindcmd", txt_keybindcmd,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), txt_keybindcmd, 72, 104);
  gtk_widget_set_uposition (txt_keybindcmd, 72, 104);
  gtk_widget_set_usize (txt_keybindcmd, 175, 19);

  btn_keyinfosave = gtk_button_new_with_label (_("Save"));
  gtk_widget_ref (btn_keyinfosave);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_keyinfosave", btn_keyinfosave,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_keyinfosave);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), btn_keyinfosave, 216, 144);
  gtk_widget_set_uposition (btn_keyinfosave, 216, 144);
  gtk_widget_set_usize (btn_keyinfosave, 80, 24);

  btn_keyinfocancel = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_ref (btn_keyinfocancel);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_keyinfocancel", btn_keyinfocancel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_keyinfocancel);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), btn_keyinfocancel, 128, 144);
  gtk_widget_set_uposition (btn_keyinfocancel, 128, 144);
  gtk_widget_set_usize (btn_keyinfocancel, 80, 24);

  lbl_keybindcombo = gtk_label_new (_("Key Combination"));
  gtk_widget_ref (lbl_keybindcombo);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_keybindcombo", lbl_keybindcombo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_keybindcombo);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), lbl_keybindcombo, 8, 16);
  gtk_widget_set_uposition (lbl_keybindcombo, 8, 16);
  gtk_widget_set_usize (lbl_keybindcombo, 128, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_keybindcombo), 7.45058e-09, 0.5);

  tgb_keybindwinkey = gtk_toggle_button_new_with_label (_("Win"));
  gtk_widget_ref (tgb_keybindwinkey);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tgb_keybindwinkey", tgb_keybindwinkey,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tgb_keybindwinkey);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), tgb_keybindwinkey, 128, 40);
  gtk_widget_set_uposition (tgb_keybindwinkey, 128, 40);
  gtk_widget_set_usize (tgb_keybindwinkey, 36, 19);

  tgb_keybindcapture = gtk_toggle_button_new_with_label (_("Capture"));
  gtk_widget_ref (tgb_keybindcapture);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tgb_keybindcapture", tgb_keybindcapture,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tgb_keybindcapture);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), tgb_keybindcapture, 8, 144);
  gtk_widget_set_uposition (tgb_keybindcapture, 8, 144);
  gtk_widget_set_usize (tgb_keybindcapture, 80, 24);

  lbl_keybindfunc = gtk_label_new (_("Function:"));
  gtk_widget_ref (lbl_keybindfunc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_keybindfunc", lbl_keybindfunc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_keybindfunc);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), lbl_keybindfunc, 8, 82);
  gtk_widget_set_uposition (lbl_keybindfunc, 8, 82);
  gtk_widget_set_usize (lbl_keybindfunc, 56, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_keybindfunc), 7.45058e-09, 0.5);

  cbo_keybindfunc = gtk_combo_new ();
  gtk_widget_ref (cbo_keybindfunc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "cbo_keybindfunc", cbo_keybindfunc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_keybindfunc);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), cbo_keybindfunc, 72, 80);
  gtk_widget_set_uposition (cbo_keybindfunc, 72, 80);
  gtk_widget_set_usize (cbo_keybindfunc, 175, 19);
  gtk_combo_set_value_in_list (GTK_COMBO (cbo_keybindfunc), TRUE, FALSE);
  /*
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeDown"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeLeft"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeRight"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("BigNudgeUp"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Close"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ExecCommand"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("HorizontalDecrement"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("HorizontalIncrement"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Lower"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeHorizontal"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeVertical"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("MaximizeWindow"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Minimize"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWindow"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWindowAllWorkspaces"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NextWorkspace"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeDown"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeLeft"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeUp"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("NudgeRight"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevWindow"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("PrevWorkspace"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Raise"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ShadeWindow"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("StickWindow"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("ToggleDecor"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace1"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace2"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace3"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace4"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace5"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace6"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace7"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace8"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace9"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace10"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace11"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("Workspace12"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("VerticalIncrement"));
  cbo_keybindfunc_items = g_list_append (cbo_keybindfunc_items, (gpointer) _("VerticalDecrement"));
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_keybindfunc), cbo_keybindfunc_items);
  g_list_free (cbo_keybindfunc_items);
  */  

  txt_cbokeybindfunc = GTK_COMBO (cbo_keybindfunc)->entry;
  gtk_widget_ref (txt_cbokeybindfunc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_cbokeybindfunc", txt_cbokeybindfunc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_cbokeybindfunc);

  lbl_keybindcmd = gtk_label_new (_("Command:"));
  gtk_widget_ref (lbl_keybindcmd);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_keybindcmd", lbl_keybindcmd,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_fixed_put (GTK_FIXED (fxd_keybindinfo), lbl_keybindcmd, 8, 106);
  gtk_widget_set_uposition (lbl_keybindcmd, 8, 106);
  gtk_widget_set_usize (lbl_keybindcmd, 57, 16);
  gtk_misc_set_alignment (GTK_MISC (lbl_keybindcmd), 7.45058e-09, 0.5);


  gtk_signal_connect (GTK_OBJECT (btn_keyadd), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_keyadd_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_keyremove), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_keyremove_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_keyclearall), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_keyclearall_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_tbrkeynavup), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_tbrkeynavup_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_tbrkeynavdown), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_tbrkeynavdown_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_tbrkeynew), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_tbrkeynew_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_tbrkeyload), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_tbrkeyload_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_tbrkeysave), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_tbrkeysave_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_tbrkeysaveas), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_tbrkeysaveas_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_tbrkeyfileinfo), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_tbrkeyfileinfo_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (txt_keybindkey), "key_press_event",
                      GTK_SIGNAL_FUNC (on_txt_keybindkey_key_press_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (txt_keybindkey), "focus_out_event",
                      GTK_SIGNAL_FUNC (on_txt_keybindkey_focus_out_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (txt_keybindkey), "key_release_event",
                      GTK_SIGNAL_FUNC (on_txt_keybindkey_key_release_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_keyinfosave), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_keyinfosave_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_keyinfocancel), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_keyinfocancel_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tgb_keybindcapture), "clicked",
                      GTK_SIGNAL_FUNC (on_tgb_keybindcapture_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (lst_keysaved), "select_row",
                      GTK_SIGNAL_FUNC (on_lst_keysaved_select_row),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (lst_keysaved), "unselect_row",
                      GTK_SIGNAL_FUNC (on_lst_keysaved_unselect_row),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (txt_cbokeybindfunc), "changed",
                      GTK_SIGNAL_FUNC (on_txt_cbokeybindfunc_changed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (txt_cbokeyfiletype), "changed",
                      GTK_SIGNAL_FUNC (on_txt_cbokeyfiletype_changed),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (tgb_keybindctrl), "pressed",
                      GTK_SIGNAL_FUNC (on_keybutton_pressed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tgb_keybindalt), "pressed",
                      GTK_SIGNAL_FUNC (on_keybutton_pressed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tgb_keybindshift), "pressed",
                      GTK_SIGNAL_FUNC (on_keybutton_pressed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tgb_keybindwinkey), "pressed",
                      GTK_SIGNAL_FUNC (on_keybutton_pressed),
                      NULL);                      

  return fxd_keymain;


}





