/*
 * libwbmenu.c
 * Menu plugin entry point and initialisation
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include "libwbmenu.h"
#include "menu.h"

#include "../../plugin.h"
#include "../../whiteBOX.h"

plugin_data plg = {
  NULL,
  "wb_menu_03",
  NULL,
  "whiteBOX menu tool component",
  "Luke Freeman",
  NULL,
  NULL
};

plugin_data *get_plugin_data( void )
{
  return &plg;
}

void
init( GtkWidget *wnd_main, int argc, char* argv[] )
{

  /* preliminary obviously ----
   * need checks for flag/arg relevant to menu loading
   *    (e.g. --menu-file=/home/user/.blackbox/menu)
   */

  /* load the current style file */
  XrmDatabase database = (XrmDatabase) 0;
  
  char bb_rc[255];
  
  gchar *res_val   = "session.menuFile";
  gchar *res_class = "Session.MenuFile";

  XrmValue value;				     /* resource value */
  char *value_type;				   /* value type ?? Sorry don't know :) */
  
  gchar *menu_file = NULL;
  
  GtkWidget *wgt_vbxmenuopts;
  
  int i;
  int no_menu = 0;
  int cur_winmgr;

  cur_winmgr = BLACKBOX_SWITCH;

  for( i = 1; i < argc; i++ ) 
  {
    
    if( !strcmp(argv[i], "-b" ))
    {
      cur_winmgr = BLACKBOX_SWITCH;
    }
    else if( !strcmp(argv[i], "-f" ))
    {
      cur_winmgr = FLUXBOX_SWITCH;
    }
    else if( !strcmp(argv[i], "-me" ) || !strcmp(argv[i], "--menu-empty" )) 
    {
      if( menu_file )
        g_print( "You cannot use the -me command when giving a menu file.\n" );
      else  
      no_menu = 1;      
    }
    else if( !strcmp(argv[i], "-mf" ) || !strcmp(argv[i], "--menu-file" )) 
    {
      if( no_menu )
      {
        g_print( "You cannot specify a menu file with the -me command.\n" );
        break;
      }
      
      i++;
      
      if( i >= argc )
        g_print( "A menu file was not specified. No menu will be loaded.\n" );
      else
        menu_file = argv[i];
    }
    else if( !strcmp( argv[i], "-met" ) || !strcmp( argv[i], "--menu-tab" ))
    {
	GtkWidget *wgt_nbkmain    = lookup_widget( wnd_main, "nbk_main" );
	GtkWidget *wgt_hpnmenuopts = lookup_widget( wnd_main, "hpn_menuopts" );
		
	gtk_notebook_set_page( (GtkNotebook *) wgt_nbkmain, 
            gtk_notebook_page_num( (GtkNotebook *) wgt_nbkmain, GTK_WIDGET( wgt_hpnmenuopts )->parent ));
    }    


  }
  
  /* if no menu file specified -- load blackbox menu */
  if(( !menu_file || argc == 1 ) && !no_menu )
  {
    strncpy( bb_rc, getenv("HOME"), 255 );
    if( cur_winmgr == BLACKBOX_SWITCH )
      strncat( bb_rc, "/.blackboxrc", 255 );
    else if( cur_winmgr == FLUXBOX_SWITCH )
      strncat( bb_rc, "/.fluxbox/init", 255 );
  
    database = XrmGetFileDatabase( bb_rc );
  
    if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
      menu_file = (gchar *) value.addr;
  
  }
  
  if( menu_file )
  {
    GtkWidget *wgt_lbl_curmenu  = lookup_widget( wnd_main, "lbl_curmenu" );
  
    build_menu_tree( wnd_main, menu_file );
    gblopen_filename = g_strdup( menu_file );
    gtk_label_set_text( (GtkLabel *) wgt_lbl_curmenu, gblopen_filename );

  }
  
  wgt_vbxmenuopts = lookup_widget( wnd_main, "vbx_menuopts" );
  gtk_widget_hide( wgt_vbxmenuopts );

  
  
}

GtkWidget *
entry_point( GtkWidget *wnd_main )
{

  GtkWidget *hpn_menuopts;
  GtkWidget *vbx_menutree;
  GtkWidget *hbx_menutitle;
  GtkWidget *lbl_menutitle;
  GtkWidget *txt_menutitle;
  GtkWidget *scw_menu;
  GtkWidget *cte_menu;
  GtkWidget *lbl_menutree;
  GtkWidget *vbx_menumain;
  GtkWidget *tbr_menuopts;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *tbn_newmenuitem;
  GtkWidget *tbn_delmenuitem;
  GtkWidget *tbn_moveupmenuitem;
  GtkWidget *tbn_movedownmenuitem;
  GtkWidget *tbn_navup;
  GtkWidget *tbn_navdown;
  GtkWidget *tbn_savemenu;
  GtkWidget *tbn_saveasmenu;
  GtkWidget *tbn_loadmenu;
  GtkWidget *vbx_menuopts;
  GtkWidget *fra_menuopts;
  GtkWidget *tbl_menuopts;
  GtkWidget *cbo_menutype;
  GList *cbo_menutype_items = NULL;
  GtkWidget *txt_cbomenutype;
  GtkWidget *txt_menulabel;
  GtkWidget *lbl_menulabel;
  GtkWidget *lbl_menuexec;
  GtkWidget *lbl_menutype;
  GtkWidget *txt_menuexec;
  GtkWidget *tbl_menuactions;
  guint btn_savemenu_key;
  GtkWidget *btn_savemenu;

  GtkWidget *lbl_curmenu;
  GtkWidget *lbl_curmenutitle;
  GtkWidget *hbx_menumain;
  GtkWidget *vbx_menu;

  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new ();

  vbx_menu = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbx_menu, "vbx_menu");
  gtk_widget_ref (vbx_menu);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "vbx_menu", vbx_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbx_menu);


  hpn_menuopts = gtk_hpaned_new ();
  gtk_widget_set_name (hpn_menuopts, "hpn_menuopts");
  gtk_widget_ref (hpn_menuopts);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "hpn_menuopts", hpn_menuopts,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hpn_menuopts);
  gtk_paned_set_handle_size (GTK_PANED (hpn_menuopts), 12);
  gtk_paned_set_gutter_size (GTK_PANED (hpn_menuopts), 12);
  gtk_paned_set_position (GTK_PANED (hpn_menuopts), 293);
  gtk_box_pack_start (GTK_BOX (vbx_menu), hpn_menuopts, TRUE, TRUE, 1);

  vbx_menutree = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbx_menutree, "vbx_menutree");
  gtk_widget_ref (vbx_menutree);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "vbx_menutree", vbx_menutree,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbx_menutree);
  gtk_paned_pack1 (GTK_PANED (hpn_menuopts), vbx_menutree, FALSE, TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (vbx_menutree), 4);

  hbx_menutitle = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbx_menutitle, "hbx_menutitle");
  gtk_widget_ref (hbx_menutitle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "hbx_menutitle", hbx_menutitle,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbx_menutitle);
  gtk_box_pack_start (GTK_BOX (vbx_menutree), hbx_menutitle, FALSE, FALSE, 4);

  lbl_menutitle = gtk_label_new (_("Menu title: "));
  gtk_widget_set_name (lbl_menutitle, "lbl_menutitle");
  gtk_widget_ref (lbl_menutitle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_menutitle", lbl_menutitle,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_menutitle);
  gtk_box_pack_start (GTK_BOX (hbx_menutitle), lbl_menutitle, FALSE, FALSE, 3);
  gtk_misc_set_alignment (GTK_MISC (lbl_menutitle), 7.45058e-09, 0.5);

  txt_menutitle = gtk_entry_new ();
  gtk_widget_set_name (txt_menutitle, "txt_menutitle");
  gtk_widget_ref (txt_menutitle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_menutitle", txt_menutitle,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_menutitle);
  gtk_box_pack_start (GTK_BOX (hbx_menutitle), txt_menutitle, TRUE, TRUE, 1);

  scw_menu = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scw_menu, "scw_menu");
  gtk_widget_ref (scw_menu);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "scw_menu", scw_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scw_menu);
  gtk_box_pack_start (GTK_BOX (vbx_menutree), scw_menu, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (scw_menu), 2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw_menu), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  cte_menu = gtk_ctree_new (1, 0);
  gtk_widget_set_name (cte_menu, "cte_menu");
  gtk_widget_ref (cte_menu);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "cte_menu", cte_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cte_menu);
  gtk_container_add (GTK_CONTAINER (scw_menu), cte_menu);
  gtk_clist_set_column_width (GTK_CLIST (cte_menu), 0, 80);
  gtk_clist_column_titles_show (GTK_CLIST (cte_menu));
  gtk_clist_set_row_height(GTK_CLIST (cte_menu), 18);


  lbl_menutree = gtk_label_new (_("Blackbox Menu"));
  gtk_widget_set_name (lbl_menutree, "lbl_menutree");
  gtk_widget_ref (lbl_menutree);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_menutree", lbl_menutree,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_menutree);
  gtk_clist_set_column_widget (GTK_CLIST (cte_menu), 0, lbl_menutree);

  vbx_menumain = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbx_menumain, "vbx_menumain");
  gtk_widget_ref (vbx_menumain);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "vbx_menumain", vbx_menumain,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbx_menumain);
  gtk_paned_pack2 (GTK_PANED (hpn_menuopts), vbx_menumain, FALSE, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (vbx_menumain), 8);

  tbr_menuopts = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_widget_set_name (tbr_menuopts, "tbr_menuopts");
  gtk_widget_ref (tbr_menuopts);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbr_menuopts", tbr_menuopts,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbr_menuopts);
  gtk_box_pack_start (GTK_BOX (vbx_menumain), tbr_menuopts, FALSE, FALSE, 0);
  gtk_widget_set_usize (tbr_menuopts, 0, 42);
  gtk_toolbar_set_space_style (GTK_TOOLBAR (tbr_menuopts), GTK_TOOLBAR_SPACE_LINE);
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (tbr_menuopts), GTK_RELIEF_NONE);


  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_open-16.xpm");
  tbn_loadmenu = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_menuopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Load"),
                                _("Load a menu"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_loadmenu, "tbn_loadmenu");
  gtk_widget_ref (tbn_loadmenu);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_loadmenu", tbn_loadmenu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_loadmenu);


  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_save-16.xpm");
  tbn_savemenu = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_menuopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Save"),
                                _("Save this menu"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_savemenu, "tbn_savemenu");
  gtk_widget_ref (tbn_savemenu);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_savemenu", tbn_savemenu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_savemenu);


  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_save_as-16.xpm");
  tbn_saveasmenu = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_menuopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Copy"),
                                _("Copy this menu (save as)"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_saveasmenu, "tbn_saveasmenu");
  gtk_widget_ref (tbn_saveasmenu);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_saveasmenu", tbn_saveasmenu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_saveasmenu);

  gtk_toolbar_append_space (GTK_TOOLBAR (tbr_menuopts));


  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_edit-16.xpm");
  tbn_newmenuitem = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_menuopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("New"),
                                _("Create a new item"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_newmenuitem, "tbn_newmenuitem");
  gtk_widget_ref (tbn_newmenuitem);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_newmenuitem", tbn_newmenuitem,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_newmenuitem);

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_delete-16.xpm");
  tbn_delmenuitem = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_menuopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Del"),
                                _("Delete an item"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_delmenuitem, "tbn_delmenuitem");
  gtk_widget_ref (tbn_delmenuitem);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_delmenuitem", tbn_delmenuitem,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_delmenuitem);

  gtk_toolbar_append_space (GTK_TOOLBAR (tbr_menuopts));

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_up-16.xpm");
  tbn_moveupmenuitem = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_menuopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Up"),
                                _("Move item up"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_moveupmenuitem, "tbn_moveupmenuitem");
  gtk_widget_ref (tbn_moveupmenuitem);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_moveupmenuitem", tbn_moveupmenuitem,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_moveupmenuitem);

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_down-16.xpm");
  tbn_movedownmenuitem = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_menuopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Down"),
                                _("Move item down"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_movedownmenuitem, "tbn_movedownmenuitem");
  gtk_widget_ref (tbn_movedownmenuitem);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_movedownmenuitem", tbn_movedownmenuitem,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_movedownmenuitem);

  gtk_toolbar_append_space (GTK_TOOLBAR (tbr_menuopts));

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_top-16.xpm");
  tbn_navup = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_menuopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Skip"),
                                _("Skip up a level"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_navup, "tbn_navup");
  gtk_widget_ref (tbn_navup);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_navup", tbn_navup,
                            (GtkDestroyNotify) gtk_widget_unref);

  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_bottom-16.xpm");
  tbn_navdown = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_menuopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Skip"),
                                _("Skip down a level"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_navdown, "tbn_navdown");
  gtk_widget_ref (tbn_navdown);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_navdown", tbn_navdown,
                            (GtkDestroyNotify) gtk_widget_unref);

  



  vbx_menuopts = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbx_menuopts, "vbx_menuopts");
  gtk_widget_ref (vbx_menuopts);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "vbx_menuopts", vbx_menuopts,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbx_menuopts);
  gtk_box_pack_start (GTK_BOX (vbx_menumain), vbx_menuopts, TRUE, TRUE, 0);

  fra_menuopts = gtk_frame_new (_("Menu Options"));
  gtk_widget_set_name (fra_menuopts, "fra_menuopts");
  gtk_widget_ref (fra_menuopts);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "fra_menuopts", fra_menuopts,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fra_menuopts);
  gtk_box_pack_start (GTK_BOX (vbx_menuopts), fra_menuopts, TRUE, TRUE, 8);
  gtk_frame_set_shadow_type (GTK_FRAME (fra_menuopts), GTK_SHADOW_ETCHED_OUT);

  tbl_menuopts = gtk_table_new (3, 2, FALSE);
  gtk_widget_set_name (tbl_menuopts, "tbl_menuopts");
  gtk_widget_ref (tbl_menuopts);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbl_menuopts", tbl_menuopts,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbl_menuopts);
  gtk_container_add (GTK_CONTAINER (fra_menuopts), tbl_menuopts);
  gtk_container_set_border_width (GTK_CONTAINER (tbl_menuopts), 8);
  gtk_table_set_row_spacings (GTK_TABLE (tbl_menuopts), 10);
  gtk_table_set_col_spacings (GTK_TABLE (tbl_menuopts), 42);

  cbo_menutype = gtk_combo_new ();
  gtk_widget_set_name (cbo_menutype, "cbo_menutype");
  gtk_widget_ref (cbo_menutype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "cbo_menutype", cbo_menutype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_menutype);
  gtk_table_attach (GTK_TABLE (tbl_menuopts), cbo_menutype, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_combo_set_value_in_list (GTK_COMBO (cbo_menutype), TRUE, TRUE);
  gtk_combo_set_use_arrows_always (GTK_COMBO (cbo_menutype), TRUE);
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Application"));
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Blackbox Configuration"));
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Exit"));
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Reconfigure"));
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Restart"));
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Spacer"));
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Style File"));
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Styles Directory"));
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Styles Menu"));
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Submenu"));
  cbo_menutype_items = g_list_append (cbo_menutype_items, (gpointer) _("Workspaces Menu"));
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_menutype), cbo_menutype_items);
  g_list_free (cbo_menutype_items);

  txt_cbomenutype = GTK_COMBO (cbo_menutype)->entry;
  gtk_widget_set_name (txt_cbomenutype, "txt_cbomenutype");
  gtk_widget_ref (txt_cbomenutype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_cbomenutype", txt_cbomenutype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_cbomenutype);
  gtk_entry_set_editable (GTK_ENTRY (txt_cbomenutype), FALSE);
  gtk_entry_set_text (GTK_ENTRY (txt_cbomenutype), _("Application"));

  txt_menulabel = gtk_entry_new ();
  gtk_widget_set_name (txt_menulabel, "txt_menulabel");
  gtk_widget_ref (txt_menulabel);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_menulabel", txt_menulabel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_menulabel);
  gtk_table_attach (GTK_TABLE (tbl_menuopts), txt_menulabel, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  lbl_menulabel = gtk_label_new (_("Item Label:"));
  gtk_widget_set_name (lbl_menulabel, "lbl_menulabel");
  gtk_widget_ref (lbl_menulabel);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_menulabel", lbl_menulabel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_menulabel);
  gtk_table_attach (GTK_TABLE (tbl_menuopts), lbl_menulabel, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (lbl_menulabel, 58, -2);
  gtk_label_set_justify (GTK_LABEL (lbl_menulabel), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lbl_menulabel), 7.45058e-09, 0.5);

  lbl_menuexec = gtk_label_new (_("Application:"));
  gtk_widget_set_name (lbl_menuexec, "lbl_menuexec");
  gtk_widget_ref (lbl_menuexec);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_menuexec", lbl_menuexec,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_menuexec);
  gtk_table_attach (GTK_TABLE (tbl_menuopts), lbl_menuexec, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (lbl_menuexec, 58, -2);
  gtk_label_set_justify (GTK_LABEL (lbl_menuexec), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lbl_menuexec), 7.45058e-09, 0.5);

  lbl_menutype = gtk_label_new (_("Item Type:"));
  gtk_widget_set_name (lbl_menutype, "lbl_menutype");
  gtk_widget_ref (lbl_menutype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_menutype", lbl_menutype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_menutype);
  gtk_table_attach (GTK_TABLE (tbl_menuopts), lbl_menutype, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (lbl_menutype, 58, -2);
  gtk_label_set_justify (GTK_LABEL (lbl_menutype), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lbl_menutype), 7.45058e-09, 0.5);

  txt_menuexec = gtk_entry_new ();
  gtk_widget_set_name (txt_menuexec, "txt_menuexec");
  gtk_widget_ref (txt_menuexec);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "txt_menuexec", txt_menuexec,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_menuexec);
  gtk_table_attach (GTK_TABLE (tbl_menuopts), txt_menuexec, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  tbl_menuactions = gtk_table_new (1, 1, FALSE);
  gtk_widget_set_name (tbl_menuactions, "tbl_menuactions");
  gtk_widget_ref (tbl_menuactions);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbl_menuactions", tbl_menuactions,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbl_menuactions);
  gtk_box_pack_start (GTK_BOX (vbx_menuopts), tbl_menuactions, FALSE, TRUE, 0);

  btn_savemenu = gtk_button_new_with_label ("");
  btn_savemenu_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (btn_savemenu)->child),
                                   _("A_pply"));
  gtk_widget_add_accelerator (btn_savemenu, "clicked", accel_group,
                              btn_savemenu_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
  gtk_widget_set_name (btn_savemenu, "btn_savemenu");
  gtk_widget_ref (btn_savemenu);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_savemenu", btn_savemenu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_savemenu);
  gtk_table_attach (GTK_TABLE (tbl_menuactions), btn_savemenu, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  GTK_WIDGET_SET_FLAGS (btn_savemenu, GTK_CAN_DEFAULT);


  hbx_menumain = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbx_menumain, "hbx_menumain");
  gtk_widget_ref (hbx_menumain);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "hbx_menumain", hbx_menumain,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_set_border_width (GTK_CONTAINER (hbx_menumain), 4);
  gtk_widget_show (hbx_menumain);
  gtk_box_pack_start (GTK_BOX (vbx_menu), hbx_menumain, FALSE, FALSE, 1);

  lbl_curmenutitle = gtk_label_new (_("Current Menu:  "));
  gtk_widget_set_name (lbl_curmenutitle, "lbl_curmenutitle");
  gtk_widget_ref (lbl_curmenutitle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_curmenutitle", lbl_curmenutitle,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_curmenutitle);
  gtk_box_pack_start (GTK_BOX (hbx_menumain), lbl_curmenutitle, FALSE, FALSE, 1);
  gtk_misc_set_alignment (GTK_MISC (lbl_curmenutitle), 7.45058e-09, 0.5);  

  lbl_curmenu = gtk_label_new (_("no menu loaded .."));
  gtk_widget_set_name (lbl_curmenu, "lbl_curmenu");
  gtk_widget_ref (lbl_curmenu);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_curmenu", lbl_curmenu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_curmenu);
  gtk_box_pack_start (GTK_BOX (hbx_menumain), lbl_curmenu, FALSE, FALSE, 2);
  gtk_misc_set_alignment (GTK_MISC (lbl_curmenu), 7.45058e-09, 0.5);  


  gtk_signal_connect (GTK_OBJECT (cte_menu), "tree_select_row",
                      GTK_SIGNAL_FUNC (on_cte_menu_tree_select_row),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (cte_menu), "tree_unselect_row",
                      GTK_SIGNAL_FUNC (on_cte_menu_tree_unselect_row),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tbn_newmenuitem), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_newmenuitem_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tbn_delmenuitem), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_delmenuitem_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tbn_moveupmenuitem), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_moveupmenuitem_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tbn_movedownmenuitem), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_movedownmenuitem_clicked),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (tbn_navup), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_navup_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tbn_navdown), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_navdown_clicked),
                      NULL);                      
                      
  gtk_signal_connect (GTK_OBJECT (tbn_savemenu), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_savemenu_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tbn_saveasmenu), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_saveasmenu_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tbn_loadmenu), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_loadmenu_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (txt_cbomenutype), "changed",
                      GTK_SIGNAL_FUNC (on_txt_cbomenutype_changed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_savemenu), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_savemenu_clicked),
                      NULL);
  
  gtk_window_add_accel_group (GTK_WINDOW (wnd_main), accel_group);


  /* set the style of the menu tree */
  gtk_ctree_set_line_style( (GtkCTree *) cte_menu, GTK_CTREE_LINES_SOLID );
  gtk_ctree_set_expander_style( (GtkCTree *) cte_menu, GTK_CTREE_EXPANDER_SQUARE );

  //return hpn_menuopts;
  return vbx_menu;

}
