/*
 * menu.c
 * Menu related callbacks and stuff
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <strings.h>

#ifdef HAVE_SYS_DIR_H
    #include <sys/dir.h>
#else
    #ifdef HAVE_SYS_NDIR_H
        #include <sys/ndir.h>
    #else
        #ifdef HAVE_DIRENT_H
            #include <dirent.h>
        #else
            #include <sys/dirent.h>
        #endif
    #endif
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "menu.h"
#include "menufunc.h"
#include "../../support.h"
#include "../../filedialog.h"
#include "../../msgbox.h"
#include "../../whiteBOX.h"

void
build_menu_tree (GtkWidget * ptr_tomain, char *menu_file)
{

	/*
	 * build_menu_tree
	 *
	 * build the intial menu tree
	 */

	static menu_item *init_item;

	GtkCTree *menu_tree;
	GtkCTreeNode *node;
	GtkCTreeNode *parent = NULL;


	GtkWidget *wgt_txtmenutitle;

	/* menu parse variables */
	char line[1024], label[1024], command[1024];
	int key = 0, in_sub = 0;
	int last_end = 0;

	FILE *file;

	char *nonop_label = "----";
	char *styles_label = "[styles dir]";

	wbTreeIcon *tree_icon;

	menu_tree = GTK_CTREE (lookup_widget (ptr_tomain, "cte_menu"));

	wgt_txtmenutitle = lookup_widget (ptr_tomain, "txt_menutitle");

	node = NULL;
	init_item = NULL;

	/*
	 * the code below is a (slightly modified) hack of code stolen 
	 * from the original blackbox sources.
	 * 
	 * for those who are interested check out Screen.cc (Line 1847)
	 * of the original Blackbox (0.61.1) sources.
	 *
	 * menu parse code will almost 100% likely be revised later. 
	 * but hey .. it works for now, and its pretty quick (i think).
	 */

	file = fopen (menu_file, "r");

	if (file != NULL)
		while (!feof (file))
		{

			memset (line, 0, 1024);
			memset (label, 0, 1024);
			memset (command, 0, 1024);

			if (fgets (line, 1024, file))
			{
				if (line[0] != '#')
				{

					register int i, parse = 0, index =
						-1, line_length =
						strlen (line), label_length =
						0, command_length = 0;

					/* determine the keyword */
					key = 0;

					for (i = 0; i < line_length; i++)
					{

						if (line[i] == '[')
							parse = 1;
						else if (line[i] == ']')
							break;
						else if (line[i] != ' ')
							if (parse)
								key += tolower
									(line
									 [i]);
					}

					/* get the label enclosed in ()'s */
					parse = 0;

					for (i = 0; i < line_length; i++)
					{

						if (line[i] == '(')
						{
							index = 0;
							parse = 1;
						}
						else if (line[i] == ')')
							break;
						else if (index++ >= 0)
						{
							if (line[i] == '\\'
							    && i <
							    line_length - 1)
								i++;
							label[index - 1] =
								line[i];
						}

						if (parse)
						{
							label[index] = '\0';
							label_length = index;
						}
						else
						{
							label[0] = '\0';
							label_length = 0;
						}
					}

					/* get the command enclosed in {}'s */
					parse = 0;
					index = -1;

					for (i = 0; i < line_length; i++)
					{
						if (line[i] == '{')
						{
							index = 0;
							parse = 1;
						}
						else if (line[i] == '}')
							break;
						else if (index++ >= 0)
						{
							if (line[i] == '\\'
							    && i <
							    line_length - 1)
								i++;
							command[index - 1] =
								line[i];
						}

						if (parse)
						{
							command[index] = '\0';
							command_length =
								index;
						}
						else
						{
							command[0] = '\0';
							command_length = 0;
						}
					}

				}
			}

			if (key == BEGIN)
			{
				gtk_entry_set_text ((GtkEntry *)
						    wgt_txtmenutitle, label);
			}

			if (key == END)
			{
				in_sub--;
				if (in_sub > 0)
				{
					if ((last_end != 0) && (in_sub > 0))
					{
						parent = GTK_CTREE_NODE
							(GTK_CTREE_ROW
							 (parent)->parent);
					}
					else
					{
						last_end = 1;
						parent = GTK_CTREE_NODE
							(GTK_CTREE_ROW
							 (GTK_CTREE_ROW
							  (node)->parent)->
							 parent);
					}
				}
			}

			/* create the menu tree */
			if (key != 0 && key != BEGIN && key != END)
			{

				init_item = new_item (key, label, command);
				last_end = 0;

				/* if first run through */
				if (node == NULL)
				{

					if (key == NONOP)
					{

						tree_icon =
							get_tree_icon (key);
						node = gtk_ctree_insert_node
							(menu_tree, parent,
							 NULL, &nonop_label,
							 8,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 FALSE, FALSE);
					}
					else if (key == STYLESDIR)
					{
						tree_icon =
							get_tree_icon (key);
						node = gtk_ctree_insert_node
							(menu_tree, NULL,
							 NULL, &styles_label,
							 8,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 FALSE, FALSE);


					}
					else
					{
						tree_icon =
							get_tree_icon (key);
						node = gtk_ctree_insert_node
							(menu_tree, NULL,
							 NULL,
							 &init_item->m_label,
							 8,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 FALSE, FALSE);

					}


					if (key == SUBMENU && in_sub == 0)
					{
						parent = node;
						in_sub++;
					}

				}
				else
				{
					if (key == SUBMENU && in_sub == 0)
					{

						tree_icon =
							get_tree_icon (key);
						node = gtk_ctree_insert_node
							(menu_tree, NULL,
							 NULL,
							 &init_item->m_label,
							 8,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 FALSE, FALSE);

						parent = node;
						in_sub++;
					}
					else
					{
						if (key == SUBMENU)
						{
							tree_icon =
								get_tree_icon
								(key);
							node = gtk_ctree_insert_node (menu_tree, parent, NULL, &init_item->m_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);

							parent = node;
							in_sub++;
						}
						else if (in_sub > 0)
						{
							if (key == NONOP)
							{

								tree_icon =
									get_tree_icon
									(key);
								node = gtk_ctree_insert_node (menu_tree, parent, NULL, &nonop_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);
							}
							else if (key ==
								 STYLESDIR)
							{

								tree_icon =
									get_tree_icon
									(key);
								node = gtk_ctree_insert_node (menu_tree, parent, NULL, &styles_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);
							}
							else
							{

								tree_icon =
									get_tree_icon
									(key);
								node = gtk_ctree_insert_node (menu_tree, parent, NULL, &init_item->m_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);
							}

						}
						else
						{
							if (key == NONOP)
							{

								tree_icon =
									get_tree_icon
									(key);
								node = gtk_ctree_insert_node (menu_tree, NULL, NULL, &nonop_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);
							}
							else if (key ==
								 STYLESDIR)
							{
								tree_icon =
									get_tree_icon
									(key);
								node = gtk_ctree_insert_node (menu_tree, NULL, NULL, &styles_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);
							}
							else
							{

								tree_icon =
									get_tree_icon
									(key);
								node = gtk_ctree_insert_node (menu_tree, NULL, NULL, &init_item->m_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);
							}
						}
					}
				}

			}

			if (node)
                            gtk_ctree_node_set_row_data (menu_tree, node, init_item);

		}		/* end while */

}


void
save_menu (GtkCTreeNode * root_node, GtkWidget * tree, FILE * save_file)
{

	GtkCTreeNode *cur_node = root_node;	/* the current node */
	GtkCTreeNode *child;	/* branch node children */

	int i = 0;		/* counter */

	menu_item *tt;		/* the menu item associated with the node */

	/* while theres still a node to play with */
	while (cur_node)
	{

		/* get the item information attached to the node */
		tt = gtk_ctree_node_get_row_data (GTK_CTREE (tree),
						  GTK_CTREE_NODE (cur_node));

		/* if item is a submenu without children */
		if ((tt->m_type == SUBMENU)
		    && (!GTK_CTREE_ROW (cur_node)->children))
		{
			insub++;	/* show submenu nesting level */
			parent[insub] = cur_node;	/* set as parent node */

			/* write to file */
			fprintf (save_file, "[%s] (%s)\n",
				 get_char_type (tt->m_type), tt->m_label);
		}
		else
		{

			/* if current node has children (branch) */
			if (GTK_CTREE_ROW (cur_node)->children)
			{

				/* if backed out of sub's and started at root level nesting */
				if ((GTK_CTREE_ROW (cur_node)->level == 1)
				    && (insub > 0))
				{
					/* recursively build [end]ing tags */
					for (i = 0; i < insub; i++)
					{
						/* write [end]'s to file */
						fprintf (save_file,
							 "[end]\n");
					}
					insub = 0;
				}
				else if ((GTK_CTREE_ROW (cur_node)->parent !=
					  parent[insub]) && (insub > 0))
				{
					fprintf (save_file, "[end]\n");
					insub--;
				}

				if (tt->m_type == SUBMENU)
				{
					insub++;
					parent[insub] = cur_node;
					fprintf (save_file, "[%s] (%s)\n",
						 get_char_type (tt->m_type),
						 tt->m_label);
				}
				else
				{
					fprintf (save_file,
						 "[%s] (%s) {%s}\n",
						 get_char_type (tt->m_type),
						 tt->m_label, tt->m_exec);
				}

				child = GTK_CTREE_NODE (GTK_CTREE_ROW
							(cur_node)->children);
				save_menu (child, tree, save_file);
			}
			else
			{

				if ((GTK_CTREE_ROW (cur_node)->level == 1)
				    && (insub > 0))
				{
					for (i = 0; i < insub; i++)
					{
						fprintf (save_file,
							 "[end]\n");
					}
					insub = 0;
				}
				else if ((GTK_CTREE_ROW (cur_node)->parent !=
					  parent[insub]) && (insub > 0))
				{
					while (GTK_CTREE_ROW (cur_node)->
					       parent != parent[insub])
					{
						fprintf (save_file,
							 "[end]\n");
						insub--;
					}

				}

				fprintf (save_file, "[%s] (%s) {%s}\n",
					 get_char_type (tt->m_type),
					 tt->m_label, tt->m_exec);
			}
		}

		if ((cur_node == last) && (insub > 0))
		{
			for (i = 0; i < insub; i++)
			{
				fprintf (save_file, "[end]\n");
			}
		}


		tt = NULL;	/* reset the temp item */
		cur_node = GTK_CTREE_ROW (cur_node)->sibling;	/* get next node */

	}


}

menu_item *
new_item (int type, const char *label, const char *exec)
{

	/*
	 * new_item
	 *
	 * create a new menu item safely, with memory allocations
	 */

	menu_item *new_item = calloc (1, sizeof (menu_item));

	assert (type);		/* see if any errors occur */

	/* set the item data */
	new_item->m_type = type;
	new_item->m_label = g_strdup (label);
	new_item->m_exec = g_strdup (exec);

	return new_item;

}

int
get_int_type (const char *str_type)
{

	if (!strcmp (str_type, "Application"))
		return APPLICATION;
	else if (!strcmp (str_type, "Blackbox Configuration"))
		return CONFIG;
	else if (!strcmp (str_type, "Exit"))
		return EXIT;
	else if (!strcmp (str_type, "Reconfigure"))
		return RECONFIG;
	else if (!strcmp (str_type, "Restart"))
		return RESTART;
	else if (!strcmp (str_type, "Spacer"))
		return NONOP;
	else if (!strcmp (str_type, "Style File"))
		return STYLE;
	else if (!strcmp (str_type, "Styles Directory"))
		return STYLESDIR;
	else if (!strcmp (str_type, "Styles Menu"))
		return STYLESMENU;
	else if (!strcmp (str_type, "Submenu"))
		return SUBMENU;
	else if (!strcmp (str_type, "Workspaces Menu"))
		return WORKSPACES;
	else
		return -1;


}

char *
get_menu_type (const int int_type)
{

	char *name = "";

	switch (int_type)
	{
	case SUBMENU:
		name = "Submenu";
		break;

	case APPLICATION:
		name = "Application";
		break;

	case STYLE:
		name = "Style File";
		break;

	case STYLESDIR:
		name = "Styles Directory";
		break;

	case STYLESMENU:
		name = "Styles Menu";
		break;

	case WORKSPACES:
		name = "Workspaces Menu";
		break;

	case NONOP:
		name = "Spacer";
		break;

	case RESTART:
		name = "Restart";
		break;

	case RECONFIG:
		name = "Reconfigure";
		break;

	case CONFIG:
		name = "Blackbox Configuration";
		break;

	case EXIT:
		name = "Exit";
		break;

	}

	return name;

}


char *
get_char_type (const int int_type)
{

	char *name = "";

	switch (int_type)
	{
	case SUBMENU:
		name = "submenu";
		break;

	case APPLICATION:
		name = "exec";
		break;

	case STYLE:
		name = "style";
		break;

	case STYLESDIR:
		name = "stylesdir";
		break;

	case STYLESMENU:
		name = "stylesmenu";
		break;

	case WORKSPACES:
		name = "workspaces";
		break;

	case NONOP:
		name = "nop";
		break;

	case RESTART:
		name = "restart";
		break;

	case RECONFIG:
		name = "reconfig";
		break;

	case CONFIG:
		name = "config";
		break;

	case EXIT:
		name = "exit";
		break;

	}

	return name;

}


void
on_txt_cbomenutype_changed (GtkEditable * editable, gpointer user_data)
{

	char *menu_type;

	GtkWidget *wgt_lblmenulabel;
	GtkWidget *wgt_lblexec;
	GtkWidget *wgt_txtexec;

	/* get the menu type from the combo box */
	menu_type = gtk_entry_get_text (GTK_ENTRY (editable));

	if ((!strcmp (menu_type, "Submenu")) ||
	    (!strcmp (menu_type, "Blackbox Configuration")) ||
	    (!strcmp (menu_type, "Reconfigure")) ||
	    (!strcmp (menu_type, "Exit")) ||
	    (!strcmp (menu_type, "Workspaces Menu")) ||
	    (!strcmp (menu_type, "Spacer")))
	{

		wgt_lblmenulabel =
			lookup_widget (GTK_WIDGET (editable),
				       "lbl_menulabel");
		gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
				    "Item Label:");

		/* hide the application data entry due to the fact its a submenu */
		wgt_lblexec =
			lookup_widget (GTK_WIDGET (editable), "lbl_menuexec");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Application:");
		gtk_widget_hide (wgt_lblexec);

		wgt_txtexec =
			lookup_widget (GTK_WIDGET (editable), "txt_menuexec");
		gtk_widget_hide (wgt_txtexec);
	}

	if (!strcmp (menu_type, "Styles Directory") ||
	    !strcmp (menu_type, "Styles Menu") ||
	    !strcmp (menu_type, "Style File"))
	{
		/* lookup menu item widgets */
		wgt_lblmenulabel =
			lookup_widget (GTK_WIDGET (editable),
				       "lbl_menulabel");
		wgt_lblexec =
			lookup_widget (GTK_WIDGET (editable), "lbl_menuexec");
		wgt_txtexec =
			lookup_widget (GTK_WIDGET (editable), "txt_menuexec");

		if (!strcmp (menu_type, "Styles Directory"))
		{
			gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
					    "Styles Directory:");
			gtk_label_set_text ((GtkLabel *) wgt_lblexec,
					    "Application:");
		}
		else if (!strcmp (menu_type, "Styles Menu"))
		{
			gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
					    "Item Label:");
			gtk_label_set_text ((GtkLabel *) wgt_lblexec,
					    "Styles Directory:");
		}
		else if (!strcmp (menu_type, "Style File"))
		{
			gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
					    "Item Label:");
			gtk_label_set_text ((GtkLabel *) wgt_lblexec,
					    "Style File:");
		}

		if (!strcmp (menu_type, "Styles Directory"))
		{
			/* hide the application data entry due to the fact its a submenu */
			gtk_widget_hide (wgt_lblexec);
			gtk_widget_hide (wgt_txtexec);
		}
		else
		{
			gtk_widget_show (wgt_lblexec);
			gtk_widget_show (wgt_txtexec);
		}

	}

	if (!strcmp (menu_type, "Application"))
	{

		wgt_lblmenulabel =
			lookup_widget (GTK_WIDGET (editable),
				       "lbl_menulabel");
		gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
				    "Item Label:");

		wgt_lblexec =
			lookup_widget (GTK_WIDGET (editable), "lbl_menuexec");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Application:");
		gtk_widget_show (wgt_lblexec);

		wgt_txtexec =
			lookup_widget (GTK_WIDGET (editable), "txt_menuexec");
		gtk_widget_show (wgt_txtexec);
	}

	if (!strcmp (menu_type, "Restart"))
	{

		wgt_lblmenulabel =
			lookup_widget (GTK_WIDGET (editable),
				       "lbl_menulabel");
		gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
				    "Item Label:");

		wgt_lblexec =
			lookup_widget (GTK_WIDGET (editable), "lbl_menuexec");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Session:");
		gtk_widget_show (wgt_lblexec);

		wgt_txtexec =
			lookup_widget (GTK_WIDGET (editable), "txt_menuexec");
		gtk_widget_show (wgt_txtexec);
	}


}

void
on_cte_menu_tree_select_row (GtkCTree * ctree,
			     GList * node, gint column, gpointer user_data)
{

	GtkWidget *wgt_vbxmenuopts;
	GtkWidget *wgt_lblexec;
	GtkWidget *wgt_txtexec;
	GtkWidget *wgt_txtlabel;
	GtkWidget *wgt_lbllabel;
	GtkWidget *wgt_cbomenutype;


	menu_item *tt =
		gtk_ctree_node_get_row_data (ctree, GTK_CTREE_NODE (node));

	wgt_vbxmenuopts = lookup_widget (GTK_WIDGET (ctree), "vbx_menuopts");
	gtk_widget_show (wgt_vbxmenuopts);

	wgt_lblexec = lookup_widget (GTK_WIDGET (ctree), "lbl_menuexec");
	wgt_txtexec = lookup_widget (GTK_WIDGET (ctree), "txt_menuexec");
	wgt_txtlabel = lookup_widget (GTK_WIDGET (ctree), "txt_menulabel");
	wgt_lbllabel = lookup_widget (GTK_WIDGET (ctree), "lbl_menulabel");

	wgt_cbomenutype =
		lookup_widget (GTK_WIDGET (ctree), "txt_cbomenutype");

	gtk_entry_set_text ((GtkEntry *) wgt_cbomenutype,
			    get_menu_type (tt->m_type));

	if ((tt->m_type == SUBMENU) ||
	    (tt->m_type == WORKSPACES) ||
	    (tt->m_type == NONOP) ||
	    (tt->m_type == RECONFIG) ||
	    (tt->m_type == CONFIG) || (tt->m_type == EXIT))
	{

		gtk_widget_hide (wgt_lblexec);
		gtk_widget_hide (wgt_txtexec);

		gtk_label_set_text ((GtkLabel *) wgt_lbllabel, "Item Label:");
		gtk_entry_set_text (GTK_ENTRY (wgt_txtlabel), tt->m_label);

	}
	else if (tt->m_type == STYLESDIR || tt->m_type == STYLE
		 || tt->m_type == STYLESMENU)
	{

		if (tt->m_type == STYLESDIR)
		{
			gtk_widget_hide (wgt_lblexec);
			gtk_widget_hide (wgt_txtexec);
		}

		if (tt->m_type == STYLESDIR)
			gtk_label_set_text ((GtkLabel *) wgt_lbllabel,
					    "Styles Directory:");
		else if (tt->m_type == STYLE)
		{
			gtk_label_set_text ((GtkLabel *) wgt_lbllabel,
					    "Item Label:");
			gtk_label_set_text ((GtkLabel *) wgt_lbllabel,
					    "Styles Directory:");
		}
		else if (tt->m_type == STYLESMENU)
		{
			gtk_label_set_text ((GtkLabel *) wgt_lbllabel,
					    "Item Label:");
			gtk_label_set_text ((GtkLabel *) wgt_lbllabel,
					    "Style File:");
		}

		gtk_entry_set_text (GTK_ENTRY (wgt_txtlabel), tt->m_label);
		if (tt->m_type == STYLE || tt->m_type == STYLESMENU)
			gtk_entry_set_text (GTK_ENTRY (wgt_txtexec),
					    tt->m_exec);

	}
	else if (tt->m_type == RESTART)
	{

		gtk_widget_show (wgt_lblexec);
		gtk_widget_show (wgt_txtexec);

		gtk_label_set_text ((GtkLabel *) wgt_lbllabel, "Item Label:");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Session:");

		gtk_entry_set_text (GTK_ENTRY (wgt_txtlabel), tt->m_label);
		gtk_entry_set_text (GTK_ENTRY (wgt_txtexec), tt->m_exec);

	}
	else if (tt->m_type == APPLICATION)
	{

		gtk_widget_show (wgt_lblexec);
		gtk_widget_show (wgt_txtexec);

		gtk_label_set_text ((GtkLabel *) wgt_lbllabel, "Item Label:");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Application:");

		gtk_entry_set_text (GTK_ENTRY (wgt_txtlabel),
				    un_bracket_string (tt->m_label));
		gtk_entry_set_text (GTK_ENTRY (wgt_txtexec), tt->m_exec);

	}


}


void
on_btn_savemenu_clicked (GtkButton * button, gpointer user_data)
{

	GtkCTreeNode *selection;	/* current item to update */
	GtkCTree *tree;		/* the menu tree */

	GtkWidget *wgt_cbotext;
	GtkWidget *wgt_txtlabel;
	GtkWidget *wgt_txtexec;

	/* menu data */
	int type;
	char *label, *exec, *b_label;

	char *menu_type;	/* the menu type (combo box) */
	wbTreeIcon *tree_icon;	/* icon attached to menu item */

	wgt_cbotext = lookup_widget (GTK_WIDGET (button), "txt_cbomenutype");
	wgt_txtexec = lookup_widget (GTK_WIDGET (button), "txt_menuexec");
	wgt_txtlabel = lookup_widget (GTK_WIDGET (button), "txt_menulabel");

	menu_type = gtk_entry_get_text (GTK_ENTRY (wgt_cbotext));

	tree = GTK_CTREE (lookup_widget (GTK_WIDGET (button), "cte_menu"));
	selection = (GTK_CLIST (tree)->selection)->data;

	type = get_int_type (menu_type);
	label = gtk_entry_get_text (GTK_ENTRY (wgt_txtlabel));
	exec = gtk_entry_get_text (GTK_ENTRY (wgt_txtexec));

	b_label = get_bracket_string (label);

	assert (selection);	/* check for those elusive errors */
	if (selection)
	{
		menu_item *new = new_item (type, b_label, exec);
                if (selection)
                    gtk_ctree_node_set_row_data (tree, selection, new);

		g_print ("working ..\n");

		/* get the items icon */
		tree_icon = get_tree_icon (type);

		if (type == STYLESDIR)
		{
			gtk_ctree_node_set_pixtext (tree, selection, 0,
						    "[styles dir]", 8,
						    tree_icon->icon_xpm,
						    tree_icon->icon_mask);
		}
		else if ((type == NONOP) && (strlen (label) == 0))
		{
			gtk_ctree_node_set_pixtext (tree, selection, 0,
						    "----", 8,
						    tree_icon->icon_xpm,
						    tree_icon->icon_mask);
		}
		else
		{
			gtk_ctree_node_set_pixtext (tree, selection, 0, label,
						    8, tree_icon->icon_xpm,
						    tree_icon->icon_mask);
		}
	}

}


void
on_cte_menu_tree_unselect_row (GtkCTree * ctree,
			       GList * node, gint column, gpointer user_data)
{

	/* gotta review my position on this .... too much flicker? */

	GtkWidget *wgt_vbxmenuopts;

	wgt_vbxmenuopts = lookup_widget (GTK_WIDGET (ctree), "vbx_menuopts");
	gtk_widget_hide (wgt_vbxmenuopts);

}

void
on_tbn_newmenuitem_clicked (GtkButton * button, gpointer user_data)
{

	GtkWidget *new_dialog;
	GtkWidget *main_ptr;

	main_ptr = GTK_WIDGET (button);

	new_dialog = create_dlg_newitem ();
	gtk_object_set_data (GTK_OBJECT (new_dialog), "wnd_parent", main_ptr);

	gtk_widget_show (new_dialog);

}


void
on_tbn_delmenuitem_clicked (GtkButton * button, gpointer user_data)
{

	GtkCTree *tree;
	GtkCTreeNode *rem_node;

	tree = GTK_CTREE (lookup_widget (GTK_WIDGET (button), "cte_menu"));

	/* if the user has made a selection */
	if ((GTK_CLIST (tree)->selection))
	{
		rem_node = (GTK_CLIST (tree)->selection)->data;
	}
	else
	{
		rem_node = NULL;
	}

	/* check to see if a valid node is selected */
	if (rem_node != NULL)
	{
		gtk_ctree_remove_node (tree, rem_node);
		gtk_ctree_select (tree, GTK_CTREE_NODE_PREV (rem_node));
	}
	else
	{
		g_print ("error:  no node selected ... \n");
	}


}


void
on_tbn_moveupmenuitem_clicked (GtkButton * button, gpointer user_data)
{

	GtkCTree *tree;
	GtkCTreeNode *move_node;
	GtkCTreeNode *parent;
	GtkCTreeNode *sibling;

	GtkCTreeNode *kid_node;

	menu_item *sibling_data;

	tree = GTK_CTREE (lookup_widget (GTK_WIDGET (button), "cte_menu"));

	/* if the user has made a selection */
	if ((GTK_CLIST (tree)->selection))
	{
		move_node = (GTK_CLIST (tree)->selection)->data;

		//sibling = GTK_CTREE_NODE_NEXT( parent );
		sibling = GTK_CTREE_NODE_PREV (move_node);
		parent = GTK_CTREE_ROW (move_node)->parent;

		if (sibling != NULL)
		{

			sibling_data =
				gtk_ctree_node_get_row_data (tree,
							     GTK_CTREE_NODE
							     (sibling));

			if ((GTK_CTREE_ROW (sibling)->children)
			    || (sibling_data->m_type == SUBMENU))
			{

				if (sibling != parent)
				{

					if (GTK_CTREE_ROW (sibling)->children)
					{
						kid_node =
							GTK_CTREE_ROW
							(sibling)->children;
					}
					else
					{
						kid_node = NULL;
					}

					if (kid_node != NULL)
					{
						while (GTK_CTREE_ROW
						       (kid_node)->sibling !=
						       NULL)
						{
							kid_node =
								GTK_CTREE_ROW
								(kid_node)->
								sibling;
						}

						gtk_ctree_move (tree,
								move_node,
								GTK_CTREE_ROW
								(kid_node)->
								parent, NULL);
						gtk_ctree_expand (tree,
								  GTK_CTREE_ROW
								  (kid_node)->
								  parent);
					}
					else
					{
						gtk_ctree_move (tree,
								move_node,
								sibling,
								NULL);
						gtk_ctree_expand (tree,
								  sibling);
					}



				}
				else
				{
					gtk_ctree_move (tree, move_node,
							GTK_CTREE_ROW
							(parent)->parent,
							parent);
				}

			}
			else
			{
				if (GTK_CTREE_ROW (sibling)->parent != parent)
				{
					gtk_ctree_move (tree, move_node,
							GTK_CTREE_ROW
							(sibling)->parent,
							NULL);
				}

				if ((sibling != NULL) && (sibling != parent))
				{
					gtk_ctree_move (tree, move_node,
							parent, sibling);
					gtk_ctree_select (tree, move_node);
				}
				else
				{

					if (sibling != NULL)
						gtk_ctree_move (tree,
								move_node,
								NULL,
								sibling);

					gtk_ctree_select (tree, move_node);

					g_print ("reached the top ...\n");
				}
			}
		}
		else
		{
			g_print ("warning: you have reached the top.\n");
			g_print ("         upwards movement is not possible.\n\n");
		}

	}

}


void
on_tbn_movedownmenuitem_clicked (GtkButton * button, gpointer user_data)
{

    GtkCTree *tree;
    GtkCTreeNode *move_node;
    GtkCTreeNode *next;
    GtkCTreeNode *sndnext;
    gboolean *bln_expanded;
    
    menu_item *sibling_data;
    
    tree = (GtkCTree *) lookup_widget ((GtkWidget *) button, "cte_menu");

    if ((GTK_CLIST (tree)->selection))
    {
        move_node = (GTK_CLIST (tree)->selection)->data;	/* node that will be moving */
        bln_expanded = (gboolean *) GTK_CTREE_ROW (move_node)->expanded;
        
        if (bln_expanded)
        {
            gtk_ctree_collapse  (tree, move_node);
            next = GTK_CTREE_NODE_NEXT (move_node);
            gtk_ctree_expand  (tree, move_node);
        }
        else
            next = GTK_CTREE_NODE_NEXT (move_node);
                
        if (next)
        {
            sibling_data = gtk_ctree_node_get_row_data (tree, GTK_CTREE_NODE(next));
            if (sibling_data->m_type == SUBMENU) {
                gtk_ctree_expand  (tree, next);
                sndnext = GTK_CTREE_NODE_NEXT (next);
                gtk_ctree_collapse  (tree, next);
            }
            else
                sndnext = GTK_CTREE_NODE_NEXT (next);
            
            if (sndnext)
            {
                if (sndnext) {
                    if (GTK_CTREE_ROW (sndnext)->parent == GTK_CTREE_ROW (move_node)->parent)
                        gtk_ctree_move (tree, move_node, GTK_CTREE_ROW (next)->parent, sndnext);
                    else
                        if (sibling_data->m_type != SUBMENU || GTK_CTREE_ROW (next)->level != GTK_CTREE_ROW (move_node)->level)
                        {
                            if (GTK_CTREE_ROW (next)->parent != GTK_CTREE_ROW (move_node)->parent)
                                gtk_ctree_move (tree, move_node, GTK_CTREE_ROW (next)->parent, next);               /* move out of sub menu */
                            else
                                gtk_ctree_move (tree, move_node, GTK_CTREE_ROW (next)->parent, NULL);               /* move to end of sub menu */
                        }
                        else
                        {
                            gtk_ctree_expand  (tree, next);
                            gtk_ctree_move (tree, move_node, next, sndnext);
                        }
                }
                //else
                    //gtk_ctree_move (tree, move_node, GTK_CTREE_ROW (GTK_CTREE_ROW (next)->parent)->parent, NULL);
            }
            else
                gtk_ctree_move (tree, move_node, GTK_CTREE_ROW (move_node)->parent, NULL);      /* move to end of sub menu */
        }
        else
            gtk_ctree_move (tree, move_node, NULL, NULL);               /* move to last item in menu */
        
    }

}

void
on_tbn_savemenu_clicked (GtkButton * button, gpointer user_data)
{

	GtkCTreeNode *top_node;

	GtkWidget *wgt_menutree =
		lookup_widget ((GtkWidget *) button, "cte_menu");
	GtkWidget *wgt_txtmenutitle =
		lookup_widget ((GtkWidget *) button, "txt_menutitle");
	GtkWidget *wgt_lbl_curmenu =
		lookup_widget ((GtkWidget *) button, "lbl_curmenu");

	GtkCTreeNode *top = NULL;

	FILE *save_file;
	gchar *file = NULL;

	char *menu_title;

	if (gblopen_filename == NULL)
	{

		file = return_file_w_title (NULL, "Save menu file ..");

		if (file)
		{

			/* grab the first item in the list */
			top = gtk_ctree_node_nth (GTK_CTREE (wgt_menutree),
						  0);
			last = gtk_ctree_last (GTK_CTREE (wgt_menutree), top);

			menu_title = gtk_entry_get_text ((GtkEntry *) wgt_txtmenutitle);	/* get menu title */

			save_file = fopen (file, "w");	/* open the file for writing */

			if (save_file)
			{

				fprintf (save_file, "[begin] (%s)\n",
					 menu_title);
				save_menu (top, wgt_menutree, save_file);	/* save the menu */

				/* do the closing [end] or menu marker */
				fprintf (save_file, "[end]\n");
				fclose (save_file);	/* close the file -- done */

				//gblopen_filename = fdlg.fdlg_file;            /* store open file */
				gblopen_filename = g_strdup (file);	/* store open file */
				gtk_label_set_text ((GtkLabel *)
						    wgt_lbl_curmenu, file);

			}
			else
			{
				if (message_box
				    (4,
				     "The file you have selected is invalid\nor you do not have write permission.\n\nPlease select a new file and try again.",
				     "Invalid File") == MBX_OK)
					return;

			}
		}

	}

	else
	{

		if ((save_file = fopen (gblopen_filename, "w")) != NULL)
		{

			wgt_txtmenutitle =
				lookup_widget ((GtkWidget *) button,
					       "txt_menutitle");
			menu_title =
				gtk_entry_get_text ((GtkEntry *)
						    wgt_txtmenutitle);

			top_node =
				gtk_ctree_node_nth (GTK_CTREE (wgt_menutree),
						    0);

			fprintf (save_file, "[begin] (%s)\n", menu_title);
			save_menu (top_node, wgt_menutree, save_file);
			fprintf (save_file, "[end]\n");

			fclose (save_file);
		}
		else
		{

			if (message_box
			    (4,
			     "The file you have selected is invalid\nor you do not have write permission.\n\nPlease select a new file and try again.",
			     "Invalid File") == MBX_OK)
				return;

		}

	}


}


void
on_tbn_saveasmenu_clicked (GtkButton * button, gpointer user_data)
{

	GtkWidget *wgt_menutree =
		lookup_widget ((GtkWidget *) button, "cte_menu");
	GtkWidget *wgt_txtmenutitle =
		lookup_widget ((GtkWidget *) button, "txt_menutitle");
	GtkWidget *wgt_lbl_curmenu =
		lookup_widget ((GtkWidget *) button, "lbl_curmenu");

	GtkCTreeNode *top = NULL;

	FILE *save_file;
	gchar *file = NULL;

	char *menu_title;

	file = return_file_w_title (NULL, "Save menu file ..");

	if (file)
	{

		/* grab the first item in the list */
		top = gtk_ctree_node_nth (GTK_CTREE (wgt_menutree), 0);
		last = gtk_ctree_last (GTK_CTREE (wgt_menutree), top);

		menu_title = gtk_entry_get_text ((GtkEntry *) wgt_txtmenutitle);	/* get menu title */

		save_file = fopen (file, "w");	/* open the file for writing */

		if (save_file)
		{

			fprintf (save_file, "[begin] (%s)\n", menu_title);
			save_menu (top, wgt_menutree, save_file);	/* save the menu */

			/* do the closing [end] or menu marker */
			fprintf (save_file, "[end]\n");
			fclose (save_file);	/* close the file -- done */

			gblopen_filename = g_strdup (file);	/* store open file */
			gtk_label_set_text ((GtkLabel *) wgt_lbl_curmenu,
					    file);

		}
		else
		{
			if (message_box
			    (4,
			     "The file you have selected is invalid\nor you do not have write permission.\n\nPlease select a new file and try again.",
			     "Invalid File") == MBX_OK)
				return;

		}
	}


}

void
on_tbn_loadmenu_clicked (GtkButton * button, gpointer user_data)
{

	GtkWidget *wgt_menutree =
		lookup_widget ((GtkWidget *) button, "cte_menu");
	GtkWidget *wgt_lbl_curmenu =
		lookup_widget ((GtkWidget *) button, "lbl_curmenu");

	gchar *file = NULL;

	struct stat s;

	file = return_file_w_title (NULL, "Load menu file ..");

	if (file)
	{

		if ((stat (file, &s) == -1) || (S_ISDIR (s.st_mode)))
		{
			if (message_box
			    (4,
			     "The file you have selected is invalid, a directory,\nor you do not have write permission.\n\nPlease select a new file and try again.",
			     "Invalid File") == MBX_OK)
				return;
		}
		else
		{

			/* everything is OK .. open this baby */
			/* clear and freeze the tree so update goes smoothly */
			gtk_clist_freeze (GTK_CLIST
					  ((GtkCTree *) wgt_menutree));
			gtk_clist_clear (GTK_CLIST
					 ((GtkCTree *) wgt_menutree));

			/* build the new menu */
			build_menu_tree (GTK_WIDGET (button), file);

			/* thaw the tree so we can use it */
			gtk_clist_thaw (GTK_CLIST (GTK_CTREE (wgt_menutree)));

			gblopen_filename = g_strdup (file);	/* set the file modified file name -- this needs work */
			gtk_label_set_text ((GtkLabel *) wgt_lbl_curmenu,
					    file);

			file = NULL;

		}

	}


}

void
on_tbn_navup_clicked (GtkButton * button, gpointer user_data)
{

	GtkCTree *tree;
	GtkCTreeNode *move_node;
	GtkCTreeNode *parent;
	GtkCTreeNode *sibling;

	tree = GTK_CTREE (lookup_widget (GTK_WIDGET (button), "cte_menu"));

	/* if the user has made a selection */
	if ((GTK_CLIST (tree)->selection))
	{
		move_node = (GTK_CLIST (tree)->selection)->data;

		//sibling = GTK_CTREE_NODE_NEXT( parent );
		sibling = GTK_CTREE_NODE_PREV (move_node);
		parent = GTK_CTREE_ROW (move_node)->parent;

		//g_print( "parent level = %d\n", GTK_CTREE_ROW( sibling )->level );

		if ((sibling != NULL)
		    && (GTK_CTREE_ROW (sibling)->level != 1))
		{
			while (GTK_CTREE_ROW (sibling)->parent != NULL)
				sibling = GTK_CTREE_ROW (sibling)->parent;
		}

		if ((sibling != NULL) && (sibling != parent))
		{
			gtk_ctree_move (tree, move_node, parent, sibling);
			gtk_ctree_select (tree, move_node);
		}
		else
		{
			if (sibling == NULL)
			{
				g_print ("at top\n");
			}
			else
			{
				gtk_ctree_move (tree, move_node, NULL,
						sibling);
				gtk_ctree_select (tree, move_node);
			}
		}

	}


}

void
on_tbn_navdown_clicked (GtkButton * button, gpointer user_data)
{

	GtkCTree *tree;
	GtkCTreeNode *move_node;
	GtkCTreeNode *parent;

	GtkCTreeNode *nnext;
	GtkCTreeNode *next;

	gboolean has_children;
	gboolean exit_flag;
	
	
	exit_flag = FALSE;
	has_children = FALSE;
	tree = GTK_CTREE (lookup_widget (GTK_WIDGET (button), "cte_menu"));

	/* if the user has made a selection */
	if ((GTK_CLIST (tree)->selection))
	{
		move_node = (GTK_CLIST (tree)->selection)->data;
		parent = GTK_CTREE_ROW (move_node)->parent;
		next = GTK_CTREE_NODE_NEXT (move_node);

		if (!next)
			return;
		
		if(GTK_CTREE_ROW (move_node)->children)
			has_children = TRUE;
		
		if(has_children)
		{
			while(GTK_CTREE_ROW(next)->level != GTK_CTREE_ROW(move_node)->level)
			{
				next = GTK_CTREE_NODE_NEXT (next);
				if(!next)
					return;
		
			}
		}
		nnext = GTK_CTREE_NODE_NEXT (next);
	

		if ((nnext != NULL) && (GTK_CTREE_ROW (nnext)->level != 1))
		{
			while (exit_flag == FALSE)
			{
				
				if (GTK_CTREE_NODE_NEXT (nnext) == NULL)
					nnext = NULL;
				else
					nnext = GTK_CTREE_NODE_NEXT (nnext);
				
				if(nnext != NULL)
				{
					if(GTK_CTREE_ROW (nnext)->level != 1)
						exit_flag = FALSE;
					else
						exit_flag = TRUE;
							
				}
				else
					exit_flag = TRUE;
			}
		}
		
		
		if (next != NULL)
		{
			if (!nnext)
				nnext = NULL;

			if (parent != NULL)
			{
				if (next != parent)
				{
					if(!nnext)
					{
						gtk_ctree_move (tree,
								move_node,
								NULL, NULL);
					}
					else
					{
						if (GTK_CTREE_ROW (nnext)->parent == NULL)
						{
							if(GTK_CTREE_ROW(next)->level != GTK_CTREE_ROW(move_node)->level)
								gtk_ctree_move (tree,
									move_node,
									NULL, next);
															
							else
								gtk_ctree_move (tree,
									move_node,
									NULL, nnext);
							
						}
						else
							gtk_ctree_move (tree,
								move_node,
								parent,
								nnext);
					}
						
					gtk_ctree_select (tree, move_node);
				}
				else
				{
					if (next)
					{
						gtk_ctree_move (tree,
								move_node,
								NULL, next);
						gtk_ctree_select (tree,
								  move_node);
					}
				}
			}
			else
			{
				if (nnext)
					gtk_ctree_move (tree, move_node, NULL,
							nnext);
				else
					gtk_ctree_move (tree, move_node, NULL,
							NULL);

			}
		}

	}


}
