/*
 * menu.h
 * Menu header file
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include <gtk/gtk.h>
#include <X11/Xlib.h>
#include <X11/Xresource.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>

#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <strings.h>
#include <string.h>

/* define numeric menu types */
#define APPLICATION   421
#define BEGIN         517
#define CONFIG        630
#define END           311
#define EXIT          442
#define INCLUDE       740
#define NONOP         333
#define RECONFIG      845
#define RESTART       773
#define STYLE         561
#define STYLESDIR     995
#define STYLESMENU    1113
#define SUBMENU       767
#define WORKSPACES    1090


typedef struct
{
  int 	 m_type;
  char	*m_label;
  char  *m_exec;
} menu_item;

/* globals */
GtkCTreeNode *parent[50];    /* tree node parent    -- save_menu */
GtkCTreeNode *last;
int insub;                   /* inside submenu flag -- save_menu*/

char *gblopen_filename;

GtkWidget*
create_dlg_newitem (void);

void
build_menu_tree( GtkWidget *ptr_tomain, char *menu_file );

menu_item *
new_item( int type, const char *label, const char *exec );

int 
get_int_type( const char *str_type );

char *
get_menu_type( const int int_type );

char *
get_char_type( const int int_type );

void
on_txt_cbomenutype_changed             (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_txt_cbomenutype_activate            (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_cbo_menutype_show                   (GtkWidget       *widget,
                                        gpointer         user_data);

void
on_cte_menu_tree_select_row            (GtkCTree        *ctree,
                                        GList           *node,
                                        gint             column,
                                        gpointer         user_data);

void
on_btn_savemenu_clicked                (GtkButton       *button,
                                        gpointer         user_data);

void
on_tbn_newmenuitem_clicked             (GtkButton       *button,
                                        gpointer         user_data);

void
on_tbn_delmenuitem_clicked             (GtkButton       *button,
                                        gpointer         user_data);

void
on_tbn_moveupmenuitem_clicked          (GtkButton       *button,
                                        gpointer         user_data);

void
on_tbn_movedownmenuitem_clicked        (GtkButton       *button,
                                        gpointer         user_data);

void
on_tbn_savemenu_clicked                (GtkButton       *button,
                                        gpointer         user_data);

void
on_tbn_saveasmenu_clicked              (GtkButton       *button,
    				         gpointer         user_data);

void
on_tbn_loadmenu_clicked                (GtkButton       *button,
                                        gpointer         user_data);

void
on_cte_menu_tree_unselect_row          (GtkCTree        *ctree,
                                        GList           *node,
                                        gint             column,
                                        gpointer         user_data);

void
on_tbn_navup_clicked                   (GtkButton       *button,
                                        gpointer         user_data);
                                        
void
on_tbn_navdown_clicked                   (GtkButton       *button,
                                        gpointer         user_data);

