/*
 * menufunc.c
 * Global menu functions
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include "menufunc.h"
#include "menu.h"

char *
get_bracket_string( char *original_string )
{

  /* checks a string for brackets and returns a bracket 
   * friendly string */

  char line[1024];           /* checks line for brackets */
  char test[1024];
  char b_val;
  
  int i, index, line_length;
  
  /* initialisation */
  memset( line, 0, 1024 );
  memset( test, 0, 1024 );  
  
  b_val = NULL;
  
  strcpy( line, original_string );
  line_length = strlen( original_string );
            
  index = 0;
            
  for( i = 0; i < line_length; i++ ) 
  {
    if( line[i] == '(' )
      b_val = '(';
    else if( line[i] == ')' )
      b_val = ')';
    else if( line[i] == '[' )
      b_val = '[';
    else if( line[i] == ']' )
      b_val = ']';
    else if( line[i] == '{' )
      b_val = '{';
    else if( line[i] == '}' )
     b_val = '}';
  
    if( b_val != NULL )
    {
      test[index] = '\\';
      test[index + 1] = b_val;
      index += 2;
      b_val = NULL;
    }
    else {
      test[index] = line[i];
      index++;
    }
  }
  
  return (char *) test;
    
}


char *
un_bracket_string( char *original_string )
{

  /* checks a string for brackets and returns a bracket 
   * friendly string */

  char line[1024];           /* checks line for brackets */
  char test[1024];
  char b_val = NULL;
  
  int i, index, line_length;
  
  /* initialisation */
  memset(line, 0, 1024);
  memset(test, 0, 1024);  
  
  
  strcpy( line, original_string );
  line_length = strlen( original_string );
            
  index = 0;
            
  for( i = 0; i < line_length; i++ ) 
  {
    if( line[i] == '\\' )
    {
      if( line[i + 1] == '(' )
        b_val = '(';
      else if( line[i + 1] == ')' ) 
        b_val = ')';
      else if( line[i + 1] == '[' ) 
        b_val = '[';
      else if( line[i + 1] == ']' ) 
        b_val = ']';
      else if( line[i + 1] == '{' ) 
        b_val = '{';
      else if( line[i + 1] == '}' ) 
        b_val = '}';
            
      if( b_val != NULL )
      {
        test[index] = '(';
        i++;
        index++;
        b_val = NULL;
      }
      else
      {
        test[index] = line[i];
        index++;
      }
                  
    }
    else {
      test[index] = line[i];
      index++;
    }
  }
    
  return (char *) test;
  
}


int
get_key_from_name( char *menu_type )
{
  int key_val = 0;
  
  if( !strcmp( menu_type, "Submenu" ))
    key_val = SUBMENU;
  else if( !strcmp( menu_type, "Blackbox Configuration" ))
	  key_val = CONFIG;
  else if( !strcmp( menu_type, "Reconfigure" )) 
	  key_val = RECONFIG;
  else if( !strcmp( menu_type, "Exit" )) 
	  key_val = EXIT;
  else if( !strcmp( menu_type, "Workspaces Menu" )) 
	  key_val = WORKSPACES;
  else if( !strcmp( menu_type, "Spacer" )) 
	  key_val = NONOP;
  else if( !strcmp( menu_type, "Style File" )) 
	  key_val = STYLE;
  else if( !strcmp( menu_type, "Styles Directory" )) 
	  key_val = STYLESDIR;
  else if( !strcmp( menu_type, "Styles Menu" )) 
	  key_val = STYLESMENU;
  else if( !strcmp( menu_type, "Restart" ))
    key_val = RESTART;
  else if( !strcmp( menu_type, "Application" ))
    key_val = APPLICATION;
  
  return key_val;
}
