/*
 * newitem.c
 * New item window callbacks and stuff
 *
 * Author:      Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <strings.h>

#include "menu.h"
#include "menufunc.h"

#include "../../msgbox.h"
#include "../../filedialog.h"
#include "../../support.h"
#include "../../whiteBOX.h"

void
on_txt_cbonewtype_changed (GtkEditable * editable, gpointer user_data)
{

	char *menu_type;	/* the menu type */

	GtkWidget *wgt_lblmenulabel;
	GtkWidget *wgt_lblexec;
	GtkWidget *wgt_txtexec;

	/*
	 * get the menu type from the combo box 
	 */
	menu_type = gtk_entry_get_text (GTK_ENTRY (editable));

	if ((!strcmp (menu_type, "Submenu")) ||
	    (!strcmp (menu_type, "Blackbox Configuration")) ||
	    (!strcmp (menu_type, "Reconfigure")) ||
	    (!strcmp (menu_type, "Exit")) ||
	    (!strcmp (menu_type, "Workspaces Menu")) ||
	    (!strcmp (menu_type, "Spacer")))
	{

		wgt_lblmenulabel =
			lookup_widget (GTK_WIDGET (editable), "lbl_newlabel");
		gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
				    "Item Label:");

		/*
		 * hide the application data entry due to the fact its a submenu 
		 */
		wgt_lblexec =
			lookup_widget (GTK_WIDGET (editable), "lbl_newexec");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Application:");
		gtk_widget_hide (wgt_lblexec);

		wgt_txtexec =
			lookup_widget (GTK_WIDGET (editable), "hbx_newexec");
		gtk_widget_hide (wgt_txtexec);

	}

	else if (!strcmp (menu_type, "Style File"))
	{

		wgt_lblmenulabel =
			lookup_widget (GTK_WIDGET (editable), "lbl_newlabel");
		gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
				    "Label:");

		/*
		 * hide the application data entry due to the fact its a submenu 
		 */
		wgt_lblexec =
			lookup_widget (GTK_WIDGET (editable), "lbl_newexec");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Style File:");
		gtk_widget_show (wgt_lblexec);

		wgt_txtexec =
			lookup_widget (GTK_WIDGET (editable), "hbx_newexec");
		gtk_widget_show (wgt_txtexec);

	}	

	else if (!strcmp (menu_type, "Styles Directory"))
	{

		wgt_lblmenulabel =
			lookup_widget (GTK_WIDGET (editable), "lbl_newlabel");
		gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
				    "Styles Directory:");

		/*
		 * hide the application data entry due to the fact its a submenu 
		 */
		wgt_lblexec =
			lookup_widget (GTK_WIDGET (editable), "lbl_newexec");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Application:");
		gtk_widget_hide (wgt_lblexec);

		wgt_txtexec =
			lookup_widget (GTK_WIDGET (editable), "hbx_newexec");
		gtk_widget_hide (wgt_txtexec);

	}

	else if (!strcmp (menu_type, "Styles Menu"))
	{

		wgt_lblmenulabel =
			lookup_widget (GTK_WIDGET (editable), "lbl_newlabel");
		gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
				    "Label:");

		/*
		 * hide the application data entry due to the fact its a submenu 
		 */
		wgt_lblexec =
			lookup_widget (GTK_WIDGET (editable), "lbl_newexec");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Styles Directory:");
		gtk_widget_show (wgt_lblexec);

		wgt_txtexec =
			lookup_widget (GTK_WIDGET (editable), "hbx_newexec");
		gtk_widget_show (wgt_txtexec);

	}
	
	else if (!strcmp (menu_type, "Application"))
	{

		wgt_lblmenulabel =
			lookup_widget (GTK_WIDGET (editable), "lbl_newlabel");
		gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
				    "Item Label:");

		wgt_lblexec =
			lookup_widget (GTK_WIDGET (editable), "lbl_newexec");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Application:");
		gtk_widget_show (wgt_lblexec);

		wgt_txtexec =
			lookup_widget (GTK_WIDGET (editable), "hbx_newexec");
		gtk_widget_show (wgt_txtexec);

	}

	else if (!strcmp (menu_type, "Restart"))
	{

		wgt_lblmenulabel =
			lookup_widget (GTK_WIDGET (editable), "lbl_newlabel");
		gtk_label_set_text ((GtkLabel *) wgt_lblmenulabel,
				    "Item Label:");

		wgt_lblexec =
			lookup_widget (GTK_WIDGET (editable), "lbl_newexec");
		gtk_label_set_text ((GtkLabel *) wgt_lblexec, "Session:");
		gtk_widget_show (wgt_lblexec);

		wgt_txtexec =
			lookup_widget (GTK_WIDGET (editable), "hbx_newexec");
		gtk_widget_show (wgt_txtexec);

	}

}


void
on_btn_newbrowse_clicked (GtkButton * button, gpointer user_data)
{

	GtkWidget *wgt_txtnewexec =
		lookup_widget ((GtkWidget *) button, "txt_newexec");
	gchar *file = NULL;

	file = return_file_w_title (NULL, "Please select a plugin ..");

	if (file)
		gtk_entry_set_text ((GtkEntry *) wgt_txtnewexec, file);

}




void
on_btn_newok_clicked (GtkButton * button, gpointer user_data)
{

	GtkCTree *tree;		/* the menu tree */

	GtkWidget *wgt_main;	/* the main window */
	GtkWidget *wgt_parent;	/* the parent window */
	GtkWidget *wgt_txtcombo;	/* the text part of the menu type
					 * combo */

	/*
	 * check widgets 
	 */
	GtkWidget *wgt_txtlabel;	/* the specified menu item label */
	GtkWidget *wgt_txtexec;	/* the specified menu item exec */

	GtkCTreeNode *selection;
	GtkCTreeNode *parent;	/* parent of the new node */
	GtkCTreeNode *node;	/* new node */

	char *menu_type;	/* the selected menu type */
	char *item_label;
	char *item_exec;

	static menu_item *tmp_item;
	static menu_item *add_item;

	int item_key = 0;	/* numeric value of item being added (to lookup icon) */

	wbTreeIcon *tree_icon;	/* icon attached to menu item */



	wgt_txtlabel = lookup_widget (GTK_WIDGET (button), "txt_newlabel");
	wgt_txtexec = lookup_widget (GTK_WIDGET (button), "txt_newexec");

	/*
	 * get the menu type from the combo box 
	 */
	wgt_txtcombo = lookup_widget (GTK_WIDGET (button), "txt_cbonewtype");
	menu_type = gtk_entry_get_text (GTK_ENTRY (wgt_txtcombo));

	item_label = gtk_entry_get_text (GTK_ENTRY (wgt_txtlabel));
	item_exec = gtk_entry_get_text (GTK_ENTRY (wgt_txtexec));

	add_item = new_item (get_int_type (menu_type), item_label, item_exec);

	/*
	 * get the main window & the parent of the main window 
	 */
	wgt_main =
		GTK_WIDGET (GTK_WIDGET
			    (GTK_WIDGET
			     (GTK_WIDGET (GTK_WIDGET (button)->parent)->
			      parent)->parent)->parent);
	wgt_parent =
		gtk_object_get_data (GTK_OBJECT (wgt_main), "wnd_parent");

	/*
	 * get the new items icon 
	 */
	item_key = get_key_from_name (menu_type);
	tree_icon = get_tree_icon (item_key);

	if (wgt_parent)
	{

		/*
		 * link to the main screen menu tree 
		 */
		tree = GTK_CTREE (lookup_widget
				  (GTK_WIDGET (wgt_parent), "cte_menu"));
		selection = GTK_CTREE_NODE (GTK_CLIST (tree)->selection);

		// g_print( "sel: %p\n", selection );

		if ((!strcmp (menu_type, "Submenu")) ||
		    (!strcmp (menu_type, "Blackbox Configuration")) ||
		    (!strcmp (menu_type, "Reconfigure")) ||
		    (!strcmp (menu_type, "Exit")) ||
		    (!strcmp (menu_type, "Workspaces Menu")) ||
		    (!strcmp (menu_type, "Spacer")) ||
		    (!strcmp (menu_type, "Styles Directory")) ||
		    (!strcmp (menu_type, "Restart")))
		{

			if ((strlen (item_label) != 0)
			    || (!strcmp (menu_type, "Spacer")))
			{
				if (selection != NULL)
				{
					selection =
						(GTK_CLIST (tree)->
						 selection)->data;
					tmp_item =
						gtk_ctree_node_get_row_data
						(tree,
						 GTK_CTREE_NODE (selection));

					if (tmp_item->m_type == SUBMENU)
					{
						/*
						 * if adding to a submenu 
						 */
						parent = (GTK_CLIST (tree)->
							  selection)->data;
						selection = NULL;

						if (!strcmp
						    (menu_type, "Spacer"))
						{
							item_label = "----";
							node = gtk_ctree_insert_node (tree, parent, selection, &item_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);
						}
						else if (!strcmp
							 (menu_type,
							  "Styles Directory"))
						{
							item_label =
								"[styles dir]";
							node = gtk_ctree_insert_node (tree, parent, selection, &item_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);
						}
						else
						{
							node = gtk_ctree_insert_node (tree, parent, selection, &item_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);

							strcpy (add_item->
								m_label,
								get_bracket_string
								(item_label));

						}


						gtk_ctree_node_set_row_data
							(tree, node,
							 add_item);

						gtk_widget_unrealize
							(wgt_main);
						gtk_widget_hide (wgt_main);

					}
					else
					{


						/*
						 * if just adding a plain node 
						 */

						selection =
							(GTK_CLIST (tree)->
							 selection)->data;
						parent = GTK_CTREE_ROW
							(selection)->parent;

						if (!strcmp
						    (menu_type, "Spacer"))
						{
							item_label = "----";
							node = gtk_ctree_insert_node (tree, parent, selection, &item_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);
						}
						else if (!strcmp
							 (menu_type,
							  "Styles Directory"))
						{
							item_label =
								"[styles dir]";
							node = gtk_ctree_insert_node (tree, parent, selection, &item_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);
						}
						else
						{
							node = gtk_ctree_insert_node (tree, parent, selection, &item_label, 8, tree_icon->icon_xpm, tree_icon->icon_mask, tree_icon->icon_xpm, tree_icon->icon_mask, FALSE, FALSE);

							strcpy (add_item->
								m_label,
								get_bracket_string
								(item_label));
						}


						gtk_ctree_node_set_row_data
							(tree, node,
							 add_item);

						gtk_widget_unrealize
							(wgt_main);
						gtk_widget_hide (wgt_main);

					}

				}
				else
				{
					g_print ("note:  no selection available ... \n");
					g_print ("       creating a new node ... \n");


					if (!strcmp (menu_type, "Spacer"))
					{
						item_label = "----";
						node = gtk_ctree_insert_node
							(tree, NULL, NULL,
							 &item_label, 8,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 FALSE, FALSE);
					}
					else if (!strcmp
						 (menu_type,
						  "Styles Directory"))
					{
						item_label = "[styles dir]";
						node = gtk_ctree_insert_node
							(tree, NULL, NULL,
							 &item_label, 8,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 FALSE, FALSE);
					}
					else
					{
						node = gtk_ctree_insert_node
							(tree, NULL, NULL,
							 &item_label, 8,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 FALSE, FALSE);

						strcpy (add_item->m_label,
							get_bracket_string
							(item_label));

					}

					gtk_ctree_node_set_row_data (tree,
								     node,
								     add_item);

					gtk_widget_unrealize (wgt_main);
					gtk_widget_hide (wgt_main);

				}
			}
			else
			{
				message_box(4, "Please enter all of the required information\nand try again.",
					"Invalid Entry");	
			}
		}
		else if (!strcmp (menu_type, "Application") 
			  || !strcmp (menu_type, "Style File")
			  || !strcmp (menu_type, "Styles Menu"))
		
		{
			if ((strlen (item_label) != 0)
			    && (strlen (item_exec) != 0))
			{
				if (selection != NULL)
				{
					selection =
						(GTK_CLIST (tree)->
						 selection)->data;

					tmp_item =
						gtk_ctree_node_get_row_data
						(tree,
						 GTK_CTREE_NODE (selection));

					if (tmp_item->m_type == SUBMENU)
					{
						/*
						 * if adding to a submenu 
						 */
						parent = (GTK_CLIST (tree)->
							  selection)->data;
						selection = NULL;

						node = gtk_ctree_insert_node
							(tree, parent,
							 selection,
							 &item_label, 8,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 FALSE, FALSE);

						strcpy (add_item->m_label,
							get_bracket_string
							(item_label));
						gtk_ctree_node_set_row_data
							(tree, node,
							 add_item);

						gtk_widget_unrealize
							(wgt_main);
						gtk_widget_hide (wgt_main);

					}
					else
					{
						/*
						 * if just adding a plain node 
						 */

						selection =
							(GTK_CLIST (tree)->
							 selection)->data;
						parent = GTK_CTREE_ROW
							(selection)->parent;

						node = gtk_ctree_insert_node
							(tree, parent,
							 selection,
							 &item_label, 8,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 tree_icon->icon_xpm,
							 tree_icon->icon_mask,
							 FALSE, FALSE);

						strcpy (add_item->m_label,
							get_bracket_string
							(item_label));

						gtk_ctree_node_set_row_data
							(tree, node,
							 add_item);

						gtk_widget_unrealize
							(wgt_main);
						gtk_widget_hide (wgt_main);

					}

				}
				else
				{
					g_print ("note:  no selection available ... \n");
					g_print ("       creating a new node ... \n");

					node = gtk_ctree_insert_node (tree,
								      NULL,
								      NULL,
								      &item_label,
								      8,
								      tree_icon->
								      icon_xpm,
								      tree_icon->
								      icon_mask,
								      tree_icon->
								      icon_xpm,
								      tree_icon->
								      icon_mask,
								      FALSE,
								      FALSE);

					strcpy (add_item->m_label,
						get_bracket_string
						(item_label));
					gtk_ctree_node_set_row_data (tree,
								     node,
								     add_item);

					gtk_widget_unrealize (wgt_main);
					gtk_widget_hide (wgt_main);

				}

			}
			else
			{
				message_box(4, "Please enter all of the required information\nand try again.",
					"Invalid Entry");
			}
		}

	}

}

void
on_dlg_newitem_show (GtkWidget * widget, gpointer user_data)
{

}


void
on_btn_newcancel_clicked (GtkButton * button, gpointer user_data)
{

	GtkWidget *wgt_main;

	wgt_main =
		GTK_WIDGET (GTK_WIDGET
			    (GTK_WIDGET
			     (GTK_WIDGET (GTK_WIDGET (button)->parent)->
			      parent)->parent)->parent);

	gtk_widget_unrealize (wgt_main);
	gtk_widget_hide (wgt_main);

}

GtkWidget *
create_dlg_newitem (void)
{
	GtkWidget *dlg_newitem;
	GtkWidget *vbx_newmain;
	GtkWidget *tbl_newmain;
	GtkWidget *cbo_newtype;
	GList *cbo_newtype_items = NULL;
	GtkWidget *txt_cbonewtype;
	GtkWidget *txt_newlabel;
	GtkWidget *lbl_newtype;
	GtkWidget *lbl_newlabel;
	GtkWidget *lbl_newexec;
	GtkWidget *hbx_newexec;
	GtkWidget *txt_newexec;
	GtkWidget *btn_newbrowse;
	GtkWidget *daa_newbot;
	GtkWidget *hbox1;
	guint btn_newcancel_key;
	GtkWidget *btn_newcancel;
	guint btn_newok_key;
	GtkWidget *btn_newok;
	GtkAccelGroup *accel_group;

	accel_group = gtk_accel_group_new ();

	dlg_newitem = gtk_dialog_new ();
	gtk_widget_set_name (dlg_newitem, "dlg_newitem");
	gtk_object_set_data (GTK_OBJECT (dlg_newitem), "dlg_newitem",
			     dlg_newitem);
	gtk_widget_set_usize (dlg_newitem, 330, 160);
	gtk_window_set_title (GTK_WINDOW (dlg_newitem), _("Add a menu item"));
	gtk_window_set_position (GTK_WINDOW (dlg_newitem), GTK_WIN_POS_MOUSE);
	gtk_window_set_modal (GTK_WINDOW (dlg_newitem), TRUE);
	gtk_window_set_policy (GTK_WINDOW (dlg_newitem), FALSE, FALSE, FALSE);

	vbx_newmain = GTK_DIALOG (dlg_newitem)->vbox;
	gtk_widget_set_name (vbx_newmain, "vbx_newmain");
	gtk_object_set_data (GTK_OBJECT (dlg_newitem), "vbx_newmain",
			     vbx_newmain);
	gtk_widget_show (vbx_newmain);

	tbl_newmain = gtk_table_new (3, 2, FALSE);
	gtk_widget_set_name (tbl_newmain, "tbl_newmain");
	gtk_widget_ref (tbl_newmain);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "tbl_newmain",
				  tbl_newmain,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tbl_newmain);
	gtk_box_pack_start (GTK_BOX (vbx_newmain), tbl_newmain, FALSE, FALSE,
			    0);
	gtk_container_set_border_width (GTK_CONTAINER (tbl_newmain), 13);
	gtk_table_set_row_spacings (GTK_TABLE (tbl_newmain), 11);
	gtk_table_set_col_spacings (GTK_TABLE (tbl_newmain), 16);

	cbo_newtype = gtk_combo_new ();
	gtk_widget_set_name (cbo_newtype, "cbo_newtype");
	gtk_widget_ref (cbo_newtype);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "cbo_newtype",
				  cbo_newtype,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (cbo_newtype);
	gtk_table_attach (GTK_TABLE (tbl_newmain), cbo_newtype, 1, 2, 0, 1,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Application"));
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Blackbox Configuration"));
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Exit"));
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Reconfigure"));
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Restart"));
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Spacer"));
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Style File"));
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Styles Directory"));
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Styles Menu"));
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Submenu"));
	cbo_newtype_items =
		g_list_append (cbo_newtype_items, (gpointer) _("Workspaces Menu"));
	gtk_combo_set_popdown_strings (GTK_COMBO (cbo_newtype),
				       cbo_newtype_items);
	g_list_free (cbo_newtype_items);

	txt_cbonewtype = GTK_COMBO (cbo_newtype)->entry;
	gtk_widget_set_name (txt_cbonewtype, "txt_cbonewtype");
	gtk_widget_ref (txt_cbonewtype);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "txt_cbonewtype",
				  txt_cbonewtype,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (txt_cbonewtype);
	gtk_entry_set_editable (GTK_ENTRY (txt_cbonewtype), FALSE);
	gtk_entry_set_text (GTK_ENTRY (txt_cbonewtype), _("Application"));

	txt_newlabel = gtk_entry_new ();
	gtk_widget_set_name (txt_newlabel, "txt_newlabel");
	gtk_widget_ref (txt_newlabel);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "txt_newlabel",
				  txt_newlabel,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (txt_newlabel);
	gtk_table_attach (GTK_TABLE (tbl_newmain), txt_newlabel, 1, 2, 1, 2,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	lbl_newtype = gtk_label_new (_("Type:"));
	gtk_widget_set_name (lbl_newtype, "lbl_newtype");
	gtk_widget_ref (lbl_newtype);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "lbl_newtype",
				  lbl_newtype,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (lbl_newtype);
	gtk_table_attach (GTK_TABLE (tbl_newmain), lbl_newtype, 0, 1, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_widget_set_usize (lbl_newtype, 82, -2);
	gtk_misc_set_alignment (GTK_MISC (lbl_newtype), 0, 0.5);

	lbl_newlabel = gtk_label_new (_("Label:"));
	gtk_widget_set_name (lbl_newlabel, "lbl_newlabel");
	gtk_widget_ref (lbl_newlabel);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "lbl_newlabel",
				  lbl_newlabel,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (lbl_newlabel);
	gtk_table_attach (GTK_TABLE (tbl_newmain), lbl_newlabel, 0, 1, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_widget_set_usize (lbl_newlabel, 82, -2);
	gtk_misc_set_alignment (GTK_MISC (lbl_newlabel), 0, 0.5);

	lbl_newexec = gtk_label_new (_("Command:"));
	gtk_widget_set_name (lbl_newexec, "lbl_newexec");
	gtk_widget_ref (lbl_newexec);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "lbl_newexec",
				  lbl_newexec,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (lbl_newexec);
	gtk_table_attach (GTK_TABLE (tbl_newmain), lbl_newexec, 0, 1, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_widget_set_usize (lbl_newexec, 82, -2);
	gtk_misc_set_alignment (GTK_MISC (lbl_newexec), 0, 0.5);

	hbx_newexec = gtk_hbox_new (FALSE, 3);
	gtk_widget_set_name (hbx_newexec, "hbx_newexec");
	gtk_widget_ref (hbx_newexec);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "hbx_newexec",
				  hbx_newexec,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbx_newexec);
	gtk_table_attach (GTK_TABLE (tbl_newmain), hbx_newexec, 1, 2, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 0, 0);

	txt_newexec = gtk_entry_new ();
	gtk_widget_set_name (txt_newexec, "txt_newexec");
	gtk_widget_ref (txt_newexec);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "txt_newexec",
				  txt_newexec,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (txt_newexec);
	gtk_box_pack_start (GTK_BOX (hbx_newexec), txt_newexec, TRUE, TRUE,
			    0);

	btn_newbrowse = gtk_button_new_with_label (_(".."));
	gtk_widget_set_name (btn_newbrowse, "btn_newbrowse");
	gtk_widget_ref (btn_newbrowse);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "btn_newbrowse",
				  btn_newbrowse,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (btn_newbrowse);
	gtk_box_pack_start (GTK_BOX (hbx_newexec), btn_newbrowse, FALSE,
			    FALSE, 0);
	gtk_widget_set_usize (btn_newbrowse, 21, 21);

	daa_newbot = GTK_DIALOG (dlg_newitem)->action_area;
	gtk_widget_set_name (daa_newbot, "daa_newbot");
	gtk_object_set_data (GTK_OBJECT (dlg_newitem), "daa_newbot",
			     daa_newbot);
	gtk_widget_show (daa_newbot);
	gtk_widget_set_usize (daa_newbot, 42, -2);
	gtk_container_set_border_width (GTK_CONTAINER (daa_newbot), 10);

	hbox1 = gtk_hbox_new (FALSE, 5);
	gtk_widget_set_name (hbox1, "hbox1");
	gtk_widget_ref (hbox1);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "hbox1", hbox1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (daa_newbot), hbox1, FALSE, TRUE, 0);

	btn_newcancel = gtk_button_new_with_label ("");
	btn_newcancel_key =
		gtk_label_parse_uline (GTK_LABEL
				       (GTK_BIN (btn_newcancel)->child),
				       _("_Cancel"));
	gtk_widget_add_accelerator (btn_newcancel, "clicked", accel_group,
				    btn_newcancel_key, GDK_MOD1_MASK,
				    (GtkAccelFlags) 0);
	gtk_widget_set_name (btn_newcancel, "btn_newcancel");
	gtk_widget_ref (btn_newcancel);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "btn_newcancel",
				  btn_newcancel,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (btn_newcancel);
	gtk_box_pack_end (GTK_BOX (hbox1), btn_newcancel, FALSE, FALSE, 0);
	gtk_widget_set_usize (btn_newcancel, 80, 24);

	btn_newok = gtk_button_new_with_label ("");
	btn_newok_key =
		gtk_label_parse_uline (GTK_LABEL (GTK_BIN (btn_newok)->child),
				       _("_Ok"));
	gtk_widget_add_accelerator (btn_newok, "clicked", accel_group,
				    btn_newok_key, GDK_MOD1_MASK,
				    (GtkAccelFlags) 0);
	gtk_widget_set_name (btn_newok, "btn_newok");
	gtk_widget_ref (btn_newok);
	gtk_object_set_data_full (GTK_OBJECT (dlg_newitem), "btn_newok",
				  btn_newok,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (btn_newok);
	gtk_box_pack_end (GTK_BOX (hbox1), btn_newok, FALSE, FALSE, 0);
	gtk_widget_set_usize (btn_newok, 80, 24);

	gtk_signal_connect (GTK_OBJECT (dlg_newitem), "show",
			    GTK_SIGNAL_FUNC (on_dlg_newitem_show), NULL);
	gtk_signal_connect (GTK_OBJECT (txt_cbonewtype), "changed",
			    GTK_SIGNAL_FUNC (on_txt_cbonewtype_changed),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (btn_newbrowse), "clicked",
			    GTK_SIGNAL_FUNC (on_btn_newbrowse_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_newcancel), "clicked",
			    GTK_SIGNAL_FUNC (on_btn_newcancel_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (btn_newok), "clicked",
			    GTK_SIGNAL_FUNC (on_btn_newok_clicked), NULL);

	gtk_window_add_accel_group (GTK_WINDOW (dlg_newitem), accel_group);

	return dlg_newitem;

}
