/*
 * libwbstyle.c
 * Style plugin entry point and initialisation
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xresource.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>

#include "libwbstyle.h"
#include "styleframes.h"
#include "../../plugin.h"
#include "../../support.h"
#include "../../whiteBOX.h"


GtkWidget* entry_point( GtkWidget *wnd_main );
void init( GtkWidget *wnd_main, int argc, char* argv[] );

plugin_data plg2 = {
  NULL,
  "wb_style_03",
  NULL,
  "whiteBOX style editor component",
  "Luke Freeman",
  NULL,
  NULL
};

plugin_data *get_plugin_data( void )
{
  return &plg2;
}


void
init( GtkWidget *wnd_main, int argc, char* argv[] )
{

  /* load the current style file */
  XrmDatabase database = (XrmDatabase) 0;
  
  char bb_rc[255];
  
  gchar *res_val   = "session.styleFile";
  gchar *res_class = "Session.StyleFile";

  XrmValue value;				     /* resource value */
  char *value_type;				   /* value type ?? Sorry don't know :) */

  gchar *style_file = NULL;
  
  GtkWidget *wgt_trvsedit     = lookup_widget( wnd_main, "trv_sedit" );
  GtkWidget *wgt_vbxstyles    = lookup_widget( wnd_main, "vbx_styles" );
  GtkWidget *wgt_tbnseditflux = lookup_widget( wnd_main, "tbn_seditflux" );
 
  int i;
  int no_style = 0;  
  int cur_winmgr;
  
  cur_winmgr = BLACKBOX_SWITCH;

  for( i = 1; i < argc; i++ ) 
  {
    if( !strcmp(argv[i], "-b" ))
    {
      cur_winmgr = BLACKBOX_SWITCH;
    }
    else if( !strcmp(argv[i], "-f" ))
    {
      cur_winmgr = FLUXBOX_SWITCH;
    }
    else if( !strcmp(argv[i], "-se" ) || !strcmp(argv[i], "--style-empty" )) 
    {
      if( style_file )
        g_print( "You cannot use the -se command when specifying a style file.\n" );
      else  
      no_style = 1;      
    }
    else if( !strcmp(argv[i], "-sf" ) || !strcmp(argv[i], "--style-file" )) 
    {
      if( no_style )
      {
        g_print( "You cannot specify a style file with the -se or --style-empty command.\n" );
        break;
      }
      
      i++;
      
      if( i >= argc )
        g_print( "A menu file was not specified. No menu will be loaded.\n" );
      else
        style_file = argv[i];
    }
    else if( !strcmp( argv[i], "-set" ) || !strcmp( argv[i], "--style-tab" ))
    {
	GtkWidget *wgt_nbkmain    = lookup_widget( wnd_main, "nbk_main" );
	GtkWidget *wgt_hpnsedit = lookup_widget( wnd_main, "hpn_sedit" );
		
	gtk_notebook_set_page( (GtkNotebook *) wgt_nbkmain, 
            gtk_notebook_page_num( (GtkNotebook *) wgt_nbkmain, GTK_WIDGET( wgt_hpnsedit )->parent ));
    }    
    
    
  }

  
  /* set fluxbox visibility option */
  if( cur_winmgr == FLUXBOX_SWITCH )
  {
    show_defaults( wgt_trvsedit, wgt_vbxstyles, wnd_main, 1 );
    gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tbnseditflux, TRUE );
  }
  else
  {
    show_defaults( wgt_trvsedit, wgt_vbxstyles, wnd_main, 0 );
    gtk_toggle_button_set_active( (GtkToggleButton *) wgt_tbnseditflux, FALSE );
  }
  
  
  /* if no menu file specified -- load blackbox menu */
  if(( !style_file || argc == 1 ) && !no_style )
  {
    strncpy( bb_rc, getenv("HOME"), 255 );
    if( cur_winmgr == BLACKBOX_SWITCH )
      strncat( bb_rc, "/.blackboxrc", 255 );
    else if( cur_winmgr == FLUXBOX_SWITCH )
      strncat( bb_rc, "/.fluxbox/init", 255 );
  
    database = XrmGetFileDatabase( bb_rc );
  
    if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
      style_file = (gchar *) value.addr;
  
  }
  
  if( style_file )
  {
    GtkWidget *wgt_styletree    = lookup_widget( wnd_main, "trv_sedit" );
    GtkCTreeNode *top            = gtk_ctree_node_nth( GTK_CTREE( wgt_styletree ), 0 );
    GtkWidget *wgt_lbl_curstyle = lookup_widget( wnd_main, "lbl_curstyle" );
    
    gblopen_stylename = g_strdup( style_file );
    gtk_label_set_text( (GtkLabel *) wgt_lbl_curstyle, style_file );      
      
    load_style( style_file, wgt_styletree, top );
  }
   
  
}




GtkWidget *
entry_point( GtkWidget *wnd_main )
{

  GtkWidget *hpn_sedit;
  GtkWidget *vbx_styletree;
  GtkWidget *scw_sedittree;
  GtkWidget *trv_sedit;
  GtkWidget *lbl_trvsedittitle;
  GtkWidget *vbx_stylemain;
  GtkWidget *tbr_styleopts;
  GtkWidget *tbn_stylesave;
  GtkWidget *tbn_stylesaveas;
  GtkWidget *tbn_styleload;
  GtkWidget *vbx_styles;
  guint btn_styleapply_key;
  GtkWidget *btn_styleapply;
  GtkWidget *tmp_toolbar_icon;

  GtkWidget *vbx_splitstyle;
  GtkWidget *lbl_curstyle;
  GtkWidget *hbx_styletitle;
  GtkWidget *lbl_curstyletitle;

  guint tbn_seditflux_key;
  GtkWidget *tbn_seditflux;



  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new ();


  vbx_splitstyle = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name( vbx_splitstyle, "vbx_splitstyle" );
  gtk_widget_ref( vbx_splitstyle );
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "vbx_splitstyle", vbx_splitstyle,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show( vbx_splitstyle );

  hpn_sedit = gtk_hpaned_new ();
  gtk_widget_set_name (hpn_sedit, "hpn_sedit");
  gtk_widget_ref (hpn_sedit);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "hpn_sedit", hpn_sedit,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hpn_sedit);
  gtk_paned_set_handle_size (GTK_PANED (hpn_sedit), 12);
  gtk_paned_set_gutter_size (GTK_PANED (hpn_sedit), 12);
  gtk_paned_set_position (GTK_PANED (hpn_sedit), 245);

  gtk_box_pack_start (GTK_BOX( vbx_splitstyle ), hpn_sedit, TRUE, TRUE, 1 );

  vbx_styletree = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbx_styletree, "vbx_styletree");
  gtk_widget_ref (vbx_styletree);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "vbx_styletree", vbx_styletree,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbx_styletree);
  gtk_paned_pack1 (GTK_PANED (hpn_sedit), vbx_styletree, TRUE, TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (vbx_styletree), 4);

  scw_sedittree = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scw_sedittree, "scw_sedittree");
  gtk_widget_ref (scw_sedittree);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "scw_sedittree", scw_sedittree,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scw_sedittree);
  gtk_box_pack_start (GTK_BOX (vbx_styletree), scw_sedittree, TRUE, TRUE, 0);
  gtk_widget_set_usize (scw_sedittree, -2, 318);
  gtk_container_set_border_width (GTK_CONTAINER (scw_sedittree), 2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw_sedittree), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  trv_sedit = gtk_ctree_new (1, 0);
  gtk_widget_set_name (trv_sedit, "trv_sedit");
  gtk_widget_ref (trv_sedit);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "trv_sedit", trv_sedit,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (trv_sedit);
  gtk_container_add (GTK_CONTAINER (scw_sedittree), trv_sedit);
  gtk_clist_set_column_width (GTK_CLIST (trv_sedit), 0, 80);
  gtk_clist_column_titles_show (GTK_CLIST (trv_sedit));

  lbl_trvsedittitle = gtk_label_new (_("Available Styles"));
  gtk_widget_set_name (lbl_trvsedittitle, "lbl_trvsedittitle");
  gtk_widget_ref (lbl_trvsedittitle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_trvsedittitle", lbl_trvsedittitle,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_trvsedittitle);
  gtk_clist_set_column_widget (GTK_CLIST (trv_sedit), 0, lbl_trvsedittitle);

  
  /* show fluxbox stuff */

  tbn_seditflux = gtk_toggle_button_new_with_label ("");
  tbn_seditflux_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (tbn_seditflux)->child),
                                   _("_Show Fluxbox"));
  gtk_widget_add_accelerator (tbn_seditflux, "clicked", accel_group,
                              tbn_seditflux_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
  gtk_widget_set_name (tbn_seditflux, "tbn_seditflux");
  gtk_widget_ref (tbn_seditflux);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_seditflux", tbn_seditflux,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_seditflux);
  gtk_box_pack_start (GTK_BOX (vbx_styletree), tbn_seditflux, TRUE, TRUE, 1);
  gtk_widget_set_usize (tbn_seditflux, 50, 44);
  /* end fluxbox stuff */


  vbx_stylemain = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbx_stylemain, "vbx_stylemain");
  gtk_widget_ref (vbx_stylemain);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "vbx_stylemain", vbx_stylemain,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbx_stylemain);
  gtk_paned_pack2 (GTK_PANED (hpn_sedit), vbx_stylemain, FALSE, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (vbx_stylemain), 8);

  tbr_styleopts = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_widget_set_name (tbr_styleopts, "tbr_styleopts");
  gtk_widget_ref (tbr_styleopts);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbr_styleopts", tbr_styleopts,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbr_styleopts);
  gtk_box_pack_start (GTK_BOX (vbx_stylemain), tbr_styleopts, FALSE, FALSE, 0);
  gtk_widget_set_usize (tbr_styleopts, 0, 42);
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (tbr_styleopts), GTK_RELIEF_NONE);


  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_open-16.xpm");
  tbn_styleload = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_styleopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Load"),
                                _("Load a style"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_styleload, "tbn_styleload");
  gtk_widget_ref (tbn_styleload);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_styleload", tbn_styleload,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_styleload);



  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_save-16.xpm");
  tbn_stylesave = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_styleopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Save"),
                                _("Save this style"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_stylesave, "tbn_stylesave");
  gtk_widget_ref (tbn_stylesave);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_stylesave", tbn_stylesave,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_stylesave);


  tmp_toolbar_icon = create_pixmap (wnd_main, "stock_save_as-16.xpm");
  tbn_stylesaveas = gtk_toolbar_append_element (GTK_TOOLBAR (tbr_styleopts),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Copy"),
                                _("Copy this style (save as)"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_set_name (tbn_stylesaveas, "tbn_stylesaveas");
  gtk_widget_ref (tbn_stylesaveas);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "tbn_stylesaveas", tbn_stylesaveas,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbn_stylesaveas);


  vbx_styles = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbx_styles, "vbx_styles");
  gtk_widget_ref (vbx_styles);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "vbx_styles", vbx_styles,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (vbx_stylemain), vbx_styles, TRUE, TRUE, 0);

  btn_styleapply = gtk_button_new_with_label ("");
  btn_styleapply_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (btn_styleapply)->child),
                                   _("_Apply"));
  gtk_widget_add_accelerator (btn_styleapply, "clicked", accel_group,
                              btn_styleapply_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
  gtk_widget_set_name (btn_styleapply, "btn_styleapply");
  gtk_widget_ref (btn_styleapply);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "btn_styleapply", btn_styleapply,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_styleapply);
  gtk_box_pack_start (GTK_BOX (vbx_styles), btn_styleapply, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_styleapply, -2, 24);


  hbx_styletitle = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbx_styletitle, "hbx_styletitle");
  gtk_widget_ref (hbx_styletitle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "hbx_styletitle", hbx_styletitle,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_set_border_width (GTK_CONTAINER (hbx_styletitle), 4);
  gtk_widget_show (hbx_styletitle);
  gtk_box_pack_start (GTK_BOX (vbx_splitstyle), hbx_styletitle, FALSE, FALSE, 1);

  lbl_curstyletitle = gtk_label_new (_("Current Style:  "));
  gtk_widget_set_name (lbl_curstyletitle, "lbl_curstyletitle");
  gtk_widget_ref (lbl_curstyletitle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_curstyletitle", lbl_curstyletitle,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_curstyletitle);
  gtk_box_pack_start (GTK_BOX (hbx_styletitle), lbl_curstyletitle, FALSE, FALSE, 1);
  gtk_misc_set_alignment (GTK_MISC (lbl_curstyletitle), 7.45058e-09, 0.5);  



  lbl_curstyle = gtk_label_new (_("no style loaded .."));
  gtk_widget_set_name( lbl_curstyle, "lbl_curstyle" );
  gtk_widget_ref (lbl_curstyle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_main), "lbl_curstyle", lbl_curstyle,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_curstyle);
  gtk_box_pack_start( GTK_BOX( hbx_styletitle ), lbl_curstyle, FALSE, FALSE, 2 );
  gtk_misc_set_alignment (GTK_MISC (lbl_curstyle), 7.45058e-09, 0.5);  


  gtk_signal_connect (GTK_OBJECT (trv_sedit), "tree_select_row",
                      GTK_SIGNAL_FUNC (on_trv_sedit_tree_select_row),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (trv_sedit), "tree_unselect_row",
                      GTK_SIGNAL_FUNC (on_trv_sedit_tree_unselect_row),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tbn_stylesave), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_stylesave_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tbn_stylesaveas), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_stylesaveas_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (tbn_styleload), "clicked",
                      GTK_SIGNAL_FUNC (on_tbn_styleload_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (btn_styleapply), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_styleapply_clicked),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (tbn_seditflux), "toggled",
                      GTK_SIGNAL_FUNC (on_tbn_seditflux_toggled),
                      NULL);


  /* set the style of the menu tree */
  gtk_ctree_set_line_style( (GtkCTree *) trv_sedit, GTK_CTREE_LINES_SOLID );
  gtk_ctree_set_expander_style( (GtkCTree *) trv_sedit, GTK_CTREE_EXPANDER_SQUARE );

  /* show the default style tree options */
  //show_defaults( trv_sedit, vbx_styles, wnd_main, 0 );


  return vbx_splitstyle;

}
