/*
 * styleframes.c
 * Style editor frame definitions
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include "styleframes.h"
#include "styles.h"

void
on_chk_fontglobal_toggled              (GtkCheckButton  *checkbox,
                                        gpointer         user_data)
{

  GtkWidget *wgt_txtfontmnutitle = lookup_widget( (GtkWidget *) checkbox, "txt_fontmnutitle" );
  GtkWidget *wgt_txtfontwindow   = lookup_widget( (GtkWidget *) checkbox, "txt_fontwindow" );
  GtkWidget *wgt_txtfonttoolbar  = lookup_widget( (GtkWidget *) checkbox, "txt_fonttoolbar" );
  GtkWidget *wgt_txtfontmnuframe  = lookup_widget( (GtkWidget *) checkbox, "txt_fontmnuframe" );
  GtkWidget *wgt_txtfonttab  = lookup_widget( (GtkWidget *) checkbox, "txt_fonttab" );

  GtkWidget *wgt_txtfontglobal   = lookup_widget( (GtkWidget *) checkbox, "txt_fontglobal" );

  if( gtk_toggle_button_get_active( (GtkToggleButton *) checkbox ))
  {

    gtk_widget_set_sensitive( wgt_txtfontmnutitle, FALSE );
    gtk_widget_set_sensitive( wgt_txtfontmnuframe, FALSE );
    gtk_widget_set_sensitive( wgt_txtfontwindow,   FALSE );
    gtk_widget_set_sensitive( wgt_txtfonttoolbar,  FALSE );
    gtk_widget_set_sensitive( wgt_txtfonttab,      FALSE );
    
    gtk_widget_set_sensitive( wgt_txtfontglobal,   TRUE );

    gtk_entry_set_text( (GtkEntry *) wgt_txtfontmnutitle, "" );
    gtk_entry_set_text( (GtkEntry *) wgt_txtfontmnuframe, "" );
    gtk_entry_set_text( (GtkEntry *) wgt_txtfontwindow, "" );
    gtk_entry_set_text( (GtkEntry *) wgt_txtfonttoolbar, "" );
    gtk_entry_set_text( (GtkEntry *) wgt_txtfonttab, "" );
    

  }
  else
  {

    gtk_widget_set_sensitive( wgt_txtfontmnutitle, TRUE );
    gtk_widget_set_sensitive( wgt_txtfontmnuframe, TRUE );
    gtk_widget_set_sensitive( wgt_txtfontwindow,   TRUE );
    gtk_widget_set_sensitive( wgt_txtfonttoolbar,  TRUE );
    gtk_widget_set_sensitive( wgt_txtfonttab,      TRUE );
    
    gtk_widget_set_sensitive( wgt_txtfontglobal,   FALSE );

    gtk_entry_set_text( (GtkEntry *) wgt_txtfontglobal, "" );

  }

}

GtkWidget *
make_no_node( GtkWidget *wnd_toadd )
{

  GtkWidget *lbl_nonode;
 
  lbl_nonode = gtk_label_new (_("There is no style associated with this node."));
  gtk_widget_set_name (lbl_nonode, "lbl_nonode");
  gtk_widget_ref (lbl_nonode);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_nonode", lbl_nonode,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (lbl_nonode, 232, 16);
  gtk_label_set_justify (GTK_LABEL (lbl_nonode), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (lbl_nonode), 0.5, 0.5);

  return lbl_nonode;
  
}

GtkWidget *
make_bulletstyle_frame( GtkWidget *wnd_toadd )
{
 
  GtkWidget *fra_stylebullet;
  GtkWidget *tbl_sbull;
  GtkWidget *lbl_sbulltype;
  GtkWidget *cbo_sbulltype;
  GtkWidget *txt_cbosbulltype;
  GtkWidget *lbl_sbullpos;
  GtkWidget *cbo_sbullpos;
  GtkWidget *txt_cbosbullpos;

  GList     *cbo_sbulltype_items = NULL;
  GList     *cbo_sbullpos_items  = NULL;


  fra_stylebullet = gtk_frame_new (_("Bullet Style"));
  gtk_widget_set_name (fra_stylebullet, "fra_stylebullet");
  gtk_widget_ref (fra_stylebullet);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "fra_stylebullet", fra_stylebullet,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (fra_stylebullet, 384, 240);
  gtk_frame_set_shadow_type (GTK_FRAME (fra_stylebullet), GTK_SHADOW_ETCHED_OUT);

  tbl_sbull = gtk_table_new (2, 2, FALSE);
  gtk_widget_set_name (tbl_sbull, "tbl_sbull");
  gtk_widget_ref (tbl_sbull);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "tbl_sbull", tbl_sbull,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbl_sbull);
  gtk_container_add (GTK_CONTAINER (fra_stylebullet), tbl_sbull);
  gtk_container_set_border_width (GTK_CONTAINER (tbl_sbull), 10);
  gtk_table_set_row_spacings (GTK_TABLE (tbl_sbull), 4);
  gtk_table_set_col_spacings (GTK_TABLE (tbl_sbull), 18);

  lbl_sbulltype = gtk_label_new (_("Bullet Type:"));
  gtk_widget_set_name (lbl_sbulltype, "lbl_sbulltype");
  gtk_widget_ref (lbl_sbulltype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_sbulltype", lbl_sbulltype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_sbulltype);
  gtk_table_attach (GTK_TABLE (tbl_sbull), lbl_sbulltype, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_sbulltype), 0, 0.5);


  cbo_sbulltype = gtk_combo_new ();
  gtk_widget_set_name (cbo_sbulltype, "cbo_sbulltype");
  gtk_widget_ref (cbo_sbulltype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "cbo_sbulltype", cbo_sbulltype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_sbulltype);
  gtk_table_attach (GTK_TABLE (tbl_sbull), cbo_sbulltype, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  cbo_sbulltype_items = g_list_append (cbo_sbulltype_items, _("Empty"));
  cbo_sbulltype_items = g_list_append (cbo_sbulltype_items, _("Triangle"));
  cbo_sbulltype_items = g_list_append (cbo_sbulltype_items, _("Square"));
  cbo_sbulltype_items = g_list_append (cbo_sbulltype_items, _("Diamond"));
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_sbulltype), cbo_sbulltype_items);
  g_list_free (cbo_sbulltype_items);
  gtk_widget_set_usize (cbo_sbulltype, 184, 20);

  txt_cbosbulltype = GTK_COMBO (cbo_sbulltype)->entry;
  gtk_widget_set_name (txt_cbosbulltype, "txt_cbosbulltype");
  gtk_widget_ref (txt_cbosbulltype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_cbosbulltype", txt_cbosbulltype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_editable( (GtkEntry *) txt_cbosbulltype, FALSE );
  gtk_widget_show (txt_cbosbulltype);



  lbl_sbullpos = gtk_label_new (_("Bullet Position:"));
  gtk_widget_set_name (lbl_sbullpos, "lbl_sbullpos");
  gtk_widget_ref (lbl_sbullpos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_sbullpos", lbl_sbullpos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_sbullpos);
  gtk_table_attach (GTK_TABLE (tbl_sbull), lbl_sbullpos, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_sbullpos), 0, 0.5);


  cbo_sbullpos = gtk_combo_new ();
  gtk_widget_set_name (cbo_sbullpos, "cbo_sbullpos");
  gtk_widget_ref (cbo_sbullpos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "cbo_sbullpos", cbo_sbullpos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_sbullpos);
  gtk_table_attach (GTK_TABLE (tbl_sbull), cbo_sbullpos, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  cbo_sbullpos_items = g_list_append (cbo_sbullpos_items, _("Right"));
  cbo_sbullpos_items = g_list_append (cbo_sbullpos_items, _("Left"));
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_sbullpos), cbo_sbullpos_items);
  g_list_free (cbo_sbullpos_items);
  gtk_widget_set_usize (cbo_sbullpos, 184, 20);

  txt_cbosbullpos = GTK_COMBO (cbo_sbullpos)->entry;
  gtk_widget_set_name (txt_cbosbullpos, "txt_cbosbullpos");
  gtk_widget_ref (txt_cbosbullpos);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_cbosbullpos", txt_cbosbullpos,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_editable( (GtkEntry *) txt_cbosbullpos, FALSE );
  gtk_widget_show (txt_cbosbullpos);

 
  return fra_stylebullet;
  
}

GtkWidget *
make_normstyle_frame( GtkWidget *wnd_toadd, gboolean show_disabled_item )
{
 
  GtkWidget *fra_menunorm;
  GtkWidget *tbl_snorm;

  GtkWidget *hbx_snormcolour;
  GtkWidget *lbl_snormcolour;
  GtkWidget *txt_snormcolour;
  GtkWidget *btn_snormcolour;
 
  GtkWidget *chk_snormgradient;

  GtkWidget *hbx_snormcolourto;  
  GtkWidget *lbl_snormcolourto;
  GtkWidget *txt_snormcolourto;
  GtkWidget *btn_snormcolourto;  
  
  GtkWidget *lbl_snormgtype;
  GtkWidget *cbo_snormgtype;
  GtkWidget *txt_cbosnormgtype;
  GtkWidget *chk_snorminterlaced;
  
  GtkWidget *lbl_snormbevel;
  GtkWidget *cbo_snormbevel;
  GtkWidget *txt_cbosnormbevel;

  GtkWidget *hbx_snormtextcolour;  
  GtkWidget *lbl_snormtextcolour;
  GtkWidget *txt_snormtextcolour;
  GtkWidget *btn_snormtextcolour;  

  GtkWidget *cbo_snormjustify;
  GtkWidget *txt_cbosnormjustify;
  GtkWidget *lbl_snormjustify;

  GtkWidget *hbx_snormdiscolour;
  GtkWidget *lbl_snormdiscolour;
  GtkWidget *txt_snormdiscolour;
  GtkWidget *btn_snormdiscolour;


  GtkWidget *scw_normframe;
  GtkWidget *vpt_normframe;

  GList     *cbo_snormgtype_items   = NULL;
  GList     *cbo_snormbevel_items   = NULL;
  GList     *cbo_snormjustify_items = NULL;
 
 

  fra_menunorm = gtk_frame_new (_("menu_norm"));
  gtk_widget_set_name (fra_menunorm, "fra_menunorm");
  gtk_widget_ref (fra_menunorm);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "fra_menunorm", fra_menunorm,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (fra_menunorm, 384, 240);
  gtk_frame_set_shadow_type (GTK_FRAME (fra_menunorm), GTK_SHADOW_ETCHED_OUT);


  if( show_disabled_item )
  {
	scw_normframe = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (scw_normframe);
	gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "scw_normframe", scw_normframe,
							(GtkDestroyNotify) gtk_widget_unref);
	gtk_scrolled_window_set_policy( scw_normframe, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC );
	gtk_widget_show (scw_normframe);
	gtk_container_add (GTK_CONTAINER (fra_menunorm), scw_normframe);
        
        vpt_normframe = gtk_viewport_new (NULL, NULL);
        gtk_widget_ref (vpt_normframe);
        gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "vpt_normframe", vpt_normframe,
                            (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (vpt_normframe);
        gtk_container_add (GTK_CONTAINER (scw_normframe), vpt_normframe);
        gtk_viewport_set_shadow_type (GTK_VIEWPORT (vpt_normframe), GTK_SHADOW_NONE);        
        
        
	  
  }

  tbl_snorm = gtk_table_new (8, 2, FALSE);
  gtk_widget_set_name (tbl_snorm, "tbl_snorm");
  gtk_widget_ref (tbl_snorm);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "tbl_snorm", tbl_snorm,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbl_snorm);
  gtk_widget_set_usize (tbl_snorm, 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (tbl_snorm), 10);
  gtk_table_set_row_spacings (GTK_TABLE (tbl_snorm), 4);
  gtk_table_set_col_spacings (GTK_TABLE (tbl_snorm), 18);

  if( show_disabled_item )
	//gtk_scrolled_window_add_with_viewport((GtkScrolledWindow *) scw_normframe, tbl_snorm);	
        gtk_container_add (GTK_CONTAINER (vpt_normframe), tbl_snorm);
  else
	gtk_container_add (GTK_CONTAINER (fra_menunorm), tbl_snorm);
  
  
  lbl_snormcolour = gtk_label_new (_("Colour:"));
  gtk_widget_set_name (lbl_snormcolour, "lbl_snormcolour");
  gtk_widget_ref (lbl_snormcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_snormcolour", lbl_snormcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_snormcolour);
  gtk_table_attach (GTK_TABLE (tbl_snorm), lbl_snormcolour, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (lbl_snormcolour, 72, 16);
  gtk_label_set_justify (GTK_LABEL (lbl_snormcolour), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lbl_snormcolour), 7.45058e-09, 0.5);

  hbx_snormcolour = gtk_hbox_new (FALSE, 6);
  gtk_widget_set_name (hbx_snormcolour, "hbx_snormcolour");
  gtk_widget_ref (hbx_snormcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "hbx_snormcolour", hbx_snormcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbx_snormcolour);
  gtk_table_attach (GTK_TABLE (tbl_snorm), hbx_snormcolour, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  txt_snormcolour = gtk_entry_new ();
  gtk_widget_set_name (txt_snormcolour, "txt_snormcolour");
  gtk_widget_ref (txt_snormcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_snormcolour", txt_snormcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_snormcolour);
  gtk_box_pack_start (GTK_BOX (hbx_snormcolour), txt_snormcolour, TRUE, TRUE, 0);
  //gtk_widget_set_usize (txt_snormcolour, 184, 20);

  btn_snormcolour = gtk_button_new_with_label (_(".."));
  gtk_widget_set_name (btn_snormcolour, "btn_snormcolour");
  gtk_widget_ref (btn_snormcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "btn_snormcolour", btn_snormcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_snormcolour);
  gtk_box_pack_start (GTK_BOX (hbx_snormcolour), btn_snormcolour, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_snormcolour, 21, 21);


  chk_snormgradient = gtk_check_button_new_with_label (_("Gradient"));
  gtk_widget_set_name (chk_snormgradient, "chk_snormgradient");
  gtk_widget_ref (chk_snormgradient);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "chk_snormgradient", chk_snormgradient,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (chk_snormgradient);
  gtk_table_attach (GTK_TABLE (tbl_snorm), chk_snormgradient, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  lbl_snormcolourto = gtk_label_new (_("Colour To:"));
  gtk_widget_set_name (lbl_snormcolourto, "lbl_snormcolourto");
  gtk_widget_ref (lbl_snormcolourto);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_snormcolourto", lbl_snormcolourto,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_snormcolourto);
  gtk_table_attach (GTK_TABLE (tbl_snorm), lbl_snormcolourto, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (lbl_snormcolourto, 83, 16);
  gtk_label_set_justify (GTK_LABEL (lbl_snormcolourto), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lbl_snormcolourto), 7.45058e-09, 0.5);

  hbx_snormcolourto = gtk_hbox_new (FALSE, 6);
  gtk_widget_set_name (hbx_snormcolourto, "hbx_snormcolourto");
  gtk_widget_ref (hbx_snormcolourto);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "hbx_snormcolourto", hbx_snormcolourto,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbx_snormcolourto);
  gtk_table_attach (GTK_TABLE (tbl_snorm), hbx_snormcolourto, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  txt_snormcolourto = gtk_entry_new ();
  gtk_widget_set_name (txt_snormcolourto, "txt_snormcolourto");
  gtk_widget_ref (txt_snormcolourto);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_snormcolourto", txt_snormcolourto,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_snormcolourto);
  gtk_box_pack_start (GTK_BOX (hbx_snormcolourto), txt_snormcolourto, TRUE, TRUE, 0);
  gtk_widget_set_usize (txt_snormcolourto, 184, 20);

  btn_snormcolourto = gtk_button_new_with_label (_(".."));
  gtk_widget_set_name (btn_snormcolourto, "btn_snormcolourto");
  gtk_widget_ref (btn_snormcolourto);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "btn_snormcolourto", btn_snormcolourto,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_snormcolourto);
  gtk_box_pack_start (GTK_BOX (hbx_snormcolourto), btn_snormcolourto, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_snormcolourto, 21, 21);


  lbl_snormgtype = gtk_label_new (_("Gradient Type:"));
  gtk_widget_set_name (lbl_snormgtype, "lbl_snormgtype");
  gtk_widget_ref (lbl_snormgtype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_snormgtype", lbl_snormgtype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_snormgtype);
  gtk_table_attach (GTK_TABLE (tbl_snorm), lbl_snormgtype, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (lbl_snormgtype, 83, 16);
  gtk_label_set_justify (GTK_LABEL (lbl_snormgtype), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lbl_snormgtype), 7.45058e-09, 0.5);

  cbo_snormgtype = gtk_combo_new ();
  gtk_widget_set_name (cbo_snormgtype, "cbo_snormgtype");
  gtk_widget_ref (cbo_snormgtype);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "cbo_snormgtype", cbo_snormgtype,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_snormgtype);
  gtk_table_attach (GTK_TABLE (tbl_snorm), cbo_snormgtype, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  cbo_snormgtype_items = g_list_append (cbo_snormgtype_items, _(""));
  cbo_snormgtype_items = g_list_append (cbo_snormgtype_items, _("Horizontal"));
  cbo_snormgtype_items = g_list_append (cbo_snormgtype_items, _("Vertical"));
  cbo_snormgtype_items = g_list_append (cbo_snormgtype_items, _("Diagonal"));
  cbo_snormgtype_items = g_list_append (cbo_snormgtype_items, _("CrossDiagonal"));
  cbo_snormgtype_items = g_list_append (cbo_snormgtype_items, _("PipeCross"));  
  cbo_snormgtype_items = g_list_append (cbo_snormgtype_items, _("Elliptic"));  
  cbo_snormgtype_items = g_list_append (cbo_snormgtype_items, _("Rectangle"));
  cbo_snormgtype_items = g_list_append (cbo_snormgtype_items, _("Pyramid"));    
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_snormgtype), cbo_snormgtype_items);
  g_list_free (cbo_snormgtype_items);

  gtk_widget_set_usize (cbo_snormgtype, 184, 20);

  txt_cbosnormgtype = GTK_COMBO( cbo_snormgtype )->entry;
  gtk_widget_set_name( txt_cbosnormgtype, "txt_cbosnormgtype" );
  gtk_widget_ref( txt_cbosnormgtype );
  gtk_object_set_data_full( GTK_OBJECT( wnd_toadd ), "txt_cbosnormgtype", txt_cbosnormgtype,
                            (GtkDestroyNotify) gtk_widget_unref );
  gtk_entry_set_editable( GTK_ENTRY( txt_cbosnormgtype ), FALSE );
  gtk_entry_set_text( (GtkEntry *) txt_cbosnormgtype, "" );
  gtk_widget_show( txt_cbosnormgtype );

  chk_snorminterlaced = gtk_check_button_new_with_label (_("Interlaced"));
  gtk_widget_set_name (chk_snorminterlaced, "chk_snorminterlaced");
  gtk_widget_ref (chk_snorminterlaced);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "chk_snorminterlaced", chk_snorminterlaced,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (chk_snorminterlaced);
  gtk_table_attach (GTK_TABLE (tbl_snorm), chk_snorminterlaced, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  lbl_snormbevel = gtk_label_new (_("Item Bevel:"));
  gtk_widget_set_name (lbl_snormbevel, "lbl_snormbevel");
  gtk_widget_ref (lbl_snormbevel);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_snormbevel", lbl_snormbevel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_snormbevel);
  gtk_table_attach (GTK_TABLE (tbl_snorm), lbl_snormbevel, 0, 1, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (lbl_snormbevel, 72, 16);
  gtk_label_set_justify (GTK_LABEL (lbl_snormbevel), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lbl_snormbevel), 7.45058e-09, 0.5);

  lbl_snormtextcolour = gtk_label_new (_("Text Colour:"));
  gtk_widget_set_name (lbl_snormtextcolour, "lbl_snormtextcolour");
  gtk_widget_ref (lbl_snormtextcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_snormtextcolour", lbl_snormtextcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_snormtextcolour);
  gtk_table_attach (GTK_TABLE (tbl_snorm), lbl_snormtextcolour, 0, 1, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (lbl_snormtextcolour, 88, 16);
  gtk_label_set_justify (GTK_LABEL (lbl_snormtextcolour), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lbl_snormtextcolour), 7.45058e-09, 0.5);

  cbo_snormbevel = gtk_combo_new ();
  gtk_widget_set_name (cbo_snormbevel, "cbo_snormbevel");
  gtk_widget_ref (cbo_snormbevel);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "cbo_snormbevel", cbo_snormbevel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_snormbevel);
  gtk_table_attach (GTK_TABLE (tbl_snorm), cbo_snormbevel, 1, 2, 5, 6,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  cbo_snormbevel_items = g_list_append (cbo_snormbevel_items, _("Flat"));
  cbo_snormbevel_items = g_list_append (cbo_snormbevel_items, _("Raised"));
  cbo_snormbevel_items = g_list_append (cbo_snormbevel_items, _("Sunken"));
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_snormbevel), cbo_snormbevel_items);
  g_list_free (cbo_snormbevel_items);

  gtk_widget_set_usize (cbo_snormbevel, 184, 20);

  txt_cbosnormbevel = GTK_COMBO (cbo_snormbevel)->entry;
  gtk_widget_set_name (txt_cbosnormbevel, "txt_cbosnormbevel");
  gtk_widget_ref (txt_cbosnormbevel);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_cbosnormbevel", txt_cbosnormbevel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_editable( GTK_ENTRY( txt_cbosnormbevel ), FALSE );
  gtk_entry_set_text( (GtkEntry *) txt_cbosnormbevel, "" );
  gtk_widget_show (txt_cbosnormbevel);



  hbx_snormtextcolour = gtk_hbox_new (FALSE, 6);
  gtk_widget_set_name (hbx_snormtextcolour, "hbx_snormtextcolour");
  gtk_widget_ref (hbx_snormtextcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "hbx_snormtextcolour", hbx_snormtextcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbx_snormtextcolour);
  gtk_table_attach (GTK_TABLE (tbl_snorm), hbx_snormtextcolour, 1, 2, 6, 7,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  txt_snormtextcolour = gtk_entry_new ();
  gtk_widget_set_name (txt_snormtextcolour, "txt_snormtextcolour");
  gtk_widget_ref (txt_snormtextcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_snormtextcolour", txt_snormtextcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_snormtextcolour);
  gtk_box_pack_start (GTK_BOX (hbx_snormtextcolour), txt_snormtextcolour, TRUE, TRUE, 0);
  gtk_widget_set_usize (txt_snormtextcolour, 184, 20);

  btn_snormtextcolour = gtk_button_new_with_label (_(".."));
  gtk_widget_set_name (btn_snormtextcolour, "btn_snormtextcolour");
  gtk_widget_ref (btn_snormtextcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "btn_snormtextcolour", btn_snormtextcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_snormtextcolour);
  gtk_box_pack_start (GTK_BOX (hbx_snormtextcolour), btn_snormtextcolour, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_snormtextcolour, 21, 21);


  lbl_snormjustify = gtk_label_new (_("Item Justify:"));
  gtk_widget_set_name (lbl_snormjustify, "lbl_snormjustify");
  gtk_widget_ref (lbl_snormjustify);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_snormjustify", lbl_snormjustify,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_snormjustify);
  gtk_table_attach (GTK_TABLE (tbl_snorm), lbl_snormjustify, 0, 1, 7, 8,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (lbl_snormbevel, 72, 16);
  gtk_label_set_justify (GTK_LABEL (lbl_snormjustify), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lbl_snormjustify), 7.45058e-09, 0.5);



  cbo_snormjustify = gtk_combo_new ();
  gtk_widget_set_name (cbo_snormjustify, "cbo_snormjustify");
  gtk_widget_ref (cbo_snormjustify);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "cbo_snormjustify", cbo_snormjustify,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_snormjustify);
  gtk_table_attach (GTK_TABLE (tbl_snorm), cbo_snormjustify, 1, 2, 7, 8,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  cbo_snormjustify_items = g_list_append (cbo_snormjustify_items, _("Left"));
  cbo_snormjustify_items = g_list_append (cbo_snormjustify_items, _("Center"));
  cbo_snormjustify_items = g_list_append (cbo_snormjustify_items, _("Right"));
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_snormjustify), cbo_snormjustify_items);
  g_list_free (cbo_snormjustify_items);

  gtk_widget_set_usize (cbo_snormjustify, 184, 20);

  txt_cbosnormjustify = GTK_COMBO (cbo_snormjustify)->entry;
  gtk_widget_set_name (txt_cbosnormjustify, "txt_cbosnormjustify");
  gtk_widget_ref (txt_cbosnormjustify);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_cbosnormjustify", txt_cbosnormjustify,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_editable( GTK_ENTRY( txt_cbosnormjustify ), FALSE );
  gtk_entry_set_text( (GtkEntry *) txt_cbosnormjustify, "" );
  gtk_widget_show (txt_cbosnormjustify);



  lbl_snormdiscolour = gtk_label_new (_("Disabled Colour:"));
  gtk_widget_set_name (lbl_snormdiscolour, "lbl_snormdiscolour");
  gtk_widget_ref (lbl_snormdiscolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_snormdiscolour", lbl_snormdiscolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_snormdiscolour);
  gtk_table_attach (GTK_TABLE (tbl_snorm), lbl_snormdiscolour, 0, 1, 8, 9,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (lbl_snormdiscolour, 72, 16);
  gtk_label_set_justify (GTK_LABEL (lbl_snormdiscolour), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lbl_snormdiscolour), 7.45058e-09, 0.5);

  hbx_snormdiscolour = gtk_hbox_new (FALSE, 6);
  gtk_widget_set_name (hbx_snormdiscolour, "hbx_snormdiscolour");
  gtk_widget_ref (hbx_snormdiscolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "hbx_snormdiscolour", hbx_snormdiscolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbx_snormdiscolour);
  gtk_table_attach (GTK_TABLE (tbl_snorm), hbx_snormdiscolour, 1, 2, 8, 9,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  txt_snormdiscolour = gtk_entry_new ();
  gtk_widget_set_name (txt_snormdiscolour, "txt_snormdiscolour");
  gtk_widget_ref (txt_snormdiscolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_snormdiscolour", txt_snormdiscolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_snormdiscolour);
  gtk_box_pack_start (GTK_BOX (hbx_snormdiscolour), txt_snormdiscolour, TRUE, TRUE, 0);
  //gtk_widget_set_usize (txt_snormcolour, 184, 20);

  btn_snormdiscolour = gtk_button_new_with_label (_(".."));
  gtk_widget_set_name (btn_snormdiscolour, "btn_snormdiscolour");
  gtk_widget_ref (btn_snormdiscolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "btn_snormdiscolour", btn_snormdiscolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_snormdiscolour);
  gtk_box_pack_start (GTK_BOX (hbx_snormdiscolour), btn_snormdiscolour, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_snormdiscolour, 21, 21);



  gtk_signal_connect (GTK_OBJECT (btn_snormcolour), "clicked",
                      GTK_SIGNAL_FUNC( on_style_colour_btn_clicked ),
                      (gpointer) txt_snormcolour );

  gtk_signal_connect (GTK_OBJECT (btn_snormcolourto), "clicked",
                      GTK_SIGNAL_FUNC( on_style_colour_btn_clicked ),
                      (gpointer) txt_snormcolourto );

  gtk_signal_connect (GTK_OBJECT (btn_snormtextcolour), "clicked",
                      GTK_SIGNAL_FUNC( on_style_colour_btn_clicked ),
                      (gpointer) txt_snormtextcolour );

  gtk_signal_connect (GTK_OBJECT (btn_snormdiscolour), "clicked",
                      GTK_SIGNAL_FUNC( on_style_colour_btn_clicked ),
                      (gpointer) txt_snormdiscolour );

  return fra_menunorm;

}


GtkWidget *
make_justonly_frame( GtkWidget *wnd_toadd )
{
 
  GtkWidget *fra_justonly;
  GtkWidget *tbl_justonly;
  GtkWidget *lbl_justonly;
  GtkWidget *cbo_justonly;
  GtkWidget *txt_justonly;

  GList     *cbo_justonly_items = NULL;


  fra_justonly = gtk_frame_new (_("Window"));
  gtk_widget_set_name (fra_justonly, "fra_justonly");
  gtk_widget_ref (fra_justonly);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "fra_justonly", fra_justonly,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (fra_justonly, 384, 240);
  gtk_frame_set_shadow_type (GTK_FRAME (fra_justonly), GTK_SHADOW_ETCHED_OUT);

  tbl_justonly = gtk_table_new (1, 2, FALSE);
  gtk_widget_set_name (tbl_justonly, "tbl_justonly");
  gtk_widget_ref (tbl_justonly);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "tbl_justonly", tbl_justonly,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbl_justonly);
  gtk_container_add (GTK_CONTAINER (fra_justonly), tbl_justonly);
  gtk_container_set_border_width (GTK_CONTAINER (tbl_justonly), 10);
  gtk_table_set_row_spacings (GTK_TABLE (tbl_justonly), 4);
  gtk_table_set_col_spacings (GTK_TABLE (tbl_justonly), 18);

  lbl_justonly = gtk_label_new (_("Justification:"));
  gtk_widget_set_name (lbl_justonly, "lbl_justonly");
  gtk_widget_ref (lbl_justonly);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_justonly", lbl_justonly,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_justonly);
  gtk_table_attach (GTK_TABLE (tbl_justonly), lbl_justonly, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_justonly), 0, 0.5);


  cbo_justonly = gtk_combo_new ();
  gtk_widget_set_name (cbo_justonly, "cbo_justonly");
  gtk_widget_ref (cbo_justonly);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "cbo_justonly", cbo_justonly,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_justonly);
  gtk_table_attach (GTK_TABLE (tbl_justonly), cbo_justonly, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  cbo_justonly_items = g_list_append (cbo_justonly_items, _("Left"));
  cbo_justonly_items = g_list_append (cbo_justonly_items, _("Center"));
  cbo_justonly_items = g_list_append (cbo_justonly_items, _("Right"));
  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_justonly), cbo_justonly_items);
  g_list_free (cbo_justonly_items);
  gtk_widget_set_usize (cbo_justonly, 184, 20);

  txt_justonly = GTK_COMBO (cbo_justonly)->entry;
  gtk_widget_set_name (txt_justonly, "txt_justonly");
  gtk_widget_ref (txt_justonly);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_justonly", txt_justonly,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_editable( (GtkEntry *) txt_justonly, FALSE );
  gtk_widget_show (txt_justonly);

 
  return fra_justonly;
  
}


GtkWidget *
make_fonts_frame( GtkWidget *wnd_toadd )
{
 
  GtkWidget *fra_fonts;
  GtkWidget *tbl_fonts;
  GtkWidget *lbl_fontmnutitle;
  GtkWidget *txt_fontmnutitle;
  GtkWidget *lbl_fontmnuframe;
  GtkWidget *txt_fontmnuframe;
  GtkWidget *lbl_fontwindow;
  GtkWidget *txt_fontwindow;
  GtkWidget *lbl_fonttoolbar;
  GtkWidget *txt_fonttoolbar;
  GtkWidget *lbl_fonttab;
  GtkWidget *txt_fonttab;

  GtkWidget *wgt_tbnseditflux;
  GtkWidget *chk_fontglobal;

  GtkWidget *lbl_fontglobal;
  GtkWidget *txt_fontglobal;

  wgt_tbnseditflux = lookup_widget( wnd_toadd, "tbn_seditflux" );


  fra_fonts = gtk_frame_new (_("Fonts"));
  gtk_widget_set_name (fra_fonts, "fra_fonts");
  gtk_widget_ref (fra_fonts);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "fra_fonts", fra_fonts,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (fra_fonts, 384, 240);
  gtk_frame_set_shadow_type (GTK_FRAME (fra_fonts), GTK_SHADOW_ETCHED_OUT);


  tbl_fonts = gtk_table_new (1, 2, FALSE);
  gtk_widget_set_name (tbl_fonts, "tbl_fonts");
  gtk_widget_ref (tbl_fonts);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "tbl_fonts", tbl_fonts,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbl_fonts);
  gtk_container_add (GTK_CONTAINER (fra_fonts), tbl_fonts);
  gtk_container_set_border_width (GTK_CONTAINER (tbl_fonts), 10);
  gtk_table_set_row_spacings (GTK_TABLE (tbl_fonts), 4);
  gtk_table_set_col_spacings (GTK_TABLE (tbl_fonts), 18);


  /* menu title widgets */
  lbl_fontmnutitle = gtk_label_new (_("Menu title font:"));
  gtk_widget_set_name (lbl_fontmnutitle, "lbl_fontmnutitle");
  gtk_widget_ref (lbl_fontmnutitle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_fontmnutitle", 
    lbl_fontmnutitle, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_fontmnutitle);
  gtk_table_attach (GTK_TABLE (tbl_fonts), lbl_fontmnutitle, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_fontmnutitle), 0, 0.5);


  txt_fontmnutitle = gtk_entry_new ();
  gtk_widget_set_name (txt_fontmnutitle, "txt_fontmnutitle");
  gtk_widget_ref (txt_fontmnutitle);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_fontmnutitle", 
    txt_fontmnutitle, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_fontmnutitle);

  gtk_table_attach (GTK_TABLE (tbl_fonts), txt_fontmnutitle, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_fontmnutitle, 184, 20);


  /* menu frame widgets */
  lbl_fontmnuframe = gtk_label_new (_("Menu frame font:"));
  gtk_widget_set_name (lbl_fontmnuframe, "lbl_fontmnuframe");
  gtk_widget_ref (lbl_fontmnuframe);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_fontmnuframe", 
    lbl_fontmnuframe, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_fontmnuframe);
  gtk_table_attach (GTK_TABLE (tbl_fonts), lbl_fontmnuframe, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_fontmnuframe), 0, 0.5);


  txt_fontmnuframe = gtk_entry_new ();
  gtk_widget_set_name (txt_fontmnuframe, "txt_fontmnuframe");
  gtk_widget_ref (txt_fontmnuframe);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_fontmnuframe", 
    txt_fontmnuframe, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_fontmnuframe);

  gtk_table_attach (GTK_TABLE (tbl_fonts), txt_fontmnuframe, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_fontmnuframe, 184, 20);


  /* window widgets */
  lbl_fontwindow = gtk_label_new (_("Window font:"));
  gtk_widget_set_name (lbl_fontwindow, "lbl_fontwindow");
  gtk_widget_ref (lbl_fontwindow);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_fontwindow", 
    lbl_fontwindow, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_fontwindow);
  gtk_table_attach (GTK_TABLE (tbl_fonts), lbl_fontwindow, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_fontwindow), 0, 0.5);


  txt_fontwindow = gtk_entry_new ();
  gtk_widget_set_name (txt_fontwindow, "txt_fontwindow");
  gtk_widget_ref (txt_fontwindow);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_fontwindow", 
    txt_fontwindow, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_fontwindow);

  gtk_table_attach (GTK_TABLE (tbl_fonts), txt_fontwindow, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_fontwindow, 184, 20);
 

  /* menu frame widgets */
  lbl_fonttoolbar = gtk_label_new (_("Toolbar font:"));
  gtk_widget_set_name (lbl_fonttoolbar, "lbl_fonttoolbar");
  gtk_widget_ref (lbl_fonttoolbar);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_fonttoolbar", 
    lbl_fonttoolbar, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_fonttoolbar);
  gtk_table_attach (GTK_TABLE (tbl_fonts), lbl_fonttoolbar, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_fonttoolbar), 0, 0.5);


  txt_fonttoolbar = gtk_entry_new ();
  gtk_widget_set_name (txt_fonttoolbar, "txt_fonttoolbar");
  gtk_widget_ref (txt_fonttoolbar);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_fonttoolbar", 
    txt_fonttoolbar, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_fonttoolbar);

  gtk_table_attach (GTK_TABLE (tbl_fonts), txt_fonttoolbar, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_fonttoolbar, 184, 20);


  /* fluxbox tab font style */
  lbl_fonttab = gtk_label_new (_("Window tab font:"));
  gtk_widget_set_name (lbl_fonttab, "lbl_fonttab");
  gtk_widget_ref (lbl_fonttab);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_fonttab", 
    lbl_fonttab, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_fonttab);
  gtk_table_attach (GTK_TABLE (tbl_fonts), lbl_fonttab, 0, 1, 4, 5,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_fonttab), 0, 0.5);

  txt_fonttab = gtk_entry_new ();
  gtk_widget_set_name (txt_fonttab, "txt_fonttab");
  gtk_widget_ref (txt_fonttab);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_fonttab", 
    txt_fonttab, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_fonttab);

  gtk_table_attach (GTK_TABLE (tbl_fonts), txt_fonttab, 1, 2, 4, 5,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_fonttab, 184, 20);


  /* global widgets */
  chk_fontglobal = gtk_check_button_new_with_label (_("Set Fonts Globally?"));
  gtk_widget_set_name (chk_fontglobal, "chk_fontglobal");
  gtk_widget_ref (chk_fontglobal);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "chk_fontglobal", chk_fontglobal,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (chk_fontglobal);
  gtk_table_attach (GTK_TABLE (tbl_fonts), chk_fontglobal, 0, 1, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 1, 0);
  //gtk_widget_set_usize (chk_fontglobal, 184, 16);
  


  lbl_fontglobal = gtk_label_new (_("Global font:"));
  gtk_widget_set_name (lbl_fontglobal, "lbl_fontglobal");
  gtk_widget_ref (lbl_fontglobal);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_fontglobal", 
    lbl_fontglobal, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_fontglobal);
  gtk_table_attach (GTK_TABLE (tbl_fonts), lbl_fontglobal, 0, 1, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_fontglobal), 0, 0.5);



  txt_fontglobal = gtk_entry_new ();
  gtk_widget_set_name (txt_fontglobal, "txt_fontglobal");
  gtk_widget_ref (txt_fontglobal);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_fontglobal", 
    txt_fontglobal, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_fontglobal);

  gtk_table_attach (GTK_TABLE (tbl_fonts), txt_fontglobal, 1, 2, 6, 7,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_fontglobal, 184, 20);


  gtk_widget_set_sensitive( txt_fontglobal, FALSE );


  gtk_signal_connect (GTK_OBJECT (chk_fontglobal), "toggled",
                      GTK_SIGNAL_FUNC (on_chk_fontglobal_toggled),
                      NULL);


  return fra_fonts;
  
}


GtkWidget *
make_frmwindow_frame( GtkWidget *wnd_toadd )
{
 
  GtkWidget *fra_wndframe;
  GtkWidget *tbl_wndframe;
  GtkWidget *lbl_frmfocuscol;
  GtkWidget *txt_frmfocuscol;
  GtkWidget *lbl_frmufocuscol;
  GtkWidget *txt_frmufocuscol;


  fra_wndframe = gtk_frame_new (_("Window Frame"));
  gtk_widget_set_name (fra_wndframe, "fra_wndframe");
  gtk_widget_ref (fra_wndframe);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "fra_wndframe", fra_wndframe,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (fra_wndframe, 384, 240);
  gtk_frame_set_shadow_type (GTK_FRAME (fra_wndframe), GTK_SHADOW_ETCHED_OUT);


  tbl_wndframe = gtk_table_new (1, 2, FALSE);
  gtk_widget_set_name (tbl_wndframe, "tbl_wndframe");
  gtk_widget_ref (tbl_wndframe);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "tbl_wndframe", tbl_wndframe,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbl_wndframe);
  gtk_container_add (GTK_CONTAINER (fra_wndframe), tbl_wndframe);
  gtk_container_set_border_width (GTK_CONTAINER (tbl_wndframe), 10);
  gtk_table_set_row_spacings (GTK_TABLE (tbl_wndframe), 4);
  gtk_table_set_col_spacings (GTK_TABLE (tbl_wndframe), 18);


  /* frame focused colour widgets */

  lbl_frmfocuscol = gtk_label_new (_("Focused Colour:"));
  gtk_widget_set_name (lbl_frmfocuscol, "lbl_frmfocuscol");
  gtk_widget_ref (lbl_frmfocuscol);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_frmfocuscol", 
    lbl_frmfocuscol, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_frmfocuscol);
  gtk_table_attach (GTK_TABLE (tbl_wndframe), lbl_frmfocuscol, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_frmfocuscol), 0, 0.5);


  txt_frmfocuscol = gtk_entry_new ();
  gtk_widget_set_name (txt_frmfocuscol, "txt_frmfocuscol");
  gtk_widget_ref (txt_frmfocuscol);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_frmfocuscol", 
    txt_frmfocuscol, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_frmfocuscol);

  gtk_table_attach (GTK_TABLE (tbl_wndframe), txt_frmfocuscol, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_frmfocuscol, 184, 20);

  /* frame unfocused colour widgets */

  lbl_frmufocuscol = gtk_label_new (_("Unfocused Colour:"));
  gtk_widget_set_name (lbl_frmufocuscol, "lbl_frmufocuscol");
  gtk_widget_ref (lbl_frmufocuscol);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_frmufocuscol", 
    lbl_frmufocuscol, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_frmufocuscol);
  gtk_table_attach (GTK_TABLE (tbl_wndframe), lbl_frmufocuscol, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_frmufocuscol), 0, 0.5);


  txt_frmufocuscol = gtk_entry_new ();
  gtk_widget_set_name (txt_frmufocuscol, "txt_frmufocuscol");
  gtk_widget_ref (txt_frmufocuscol);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_frmufocuscol", 
    txt_frmufocuscol, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_frmufocuscol);

  gtk_table_attach (GTK_TABLE (tbl_wndframe), txt_frmufocuscol, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_frmufocuscol, 184, 20);



  return fra_wndframe;
  
}


GtkWidget *
make_frmtab_frame( GtkWidget *wnd_toadd )
{
 
  GtkWidget *fra_tabframe;
  GtkWidget *tbl_tabframe;
  GtkWidget *lbl_tabjustify;
  GtkWidget *cbo_tabjustify;
  GtkWidget *txt_tabjustify;
  GtkWidget *lbl_frmtabborderwidth;
  GtkWidget *txt_frmtabborderwidth;

  GtkWidget *hbx_frmtabbordercolour;
  GtkWidget *lbl_frmtabbordercolour;
  GtkWidget *txt_frmtabbordercolour;
  GtkWidget *btn_frmtabbordercolour;
  
  GList     *cbo_tabjustify_items = NULL;


  fra_tabframe = gtk_frame_new (_("Window Tab"));
  gtk_widget_set_name (fra_tabframe, "fra_tabframe");
  gtk_widget_ref (fra_tabframe);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "fra_tabframe", fra_tabframe,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (fra_tabframe, 384, 240);
  gtk_frame_set_shadow_type (GTK_FRAME (fra_tabframe), GTK_SHADOW_ETCHED_OUT);


  tbl_tabframe = gtk_table_new (1, 2, FALSE);
  gtk_widget_set_name (tbl_tabframe, "tbl_tabframe");
  gtk_widget_ref (tbl_tabframe);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "tbl_tabframe", tbl_tabframe,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbl_tabframe);
  gtk_container_add (GTK_CONTAINER (fra_tabframe), tbl_tabframe);
  gtk_container_set_border_width (GTK_CONTAINER (tbl_tabframe), 10);
  gtk_table_set_row_spacings (GTK_TABLE (tbl_tabframe), 4);
  gtk_table_set_col_spacings (GTK_TABLE (tbl_tabframe), 18);


  /* show tab justification */

  lbl_tabjustify = gtk_label_new (_("Justification:"));
  gtk_widget_set_name (lbl_tabjustify, "lbl_tabjustify");
  gtk_widget_ref (lbl_tabjustify);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_tabjustify", lbl_tabjustify,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_tabjustify);
  gtk_table_attach (GTK_TABLE (tbl_tabframe), lbl_tabjustify, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_tabjustify), 0, 0.5);


  cbo_tabjustify = gtk_combo_new ();
  gtk_widget_set_name (cbo_tabjustify, "cbo_tabjustify");
  gtk_widget_ref (cbo_tabjustify);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "cbo_tabjustify", cbo_tabjustify,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbo_tabjustify);
  gtk_table_attach (GTK_TABLE (tbl_tabframe), cbo_tabjustify, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  
  cbo_tabjustify_items = g_list_append (cbo_tabjustify_items, _("Left"));
  cbo_tabjustify_items = g_list_append (cbo_tabjustify_items, _("Center"));
  cbo_tabjustify_items = g_list_append (cbo_tabjustify_items, _("Right"));


  gtk_combo_set_popdown_strings (GTK_COMBO (cbo_tabjustify), cbo_tabjustify_items);
  g_list_free (cbo_tabjustify_items);
  gtk_widget_set_usize (cbo_tabjustify, 184, 20);

  txt_tabjustify = GTK_COMBO (cbo_tabjustify)->entry;
  gtk_widget_set_name (txt_tabjustify, "txt_tabjustify");
  gtk_widget_ref (txt_tabjustify);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_tabjustify", txt_tabjustify,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_editable( (GtkEntry *) txt_tabjustify, FALSE );
  gtk_widget_show (txt_tabjustify);


  /* tab border width widgets */

  lbl_frmtabborderwidth = gtk_label_new (_("Border Width:"));
  gtk_widget_set_name (lbl_frmtabborderwidth, "lbl_frmtabborderwidth");
  gtk_widget_ref (lbl_frmtabborderwidth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_frmtabborderwidth", 
    lbl_frmtabborderwidth, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_frmtabborderwidth);
  gtk_table_attach (GTK_TABLE (tbl_tabframe), lbl_frmtabborderwidth, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_frmtabborderwidth), 0, 0.5);


  txt_frmtabborderwidth = gtk_entry_new ();
  gtk_widget_set_name (txt_frmtabborderwidth, "txt_frmtabborderwidth");
  gtk_widget_ref (txt_frmtabborderwidth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_frmtabborderwidth", 
    txt_frmtabborderwidth, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_frmtabborderwidth);

  gtk_table_attach (GTK_TABLE (tbl_tabframe), txt_frmtabborderwidth, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (txt_frmtabborderwidth, 184, 20);


  /* tab border colour widgets */

  lbl_frmtabbordercolour = gtk_label_new (_("Border Colour:"));
  gtk_widget_set_name (lbl_frmtabbordercolour, "lbl_frmtabbordercolour");
  gtk_widget_ref (lbl_frmtabbordercolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_frmtabbordercolour", 
    lbl_frmtabbordercolour, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_frmtabbordercolour);
  gtk_table_attach (GTK_TABLE (tbl_tabframe), lbl_frmtabbordercolour, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_frmtabbordercolour), 0, 0.5);

  hbx_frmtabbordercolour = gtk_hbox_new (FALSE, 6);
  gtk_widget_set_name (hbx_frmtabbordercolour, "hbx_frmtabbordercolour");
  gtk_widget_ref (hbx_frmtabbordercolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "hbx_frmtabbordercolour", hbx_frmtabbordercolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbx_frmtabbordercolour);
  gtk_table_attach (GTK_TABLE (tbl_tabframe), hbx_frmtabbordercolour, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  txt_frmtabbordercolour = gtk_entry_new ();
  gtk_widget_set_name (txt_frmtabbordercolour, "txt_frmtabbordercolour");
  gtk_widget_ref (txt_frmtabbordercolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_frmtabbordercolour", 
    txt_frmtabbordercolour, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_frmtabbordercolour);
  gtk_box_pack_start (GTK_BOX (hbx_frmtabbordercolour), txt_frmtabbordercolour, TRUE, TRUE, 0); 
  gtk_widget_set_usize (txt_frmtabbordercolour, 184, 20);

  btn_frmtabbordercolour = gtk_button_new_with_label (_(".."));
  gtk_widget_set_name (btn_frmtabbordercolour, "btn_frmtabbordercolour");
  gtk_widget_ref (btn_frmtabbordercolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "btn_frmtabbordercolour", btn_frmtabbordercolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_frmtabbordercolour);
  gtk_box_pack_start (GTK_BOX (hbx_frmtabbordercolour), btn_frmtabbordercolour, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_frmtabbordercolour, 21, 21);

  gtk_signal_connect (GTK_OBJECT (btn_frmtabbordercolour), "clicked",
                      GTK_SIGNAL_FUNC( on_style_colour_btn_clicked ),
                      (gpointer) txt_frmtabbordercolour );

  return fra_tabframe;
  
}


void
on_chk_smiscusebgset_clicked( GtkButton       *button,
                              gpointer         user_data)
{

	GtkWidget *wgt_txtsmiscrootcomm = lookup_widget( (GtkWidget *) button, "txt_smiscrootcomm" );
	

	if( gtk_toggle_button_get_active( (GtkToggleButton *) button ))
		gtk_entry_set_text( (GtkEntry *) wgt_txtsmiscrootcomm, "whiteBOX -bgs" );
	else
		gtk_entry_set_text( (GtkEntry *) wgt_txtsmiscrootcomm, "" );

	gtk_widget_set_sensitive( wgt_txtsmiscrootcomm, !gtk_toggle_button_get_active( (GtkToggleButton *) button ));
	
}



GtkWidget *
make_misc_frame( GtkWidget *wnd_toadd )
{
 
  GtkWidget *fra_misc;
  GtkWidget *tbl_misc;
  
  GtkWidget *lbl_smiscbordwidth;
  GtkWidget *txt_smiscbordwidth;

  GtkWidget *lbl_smiscbevelwidth;
  GtkWidget *txt_smiscbevelwidth;

  GtkWidget *lbl_smischandlewidth;
  GtkWidget *txt_smischandlewidth;

  GtkWidget *lbl_smiscframewidth;
  GtkWidget *txt_smiscframewidth;

  GtkWidget *lbl_smiscbordcolour;
  GtkWidget *hbx_smiscbordcolour;
  GtkWidget *txt_smiscbordcolour;
  GtkWidget *btn_smiscbordcolour;

  GtkWidget *chk_smiscusebgset;

  GtkWidget *lbl_smiscrootcomm;
  GtkWidget *txt_smiscrootcomm;


  fra_misc = gtk_frame_new (_("Miscellaneous"));
  gtk_widget_set_name (fra_misc, "fra_misc");
  gtk_widget_ref (fra_misc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "fra_misc", fra_misc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (fra_misc, 384, 240);
  gtk_frame_set_shadow_type (GTK_FRAME (fra_misc), GTK_SHADOW_ETCHED_OUT);


  tbl_misc = gtk_table_new (5, 5, FALSE);
  gtk_widget_set_name (tbl_misc, "tbl_misc");
  gtk_widget_ref (tbl_misc);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "tbl_misc", tbl_misc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tbl_misc);
  gtk_container_add (GTK_CONTAINER (fra_misc), tbl_misc);
  gtk_container_set_border_width (GTK_CONTAINER (tbl_misc), 10);
  gtk_table_set_row_spacings (GTK_TABLE (tbl_misc), 4);
  gtk_table_set_col_spacings (GTK_TABLE (tbl_misc), 18);



  lbl_smiscbevelwidth = gtk_label_new (_("Bevel Width:"));
  gtk_widget_set_name (lbl_smiscbevelwidth, "lbl_smiscbevelwidth");
  gtk_widget_ref (lbl_smiscbevelwidth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_smiscbevelwidth", lbl_smiscbevelwidth,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_smiscbevelwidth);
  gtk_table_attach (GTK_TABLE (tbl_misc), lbl_smiscbevelwidth, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_smiscbevelwidth), 0, 0.5);


  txt_smiscbevelwidth = gtk_entry_new ();
  gtk_widget_set_name (txt_smiscbevelwidth, "txt_smiscbevelwidth");
  gtk_widget_ref (txt_smiscbevelwidth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_smiscbevelwidth", 
    txt_smiscbevelwidth, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_smiscbevelwidth);
  gtk_table_attach (GTK_TABLE (tbl_misc), txt_smiscbevelwidth, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);  
  gtk_widget_set_usize (txt_smiscbevelwidth, 1, 20);




  lbl_smischandlewidth = gtk_label_new (_("Handle Width:"));
  gtk_widget_set_name (lbl_smischandlewidth, "lbl_smischandlewidth");
  gtk_widget_ref (lbl_smischandlewidth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_smischandlewidth", lbl_smischandlewidth,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_smischandlewidth);
  gtk_table_attach (GTK_TABLE (tbl_misc), lbl_smischandlewidth, 3, 4, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_smischandlewidth), 0, 0.5);


  txt_smischandlewidth = gtk_entry_new ();
  gtk_widget_set_name (txt_smischandlewidth, "txt_smischandlewidth");
  gtk_widget_ref (txt_smischandlewidth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_smischandlewidth", 
    txt_smischandlewidth, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_smischandlewidth);
  gtk_table_attach (GTK_TABLE (tbl_misc), txt_smischandlewidth, 4, 5, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);  
  gtk_widget_set_usize (txt_smischandlewidth, 1, 20);



  lbl_smiscframewidth = gtk_label_new (_("Frame Width:"));
  gtk_widget_set_name (lbl_smiscframewidth, "lbl_smiscframewidth");
  gtk_widget_ref (lbl_smiscframewidth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_smiscframewidth", lbl_smiscframewidth,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_smiscframewidth);
  gtk_table_attach (GTK_TABLE (tbl_misc), lbl_smiscframewidth, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_smiscframewidth), 0, 0.5);


  txt_smiscframewidth = gtk_entry_new ();
  gtk_widget_set_name (txt_smiscframewidth, "txt_smiscframewidth");
  gtk_widget_ref (txt_smiscframewidth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_smiscframewidth", 
    txt_smiscframewidth, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_smiscframewidth);
  gtk_table_attach (GTK_TABLE (tbl_misc), txt_smiscframewidth, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);  
  gtk_widget_set_usize (txt_smiscframewidth, 1, 20);




  lbl_smiscbordwidth = gtk_label_new (_("Border Width:"));
  gtk_widget_set_name (lbl_smiscbordwidth, "lbl_smiscbordwidth");
  gtk_widget_ref (lbl_smiscbordwidth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_smiscbordwidth", lbl_smiscbordwidth,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_smiscbordwidth);
  gtk_table_attach (GTK_TABLE (tbl_misc), lbl_smiscbordwidth, 3, 4, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_smiscbordwidth), 0, 0.5);


  txt_smiscbordwidth = gtk_entry_new ();
  gtk_widget_set_name (txt_smiscbordwidth, "txt_smiscbordwidth");
  gtk_widget_ref (txt_smiscbordwidth);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_smiscbordwidth", 
    txt_smiscbordwidth, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_smiscbordwidth);
  gtk_table_attach (GTK_TABLE (tbl_misc), txt_smiscbordwidth, 4, 5, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);  
  gtk_widget_set_usize (txt_smiscbordwidth, 1, 20);




  lbl_smiscbordcolour = gtk_label_new (_("Border Colour:"));
  gtk_widget_set_name (lbl_smiscbordcolour, "lbl_smiscbordcolour");
  gtk_widget_ref (lbl_smiscbordcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_smiscbordcolour", lbl_smiscbordcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_smiscbordcolour);
  gtk_table_attach (GTK_TABLE (tbl_misc), lbl_smiscbordcolour, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_smiscbordcolour), 0, 0.5);



  hbx_smiscbordcolour = gtk_hbox_new (FALSE, 6);
  gtk_widget_set_name (hbx_smiscbordcolour, "hbx_smiscbordcolour");
  gtk_widget_ref (hbx_smiscbordcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "hbx_smiscbordcolour", hbx_smiscbordcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbx_smiscbordcolour);
  gtk_table_attach (GTK_TABLE (tbl_misc), hbx_smiscbordcolour, 1, 5, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);


  txt_smiscbordcolour = gtk_entry_new ();
  gtk_widget_set_name (txt_smiscbordcolour, "txt_smiscbordcolour");
  gtk_widget_ref (txt_smiscbordcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_smiscbordcolour", 
    txt_smiscbordcolour, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_smiscbordcolour);
  gtk_box_pack_start (GTK_BOX (hbx_smiscbordcolour), txt_smiscbordcolour, TRUE, TRUE, 0); 
  gtk_widget_set_usize (txt_smiscbordcolour, 184, 20);

  btn_smiscbordcolour = gtk_button_new_with_label (_(".."));
  gtk_widget_set_name (btn_smiscbordcolour, "btn_smiscbordcolour");
  gtk_widget_ref (btn_smiscbordcolour);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "btn_smiscbordcolour", btn_smiscbordcolour,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_smiscbordcolour);
  gtk_box_pack_start (GTK_BOX (hbx_smiscbordcolour), btn_smiscbordcolour, FALSE, FALSE, 0);
  gtk_widget_set_usize (btn_smiscbordcolour, 21, 21);



  chk_smiscusebgset = gtk_check_button_new_with_label (_("Use whiteBOX Background (single only)"));
  gtk_widget_set_name (chk_smiscusebgset, "chk_smiscusebgset");
  gtk_widget_ref (chk_smiscusebgset);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "chk_smiscusebgset", chk_smiscusebgset,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (chk_smiscusebgset);
  gtk_table_attach (GTK_TABLE (tbl_misc), chk_smiscusebgset, 0, 5, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);




  lbl_smiscrootcomm = gtk_label_new (_("Root Command:"));
  gtk_widget_set_name (lbl_smiscrootcomm, "lbl_smiscrootcomm");
  gtk_widget_ref (lbl_smiscrootcomm);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "lbl_smiscrootcomm", lbl_smiscrootcomm,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lbl_smiscrootcomm);
  gtk_table_attach (GTK_TABLE (tbl_misc), lbl_smiscrootcomm, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lbl_smiscrootcomm), 0, 0.5);


  txt_smiscrootcomm = gtk_entry_new ();
  gtk_widget_set_name (txt_smiscrootcomm, "txt_smiscrootcomm");
  gtk_widget_ref (txt_smiscrootcomm);
  gtk_object_set_data_full (GTK_OBJECT (wnd_toadd), "txt_smiscrootcomm", 
    txt_smiscrootcomm, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txt_smiscrootcomm);
  gtk_table_attach (GTK_TABLE (tbl_misc), txt_smiscrootcomm, 1, 5, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);  
  gtk_widget_set_usize (txt_smiscrootcomm, 1, 20);



  gtk_signal_connect (GTK_OBJECT (chk_smiscusebgset), "clicked",
                      GTK_SIGNAL_FUNC( on_chk_smiscusebgset_clicked ),
                      NULL );

  gtk_signal_connect (GTK_OBJECT (btn_smiscbordcolour), "clicked",
                      GTK_SIGNAL_FUNC( on_style_colour_btn_clicked ),
                      (gpointer) txt_smiscbordcolour );


  return fra_misc;
  
}
