/*
 * styles.c
 * Styles related callbacks and stuff
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <strings.h>
#include <string.h>

#ifdef HAVE_SYS_DIR_H
    #include <sys/dir.h>
#else
    #ifdef HAVE_SYS_NDIR_H
        #include <sys/ndir.h>
    #else
        #ifdef HAVE_DIRENT_H
            #include <dirent.h>
        #else
            #include <sys/dirent.h>
        #endif
    #endif
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <X11/Xlib.h>
#include <X11/Xresource.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>

#include "styles.h"
#include "styleframes.h"
#include "../../support.h"
#include "../../filedialog.h"
#include "../../msgbox.h"
#include "../../colourdialog.h"
#include "../../whiteBOX.h"

GtkCTreeNode *nde_fluxpos = NULL;
GtkCTreeNode *nde_fluxadd = NULL;

/* keep the fluxbox related styles in memory */
style_item *flx_tabstyle;
style_item *flx_lblfocusstyle;
style_item *flx_lblufocusstyle;
char *flx_tabfont;


char *style_texts[29] = { "toolbar", "toolbar.label", "toolbar.windowLabel",
	"toolbar.button", "toolbar.button.pressed", "toolbar.clock",
	"menu.title", "menu.bullet", "menu.frame",
	"menu.hilite", "window", "window.title.focus", "window.title.unfocus",
	"window.label.focus", "window.label.unfocus", "window.handle.focus",
	"window.handle.unfocus", "window.grip.focus", "window.grip.unfocus",
	"window.button.focus", "window.button.unfocus",
	"window.button.pressed", "window.frame", "", "", "", "window.tab", "window.tab.label.focus", "window.tab.label.unfocus"
	};

char *style_classes[29] = { "Toolbar", "Toolbar.Label", "Toolbar.WindowLabel",
	"Toolbar.Button", "Toolbar.Button.Pressed", "Toolbar.Clock",
	"Menu.Title", "Menu.Bullet", "Menu.Frame",
	"Menu.Hilite", "Window", "Window.Title.Focus", "Window.Title.Unfocus",
	"Window.Label.Focus", "Window.Label.Unfocus", "Window.Handle.Focus",
	"Window.Handle.Unfocus", "Window.Grip.Focus", "Window.Grip.Unfocus",
	"Window.Button.Focus", "Window.Button.Unfocus",
	"Window.Button.Pressed", "Window.Frame", "", "", "", "Window.Tab", "Window.Tab.Label.Focus", "Window.Tab.Label.Unfocus"
	};


/* various conditions to search for */
static char *gradient_types[] = { "horizontal", "diagonal", "vertical", "crossdiagonal",
                                  "pipecross", "elliptic", "rectangle", "pyramid" };
static char *bevel_types[]    = { "raised", "sunken", "flat" };

int NUM_GRADS = 8;                                /* total number of possible gradients */
int NUM_BEVEL = 3;


//new_item->s_bulletPos    = g_strdup( bulletPos );

font_item *
new_font_item( int type, const char *f_mtitle, const char *f_mframe,
          const char *f_wnd, const char *f_tbr, const char *f_tab, char *f_gbl )
{

	/*
	 * new_font_item
	 * ---------------------------------
	 * create a new font style item safely, with memory allocations
	 */

	font_item *new_item = calloc( 1, sizeof( font_item ));
	
	assert( type );        /* see if any errors occur */
	
	/* set the item data */
	new_item->s_type       = type;
	new_item->f_menutitle  = g_strdup( f_mtitle );
	new_item->f_menuframe  = g_strdup( f_mframe );
	new_item->f_window     = g_strdup( f_wnd );
	new_item->f_toolbar    = g_strdup( f_tbr );
	new_item->f_wintab     = g_strdup( f_tab );
	new_item->f_global     = g_strdup( f_gbl );
	
	return new_item;

}


misc_item *
new_misc_item( int type, int m_bWidth, const char *m_bColour, int m_bvWidth,
          int m_hWidth, int m_fWidth, int m_uBGSet, const char *m_rCommand )
{

	/*
	 * new_misc_item
	 * ---------------------------------
	 * create a new miscellaneous style item safely, with memory allocations
	 */

	misc_item *mm_item = calloc( 1, sizeof( misc_item ));
	
	assert( type );        /* see if any errors occur */

	/* set the item data */
	mm_item->s_type  = type;
    mm_item->m_borderWidth  = m_bWidth;
	mm_item->m_borderColour = g_strdup( m_bColour );
	mm_item->m_bevelWidth   = m_bvWidth;
	mm_item->m_handleWidth  = m_hWidth;
	mm_item->m_frameWidth   = m_fWidth;
	mm_item->m_useBGSet     = m_uBGSet;
	mm_item->m_rootCommand  = g_strdup( m_rCommand );
	
	return mm_item;

}



style_item *
new_style_item( int type, const char *bevel, int gradient, const char *gradientType,
                int interlaced, const char *colour, const char *colourTo,
                const char *textColour, const char *justify, const char *bulletType,
                const char *bulletPos, const char *disColour )
{
	/*
	 * new_style_item
	 * ---------------------------------
	 * create a new style item safely, with memory allocations
	 */
	
	style_item *new_item = calloc( 1, sizeof( style_item ));
	
	assert( type );	/* see if any errors occur */
	
	/* set the item data */
	new_item->s_type         = type;
	new_item->s_bevel        = g_strdup( bevel );
	new_item->s_gradient     = gradient;
	new_item->s_gradientType = g_strdup( gradientType );
	new_item->s_interlaced   = interlaced;
	new_item->s_colour       = g_strdup( colour );
	new_item->s_colourTo     = g_strdup( colourTo );
	new_item->s_textColour   = g_strdup( textColour );
	new_item->s_justify      = g_strdup( justify );
	new_item->s_bulletType   = g_strdup( bulletType );
	new_item->s_bulletPos    = g_strdup( bulletPos );
	new_item->s_disColour    = g_strdup( disColour );

	return new_item;

}



void
show_defaults( GtkWidget *widget, GtkWidget *vbx, GtkWidget *wnd_main, int show_flux )
{

    GtkWidget *wgt_vbxstyles;

    /* style property dialogs */
    GtkWidget *wgt_normframe;
    GtkWidget *wgt_bulletframe;
    GtkWidget *wgt_nonopframe;
    GtkWidget *wgt_justonlyframe;
    GtkWidget *wgt_fontframe;
    GtkWidget *wgt_wndframe;
    GtkWidget *wgt_tabframe;
	GtkWidget *wgt_miscframe;

    GtkCTree  *trv_styles;

    GtkCTreeNode *nde_menuroot;
    GtkCTreeNode *nde_addnode;

    char *styles_label;

    static style_item *style_item;

    //wgt_vbxmenuopts = lookup_widget( widget, "vbx_menuopts" );

    //gtk_widget_hide( wgt_vbxmenuopts );
    


    //trv_styles  = (GtkCTree *) lookup_widget( widget, "trv_sedit" );
    trv_styles = (GtkCTree *) widget;

    /* create the menu style nodes */
    styles_label = "Menu";
    nde_menuroot = gtk_ctree_insert_node( trv_styles, NULL, NULL, &styles_label, 8,
                                          NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_menuroot, style_item );
   
   
    styles_label = ".title";
    nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
    	                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( MENU_TITLE, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    
    styles_label = ".frame";
    nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
                                				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( MENU_FRAME, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );


    styles_label = ".hilite";
    nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
    	                           				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( MENU_HILITE, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );


    styles_label = ".bullet";
    nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
    	                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    
    style_item = new_style_item( MENU_BULLET, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );



    styles_label = "Toolbar";
    
    nde_menuroot = gtk_ctree_insert_node( trv_styles, NULL, NULL, &styles_label, 8,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( TOOL_ROOT, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_menuroot, style_item );
    

    styles_label = ".button";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( TOOL_BUTTON, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );


    styles_label = ".pressed";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_addnode, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( TOOL_PRESSED, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );




    styles_label = ".clock";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( TOOL_CLOCK, NULL, 0, NULL,0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );


    styles_label = ".label";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( TOOL_LABEL, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );


    styles_label = ".windowlabel";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( TOOL_WLABEL, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );


    styles_label = "Window";
		nde_menuroot = gtk_ctree_insert_node( trv_styles, NULL, NULL, &styles_label, 8,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_ROOT, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_menuroot, style_item );
    nde_fluxadd = nde_menuroot;


    styles_label = ".title";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".focus";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_addnode, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_TTLFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".unfocus";
		nde_addnode = gtk_ctree_insert_node( trv_styles, GTK_CTREE_ROW(nde_addnode)->parent, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_TTLUFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );


    styles_label = ".label";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".focus";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_addnode, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_LBLFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".unfocus";
		nde_addnode = gtk_ctree_insert_node( trv_styles, GTK_CTREE_ROW(nde_addnode)->parent, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_LBLUFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );


    styles_label = ".handle";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".focus";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_addnode, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_HDLFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".unfocus";
		nde_addnode = gtk_ctree_insert_node( trv_styles, GTK_CTREE_ROW(nde_addnode)->parent, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_HDLUFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );


    styles_label = ".grip";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".focus";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_addnode, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_GRPFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".unfocus";
		nde_addnode = gtk_ctree_insert_node( trv_styles, GTK_CTREE_ROW(nde_addnode)->parent, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_GRPUFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );


    styles_label = ".button";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".focus";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_addnode, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_BTNFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".unfocus";
		nde_addnode = gtk_ctree_insert_node( trv_styles, GTK_CTREE_ROW(nde_addnode)->parent, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_BTNUFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".pressed";
		nde_addnode = gtk_ctree_insert_node( trv_styles, GTK_CTREE_ROW(nde_addnode)->parent, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_BTNPRESS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

    styles_label = ".frame";
		nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( WND_FRMFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

  
    /* show fluxbox styles ?? */
    //if( show_flux )
    //{
      /* fluxbox tab styles */
      /*
      styles_label = ".tab";
		  nde_addnode = gtk_ctree_insert_node( trv_styles, nde_menuroot, NULL, &styles_label, 4,
		                              				 NULL, NULL, NULL, NULL, FALSE, FALSE );
      style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL );
      gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );
      nde_fluxpos = nde_addnode;
      

      styles_label = ".label";
		  nde_addnode = gtk_ctree_insert_node( trv_styles, nde_addnode, NULL, &styles_label, 4,
		                              				 NULL, NULL, NULL, NULL, FALSE, FALSE );
      style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL );
      gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );
  
      styles_label = ".focus";
		  nde_addnode = gtk_ctree_insert_node( trv_styles, nde_addnode, NULL, &styles_label, 4,
		                              				 NULL, NULL, NULL, NULL, FALSE, FALSE );
      style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL );
      gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );

      styles_label = ".unfocus";
		  nde_addnode = gtk_ctree_insert_node( trv_styles, GTK_CTREE_ROW(nde_addnode)->parent, NULL, &styles_label, 4,
		                              				 NULL, NULL, NULL, NULL, FALSE, FALSE );
      style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL );
      gtk_ctree_node_set_row_data( trv_styles, nde_addnode, style_item );
      */ 
      /* end fluxbox styles */
      
    //}


    styles_label = "Fonts";
    nde_menuroot = gtk_ctree_insert_node( trv_styles, NULL, NULL, &styles_label, 8,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( FNT_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_menuroot, style_item );


    styles_label = "Globals";
		nde_menuroot = gtk_ctree_insert_node( trv_styles, NULL, NULL, &styles_label, 8,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( trv_styles, nde_menuroot, style_item );

    styles_label = "Miscellaneous";
	nde_menuroot = gtk_ctree_insert_node( trv_styles, NULL, NULL, &styles_label, 8,
		                            				 NULL, NULL, NULL, NULL, FALSE, FALSE );
	style_item = new_style_item( STY_MISC, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );    
    gtk_ctree_node_set_row_data( trv_styles, nde_menuroot, style_item );


    //wgt_vbxstyles = lookup_widget( widget, "vbx_styles" );
    wgt_vbxstyles = vbx;

    wgt_normframe     = make_normstyle_frame( wnd_main, TRUE );
    wgt_bulletframe   = make_bulletstyle_frame( wnd_main );
    wgt_nonopframe    = make_no_node( wnd_main );
    wgt_justonlyframe = make_justonly_frame( wnd_main );
    wgt_fontframe     = make_fonts_frame( wnd_main );
    wgt_wndframe      = make_frmwindow_frame( wnd_main );
    wgt_tabframe      = make_frmtab_frame( wnd_main );
    wgt_miscframe     = make_misc_frame( wnd_main );

    gtk_box_pack_start (GTK_BOX (wgt_vbxstyles), wgt_normframe, TRUE, TRUE, 8);
    gtk_box_reorder_child( GTK_BOX( wgt_vbxstyles ), wgt_normframe, 0 );

    gtk_box_pack_start (GTK_BOX (wgt_vbxstyles), wgt_bulletframe, TRUE, TRUE, 8 );
    gtk_box_reorder_child( GTK_BOX( wgt_vbxstyles ), wgt_bulletframe, 0 );

    gtk_box_pack_start (GTK_BOX (wgt_vbxstyles), wgt_nonopframe, TRUE, TRUE, 8);
    gtk_box_reorder_child( GTK_BOX( wgt_vbxstyles ), wgt_nonopframe, 0 );

    gtk_box_pack_start (GTK_BOX (wgt_vbxstyles), wgt_justonlyframe, TRUE, TRUE, 8);
    gtk_box_reorder_child( GTK_BOX( wgt_vbxstyles ), wgt_justonlyframe, 0 );

    gtk_box_pack_start (GTK_BOX (wgt_vbxstyles), wgt_fontframe, TRUE, TRUE, 8);
    gtk_box_reorder_child( GTK_BOX( wgt_vbxstyles ), wgt_fontframe, 0 );

    gtk_box_pack_start (GTK_BOX (wgt_vbxstyles), wgt_wndframe, TRUE, TRUE, 8);
    gtk_box_reorder_child( GTK_BOX( wgt_vbxstyles ), wgt_wndframe, 0 );
    
    gtk_box_pack_start (GTK_BOX (wgt_vbxstyles), wgt_tabframe, TRUE, TRUE, 8);
    gtk_box_reorder_child( GTK_BOX( wgt_vbxstyles ), wgt_tabframe, 0 );

    gtk_box_pack_start (GTK_BOX (wgt_vbxstyles), wgt_miscframe, TRUE, TRUE, 8);
    gtk_box_reorder_child( GTK_BOX( wgt_vbxstyles ), wgt_miscframe, 0 );

}

int
check_condition( char *line_to_parse, const char *condition )
{

  /* not exactly portable ..
     but i suppose you'll need gtk anyway? */

  g_strdown( line_to_parse );              /* convert the parent string to lowercase */

  if( strstr( line_to_parse, condition ))  /* check for condition in parent string */
    return TRUE;
  else
    return FALSE;

}

void
load_style( gchar* style_file, GtkWidget *tree, GtkCTreeNode *root_node )
{

  static style_item *temp_sitem;                     /* the menu item associated with the node */

  GtkCTreeNode *cur_node  = root_node;               /* the current node */
  GtkCTreeNode *child;				     /* children of any parent item */

  style_item *add_item;

  XrmValue value;				     /* resource value */
  char *value_type;				     /* value type ?? Sorry don't know :) */
	
  char *style_text  = "";
  char *style_class = "";

  char res_val[60];
  char res_class[60];

  char *item_col     = "";                          /* item colour */
  char *item_colto   = "";                          /* gradient to colour */
  char *item_tpcol   = "";                          /* text/pic colour */
  int   grad_val     = 0;                           /* gradient on/off */
  char *grad_type    = "";                          /* gradient type */
  char *bevel_type   = "";                          /* bevel type */
  int   int_val      = 0;                           /* interlace value */
  char *item_justify = "";                          /* item justification */
  char *blt_type     = "";                          /* bullet type */
  char *blt_pos      = "";                          /* bullet position */
  char *disColour    = "";							/* disable colour */

  char *root_line    = "";                          /* root style item (full line value) */

  int i;                                            /* counter */

  XrmDatabase database = (XrmDatabase) 0;	    /* resource(style) database */

  gchar *fle = style_file;

		
  /* get the style data and store it */
  database = XrmGetFileDatabase( fle );

  /* while theres still a node to play with */
  while( cur_node )
  {

    /* get the item information attached to the node */
    temp_sitem = gtk_ctree_node_get_row_data( GTK_CTREE( tree ), GTK_CTREE_NODE( cur_node ));

    style_text  = style_texts[temp_sitem->s_type - 100];
    style_class = style_classes[temp_sitem->s_type - 100];

    //add_item->s_type = temp_sitem->s_type;

    if( temp_sitem->s_type == STY_MISC )
    {
		int borderWidth = 0;
		char *borderColour = NULL;
		int bevelWidth = 0;
		int handleWidth = 0;
		int frameWidth = 0;
		int useBGSet = 0;
		char *rootCommand = NULL;

		misc_item *m_item;

		sprintf( res_val, "borderWidth" );
		sprintf( res_class, "BorderWidth" );

		if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
			borderWidth = atoi((char *) value.addr);
		
        else
			borderWidth = -9999;


		sprintf( res_val, "borderColor" );
		sprintf( res_class, "borderColor" );

		if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
			borderColour = (char *) value.addr;
        else
			borderColour = NULL;

		sprintf( res_val, "bevelWidth" );
		sprintf( res_class, "BevelWidth" );

		if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
			bevelWidth = atoi((char *) value.addr);
        else
			bevelWidth = -9999;

		sprintf( res_val, "handleWidth" );
		sprintf( res_class, "HandleWidth" );

		if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
			handleWidth = atoi((char *) value.addr);
        else
			handleWidth = -9999;

		sprintf( res_val, "frameWidth" );
		sprintf( res_class, "FrameWidth" );

		if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
			frameWidth = atoi((char *) value.addr);
        else
			frameWidth = -9999;


		sprintf( res_val, "useBGSet" );
		sprintf( res_class, "UseBGSet" );

		if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
			useBGSet = atoi((char *) value.addr);
        else
			useBGSet = 0;

		sprintf( res_val, "rootCommand" );
		sprintf( res_class, "RootCommand" );

		if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
			rootCommand = (char *) value.addr;
        else
			rootCommand = NULL;

        
		m_item = new_misc_item( STY_MISC, borderWidth, borderColour, bevelWidth, handleWidth, frameWidth, useBGSet, rootCommand );
        gtk_ctree_node_set_row_data( (GtkCTree *) tree, (GtkCTreeNode *) cur_node, m_item );


	}
	/* if the style item is a font item */
	else if( temp_sitem->s_type == FNT_ITEM )
    {

      char *gbl_font;

      char *wnd_font;
      char *tbr_font;
      char *mtl_font;
      char *mfm_font;
      char *wtb_font;

      font_item *fnt_item;

      sprintf( res_val, "*font" );
      sprintf( res_class, "*Font" );

      if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
        gbl_font = (char *) value.addr;
      else
        gbl_font = NULL;

      if( !gbl_font )
      {

        sprintf( res_val, "window.font" );
        sprintf( res_class, "Window.Font" );

        if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
          wnd_font = (char *) value.addr;
        else
          wnd_font = NULL;


        sprintf( res_val, "menu.title.font" );
        sprintf( res_class, "Menu.Title.Font" );

        if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
          mtl_font = (char *) value.addr;
        else
          mtl_font = NULL;


        sprintf( res_val, "menu.frame.font" );
        sprintf( res_class, "Menu.Frame.Font" );

        if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
          mfm_font = (char *) value.addr;
        else
          mfm_font = NULL;


        sprintf( res_val, "toolbar.font" );
        sprintf( res_class, "Toolbar.Font" );

        if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
          tbr_font = (char *) value.addr;
        else
          tbr_font = NULL;


        sprintf( res_val, "window.tab.font" );
        sprintf( res_class, "Window.Tab.Font" );

        if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
          wtb_font = (char *) value.addr;
        else
          wtb_font = NULL;        
        
        
        fnt_item = new_font_item( FNT_ITEM, mtl_font, mfm_font, wnd_font,
          tbr_font, wtb_font, NULL );

        gtk_ctree_node_set_row_data( (GtkCTree *) tree,
          (GtkCTreeNode *) cur_node, fnt_item );

      }
      else
      {


        fnt_item = new_font_item( FNT_ITEM, NULL, NULL, NULL, NULL, NULL,
          (char *) gbl_font );

        gtk_ctree_node_set_row_data( (GtkCTree *) tree,
          (GtkCTreeNode *) cur_node, fnt_item );

      }

    }
    else if( temp_sitem->s_type == WND_FRMFOCUS )
    {
      static style_item *tmp_style;

      char *fcs_col;
      char *ufs_col;

      /* get colour value */
      sprintf( res_val, "%s.focuscolor", style_text );
      sprintf( res_class, "%s.focusColor", style_class );
		
      if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
        fcs_col = (char *) value.addr;
      else
        fcs_col = NULL;

      /* get colour value */
      sprintf( res_val, "%s.unfocuscolor", style_text );
      sprintf( res_class, "%s.unfocusColor", style_class );
		
      if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
        ufs_col = (char *) value.addr;
      else
        ufs_col = NULL;


      tmp_style = new_style_item( WND_FRMFOCUS, NULL, 0, NULL, 0,
					      fcs_col, ufs_col, NULL, NULL, NULL, NULL, NULL );


      gtk_ctree_node_set_row_data( (GtkCTree *) tree,
        (GtkCTreeNode *) cur_node, tmp_style );

    }

    
    else if( temp_sitem->s_type == WND_TAB )
    {

      int   b_width;
      char *b_colour;
      char *b_justify;

      
      /* get justify value */
      sprintf( res_val, "%s.justify", style_text );
      sprintf( res_class, "%s.justify", style_class );
		
      if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
        b_justify = (char *) value.addr;
      else
        b_justify = NULL;
      
      
      /* get border width value */
      sprintf( res_val, "%s.borderwidth", style_text );
      sprintf( res_class, "%s.borderWidth", style_class );
		
      if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
        b_width = atoi((char *) value.addr );
      else
        b_width = 0;
 

      /* get colour value */
      sprintf( res_val, "%s.bordercolor", style_text );
      sprintf( res_class, "%s.borderColor", style_class );
		
      if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
        b_colour = (char *) value.addr;
      else
        b_colour = NULL;

      
      flx_tabstyle = new_style_item( WND_TAB, NULL, 0, NULL, b_width,
                  b_colour, NULL, NULL, b_justify, NULL, NULL, NULL );

      gtk_ctree_node_set_row_data( (GtkCTree *) tree,
        (GtkCTreeNode *) cur_node, flx_tabstyle );

    }    
    
    
    /* check to see if the node is non-operative */
    else if( temp_sitem->s_type != NOS_ITEM )
    {

      //int type, const char *bevel, int gradient, const char *gradientType,
      //          int interlaced, const char *colour, const char *colourTo,
      //          const char *textColour, const char *justify, const char *bulletType,
      //          const char *bulletPos )


      /* get colour value */
      sprintf( res_val, "%s.color", style_text );
      sprintf( res_class, "%s.Color", style_class );
		
      if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
        item_col = (char *) value.addr;
      else
        item_col = NULL;



      /* get root values */
      sprintf( res_val, "%s", style_text );
      sprintf( res_class, "%s", style_class );

      if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
      {

        root_line = (char *) value.addr;

        /* check for gradient */
        if( check_condition( root_line, "gradient" ))
          grad_val = 1;
	    else
          grad_val = 0;


        grad_type = NULL;        /* intital gradient type */

        /* check gradient type */
        for( i = 0; i < NUM_GRADS; i++ )
        {
          if( check_condition( root_line, gradient_types[i] ))
            grad_type = gradient_types[i];

	      }

        bevel_type = NULL;        /* intital bevel type */

        /* check bevel type */
        for( i = 0; i < NUM_BEVEL; i++ )
        {
          if( check_condition( root_line, bevel_types[i] ))
            bevel_type = bevel_types[i];

	      }

        /* check for gradient */
        if( check_condition( root_line, "interlaced" ))
          int_val = 1;
	      else
          int_val = 0;

      }

      /* get colourTo value */
      sprintf( res_val, "%s.colorTo", style_text );
      sprintf( res_class, "%s.ColorTo", style_class );
	
      if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
        item_colto = (char *) value.addr;
      else
        item_colto = NULL;

      /* get text/pic colour */

      if(( temp_sitem->s_type == TOOL_BUTTON  ) ||
         ( temp_sitem->s_type == WND_BTNFOCUS ) ||
         ( temp_sitem->s_type == WND_BTNUFOCUS ))
      {

        sprintf( res_val, "%s.picColor", style_text );
        sprintf( res_class, "%s.PicColor", style_class );
		
        if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
          item_tpcol = (char *) value.addr;
        else
          item_tpcol = NULL;

      }
      else
      {

        sprintf( res_val, "%s.textColor", style_text );
        sprintf( res_class, "%s.TextColor", style_class );
		
        if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
          item_tpcol = strdup( (char *) value.addr );
	      else
          item_tpcol = NULL;
      }

	
	  disColour = NULL;
	  if( temp_sitem->s_type == MENU_FRAME )
      {
        sprintf( res_val, "%s.disableColor", style_text );
        sprintf( res_class, "%s.DisableColor", style_class );
		
        if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
          disColour = strdup( (char *) value.addr );
	    else
          disColour = NULL;		
      }

      /* item justification */
      sprintf( res_val, "%s.justify", style_text );
      sprintf( res_class, "%s.Justify", style_class );
		
      if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
        item_justify = strdup( (char *) value.addr );
      else
        item_justify = NULL;


      if( temp_sitem->s_type == MENU_BULLET )
      {

        /* get bullet type value */
        sprintf( res_val, "%s", style_text );
        sprintf( res_class, "%s", style_class );
	
        if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
          blt_type = (char *) value.addr;
        else
          blt_type = NULL;


        /* get bullet position value */
        sprintf( res_val, "%s.position", style_text );
        sprintf( res_class, "%s.Position", style_class );
	
        if( XrmGetResource( database, res_val, res_class, &value_type, &value ))
          blt_pos = (char *) value.addr;
        else
          blt_pos = NULL;

      }


      /* -- DEBUG -- */
      if( DEBUG_FLAG )
      {
        g_print( "\n" );
        g_print( "ic:  %s\n", item_col );
        g_print( "ic2: %s\n", item_colto );
        g_print( "itp: %s\n", item_tpcol );
        g_print( "gv:  %d\n", grad_val );
        g_print( "gt:  %s\n", grad_type ); 
      }
      
      if( temp_sitem->s_type == WND_TAB_LAB_FOCUS )
      {
        flx_lblfocusstyle = new_style_item( temp_sitem->s_type, bevel_type, grad_val, grad_type, int_val, item_col,
                                            item_colto, item_tpcol, item_justify, blt_type, blt_pos, NULL );      

        gtk_ctree_node_set_row_data( (GtkCTree *) tree, (GtkCTreeNode *) cur_node, flx_lblfocusstyle );
      }
      else if( temp_sitem->s_type == WND_TAB_LAB_UFOCUS )
      {
        flx_lblufocusstyle = new_style_item( temp_sitem->s_type, bevel_type, grad_val, grad_type, int_val, item_col,
                                             item_colto, item_tpcol, item_justify, blt_type, blt_pos, NULL );      

        gtk_ctree_node_set_row_data( (GtkCTree *) tree, (GtkCTreeNode *) cur_node, flx_lblufocusstyle );
      }  
      else
      {
        add_item = new_style_item( temp_sitem->s_type, bevel_type, grad_val, grad_type, int_val, item_col,
                                   item_colto, item_tpcol, item_justify, blt_type, blt_pos, disColour );
      
        gtk_ctree_node_set_row_data( (GtkCTree *) tree, (GtkCTreeNode *) cur_node, add_item );
      }
    
    }	
			
    /* if the node has children then do a sub search */
    if( GTK_CTREE_ROW( cur_node )->children ) {
    	
      /* get the child, start the sub search, then continue */
      child = GTK_CTREE_NODE( GTK_CTREE_ROW( cur_node )->children );
      load_style( fle, tree, child );
	     	
    }

    add_item = NULL;
    temp_sitem = NULL;					/* set style item to NULL */
    cur_node   = GTK_CTREE_ROW( cur_node )->sibling; 	/* get next item */
	
    }

}
	
void
write_style_line( style_item *tmp_item, FILE *out_file )
{
	
  char *style_text = "";
  char *s_grad     = "";
  char *s_intlcd   = "";
  char *s_bevel    = "";
  char *s_gradtype = "";
	
  style_text = style_texts[tmp_item->s_type - 100];

  if( tmp_item->s_type != STY_MISC )
  {
	  if( tmp_item->s_gradient )
		s_grad   = "gradient";
	  else
		s_grad   = NULL;
	
	
	  if( tmp_item->s_gradientType && strlen( tmp_item->s_gradientType ) != 0 )
		s_gradtype = tmp_item->s_gradientType;
	   else
		s_gradtype = NULL;
	
	
	  if( tmp_item->s_interlaced )
		s_intlcd = "interlaced";
	  else
		s_intlcd = NULL;
	
	
	  if( tmp_item->s_bevel && strlen( tmp_item->s_bevel ) != 0 )
		s_bevel = tmp_item->s_bevel;
	   else
		s_bevel = NULL;
  }


  /* might need some sting length checking if problems arise */
  if( tmp_item->s_type == MENU_BULLET )
  {
    if( tmp_item->s_bulletType )
      fprintf( out_file, "%s: %s\n", style_text, tmp_item->s_bulletType );

    if( tmp_item->s_bulletPos )
      fprintf( out_file, "%s.position: %s\n", style_text, tmp_item->s_bulletPos );

  }
  else if( tmp_item->s_type == WND_TAB )
  {
    if( tmp_item->s_justify )
      fprintf( out_file, "%s.justify: %s\n", style_text, tmp_item->s_justify );

    if( tmp_item->s_interlaced )
      fprintf( out_file, "%s.borderWidth: %d\n", style_text, tmp_item->s_interlaced );

    if( tmp_item->s_colour )
      fprintf( out_file, "%s.borderColor: %s\n", style_text, tmp_item->s_colour );

    
  }
  else if( tmp_item->s_type == WND_FRMFOCUS )
  {
    if( tmp_item->s_colour )
      fprintf( out_file, "%s.focusColor: %s\n", style_text, tmp_item->s_colour );

    if( tmp_item->s_colourTo )
      fprintf( out_file, "%s.unfocusColor: %s\n", style_text, tmp_item->s_colourTo );

  }
  else if( tmp_item->s_type == STY_MISC )
  {
	if( ((misc_item *)tmp_item)->m_borderWidth > 0 )
		fprintf( out_file, "borderWidth: %d\n", ((misc_item *)tmp_item)->m_borderWidth );
	if( ((misc_item *)tmp_item)->m_borderColour )
		fprintf( out_file, "borderColor: %s\n", ((misc_item *)tmp_item)->m_borderColour );
	if( ((misc_item *)tmp_item)->m_bevelWidth > 0 )
		fprintf( out_file, "bevelWidth: %d\n", ((misc_item *)tmp_item)->m_bevelWidth );
	if( ((misc_item *)tmp_item)->m_handleWidth > 0 )
		fprintf( out_file, "handleWidth: %d\n", ((misc_item *)tmp_item)->m_handleWidth );
	if( ((misc_item *)tmp_item)->m_frameWidth > 0 )
		fprintf( out_file, "frameWidth: %d\n", ((misc_item *)tmp_item)->m_frameWidth );
	if( ((misc_item *)tmp_item)->m_rootCommand )
		fprintf( out_file, "rootCommand: %s\n", ((misc_item *)tmp_item)->m_rootCommand );
	fprintf( out_file, "useBGSet: %d\t! convenience flag -- not part of BB/FB\n", ((misc_item *)tmp_item)->m_useBGSet );

  }
  else if( tmp_item->s_type == FNT_ITEM )
  {

    if( !((font_item *) tmp_item )->f_global )
    {
      if( ((font_item *) tmp_item )->f_menutitle )
        fprintf( out_file, "menu.title.font: %s\n", ((font_item *) tmp_item )->f_menutitle );

      if( ((font_item *) tmp_item )->f_menuframe )
        fprintf( out_file, "menu.frame.font: %s\n", ((font_item *) tmp_item )->f_menuframe );

      if( ((font_item *) tmp_item )->f_window )
        fprintf( out_file, "window.font: %s\n",     ((font_item *) tmp_item )->f_window );

      if( ((font_item *) tmp_item )->f_toolbar )
        fprintf( out_file, "toolbar.font: %s\n",    ((font_item *) tmp_item )->f_toolbar );
      
      if( ((font_item *) tmp_item )->f_wintab )
        fprintf( out_file, "window.tab.font: %s\n", ((font_item *) tmp_item )->f_wintab );
      
    }
    else
    {
      fprintf( out_file, "*font: %s\n", ((font_item *) tmp_item )->f_global );
    }

  }
  else
  {

    if(( s_bevel ) ||
       ( s_grad ) ||
       ( s_gradtype ) ||
       ( s_intlcd ))
    {

      fprintf( out_file, "%s: ", style_text );

      if( tmp_item->s_bevel )
        fprintf( out_file, "%s ", s_bevel );

      if( s_grad )
        fprintf( out_file, "%s ", s_grad );

      if( tmp_item->s_gradientType )
        fprintf( out_file, "%s ", s_gradtype );

      if( s_intlcd )
        fprintf( out_file, "%s ", s_intlcd );

      fprintf( out_file, "\n" );

    }

    if( tmp_item->s_colour )
      fprintf( out_file, "%s.color: %s\n", style_text, tmp_item->s_colour );

    if( tmp_item->s_colourTo )
      fprintf( out_file, "%s.colorTo: %s\n", style_text, tmp_item->s_colourTo );

    if( tmp_item->s_type == TOOL_BUTTON   ||
        tmp_item->s_type == TOOL_PRESSED  ||
        tmp_item->s_type == WND_BTNFOCUS  ||
        tmp_item->s_type == WND_BTNUFOCUS )
    {

      if( tmp_item->s_textColour )
        fprintf( out_file, "%s.picColor: %s\n", style_text, tmp_item->s_textColour );
    }
    else
      if( tmp_item->s_textColour )
        fprintf( out_file, "%s.textColor: %s\n", style_text, tmp_item->s_textColour );


	if( tmp_item->s_disColour )
		fprintf( out_file, "%s.disableColor: %s\n", style_text, tmp_item->s_disColour );


    if( tmp_item->s_justify )
      fprintf( out_file, "%s.justify: %s\n", style_text, tmp_item->s_justify );

		
  }

  //fprintf( out_file, "\n\n" );
	
  tmp_item = NULL;
	
}

void
save_style( GtkCTreeNode *root_node, GtkWidget *tree, FILE *save_file )
{


  GtkCTreeNode *cur_node  = root_node;               /* the current node */
  GtkCTreeNode *child;				     /* children of any parent item */

  style_item *temp_sitem;                            /* the menu item associated with the node */

  /* while theres still a node to play with */
  while( cur_node )
  {

    /* get the item information attached to the node */
    temp_sitem = gtk_ctree_node_get_row_data( GTK_CTREE( tree ), GTK_CTREE_NODE( cur_node ));


    /* check to see if the node is non-operative */
    if( temp_sitem->s_type != NOS_ITEM )
		write_style_line( temp_sitem, save_file );	/* write the style info to the style file */

    /* if the node has children then do a sub search */
    if( GTK_CTREE_ROW( cur_node )->children ) {
			
      /* get the child, start the sub search, then continue */
      child = GTK_CTREE_NODE( GTK_CTREE_ROW( cur_node )->children );
      save_style( child, tree, save_file );
		
    }

    temp_sitem = NULL;	                                /* set style item to NULL */
    cur_node = GTK_CTREE_ROW( cur_node )->sibling;      /* get next item */
		
  }

	
}


void
show_style_item                        (GtkWidget       *ptr_to_main,
                                        GtkCTreeNode    *cur_node)
{

  /* style tree */
  GtkWidget *wgt_styletree      = lookup_widget( ptr_to_main, "trv_sedit" );

  /* the style properties items */
  GtkWidget *wgt_txtsnormcolour      = lookup_widget( ptr_to_main, "txt_snormcolour" );
  GtkWidget *wgt_txtsnormcolourto    = lookup_widget( ptr_to_main, "txt_snormcolourto" );
  GtkWidget *wgt_txtcbosnormbevel    = lookup_widget( ptr_to_main, "txt_cbosnormbevel" );
  GtkWidget *wgt_chksnormgradient    = lookup_widget( ptr_to_main, "chk_snormgradient" );
  GtkWidget *wgt_txtcbosnormgtype    = lookup_widget( ptr_to_main, "txt_cbosnormgtype" );
  GtkWidget *wgt_chksnorminterlaced  = lookup_widget( ptr_to_main, "chk_snorminterlaced" );
  GtkWidget *wgt_txtsnormtextcolour  = lookup_widget( ptr_to_main, "txt_snormtextcolour" );
  GtkWidget *wgt_txtcbojustonly      = lookup_widget( ptr_to_main, "txt_justonly" );
  GtkWidget *wgt_txtsnormdiscolour   = lookup_widget( ptr_to_main, "txt_snormdiscolour" );

  GtkWidget *wgt_txtsbulltype;
  GtkWidget *wgt_txtsbullpos;

  GtkWidget *wgt_txtcbosnormjustify  = lookup_widget( ptr_to_main, "txt_cbosnormjustify" );

  style_item *show_item = gtk_ctree_node_get_row_data( (GtkCTree *) wgt_styletree, cur_node );

  int gradient_flag  = show_item->s_gradient;
  int interlace_flag = show_item->s_interlaced;



  if( show_item->s_type != MENU_BULLET    && 
      show_item->s_type != FNT_ITEM       && 
      show_item->s_type != WND_FRMFOCUS   &&
	  show_item->s_type != STY_MISC		  &&
      show_item->s_type != WND_TAB )
  {

    if(( show_item->s_type == MENU_TITLE ) ||
       ( show_item->s_type == MENU_FRAME ) ||
       ( show_item->s_type == TOOL_ROOT ))
    {
      if( show_item->s_justify )
        gtk_entry_set_text( (GtkEntry *) wgt_txtcbosnormjustify, show_item->s_justify );
      else
        gtk_entry_set_text( (GtkEntry *) wgt_txtcbosnormjustify, "" );

	  if( show_item->s_type == MENU_FRAME )
	  {
        if( show_item->s_disColour )
		  gtk_entry_set_text( (GtkEntry *) wgt_txtsnormdiscolour, show_item->s_disColour );
		else
		  gtk_entry_set_text( (GtkEntry *) wgt_txtsnormdiscolour, "" );
	  }
    }


    if( show_item->s_colour )
      gtk_entry_set_text( (GtkEntry *) wgt_txtsnormcolour, show_item->s_colour );
    else
      gtk_entry_set_text( (GtkEntry *) wgt_txtsnormcolour, "" );


    /* show the gradient to colour  -- 
     * note: 
     * we dont want to check for a gradient flag .. this way we can have funky
     * interlaced items .. you know you want it :)
     */
    if( show_item->s_colourTo ) 
      gtk_entry_set_text( (GtkEntry *) wgt_txtsnormcolourto, show_item->s_colourTo );
    else
      gtk_entry_set_text( (GtkEntry *) wgt_txtsnormcolourto, "" );
		
    
    if( gradient_flag ) {  

      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_chksnormgradient, TRUE );

      /* show the gradient to colour */
      //if( show_item->s_colourTo )
      //  gtk_entry_set_text( (GtkEntry *) wgt_txtsnormcolourto, show_item->s_colourTo );
      //else
      //  gtk_entry_set_text( (GtkEntry *) wgt_txtsnormcolourto, "" );
  
      /* show the gradient type */
      if( show_item->s_gradientType )
        gtk_entry_set_text( (GtkEntry *) wgt_txtcbosnormgtype, show_item->s_gradientType );
      else
        gtk_entry_set_text( (GtkEntry *) wgt_txtcbosnormgtype, _("Horizontal") );
 
    }
    else 
    {
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_chksnormgradient, FALSE );
      //gtk_entry_set_text( (GtkEntry *) wgt_txtsnormcolourto, "" );
      gtk_entry_set_text( (GtkEntry *) wgt_txtcbosnormgtype, _("") );
    }  

    if( interlace_flag )
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_chksnorminterlaced, TRUE );    
    else
      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_chksnorminterlaced, FALSE );    
 
    if( show_item->s_bevel )
      gtk_entry_set_text( (GtkEntry *) wgt_txtcbosnormbevel, show_item->s_bevel );    
    else
      gtk_entry_set_text( (GtkEntry *) wgt_txtcbosnormbevel, _("") );    

    if( show_item->s_textColour )
      gtk_entry_set_text( (GtkEntry *) wgt_txtsnormtextcolour, show_item->s_textColour );    
    else
      gtk_entry_set_text( (GtkEntry *) wgt_txtsnormtextcolour, "" );        


    if( show_item->s_justify )
      gtk_entry_set_text( (GtkEntry *) wgt_txtcbojustonly, show_item->s_justify );
    else
      gtk_entry_set_text( (GtkEntry *) wgt_txtcbojustonly, "" );

  }

  else if( show_item->s_type == WND_TAB )
  {
    GtkWidget *wgt_txttabjustify = lookup_widget( ptr_to_main, "txt_tabjustify" );
    GtkWidget *wgt_txtfrmtabborderwidth = lookup_widget( ptr_to_main, "txt_frmtabborderwidth" );
    GtkWidget *wgt_txtfrmtabbordercolour = lookup_widget( ptr_to_main, "txt_frmtabbordercolour" );
        
    char *b_widthstr = "";
            
    if( show_item->s_justify )
      gtk_entry_set_text( (GtkEntry *) wgt_txttabjustify, show_item->s_justify );
    else
      gtk_entry_set_text( (GtkEntry *) wgt_txttabjustify, "" );    

    
    if( show_item->s_interlaced ) {
      sprintf( b_widthstr, "%d", show_item->s_interlaced );
      gtk_entry_set_text( (GtkEntry *) wgt_txtfrmtabborderwidth, b_widthstr );
    }
    else
      gtk_entry_set_text( (GtkEntry *) wgt_txtfrmtabborderwidth, "" );    

    
    if( show_item->s_colour )
      gtk_entry_set_text( (GtkEntry *) wgt_txtfrmtabbordercolour, show_item->s_colour );
    else
      gtk_entry_set_text( (GtkEntry *) wgt_txtfrmtabbordercolour, "" );    
    
    
  }
  
  else if( show_item->s_type == WND_FRMFOCUS )
  {
    GtkWidget *wgt_txtfrmfocuscol  = lookup_widget( ptr_to_main, "txt_frmfocuscol" );
    GtkWidget *wgt_txtfrmufocuscol = lookup_widget( ptr_to_main, "txt_frmufocuscol" );


    if( show_item->s_colour )
      gtk_entry_set_text( (GtkEntry *) wgt_txtfrmfocuscol, show_item->s_colour );
    else
      gtk_entry_set_text( (GtkEntry *) wgt_txtfrmfocuscol, "" );

    if( show_item->s_colourTo )
      gtk_entry_set_text( (GtkEntry *) wgt_txtfrmufocuscol, show_item->s_colourTo );
    else
      gtk_entry_set_text( (GtkEntry *) wgt_txtfrmufocuscol, "" );

  }

  /* if style node is fonts definition */
  else if( show_item->s_type == FNT_ITEM ) {

    GtkWidget *wgt_txtfontmnutitle = lookup_widget( ptr_to_main, "txt_fontmnutitle" );
    GtkWidget *wgt_txtfontmnuframe = lookup_widget( ptr_to_main, "txt_fontmnuframe" );
    GtkWidget *wgt_txtfontwindow   = lookup_widget( ptr_to_main, "txt_fontwindow" );
    GtkWidget *wgt_txtfonttoolbar  = lookup_widget( ptr_to_main, "txt_fonttoolbar" );
    GtkWidget *wgt_txtfontglobal   = lookup_widget( ptr_to_main, "txt_fontglobal" );
    GtkWidget *wgt_txtfonttab      = lookup_widget( ptr_to_main, "txt_fonttab" );
    
    GtkWidget *wgt_chkfontglobal   = lookup_widget( ptr_to_main, "chk_fontglobal" );


    if( !((font_item *) show_item )->f_global )
    {
      if( ((font_item *) show_item )->f_menutitle )
        gtk_entry_set_text( (GtkEntry *) wgt_txtfontmnutitle, ((font_item *) show_item )->f_menutitle );

      if( ((font_item *) show_item )->f_menuframe )
        gtk_entry_set_text( (GtkEntry *) wgt_txtfontmnuframe, ((font_item *) show_item )->f_menuframe );

      if( ((font_item *) show_item )->f_window )
        gtk_entry_set_text( (GtkEntry *) wgt_txtfontwindow, ((font_item *) show_item )->f_window );

      if( ((font_item *) show_item )->f_toolbar )
        gtk_entry_set_text( (GtkEntry *) wgt_txtfonttoolbar, ((font_item *) show_item )->f_toolbar );

      if( ((font_item *) show_item )->f_wintab )
        gtk_entry_set_text( (GtkEntry *) wgt_txtfonttab, ((font_item *) show_item )->f_wintab );      
      
    }
    else
    {

      gtk_toggle_button_set_active( (GtkToggleButton *) wgt_chkfontglobal, TRUE );

      if( ((font_item *) show_item )->f_global )
        gtk_entry_set_text( (GtkEntry *) wgt_txtfontglobal, ((font_item *) show_item )->f_global );

    }

  }

  /* if the style type is menu.bullet */
  else if( show_item->s_type == MENU_BULLET ) {
    
    wgt_txtsbulltype = lookup_widget( ptr_to_main, "txt_cbosbulltype" );
    wgt_txtsbullpos  = lookup_widget( ptr_to_main, "txt_cbosbullpos" );

    if( show_item->s_bulletType ) {
      gtk_entry_set_text( (GtkEntry *) wgt_txtsbulltype, show_item->s_bulletType );
    }
    else {
      gtk_entry_set_text( (GtkEntry *) wgt_txtsbulltype, "" );
    }
    
    if( show_item->s_bulletPos ) {
      gtk_entry_set_text( (GtkEntry *) wgt_txtsbullpos, show_item->s_bulletPos );
    }
    else {
      gtk_entry_set_text( (GtkEntry *) wgt_txtsbullpos, "" ); 
    }
    
  }

  else if( show_item->s_type == STY_MISC )
  {

	GtkWidget *wgt_txtsmiscbordwidth   = lookup_widget( (GtkWidget *) ptr_to_main, "txt_smiscbordwidth" );
	GtkWidget *wgt_txtsmiscbevelwidth  = lookup_widget( (GtkWidget *) ptr_to_main, "txt_smiscbevelwidth" );
	GtkWidget *wgt_txtsmischandlewidth = lookup_widget( (GtkWidget *) ptr_to_main, "txt_smischandlewidth" );
	GtkWidget *wgt_txtsmiscframewidth  = lookup_widget( (GtkWidget *) ptr_to_main, "txt_smiscframewidth" );
	GtkWidget *wgt_txtsmiscbordcolour  = lookup_widget( (GtkWidget *) ptr_to_main, "txt_smiscbordcolour" );
	GtkWidget *wgt_chksmiscusebgset    = lookup_widget( (GtkWidget *) ptr_to_main, "chk_smiscusebgset" );
	GtkWidget *wgt_txtsmiscrootcomm    = lookup_widget( (GtkWidget *) ptr_to_main, "txt_smiscrootcomm" );


	char int_borderWidth[255];
	char int_bevelWidth[255];
	char int_frameWidth[255];
	char int_handleWidth[255];

	strcpy(int_borderWidth, "");
	strcpy(int_bevelWidth, "");
	strcpy(int_frameWidth, "");	
	strcpy(int_handleWidth, "");


	if( ((misc_item *) show_item)->m_borderWidth )
	{
		if( ((misc_item *) show_item)->m_borderWidth > 0 )
		{
			sprintf( int_borderWidth, "%d", ((misc_item *) show_item)->m_borderWidth );
			gtk_entry_set_text( (GtkEntry *) wgt_txtsmiscbordwidth, int_borderWidth );
		}
	}


	if( ((misc_item *) show_item)->m_bevelWidth )
	{
		if( ((misc_item *) show_item)->m_bevelWidth > 0 )
		{
			sprintf( int_bevelWidth, "%d", ((misc_item *) show_item)->m_bevelWidth );
			gtk_entry_set_text( (GtkEntry *) wgt_txtsmiscbevelwidth, int_bevelWidth );
		}
	}


	if( ((misc_item *) show_item)->m_frameWidth )
	{
		if( ((misc_item *) show_item)->m_frameWidth > 0 )
		{
			sprintf( int_frameWidth, "%d", ((misc_item *) show_item)->m_frameWidth );
			gtk_entry_set_text( (GtkEntry *) wgt_txtsmiscframewidth, int_frameWidth );
		}
	}


	if( ((misc_item *) show_item)->m_handleWidth )
	{	
		if( ((misc_item *) show_item)->m_handleWidth > 0 )
		{
			sprintf( int_handleWidth, "%d", ((misc_item *) show_item)->m_handleWidth );
			gtk_entry_set_text( (GtkEntry *) wgt_txtsmischandlewidth, int_handleWidth );
		}
	}


	if( ((misc_item *) show_item)->m_borderColour )
	{
		if( strlen(((misc_item *) show_item)->m_borderColour) > 0 )
			gtk_entry_set_text( (GtkEntry *) wgt_txtsmiscbordcolour, ((misc_item *) show_item)->m_borderColour );
	}

	if( (int) ((misc_item *) show_item)->m_useBGSet == 1 )
	{
		gtk_toggle_button_set_active( (GtkToggleButton *) wgt_chksmiscusebgset, TRUE );
		gtk_entry_set_text( (GtkEntry *) wgt_txtsmiscrootcomm, "whiteBOX -bgs" );
	}
	else
	{
		gtk_entry_set_text( (GtkEntry *) wgt_txtsmiscrootcomm, ((misc_item *) show_item)->m_rootCommand );
	}

  }

  
}



void
on_style_colour_btn_clicked( GtkWidget *widget, gpointer txt )
{
  
  GtkWidget *wgt_txtcolour = NULL;
  gchar *s_colour;
  cdlg_struct clr;
  
  clr = colour_dialog( "Please select a colour .." );
  
  s_colour = clr.cdlg_colour;
  
  wgt_txtcolour = (GtkWidget *) txt;
  
  if( wgt_txtcolour )
    gtk_entry_set_text( (GtkEntry *) wgt_txtcolour, s_colour );  
  else
    if( DEBUG_FLAG )
      g_print( "error: styles.c\non_style_colour_btn_clicked: could not grab widget ..\n" );
}

void
on_trv_sedit_tree_select_row           (GtkCTree        *ctree,
                                        GList           *node,
                                        gint             column,
                                        gpointer         user_data)
{

  /* the different style property displays */
  GtkWidget *wgt_normframe       = lookup_widget( (GtkWidget *) ctree, "fra_menunorm" );
  GtkWidget *wgt_bulletframe     = lookup_widget( (GtkWidget *) ctree, "fra_stylebullet" );
  GtkWidget *wgt_justonlyframe   = lookup_widget( (GtkWidget *) ctree, "fra_justonly" );
  GtkWidget *wgt_fontframe       = lookup_widget( (GtkWidget *) ctree, "fra_fonts" );
  GtkWidget *wgt_wndframe        = lookup_widget( (GtkWidget *) ctree, "fra_wndframe" );
  GtkWidget *wgt_tabframe        = lookup_widget( (GtkWidget *) ctree, "fra_tabframe" );
  GtkWidget *wgt_lblnostyle      = lookup_widget( (GtkWidget *) ctree, "lbl_nonode" );
  GtkWidget *wgt_miscframe       = lookup_widget( (GtkWidget *) ctree, "fra_misc" );  

  GtkWidget *wgt_tbnseditflux    = lookup_widget( (GtkWidget *) ctree, "tbn_seditflux" );
  
  /* the window tab font widgets */
  GtkWidget *wgt_lblfonttab      = lookup_widget( (GtkWidget *) ctree, "lbl_fonttab" );
  GtkWidget *wgt_txtfonttab      = lookup_widget( (GtkWidget *) ctree, "txt_fonttab" );
  
  GtkWidget *wgt_applybutton     = lookup_widget( (GtkWidget *) ctree, "btn_styleapply" );
  
  GtkWidget *wgt_hbxsnormtext    = NULL;
  GtkWidget *wgt_lblsnormtext    = NULL;
  
  GtkWidget *wgt_cbosnormjustify = lookup_widget( (GtkWidget *) ctree, "cbo_snormjustify" );
  GtkWidget *wgt_lblsnormjustify = lookup_widget( (GtkWidget *) ctree, "lbl_snormjustify" );

  GtkWidget *wgt_vbxstyles;    /* style properties packing box */

  GtkWidget *wgt_hbxsnormdiscolour = lookup_widget( (GtkWidget *) ctree, "hbx_snormdiscolour" );
  GtkWidget *wgt_lblsnormdiscolour = lookup_widget( (GtkWidget *) ctree, "lbl_snormdiscolour" );
  GtkWidget *wgt_scwnormframe      = lookup_widget( (GtkWidget *) ctree, "scw_normframe" );


  style_item *got_item = NULL; 

  wgt_vbxstyles = lookup_widget( (GtkWidget *) ctree, "vbx_styles" );

  if( !GTK_WIDGET_VISIBLE( wgt_vbxstyles )) {
    gtk_widget_show( wgt_vbxstyles );
  }
  
  got_item = gtk_ctree_node_get_row_data( ctree, GTK_CTREE_NODE( node ));

  /* All style type (typdefs) have a common s_type as a way of linking
   * them all together ... please someone tell me if there is a better
   * way of doing this ... 
   */
  
  if( got_item ) {

	if( got_item->s_type != MENU_FRAME )
	{
		gtk_widget_hide( wgt_hbxsnormdiscolour );
		gtk_widget_hide( wgt_lblsnormdiscolour );
		gtk_scrolled_window_set_policy( (GtkScrolledWindow *) wgt_scwnormframe, GTK_POLICY_NEVER, GTK_POLICY_NEVER );

	}
	else
	{
		gtk_widget_show( wgt_hbxsnormdiscolour );
		gtk_widget_show( wgt_lblsnormdiscolour );
		gtk_scrolled_window_set_policy( (GtkScrolledWindow *) wgt_scwnormframe, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC );
	}

	

    if(( got_item->s_type == MENU_TITLE ) ||
       ( got_item->s_type == MENU_FRAME ) ||
       ( got_item->s_type == TOOL_ROOT  ))
    {
      gtk_widget_show( wgt_cbosnormjustify );
      gtk_widget_show( wgt_lblsnormjustify );      
    }
    else
    {
      gtk_widget_hide( wgt_cbosnormjustify );
      gtk_widget_hide( wgt_lblsnormjustify );
    }

	
	if( got_item->s_type != STY_MISC )
		gtk_widget_hide( wgt_miscframe );


    switch( got_item->s_type )
    {

	  case STY_MISC:
        
		if( GTK_WIDGET_VISIBLE( wgt_tabframe ))
          gtk_widget_hide( wgt_tabframe );
      
        if( GTK_WIDGET_VISIBLE( wgt_lblnostyle ))
          gtk_widget_hide( wgt_lblnostyle );
                
        if( GTK_WIDGET_VISIBLE( wgt_bulletframe )) 
          gtk_widget_hide( wgt_bulletframe );

        if( GTK_WIDGET_VISIBLE( wgt_justonlyframe ))
          gtk_widget_hide( wgt_justonlyframe );

        if( GTK_WIDGET_VISIBLE( wgt_normframe ))
          gtk_widget_hide( wgt_normframe );

        if( GTK_WIDGET_VISIBLE( wgt_fontframe ))
          gtk_widget_hide( wgt_fontframe );

		gtk_widget_show( wgt_miscframe );
        gtk_widget_show( wgt_applybutton );

		show_style_item( (GtkWidget *) ctree, (GtkCTreeNode *) node );

		break;


      case TOOL_ROOT:
      case WND_HDLFOCUS:
      case WND_HDLUFOCUS:
      case WND_GRPFOCUS:
      case WND_GRPUFOCUS:
      case WND_BTNPRESS:
      case WND_TTLFOCUS:
      case WND_TTLUFOCUS:

        if( GTK_WIDGET_VISIBLE( wgt_tabframe ))
          gtk_widget_hide( wgt_tabframe );
      
        if( GTK_WIDGET_VISIBLE( wgt_lblnostyle ))
          gtk_widget_hide( wgt_lblnostyle );
                
        if( GTK_WIDGET_VISIBLE( wgt_bulletframe )) 
          gtk_widget_hide( wgt_bulletframe );

        if( GTK_WIDGET_VISIBLE( wgt_justonlyframe ))
          gtk_widget_hide( wgt_justonlyframe );

        if( !GTK_WIDGET_VISIBLE( wgt_normframe ))
          gtk_widget_show( wgt_normframe );

        if( GTK_WIDGET_VISIBLE( wgt_fontframe ))
          gtk_widget_hide( wgt_fontframe );
		

        wgt_hbxsnormtext = lookup_widget( (GtkWidget *) ctree, "hbx_snormtextcolour" );
        wgt_lblsnormtext = lookup_widget( (GtkWidget *) ctree, "lbl_snormtextcolour" );
        
        gtk_label_set_text( (GtkLabel *) wgt_lblsnormtext, "Text Colour:" );

        gtk_widget_hide( wgt_hbxsnormtext );
        gtk_widget_hide( wgt_lblsnormtext );
        
        gtk_frame_set_label( (GtkFrame *) wgt_normframe, "Toolbar Style" );

        gtk_widget_show( wgt_applybutton );

        if( GTK_WIDGET_VISIBLE( wgt_wndframe ))
          gtk_widget_hide( wgt_wndframe );

        show_style_item( (GtkWidget *) ctree, (GtkCTreeNode *) node );
     
        break;
      
      case TOOL_LABEL:
      case TOOL_WLABEL:
      case TOOL_CLOCK:
      case TOOL_BUTTON:
      case WND_LBLFOCUS:
      case WND_LBLUFOCUS:
      case WND_BTNFOCUS:
      case WND_BTNUFOCUS:
      case TOOL_PRESSED:
      case WND_TAB_LAB_FOCUS:
      case WND_TAB_LAB_UFOCUS:
      
        if( GTK_WIDGET_VISIBLE( wgt_tabframe ))
          gtk_widget_hide( wgt_tabframe );
      
        if( GTK_WIDGET_VISIBLE( wgt_lblnostyle ))
          gtk_widget_hide( wgt_lblnostyle );
                
        if( GTK_WIDGET_VISIBLE( wgt_bulletframe )) 
          gtk_widget_hide( wgt_bulletframe );

        if( GTK_WIDGET_VISIBLE( wgt_justonlyframe ))
          gtk_widget_hide( wgt_justonlyframe );

        if( !GTK_WIDGET_VISIBLE( wgt_normframe ))
          gtk_widget_show( wgt_normframe );

        if( GTK_WIDGET_VISIBLE( wgt_fontframe ))
          gtk_widget_hide( wgt_fontframe );

        wgt_hbxsnormtext = lookup_widget( (GtkWidget *) ctree, "hbx_snormtextcolour" );
        wgt_lblsnormtext = lookup_widget( (GtkWidget *) ctree, "lbl_snormtextcolour" );
                
        if(( got_item->s_type == TOOL_BUTTON   ) ||
           ( got_item->s_type == TOOL_PRESSED  ) ||
           ( got_item->s_type == WND_BTNFOCUS  ) ||
           ( got_item->s_type == WND_BTNUFOCUS ))
        {
          gtk_label_set_text( (GtkLabel *) wgt_lblsnormtext, "Picture Colour:" );
        }
        else {
          gtk_label_set_text( (GtkLabel *) wgt_lblsnormtext, "Text Colour:" );
        }
        
        gtk_widget_show( wgt_lblsnormtext );
        gtk_widget_show( wgt_hbxsnormtext );
        
        gtk_frame_set_label( (GtkFrame *) wgt_normframe, "Toolbar Style" );
        gtk_widget_show( wgt_applybutton );

        if( GTK_WIDGET_VISIBLE( wgt_wndframe ))
          gtk_widget_hide( wgt_wndframe );

        show_style_item( (GtkWidget *) ctree, (GtkCTreeNode *) node );
        
        break;
      

      case MENU_TITLE:
      case MENU_HILITE:
      case MENU_FRAME:

        if( GTK_WIDGET_VISIBLE( wgt_tabframe ))
          gtk_widget_hide( wgt_tabframe );
      
        if( GTK_WIDGET_VISIBLE( wgt_lblnostyle ))
          gtk_widget_hide( wgt_lblnostyle );
                
        if( GTK_WIDGET_VISIBLE( wgt_bulletframe )) 
          gtk_widget_hide( wgt_bulletframe );

        if( GTK_WIDGET_VISIBLE( wgt_justonlyframe ))
          gtk_widget_hide( wgt_justonlyframe );

        if( !GTK_WIDGET_VISIBLE( wgt_normframe ))
          gtk_widget_show( wgt_normframe );

        if( GTK_WIDGET_VISIBLE( wgt_fontframe ))
          gtk_widget_hide( wgt_fontframe );

        wgt_hbxsnormtext = lookup_widget( (GtkWidget *) ctree, "hbx_snormtextcolour" );
        wgt_lblsnormtext = lookup_widget( (GtkWidget *) ctree, "lbl_snormtextcolour" );
        
        gtk_label_set_text( (GtkLabel *) wgt_lblsnormtext, "Text Colour:" );
        
        gtk_widget_show( wgt_hbxsnormtext );
        gtk_widget_show( wgt_lblsnormtext );
        
        gtk_frame_set_label( (GtkFrame *) wgt_normframe, "Menu Style" );
        gtk_widget_show( wgt_applybutton );

        if( GTK_WIDGET_VISIBLE( wgt_wndframe ))
          gtk_widget_hide( wgt_wndframe );

        show_style_item( (GtkWidget *) ctree, (GtkCTreeNode *) node );

        break;

      case MENU_BULLET:

        if( GTK_WIDGET_VISIBLE( wgt_tabframe ))
          gtk_widget_hide( wgt_tabframe );
      
        if( GTK_WIDGET_VISIBLE( wgt_lblnostyle ))
          gtk_widget_hide( wgt_lblnostyle );

        if( GTK_WIDGET_VISIBLE( wgt_normframe ))
          gtk_widget_hide( wgt_normframe );

        if( GTK_WIDGET_VISIBLE( wgt_justonlyframe ))
          gtk_widget_hide( wgt_justonlyframe );
                
        if( !GTK_WIDGET_VISIBLE( wgt_bulletframe )) 
          gtk_widget_show( wgt_bulletframe );

        if( GTK_WIDGET_VISIBLE( wgt_fontframe ))
          gtk_widget_hide( wgt_fontframe );

        gtk_frame_set_label( (GtkFrame *) wgt_bulletframe, "Menu Bullet Style" );
        gtk_widget_show( wgt_applybutton );
 
        if( GTK_WIDGET_VISIBLE( wgt_wndframe ))
          gtk_widget_hide( wgt_wndframe );

        show_style_item( (GtkWidget *) ctree, (GtkCTreeNode *) node );
       
        break;


      case WND_ROOT:

        if( GTK_WIDGET_VISIBLE( wgt_tabframe ))
          gtk_widget_hide( wgt_tabframe );
      
        if( GTK_WIDGET_VISIBLE( wgt_lblnostyle ))
          gtk_widget_hide( wgt_lblnostyle );

        if( GTK_WIDGET_VISIBLE( wgt_normframe ))
          gtk_widget_hide( wgt_normframe );
                
        if( GTK_WIDGET_VISIBLE( wgt_bulletframe )) 
          gtk_widget_hide( wgt_bulletframe );      

        if( !GTK_WIDGET_VISIBLE( wgt_justonlyframe ))
          gtk_widget_show( wgt_justonlyframe );

        if( GTK_WIDGET_VISIBLE( wgt_fontframe ))
          gtk_widget_hide( wgt_fontframe );

        if( GTK_WIDGET_VISIBLE( wgt_wndframe ))
          gtk_widget_hide( wgt_wndframe );

        gtk_widget_show( wgt_applybutton );

        show_style_item( (GtkWidget *) ctree, (GtkCTreeNode *) node );

        break;

      case WND_TAB:

      
        if( !GTK_WIDGET_VISIBLE( wgt_tabframe ))
          gtk_widget_show( wgt_tabframe );
      
        if( GTK_WIDGET_VISIBLE( wgt_lblnostyle ))
          gtk_widget_hide( wgt_lblnostyle );

        if( GTK_WIDGET_VISIBLE( wgt_normframe ))
          gtk_widget_hide( wgt_normframe );
                
        if( GTK_WIDGET_VISIBLE( wgt_bulletframe )) 
          gtk_widget_hide( wgt_bulletframe );      

        if( GTK_WIDGET_VISIBLE( wgt_justonlyframe ))
          gtk_widget_hide( wgt_justonlyframe );

        if( GTK_WIDGET_VISIBLE( wgt_fontframe ))
          gtk_widget_hide( wgt_fontframe );

        if( GTK_WIDGET_VISIBLE( wgt_wndframe ))
          gtk_widget_hide( wgt_wndframe );

        gtk_widget_show( wgt_applybutton );

        show_style_item( (GtkWidget *) ctree, (GtkCTreeNode *) node );

        break;

        
      case NOS_ITEM:

        if( GTK_WIDGET_VISIBLE( wgt_tabframe ))
          gtk_widget_hide( wgt_tabframe );

      
        if( GTK_WIDGET_VISIBLE( wgt_bulletframe )) 
          gtk_widget_hide( wgt_bulletframe );

        if( GTK_WIDGET_VISIBLE( wgt_normframe ))
          gtk_widget_hide( wgt_normframe );

        if( GTK_WIDGET_VISIBLE( wgt_justonlyframe ))
          gtk_widget_hide( wgt_justonlyframe );

        if( !GTK_WIDGET_VISIBLE( wgt_lblnostyle ))
          gtk_widget_show( wgt_lblnostyle );

        if( GTK_WIDGET_VISIBLE( wgt_fontframe ))
          gtk_widget_hide( wgt_fontframe );

        if( GTK_WIDGET_VISIBLE( wgt_wndframe ))
          gtk_widget_hide( wgt_wndframe );

        gtk_widget_hide( wgt_applybutton );

        break;


      case FNT_ITEM:
        
        if( !gtk_toggle_button_get_active((GtkToggleButton *) wgt_tbnseditflux ))
        {
          gtk_widget_hide( wgt_lblfonttab );
          gtk_widget_hide( wgt_txtfonttab );
        }
        else
        {
          gtk_widget_show( wgt_lblfonttab );
          gtk_widget_show( wgt_txtfonttab );
        }
        
        if( GTK_WIDGET_VISIBLE( wgt_tabframe ))
          gtk_widget_hide( wgt_tabframe );      
      
        if( GTK_WIDGET_VISIBLE( wgt_bulletframe )) 
          gtk_widget_hide( wgt_bulletframe );

        if( GTK_WIDGET_VISIBLE( wgt_normframe ))
          gtk_widget_hide( wgt_normframe );

        if( GTK_WIDGET_VISIBLE( wgt_justonlyframe ))
          gtk_widget_hide( wgt_justonlyframe );

        if( GTK_WIDGET_VISIBLE( wgt_lblnostyle ))
          gtk_widget_hide( wgt_lblnostyle );

        if( !GTK_WIDGET_VISIBLE( wgt_fontframe ))
          gtk_widget_show( wgt_fontframe );

        if( GTK_WIDGET_VISIBLE( wgt_wndframe ))
          gtk_widget_hide( wgt_wndframe );

        gtk_widget_show( wgt_applybutton );

        show_style_item( (GtkWidget *) ctree, (GtkCTreeNode *) node );

        break;

      case WND_FRMFOCUS:

        if( GTK_WIDGET_VISIBLE( wgt_tabframe ))
          gtk_widget_hide( wgt_tabframe );      
      
        if( GTK_WIDGET_VISIBLE( wgt_bulletframe )) 
          gtk_widget_hide( wgt_bulletframe );

        if( GTK_WIDGET_VISIBLE( wgt_normframe ))
          gtk_widget_hide( wgt_normframe );

        if( GTK_WIDGET_VISIBLE( wgt_justonlyframe ))
          gtk_widget_hide( wgt_justonlyframe );

        if( GTK_WIDGET_VISIBLE( wgt_lblnostyle ))
          gtk_widget_hide( wgt_lblnostyle );

        if( GTK_WIDGET_VISIBLE( wgt_fontframe ))
          gtk_widget_hide( wgt_fontframe );

        if( !GTK_WIDGET_VISIBLE( wgt_wndframe ))
          gtk_widget_show( wgt_wndframe );

        gtk_widget_show( wgt_applybutton );

        show_style_item( (GtkWidget *) ctree, (GtkCTreeNode *) node );


    }
  }

}

void
on_trv_sedit_tree_unselect_row         (GtkCTree        *ctree,
                                        GList           *node,
                                        gint             column,
                                        gpointer         user_data)
{

  /* gotta review my position on this .... too much flicker? */

  GtkWidget *wgt_vbxstyles;

  wgt_vbxstyles = lookup_widget( GTK_WIDGET( ctree ), "vbx_styles" );
  gtk_widget_hide( wgt_vbxstyles );

}

void
on_btn_styleapply_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{

  /*
   * IMPORTANT:
   * this needs to be re done and optimized eventually.
   *  includes:
   *    - 2nd if = else ?
   *    - check for availibility of values
   *    - writing of NULLS
   */


  GtkCTreeNode *selection;		   /* current item to update */
  GtkCTree     *tree;			   /* the menu tree */

  /* get the style attribs */
  GtkWidget    *wgt_txtsbulltype;          /* bullet type */
  GtkWidget    *wgt_txtsbullpos;           /* bullet position */
  GtkWidget    *wgt_txtsnormcolour;
  GtkWidget    *wgt_txtsnormcolourto;
  GtkWidget    *wgt_chksnormgradient;
  GtkWidget    *wgt_txtcbosnormgtype;
  GtkWidget    *wgt_chksnorminterlaced;
  GtkWidget    *wgt_txtcbosnormbevel;
  GtkWidget    *wgt_txtsnormtextcolour;
  GtkWidget    *wgt_txtcbojustonly;
  GtkWidget    *wgt_txtcbosnormjustify;
  GtkWidget    *wgt_txtsnormdiscolour;

  /* style data */
  char *b_type;           /* bullet type */
  char *b_pos;            /* bullet position */
  char *str_colourTo;     /* if a gradient -- the to colour */
  char *item_justify;     /* item justification */
  int gradient_flag;      /* gradient ? */
  int interlace_flag;     /* interlaced ? */

  char *text_colour;
  char *n_colour;
  char *bevel;
  char *grad_type;
  char *dis_colour = "";

  style_item *s_apply    = NULL;

  style_item *style_item = NULL;
  font_item  *f_main      = NULL;
  misc_item  *m_item	 = NULL;
  
  int item_type;


  tree = GTK_CTREE( lookup_widget( GTK_WIDGET( button ), "trv_sedit" ));
  selection = (GTK_CLIST( tree )->selection )->data;

  assert( selection );		/* check for those elusive errors */
  if( selection )
  {
   
	s_apply = gtk_ctree_node_get_row_data( tree, selection );
	item_type = s_apply->s_type;  

    if( item_type == WND_FRMFOCUS )
    {

      GtkWidget *wgt_txtfrmfocuscol  = lookup_widget( (GtkWidget *) button, "txt_frmfocuscol" );
      GtkWidget *wgt_txtfrmufocuscol = lookup_widget( (GtkWidget *) button, "txt_frmufocuscol" );

      style_item = new_style_item( item_type, NULL, 0, NULL, 0, 
                                   gtk_entry_get_text( (GtkEntry *) wgt_txtfrmfocuscol ),
                                   gtk_entry_get_text( (GtkEntry *) wgt_txtfrmufocuscol ),
                                   NULL, NULL, NULL, NULL, NULL );

    }

	else if( item_type == STY_MISC )
	{
		GtkWidget *wgt_txtsmiscbordwidth   = lookup_widget( (GtkWidget *) button, "txt_smiscbordwidth" );
		GtkWidget *wgt_txtsmiscbevelwidth  = lookup_widget( (GtkWidget *) button, "txt_smiscbevelwidth" );
		GtkWidget *wgt_txtsmischandlewidth = lookup_widget( (GtkWidget *) button, "txt_smischandlewidth" );
		GtkWidget *wgt_txtsmiscframewidth  = lookup_widget( (GtkWidget *) button, "txt_smiscframewidth" );
		GtkWidget *wgt_txtsmiscbordcolour  = lookup_widget( (GtkWidget *) button, "txt_smiscbordcolour" );
		GtkWidget *wgt_chksmiscusebgset    = lookup_widget( (GtkWidget *) button, "chk_smiscusebgset" );
		GtkWidget *wgt_txtsmiscrootcomm    = lookup_widget( (GtkWidget *) button, "txt_smiscrootcomm" );

		int m_borderWidth = -9999;
		char *m_borderColour = NULL;
		int m_bevelWidth = -9999;
		int m_handleWidth = -9999;
		int m_frameWidth = -9999;
		int m_BGSet = -9999;
		char *m_rootCommand  = NULL;

		m_borderWidth  = atoi( gtk_entry_get_text( (GtkEntry *) wgt_txtsmiscbordwidth ));
		if( strlen(gtk_entry_get_text( (GtkEntry *) wgt_txtsmiscbordcolour )) > 0 )
			m_borderColour = gtk_entry_get_text( (GtkEntry *) wgt_txtsmiscbordcolour );
		m_bevelWidth   = atoi( gtk_entry_get_text( (GtkEntry *) wgt_txtsmiscbevelwidth ));
		m_handleWidth  = atoi( gtk_entry_get_text( (GtkEntry *) wgt_txtsmischandlewidth ));
		m_frameWidth   = atoi( gtk_entry_get_text( (GtkEntry *) wgt_txtsmiscframewidth ));		

		if( m_borderWidth < 0 && strlen(gtk_entry_get_text( (GtkEntry *) wgt_txtsmiscbordwidth )) > 0 )
		{
			message_box( 4, "The border width must be a number > 0.", "Error!" );
			return;
		}

		if( m_bevelWidth <= 0 && strlen(gtk_entry_get_text( (GtkEntry *) wgt_txtsmiscbevelwidth )) > 0 )
		{
			message_box( 4, "The bevel width must be a number > 0.", "Error!" );
			return;
		}

		if( m_handleWidth <= 0 && strlen(gtk_entry_get_text( (GtkEntry *) wgt_txtsmischandlewidth )) > 0 )
		{
			message_box( 4, "The handle width must be a number > 0.", "Error!" );
			return;
		}

		if( m_frameWidth < 0 && strlen(gtk_entry_get_text( (GtkEntry *) wgt_txtsmiscframewidth )) > 0 )
		{
			message_box( 4, "The frame width must be a number > 0.", "Error!" );
			return;
		}



		if( gtk_toggle_button_get_active( (GtkToggleButton *) wgt_chksmiscusebgset ))
			m_BGSet = 1;
		else
			m_BGSet = 0;
	
		if( m_BGSet == 0 )
		{
			if( strlen(gtk_entry_get_text( (GtkEntry *) wgt_txtsmiscrootcomm )) > 0 )
				m_rootCommand  = gtk_entry_get_text( (GtkEntry *) wgt_txtsmiscrootcomm );
		}
		else
			m_rootCommand  = "whiteBOX -bgs";

		m_item = new_misc_item( STY_MISC, m_borderWidth, m_borderColour, m_bevelWidth, m_handleWidth, 
								   m_frameWidth, m_BGSet, m_rootCommand );
		

	}

    else if( item_type == FNT_ITEM )
    {

      GtkWidget *wgt_txtfontmnutitle = lookup_widget( (GtkWidget *) button, "txt_fontmnutitle" );
      GtkWidget *wgt_txtfontmnuframe = lookup_widget( (GtkWidget *) button, "txt_fontmnuframe" );
      GtkWidget *wgt_txtfontwindow   = lookup_widget( (GtkWidget *) button, "txt_fontwindow" );
      GtkWidget *wgt_txtfonttoolbar  = lookup_widget( (GtkWidget *) button, "txt_fonttoolbar" );
      GtkWidget *wgt_txtfontglobal   = lookup_widget( (GtkWidget *) button, "txt_fontglobal" );
      GtkWidget *wgt_txtfonttab      = lookup_widget( (GtkWidget *) button, "txt_fonttab" );

      char *tmp_font[6];

      tmp_font[0] = gtk_entry_get_text( (GtkEntry *) wgt_txtfontmnutitle );
      tmp_font[1] = gtk_entry_get_text( (GtkEntry *) wgt_txtfontmnuframe );
      tmp_font[2] = gtk_entry_get_text( (GtkEntry *) wgt_txtfontwindow );
      tmp_font[3] = gtk_entry_get_text( (GtkEntry *) wgt_txtfonttoolbar );
      tmp_font[4] = gtk_entry_get_text( (GtkEntry *) wgt_txtfonttab );
      tmp_font[5] = gtk_entry_get_text( (GtkEntry *) wgt_txtfontglobal );
      
   
      if( !tmp_font[5] || strlen( tmp_font[5] ) == 0 )
        f_main = new_font_item( FNT_ITEM, tmp_font[0], tmp_font[1], tmp_font[2], tmp_font[3], tmp_font[4], NULL );
      else
        f_main = new_font_item( FNT_ITEM, NULL, NULL, NULL, NULL, NULL, tmp_font[5] );
      
    }
    
    /* do bullet application */
    if( item_type == MENU_BULLET )
    {
      wgt_txtsbulltype = lookup_widget( (GtkWidget *) button, "txt_cbosbulltype" );
      wgt_txtsbullpos  = lookup_widget( (GtkWidget *) button, "txt_cbosbullpos" );
    
      b_type = gtk_entry_get_text( (GtkEntry *) wgt_txtsbulltype );
      b_pos  = gtk_entry_get_text( (GtkEntry *) wgt_txtsbullpos );
      style_item = new_style_item( MENU_BULLET, NULL, 0, NULL, 0, NULL, NULL, NULL, 
                                   NULL, b_type, b_pos, NULL );
    }

    if( item_type == WND_TAB )
    {
      
      GtkWidget *wgt_txttabjustify;
      GtkWidget *wgt_txtfrmtabborderwidth;
      GtkWidget *wgt_txtfrmtabbordercolour;
      
      int   t_border;
      char *t_colour;
      char *t_justify;
      
      wgt_txttabjustify = lookup_widget( (GtkWidget *) button, "txt_tabjustify" );
      wgt_txtfrmtabborderwidth = lookup_widget( (GtkWidget *) button, "txt_frmtabborderwidth" );
      wgt_txtfrmtabbordercolour  = lookup_widget( (GtkWidget *) button, "txt_frmtabbordercolour" );
      
      t_border  = atoi( gtk_entry_get_text( (GtkEntry *) wgt_txtfrmtabborderwidth ));
      t_colour  = gtk_entry_get_text( (GtkEntry *) wgt_txtfrmtabbordercolour );
      t_justify = gtk_entry_get_text( (GtkEntry *) wgt_txttabjustify );
                   
      flx_tabstyle = new_style_item( item_type, NULL, 0, NULL, t_border, 
                                   t_colour, NULL, NULL, t_justify, NULL, NULL, NULL );      
      
    }
      
    if(( item_type == TOOL_LABEL    ) || 
       ( item_type == TOOL_WLABEL   ) || 
       ( item_type == TOOL_CLOCK    ) || 
       ( item_type == TOOL_BUTTON   ) ||
       ( item_type == TOOL_ROOT     ) ||
       ( item_type == TOOL_PRESSED  ) ||
       ( item_type == MENU_TITLE    ) ||
       ( item_type == MENU_HILITE   ) ||
       ( item_type == MENU_FRAME    ) ||
       ( item_type == WND_TTLFOCUS  ) ||
       ( item_type == WND_TTLUFOCUS ) ||
       ( item_type == WND_LBLFOCUS  ) ||
       ( item_type == WND_LBLUFOCUS ) ||
       ( item_type == WND_HDLFOCUS  ) ||
       ( item_type == WND_HDLUFOCUS ) ||
       ( item_type == WND_GRPFOCUS  ) ||
       ( item_type == WND_GRPUFOCUS ) ||
       ( item_type == WND_BTNFOCUS  ) ||
       ( item_type == WND_BTNUFOCUS ) ||
       ( item_type == WND_BTNPRESS  ) ||
       ( item_type == WND_ROOT )      ||
       ( item_type == WND_TAB_LAB_FOCUS ) ||
       ( item_type == WND_TAB_LAB_UFOCUS ))

    {

      wgt_txtcbosnormbevel    = lookup_widget( (GtkWidget *) button, "txt_cbosnormbevel" );
      wgt_txtsnormcolour      = lookup_widget( (GtkWidget *) button, "txt_snormcolour" );
      wgt_txtsnormcolourto    = lookup_widget( (GtkWidget *) button, "txt_snormcolourto" );
      wgt_chksnormgradient    = lookup_widget( (GtkWidget *) button, "chk_snormgradient" );
      wgt_txtcbosnormgtype    = lookup_widget( (GtkWidget *) button, "txt_cbosnormgtype" );
      wgt_chksnorminterlaced  = lookup_widget( (GtkWidget *) button, "chk_snorminterlaced" );
      wgt_txtsnormtextcolour  = lookup_widget( (GtkWidget *) button, "txt_snormtextcolour" );
      wgt_txtcbojustonly      = lookup_widget( (GtkWidget *) button, "txt_justonly" );
      wgt_txtcbosnormjustify  = lookup_widget( (GtkWidget *) button, "txt_cbosnormjustify" );
	  
	  /* grab disabled colour for menu frame */
	  if( s_apply->s_type == MENU_FRAME )
	  {
		wgt_txtsnormdiscolour   = lookup_widget( (GtkWidget *) button, "txt_snormdiscolour" );
		dis_colour = gtk_entry_get_text( (GtkEntry *) wgt_txtsnormdiscolour ); 
	  }

      gradient_flag  = (int) gtk_toggle_button_get_active( (GtkToggleButton *) wgt_chksnormgradient );
      interlace_flag = (int) gtk_toggle_button_get_active( (GtkToggleButton *) wgt_chksnorminterlaced );

      text_colour    = gtk_entry_get_text( (GtkEntry *) wgt_txtsnormtextcolour );
      n_colour       = gtk_entry_get_text( (GtkEntry *) wgt_txtsnormcolour );
      str_colourTo   = gtk_entry_get_text( (GtkEntry *) wgt_txtsnormcolourto );
      bevel          = gtk_entry_get_text( (GtkEntry *) wgt_txtcbosnormbevel );
      grad_type      = gtk_entry_get_text( (GtkEntry *) wgt_txtcbosnormgtype );

      /* FOR MY REFERENCE ONLY ---------------------------------
        new_item->s_type         = type;
        new_item->s_bevel        = g_strdup( bevel );
        new_item->s_gradient     = gradient;
        new_item->s_gradientType = g_strdup( gradientType );
        new_item->s_interlaced   = interlaced;
        new_item->s_colour       = g_strdup( colour );
        new_item->s_colourTo     = g_strdup( colourTo );
        new_item->s_textColour   = g_strdup( textColour );
        new_item->s_justify      = g_strdup( justify );
        new_item->s_bulletType   = g_strdup( bulletType );
        new_item->s_bulletPos    = g_strdup( bulletPos );
	    new_item->s_disColour    = g_strdup( disColour );
       ---------------------------------------------------------- */
      
      //if( !gradient_flag ) {
      //  str_colourTo = NULL;
      //}
      //else {
      //}

      if( s_apply->s_type == WND_ROOT )
        item_justify =  gtk_entry_get_text( (GtkEntry *) wgt_txtcbojustonly );
      else
        item_justify =  gtk_entry_get_text( (GtkEntry *) wgt_txtcbosnormjustify );

      if( strlen( item_justify ) == 0 )
        item_justify = NULL;

      if( strlen( text_colour ) == 0 )
        text_colour = NULL;

      if( strlen( n_colour ) == 0 )
        n_colour = NULL;

      if( strlen( str_colourTo ) == 0 )
        str_colourTo = NULL;

      if( strlen( bevel ) == 0 )
        bevel = NULL;

      if( strlen( grad_type ) == 0 )
        grad_type = NULL;

       /* REFERENCE ONLY --
    - flx_tabstyle;
    - flx_lblfocusstyle;
    - flx_lblufocusstyle;
  */
      
      if( s_apply->s_type == WND_TAB_LAB_FOCUS )
      {
        flx_lblfocusstyle = new_style_item( item_type, bevel, gradient_flag, grad_type,
                                   interlace_flag, n_colour, str_colourTo, text_colour, item_justify,
                                   NULL, NULL, NULL );
      }
      else if( s_apply->s_type == WND_TAB_LAB_UFOCUS )
      {
        flx_lblufocusstyle = new_style_item( item_type, bevel, gradient_flag, grad_type,
                                   interlace_flag, n_colour, str_colourTo, text_colour, item_justify,
                                   NULL, NULL, NULL );
      }
	  else if( s_apply->s_type == MENU_FRAME )
	  {
        style_item = new_style_item( item_type, bevel, gradient_flag, grad_type,
                                   interlace_flag, n_colour, str_colourTo, text_colour, item_justify,
                                   NULL, NULL, dis_colour );		  
	  }
      else 
      {
        style_item = new_style_item( item_type, bevel, gradient_flag, grad_type,
                                   interlace_flag, n_colour, str_colourTo, text_colour, item_justify,
                                   NULL, NULL, NULL );
      }

    }        

  
  if( item_type != FNT_ITEM )
  {

	if( item_type == STY_MISC )
		gtk_ctree_node_set_row_data( tree, selection, m_item );
	else
	{
		if( item_type == WND_TAB )
		  gtk_ctree_node_set_row_data( tree, selection, flx_tabstyle );
		else if( item_type == WND_TAB_LAB_FOCUS )
		  gtk_ctree_node_set_row_data( tree, selection, flx_lblfocusstyle );
		else if( item_type == WND_TAB_LAB_UFOCUS )
		  gtk_ctree_node_set_row_data( tree, selection, flx_lblufocusstyle );
		else
		  gtk_ctree_node_set_row_data( tree, selection, style_item );
    }
  }
  else
    gtk_ctree_node_set_row_data( tree, selection, f_main );

  }

}

void
on_tbn_stylesave_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget *main_ptr;
  GtkWidget *wgt_styletree;
  GtkCTreeNode *top;

  GtkWidget *wgt_lbl_curstyle = lookup_widget( (GtkWidget *) button, "lbl_curstyle" );
  
  gchar *file = NULL;
  
  FILE *out_file;
  
  struct stat s;

  
  main_ptr = GTK_WIDGET( button );
    
  if( gblopen_stylename == NULL )
  {

    file = return_file_w_title( NULL, "Save style file .." );

    if( file )
    {

      out_file = fopen( file, "w" );    /* open the file for writing */
      
      if( out_file )
      {

        if (( stat( file, &s ) == -1 ) || ( S_ISDIR( s.st_mode )))
       {
         if( message_box( 4, "The file you have selected is invalid, a directory,\nor you do not have write permission.\n\nPlease select a new file and try again.", "Invalid File" ) == MBX_OK )
           return;
       }
       else
       {          
        
          fprintf( out_file, "! style name: <goes here>\n" );
          fprintf( out_file, "! copyright:  <author>\n\n" );

          fprintf( out_file, "! This style was generated using whiteBOX.\n" );
          fprintf( out_file, "! See the whiteBOX homepage for more information.\n" );
          fprintf( out_file, "! http://whitebox.sourceforge.net/\n\n" );        

          wgt_styletree = lookup_widget( (GtkWidget *) main_ptr, "trv_sedit" );
          top = gtk_ctree_node_nth( GTK_CTREE( wgt_styletree ), 0 );
       
          save_style( top, wgt_styletree, out_file );
          gblopen_stylename = g_strdup( file );
          gtk_label_set_text( (GtkLabel *) wgt_lbl_curstyle, file );
  
          fclose( out_file );        
        }
      }
      else
      {
        /* cannot write to the file */
        message_box( 4, "There was an error writing to the specified file.\n\nPlease check you have permission and try again.",
                     "Error writing file .." );
      }
    }
  }
  else
  {

    if (( stat( gblopen_stylename, &s ) == -1 ) || ( S_ISDIR( s.st_mode )))
    {
      if( message_box( 4, "The file you have selected is invalid, a directory,\nor you do not have write permission.\n\nPlease select a new file and try again.", "Invalid File" ) == MBX_OK )
        return;
    }
    else
    {                 
      out_file = fopen( gblopen_stylename, "w" );    /* open the file for writing */

      if( out_file ) 
      {
      
        fprintf( out_file, "! style name: <goes here>\n" );
        fprintf( out_file, "! copyright:  <author>\n\n" );

        fprintf( out_file, "! This style was generated using whiteBOX.\n" );
        fprintf( out_file, "! See the whiteBOX homepage for more information.\n" );
        fprintf( out_file, "! http://whitebox.sourceforge.net/\n\n" );

        wgt_styletree = lookup_widget( (GtkWidget *) main_ptr, "trv_sedit" );
        top = gtk_ctree_node_nth( GTK_CTREE( wgt_styletree ), 0 );
       
        save_style( top, wgt_styletree, out_file );
        gblopen_stylename = g_strdup( gblopen_stylename );

        fclose( out_file );
      }
      else
      {

        /* cannot write to the file */
        message_box( 4, "There was an error writing to the specified file.\n\nPlease check you have permission and try again.",
                     "Error writing file .." );

      }
     
    } 

  }
  
}

void
on_tbn_stylesaveas_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget *main_ptr;
  GtkWidget *wgt_styletree;
  GtkCTreeNode *top;

  GtkWidget *wgt_lbl_curstyle = lookup_widget( (GtkWidget *) button, "lbl_curstyle" );
  
  gchar *file = NULL;
  
  FILE *out_file;
  
  struct stat s;

  
  main_ptr = GTK_WIDGET( button );

  file = return_file_w_title( NULL, "Save style file .." );
  
  if( file )
  {

    out_file = fopen( file, "w" );    /* open the file for writing */
    
    if( out_file )
    {

      if (( stat( file, &s ) == -1 ) || ( S_ISDIR( s.st_mode )))
      {
        if( message_box( 4, "The file you have selected is invalid, a directory,\nor you do not have write permission.\n\nPlease select a new file and try again.", "Invalid File" ) == MBX_OK )
           return;
      }
      else
      {          
        
        fprintf( out_file, "! style name: <goes here>\n" );
        fprintf( out_file, "! copyright:  <author>\n\n" );

        fprintf( out_file, "! This style was generated using whiteBOX.\n" );
        fprintf( out_file, "! See the whiteBOX homepage for more information.\n" );
        fprintf( out_file, "! http://whitebox.sourceforge.net/\n\n" );        

        wgt_styletree = lookup_widget( (GtkWidget *) main_ptr, "trv_sedit" );
        top = gtk_ctree_node_nth( GTK_CTREE( wgt_styletree ), 0 );
       
        save_style( top, wgt_styletree, out_file );
        gblopen_stylename = g_strdup( file );
        gtk_label_set_text( (GtkLabel *) wgt_lbl_curstyle, file );
  
        fclose( out_file );        
      }
    }
    else
    {
      /* cannot write to the file */
      message_box( 4, "There was an error writing to the specified file.\n\nPlease check you have permission and try again.",
                   "Error writing file .." );
    }
  }
  
}


void
on_tbn_styleload_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget *wgt_styletree    = lookup_widget( (GtkWidget *) button, "trv_sedit" );
  GtkCTreeNode *top              = gtk_ctree_node_nth( GTK_CTREE( wgt_styletree ), 0 );
  GtkWidget *wgt_lbl_curstyle = lookup_widget( (GtkWidget *) button, "lbl_curstyle" );

  gchar *file = NULL;
  
  struct stat s;

  file = return_file_w_title( NULL, "Load style file .." );
   
  if( file )
  {
    if( DEBUG_FLAG )
      g_print( "filename = %s\n", file );
    
    if (( stat( file, &s ) == -1 ) || ( S_ISDIR( s.st_mode )))
    {
      if( message_box( 4, "The file you have selected is invalid, a directory,\nor you do not have write permission.\n\nPlease select a new file and try again.", "Invalid File" ) == MBX_OK )
        return;
    }
    else
    {  
      /* open style file */
      gblopen_stylename = g_strdup( file );
      gtk_label_set_text( (GtkLabel *) wgt_lbl_curstyle, file );      
      
      load_style( file, wgt_styletree, top );
     
    }

  }      
  
}

void
on_tbn_seditflux_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

  char style_rc[255];
  XrmDatabase n_database = (XrmDatabase) 0;
  XrmDatabase o_database = (XrmDatabase) 0;

  GtkCTree *wgt_styletree = (GtkCTree *) lookup_widget( (GtkWidget *) togglebutton, "trv_sedit" );
 
  /* the window tab font widgets */
  GtkWidget *wgt_lblfonttab = lookup_widget( (GtkWidget *) togglebutton, "lbl_fonttab" );
  GtkWidget *wgt_txtfonttab = lookup_widget( (GtkWidget *) togglebutton, "txt_fonttab" );
    
  
  gchar *res_nme = "style.ShowFluxbox";
  gchar *res_val = NULL;

  GtkCTreeNode *nde_addnode;
  char *styles_label;
  static style_item *style_item;


  /* get the current state of the button */
  if( gtk_toggle_button_get_active( togglebutton ))
    res_val = "1";
  else  
    res_val = "0";


  /* REFERENCE ONLY --
    - flx_tabstyle;
    - flx_lblfocusstyle;
    - flx_lblufocusstyle;
  */


  /* check to see if fluxbox defaults have not been set */
  if(( !flx_tabstyle ) ||
     ( !flx_lblfocusstyle ) ||
     ( !flx_lblufocusstyle ))
  {
  
    /* create the default fluxbox styles */
    flx_tabstyle       = new_style_item( WND_TAB, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    flx_lblfocusstyle  = new_style_item( WND_TAB_LAB_FOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    flx_lblufocusstyle = new_style_item( WND_TAB_LAB_UFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
  
  }
    

  /* show fluxbox styles */
  if( gtk_toggle_button_get_active( togglebutton ))
  {

    /* show the fluxbox fonts stuff */    
    gtk_widget_show( wgt_lblfonttab );
    gtk_widget_show( wgt_txtfonttab );
    
    
    /* fluxbox tab styles */
    styles_label = ".tab";
    nde_addnode = gtk_ctree_insert_node( wgt_styletree, nde_fluxadd, NULL, &styles_label, 4,
	                              				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    
    style_item = flx_tabstyle;
    //style_item = new_style_item( WND_TAB, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( wgt_styletree, nde_addnode, style_item );
    nde_fluxpos = nde_addnode;
      

    styles_label = ".label";
		nde_addnode = gtk_ctree_insert_node( wgt_styletree, nde_addnode, NULL, &styles_label, 4,
		                              				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = new_style_item( NOS_ITEM, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( wgt_styletree, nde_addnode, style_item );
  
    styles_label = ".focus";
		nde_addnode = gtk_ctree_insert_node( wgt_styletree, nde_addnode, NULL, &styles_label, 4,
		                             				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = flx_lblfocusstyle;
    //style_item = new_style_item( WND_TAB_LAB_FOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( wgt_styletree, nde_addnode, style_item );

    styles_label = ".unfocus";
		nde_addnode = gtk_ctree_insert_node( wgt_styletree, GTK_CTREE_ROW(nde_addnode)->parent, NULL, &styles_label, 4,
		                             				 NULL, NULL, NULL, NULL, FALSE, FALSE );
    style_item = flx_lblufocusstyle;
    //style_item = new_style_item( WND_TAB_LAB_UFOCUS, NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL );
    gtk_ctree_node_set_row_data( wgt_styletree, nde_addnode, style_item );
       
    /* end fluxbox styles */
      
  }
  else
  {

    /* hide the fluxbox fonts stuff */
    gtk_widget_hide( wgt_lblfonttab );
    gtk_widget_hide( wgt_txtfonttab );
    
    gtk_ctree_remove_node( wgt_styletree, nde_fluxpos );  
  }


  
  /* get the filename of the style options database */
  strncpy( style_rc, getenv("HOME"), 255 );
  strncat( style_rc, "/.whiteBOX/stylerc", 255 );

  /* grab 2 copies of the database - new and old */
  o_database = XrmGetFileDatabase( style_rc );
  n_database = XrmGetFileDatabase( style_rc );

  /* set the option */
  XrmPutStringResource( &n_database,  res_nme, res_val );
  
  /* re-write the options */
  XrmMergeDatabases ( n_database, &o_database );
  XrmPutFileDatabase( o_database, style_rc );
  XrmDestroyDatabase( o_database );


}

