/*
 * styles.h
 * Styles header file
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>

/* define numeric style types */
#define TOOL_ROOT     100
#define TOOL_LABEL    101
#define TOOL_WLABEL   102
#define TOOL_BUTTON   103
#define TOOL_PRESSED  104
#define TOOL_CLOCK    105

#define MENU_TITLE    106
#define MENU_BULLET   107
#define MENU_FRAME    108
#define MENU_HILITE   109

#define WND_ROOT      110
#define WND_TTLFOCUS  111
#define WND_TTLUFOCUS 112
#define WND_LBLFOCUS  113
#define WND_LBLUFOCUS 114
#define WND_HDLFOCUS  115
#define WND_HDLUFOCUS 116
#define WND_GRPFOCUS  117
#define WND_GRPUFOCUS 118
#define WND_BTNFOCUS  119
#define WND_BTNUFOCUS 120
#define WND_BTNPRESS  121
#define WND_FRMFOCUS  122
#define WND_FRMUFOCUS 123

#define NOS_ITEM      124
#define FNT_ITEM      125


/* fluxbox styles */

#define WND_TAB            126
#define WND_TAB_LAB_FOCUS  127
#define WND_TAB_LAB_UFOCUS 128

#define STY_MISC			129

typedef struct
{
  int    s_type;
  char	*s_bevel;
  int    s_gradient;
  char	*s_gradientType;
  int    s_interlaced;
  char	*s_colour;
  char  *s_colourTo;
  char  *s_textColour;
  char  *s_justify;
  char  *s_bulletType;
  char  *s_bulletPos;
  char  *s_disColour;
} style_item;

typedef struct
{
  int   s_type;
  char *f_menutitle;
  char *f_menuframe;
  char *f_window;
  char *f_toolbar;
  char *f_wintab;
  char *f_global;
} font_item;

typedef struct
{
  int	s_type;
  int   m_borderWidth;
  char *m_borderColour;
  int   m_bevelWidth;
  int   m_handleWidth;
  int   m_frameWidth;
  int   m_useBGSet;
  char *m_rootCommand;
} misc_item;


typedef struct
{
  int    mreload_flag;
  int    mmake_bak;
} config_info;


gchar *gblopen_stylename;


font_item *
new_font_item( int type, const char *f_mtitle, const char *f_mframe,
          const char *f_wnd, const char *f_tbr, const char *f_tab, char *f_gbl );

misc_item *
new_misc_item( int type, int m_bWidth, const char *m_bColour, int m_bvWidth,
          int m_hWidth, int m_fWidth, int m_uBGSet, const char *m_rCommand );

style_item *
new_style_item( int type, const char *bevel, int gradient, const char *gradientType,
                int interlaced, const char *colour, const char *colourTo,
                const char *textColour, const char *justify, const char *bulletType,
                const char *bulletPos, const char *disColour );

void
show_defaults( GtkWidget *widget, GtkWidget *vbx, GtkWidget *wnd_main, int show_flux );

void
load_style( gchar* style_file, GtkWidget *tree, GtkCTreeNode *root_node );

void
save_menu( GtkCTreeNode *root_node, GtkWidget *tree, FILE *save_file );

void
save_style( GtkCTreeNode *root_node, GtkWidget *tree, FILE *save_file );


void
on_style_colour_btn_clicked( GtkWidget *widget, gpointer txt );


void
on_trv_sedit_tree_select_row           (GtkCTree        *ctree,
                                        GList           *node,
                                        gint             column,
                                        gpointer         user_data);                                        
                                        
void
on_trv_sedit_tree_unselect_row         (GtkCTree        *ctree,
                                        GList           *node,
                                        gint             column,
                                        gpointer         user_data);

void
on_btn_styleapply_clicked              (GtkButton       *button,
                                        gpointer         user_data);
                                        
void
on_tbn_stylesave_clicked               (GtkButton       *button,
                                        gpointer         user_data);

void
on_tbn_stylesaveas_clicked             (GtkButton       *button,
                                        gpointer         user_data);

void
on_tbn_styleload_clicked               (GtkButton       *button,
                                        gpointer         user_data);
                                        
void
on_tbn_seditflux_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data);
