/*
 * wbmain.c
 * Main window callbacks and stuff
 *
 * Author: 	Luke Freeman
 * Copyright 2000 - 2002 Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <strings.h>

#include "wbmain.h"
#include "interface.h"
#include "support.h"

#define MENU_LOC "/usr/local/share/Blackbox/menu"


void
on_mnu_file_options_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

  GtkWidget *wgt_wndoptions; 
  GtkWidget *wgt_main;

  wgt_wndoptions = create_wnd_options();
  wgt_main = lookup_widget( (GtkWidget *) menuitem, "wnd_main" );
  gtk_object_set_data( (GtkObject *) wgt_wndoptions, "wnd_main", wgt_main );  
    
  gtk_widget_show( wgt_wndoptions ); 

}


void
on_mnu_file_exit_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

  gtk_main_quit();

}


void
on_mnu_about_licence_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *wnd_info;
  GtkWidget *wgt_content;

  GdkFont *fnt_title;
  GdkFont *fnt_subtitle;  
  GdkFont *fnt_body;  
  
  char in_line[1024];
    
  FILE *licence_info = fopen( PACKAGEDIR "/docs/gpl.txt", "r" );
  
  /* check to see if the licence information can be read --- change this check */
  if( !licence_info )
    return;
  
  fnt_title    = gdk_font_load( "-adobe-helvetica-bold-r-normal-*-*-180-*-*-p-*-iso8859-1" );
  fnt_subtitle = gdk_font_load( "-b&h-lucida-bold-r-normal-*-*-100-*-*-p-*-iso8859-1" );
  fnt_body     = gdk_font_load( "-*-courier-medium-r-normal-*-*-120-*-*-m-*-iso8859-1" );

  wnd_info = create_dlg_info();
  gtk_window_set_title( GTK_WINDOW( wnd_info ), "info: Licence" );
  gtk_window_set_default_size( GTK_WINDOW( wnd_info ), 575, 260 );
  /* 575 x 260 */
  gtk_widget_show( wnd_info );

  wgt_content = lookup_widget( wnd_info, "txt_infocontent" );
  gtk_text_insert( GTK_TEXT( wgt_content ), fnt_title, NULL, NULL, "\n Licence Information\n\n", -1 );
  gtk_text_insert( GTK_TEXT( wgt_content ), fnt_subtitle, NULL, NULL, " GNU Public Licence\n\n", -1 );
 
  gtk_text_freeze( GTK_TEXT( wgt_content ));
  while( fgets( in_line, 1024, licence_info ))
    gtk_text_insert( GTK_TEXT( wgt_content ), fnt_body, NULL, NULL, in_line, -1 );  

  gtk_text_thaw( GTK_TEXT( wgt_content ));
  fclose( licence_info );
  
  gdk_font_unref( fnt_title );
  gdk_font_unref( fnt_subtitle );
  gdk_font_unref( fnt_body );


}


void
on_mnu_about_website_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  /*
   * NOTE: this is a pretty shoddy way of doing things and will 
   * 	   be addressed later.
   */
	
  char *href = "http://whitebox.sourceforge.net/";
  char *run_string = g_strdup_printf( "mozilla %s &", href );

  system( run_string );
  g_free( run_string );

}


void
on_mnu_about_prog_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

	GtkWidget *wgt_dlgabout;
	
	wgt_dlgabout = create_dlg_about();
	gtk_widget_show( wgt_dlgabout );

}


void
on_btn_quit_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{

  gtk_main_quit();

}



void
on_wnd_main_show                       (GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void
on_btn_infook_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget *wgt_main;

  /* get the main window & the parent of the main window*/
	wgt_main     = GTK_WIDGET(GTK_WIDGET(GTK_WIDGET(GTK_WIDGET( button )->parent)->parent)->parent);

  /* hide this dialog */
  gtk_widget_unrealize( wgt_main );
  gtk_widget_hide( wgt_main );
  gtk_widget_destroy( wgt_main );
  
}

void
on_wnd_main_destroy                    (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_main_quit();
}



