/*
 * whiteBOX.c
 * whiteBOX global routines
 *
 * Author: 	Luke Freeman (except where specified)
 * Copyright 2001, Luke Freeman.
 *
 * This file is part of whiteBOX.
 *
 * whiteBOX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * whiteBOX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 *
 *
 * Bluefish HTML Editor
 *
 * Copyright (C) 1999-2001 Olivier Sessink
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <strings.h>

#ifdef HAVE_SYS_DIR_H
    #include <sys/dir.h>
#else
    #ifdef HAVE_SYS_NDIR_H
        #include <sys/ndir.h>
    #else
        #include <dirent.h>
    #endif
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "whiteBOX.h"

#include "plugins/menu/menu.h"
#include "plugins/style/styles.h"

wbTreeIcon *
get_tree_icon( int key )
{

  GdkColormap *colormap;         /* colourmap to use for icons */
  GdkBitmap *mask;               /* storage for icon mask */
   
  GdkPixmap *gdkpixmap = NULL;  /* pixmap to return */
  
  char *file_name = NULL;       /* pixmap file filename */

  wbTreeIcon *tmp_icon = calloc( 1, sizeof( wbTreeIcon ));
  
  colormap = gdk_colormap_get_system();

  switch( key )
  {
    case SUBMENU:
      file_name = PACKAGE_DATA_DIR "/pixmaps/stock_open-16.xpm"; break;
    
    case APPLICATION:
      file_name = PACKAGE_DATA_DIR "/pixmaps/stock_exec-16.xpm"; break;
    
    case STYLE:
	  file_name = PACKAGE_DATA_DIR "/pixmaps/stock_insert-image-16.xpm"; break;
	
	case STYLESDIR:
      file_name = PACKAGE_DATA_DIR "/pixmaps/stock_style-dir-16.xpm"; break;
    
	case STYLESMENU:
	  file_name = PACKAGE_DATA_DIR "/pixmaps/stock_style-dir-16.xpm"; break;
	
    case WORKSPACES:
      file_name = PACKAGE_DATA_DIR "/pixmaps/stock_new-16.xpm"; break;    
    
    case NONOP:
      file_name = PACKAGE_DATA_DIR "/pixmaps/stock_broken_image-16.xpm"; break;
    
    case RESTART:
      file_name = PACKAGE_DATA_DIR "/pixmaps/stock_refresh-16.xpm"; break;    
    
    case RECONFIG:
      file_name = PACKAGE_DATA_DIR "/pixmaps/stock_convert-16.xpm"; break;
    
    case CONFIG:
      file_name = PACKAGE_DATA_DIR "/pixmaps/stock_preferences-16.xpm"; break;
    
    case EXIT:
      file_name = PACKAGE_DATA_DIR "/pixmaps/stock_exit-16.xpm"; break;   
    
  }

  gdkpixmap = gdk_pixmap_colormap_create_from_xpm( NULL, colormap, &mask,
                NULL, file_name );          
  
  tmp_icon->icon_xpm  = gdkpixmap;
  tmp_icon->icon_mask = mask;
  
  return tmp_icon;
}

int 
dir_stat( gchar *file )
{

  struct stat s;
  
  if( !file ) 
    return FALSE;
  
  if (( stat( file, &s ) == 0 ) && ( S_ISDIR( s.st_mode )))
    return TRUE;
  else
    return FALSE;
  
}



/*
 * Global calls for bluefish file dialogs (all code below)
 * Download bluefish at http://bluefish.openoffice.nl
 * 
 * Copyright 1999 - 2001, Olivier Sessink. 
 * Licenced under GPL (see above notice for more details).
 */

gint free_stringlist(GList * which_list)
{
	GList *tmplist;

	tmplist = g_list_first(which_list);
	while (tmplist != NULL) {

		g_free(tmplist->data);
		tmplist = g_list_next(tmplist);
	}

	g_list_free(which_list);
	which_list = NULL;
	return 1;
}

void window_destroy(GtkWidget * windowname)
{
	
	gtk_signal_handlers_destroy(GTK_OBJECT(windowname));
	gtk_grab_remove(windowname);
	gtk_widget_destroy(windowname);
}

